@@ -24,6 +24,8 @@
 #undef DEBUG
 
 #define __NO_VERSION__
+#include "comedi_fops.h"
+
 #include <linux/module.h>
 #include <linux/errno.h>
 #include <linux/kernel.h>
@@ -36,12 +38,11 @@
 #include <linux/kmod.h>
 #include <linux/poll.h>
 #include <linux/init.h>
-#include <linux/devfs_fs_kernel.h>
 #include <linux/device.h>
 #include <linux/vmalloc.h>
 #include <linux/fs.h>
-
 #include <linux/comedidev.h>
+#include <linux/cdev.h>
 
 #include <asm/io.h>
 #include <asm/uaccess.h>
@@ -60,9 +61,9 @@ module_param(comedi_debug, int, 0644);
 comedi_device *comedi_devices;
 spinlock_t big_comedi_lock; /* Dynamic initialization */
 
-static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg,unsigned int minor);
+static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg);
 static int do_bufconfig_ioctl(comedi_device *dev,void *arg);
-static int do_devinfo_ioctl(comedi_device *dev,comedi_devinfo *arg);
+static int do_devinfo_ioctl(comedi_device *dev,comedi_devinfo *arg, struct file *file);
 static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file);
 static int do_chaninfo_ioctl(comedi_device *dev,comedi_chaninfo *arg);
 static int do_bufinfo_ioctl(comedi_device *dev,void *arg);
@@ -79,18 +80,17 @@ void do_become_nonbusy(comedi_device *dev,comedi_subdevice *s);
 static int do_cancel(comedi_device *dev,comedi_subdevice *s);
 
 static int comedi_fasync (int fd, struct file *file, int on);
-static void init_async_buf( comedi_async *async );
 
 static int comedi_ioctl(struct inode * inode,struct file * file,
-	unsigned int cmd,unsigned long arg)
+	unsigned int cmd, unsigned long arg)
 {
-	unsigned int minor=MINOR(inode->i_rdev);
-	comedi_device *dev=comedi_get_device_by_minor(minor);
+	const unsigned minor = iminor(inode);
+	comedi_device *dev = comedi_get_device_by_minor(minor);
 
 	/* Device config is special, because it must work on
 	 * an unconfigured device. */
 	if(cmd==COMEDI_DEVCONFIG){
-		return do_devconfig_ioctl(dev,(void *)arg,minor);
+		return do_devconfig_ioctl(dev,(void *)arg);
 	}
 
 	if(!dev->attached){
@@ -103,7 +103,7 @@ static int comedi_ioctl(struct inode * inode,struct file * file,
 	case COMEDI_BUFCONFIG:
 		return do_bufconfig_ioctl(dev,(void*)arg);
 	case COMEDI_DEVINFO:
-		return do_devinfo_ioctl(dev,(void *)arg);
+		return do_devinfo_ioctl(dev,(void *)arg, file);
 	case COMEDI_SUBDINFO:
 		return do_subdinfo_ioctl(dev,(void *)arg,file);
 	case COMEDI_CHANINFO:
@@ -147,7 +147,7 @@ static int comedi_ioctl(struct inode * inode,struct file * file,
 	writes:
 		none
 */
-static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg, unsigned int minor)
+static int do_devconfig_ioctl(comedi_device *dev, comedi_devconfig *arg)
 {
 	comedi_devconfig it;
 	int ret;
@@ -296,9 +296,12 @@ copyback:
 		devinfo structure
 
 */
-static int do_devinfo_ioctl(comedi_device *dev,comedi_devinfo *arg)
+static int do_devinfo_ioctl(comedi_device *dev, comedi_devinfo *arg, struct file *file)
 {
 	comedi_devinfo devinfo;
+	const unsigned minor = iminor(file->f_dentry->d_inode);
+	comedi_subdevice *read_subdev = comedi_get_read_subdevice(dev, minor);
+	comedi_subdevice *write_subdev = comedi_get_write_subdevice(dev, minor);
 
 	memset(&devinfo,0,sizeof(devinfo));
 
@@ -308,13 +311,13 @@ static int do_devinfo_ioctl(comedi_device *dev,comedi_devinfo *arg)
 	memcpy(devinfo.driver_name,dev->driver->driver_name,COMEDI_NAMELEN);
 	memcpy(devinfo.board_name,dev->board_name,COMEDI_NAMELEN);
 
-	if(dev->read_subdev){
-		devinfo.read_subdevice = dev->read_subdev - dev->subdevices;
+	if(read_subdev){
+		devinfo.read_subdevice = read_subdev - dev->subdevices;
 	}else{
 		devinfo.read_subdevice = -1;
 	}
-	if(dev->write_subdev){
-		devinfo.write_subdevice = dev->write_subdev - dev->subdevices;
+	if(write_subdev){
+		devinfo.write_subdevice = write_subdev - dev->subdevices;
 	}else{
 		devinfo.write_subdevice = -1;
 	}
@@ -347,12 +350,10 @@ static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file)
 	comedi_subdevice *s;
 
 
-	tmp=kmalloc(dev->n_subdevices*sizeof(comedi_subdinfo),GFP_KERNEL);
+	tmp=kcalloc(dev->n_subdevices,sizeof(comedi_subdinfo),GFP_KERNEL);
 	if(!tmp)
 		return -ENOMEM;
 
-	memset(tmp,0,sizeof(comedi_subdinfo)*dev->n_subdevices);
-
 	/* fill subdinfo structs */
 	for(i=0;i<dev->n_subdevices;i++){
 		s=dev->subdevices+i;
@@ -361,13 +362,15 @@ static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file)
 		us->type		= s->type;
 		us->n_chan		= s->n_chan;
 		us->subd_flags		= s->subdev_flags;
+		if(comedi_get_subdevice_runflags(s) & SRF_RUNNING)
+			us->subd_flags |= SDF_RUNNING;
 #define TIMER_nanosec 5	/* backwards compatibility */
 		us->timer_type		= TIMER_nanosec;
 		us->len_chanlist	= s->len_chanlist;
 		us->maxdata		= s->maxdata;
 		if(s->range_table){
-			us->range_type	= (dev->minor<<28)|(i<<24)|(0<<16)|
-					(s->range_table->length);
+			us->range_type	= (dev->minor << 28) | (i << 24) | (0 << 16) |
+				(s->range_table->length);
 		}else{
 			us->range_type	= 0; /* XXX */
 		}
@@ -447,7 +450,7 @@ static int do_chaninfo_ioctl(comedi_device *dev,comedi_chaninfo *arg)
 		for(i=0;i<s->n_chan;i++){
 			int x;
 
-			x=(dev->minor<<28)|(it.subdev<<24)|(i<<16)|
+			x=(dev->minor << 28) | (it.subdev << 24) | (i << 16) |
 				(s->range_table_list[i]->length);
 			put_user(x,it.rangelist+i);
 		}
@@ -484,10 +487,8 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 	if(bi.subdevice >= dev->n_subdevices || bi.subdevice < 0)
 		return -EINVAL;
 
-	s=dev->subdevices + bi.subdevice;
-	async=s->async;
-
-	if(s!=dev->read_subdev && s!=dev->write_subdev)return -EINVAL;
+	s = dev->subdevices + bi.subdevice;
+	async = s->async;
 
 	if(!async){
 		DPRINTK("subdevice does not have async capability\n");
@@ -498,19 +499,18 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 		goto copyback;
 	}
 
-	if(bi.bytes_read && s==dev->read_subdev){
+	if(bi.bytes_read && (s->subdev_flags & SDF_CMD_READ)){
+		bi.bytes_read = comedi_buf_read_alloc(async, bi.bytes_read);
 		comedi_buf_read_free(async, bi.bytes_read);
 
-		if(!(s->subdev_flags&SDF_RUNNING) &&
-		   !(s->runflags & SRF_ERROR) &&
-		   async->buf_write_count==async->buf_read_count){
+		if(!(comedi_get_subdevice_runflags(s) & (SRF_ERROR | SRF_RUNNING)) &&
+		   async->buf_write_count == async->buf_read_count){
 			do_become_nonbusy(dev,s);
 		}
 	}
 
-	if(bi.bytes_written && s==dev->write_subdev){
+	if(bi.bytes_written && (s->subdev_flags & SDF_CMD_WRITE)){
 		bi.bytes_written = comedi_buf_write_alloc( async, bi.bytes_written );
-		comedi_buf_munge(dev, s, async->buf_write_alloc_count - async->munge_count);
 		comedi_buf_write_free(async, bi.bytes_written);
 	}
 
@@ -518,12 +518,6 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 	bi.buf_write_ptr = async->buf_write_ptr;
 	bi.buf_read_count = async->buf_read_count;
 	bi.buf_read_ptr = async->buf_read_ptr;
-	if(s==dev->read_subdev){
-		unsigned int n_munge_bytes;
-
-		n_munge_bytes = bi.buf_write_count - s->async->munge_count;
-		comedi_buf_munge(dev, s, n_munge_bytes);
-	}
 
 copyback:
 	if(copy_to_user(arg, &bi, sizeof(comedi_bufinfo)))
@@ -598,11 +592,6 @@ static int do_insnlist_ioctl(comedi_device *dev,void *arg,void *file)
 		}
 		ret = parse_insn(dev,insns+i,data,file);
 		if(ret<0)goto error;
-		if(ret!=insns[i].n){
-			printk("BUG: result of insn != insn.n\n");
-			ret=-EINVAL;
-			goto error;
-		}
 		if(insns[i].insn&INSN_MASK_READ){
 			if(copy_to_user(insns[i].data,data,
 					insns[i].n*sizeof(lsampl_t))){
@@ -629,23 +618,41 @@ static int check_insn_config_length(comedi_insn *insn, lsampl_t *data)
 	{
 	case INSN_CONFIG_DIO_OUTPUT:
 	case INSN_CONFIG_DIO_INPUT:
+	case INSN_CONFIG_DISARM:
+	case INSN_CONFIG_RESET:
 		if(insn->n == 1) return 0;
 		break;
+	case INSN_CONFIG_ARM:
 	case INSN_CONFIG_DIO_QUERY:
 	case INSN_CONFIG_BLOCK_SIZE:
+	case INSN_CONFIG_FILTER:
 	case INSN_CONFIG_SERIAL_CLOCK:
 	case INSN_CONFIG_BIDIRECTIONAL_DATA:
-	case INSN_CONFIG_SET_RTSI_CLOCK_MODE:
 	case INSN_CONFIG_ALT_SOURCE:
-		if( insn->n == 2 ) return 0;
+	case INSN_CONFIG_SET_COUNTER_MODE:
+	case INSN_CONFIG_8254_READ_STATUS:
+	case INSN_CONFIG_SET_ROUTING:
+	case INSN_CONFIG_GET_ROUTING:
+		if(insn->n == 2) return 0;
+		break;
+	case INSN_CONFIG_SET_GATE_SRC:
+	case INSN_CONFIG_GET_GATE_SRC:
+	case INSN_CONFIG_SET_CLOCK_SRC:
+	case INSN_CONFIG_GET_CLOCK_SRC:
+	case INSN_CONFIG_SET_OTHER_SRC:
+	case INSN_CONFIG_GET_COUNTER_STATUS:
+		if(insn->n == 3) return 0;
 		break;
 	case INSN_CONFIG_PWM_OUTPUT:
+	case INSN_CONFIG_ANALOG_TRIG:
 		if(insn->n == 5) return 0;
 		break;
 	//by default we allow the insn since we don't have checks for all possible cases yet
 	default:
-		rt_printk("comedi: no check for data length of config insn id %i implemented.  Assuming n=%i is correct.\n",
-				data[0], insn->n);
+		rt_printk("comedi: no check for data length of config insn id %i is implemented.\n"
+			" Add a check to %s in %s.\n"
+			" Assuming n=%i is correct.\n",
+			data[0], __FUNCTION__, __FILE__, insn->n);
 		return 0;
 		break;
 	}
@@ -656,6 +663,7 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
 {
 	comedi_subdevice *s;
 	int ret = 0;
+	int i;
 
 	if(insn->insn&INSN_MASK_SPECIAL){
 		/* a non-subdevice instruction */
@@ -753,10 +761,19 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
 				ret=s->insn_read(dev,s,insn,data);
 				break;
 			case INSN_WRITE:
-				//XXX check against subdevice's maxdata
-				ret=s->insn_write(dev,s,insn,data);
+				for(i = 0; i < insn->n; ++i)
+				{
+					if(data[i] > s->maxdata) ret = -EINVAL;
+				}
+				if(ret == 0)
+					ret = s->insn_write(dev,s,insn,data);
 				break;
 			case INSN_BITS:
+				if(insn->n != 2)
+				{
+					ret=-EINVAL;
+					break;
+				}
 				ret=s->insn_bits(dev,s,insn,data);
 				break;
 			case INSN_CONFIG:
@@ -875,7 +892,7 @@ static int do_cmd_ioctl(comedi_device *dev,void *arg,void *file)
 		return -EIO;
 	}
 
-	if(!s->do_cmd || !s->async){
+	if(!s->do_cmd || !s->do_cmdtest || !s->async){
 		DPRINTK("subdevice %i does not support commands\n", user_cmd.subdev);
 		return -EIO;
 	}
@@ -946,26 +963,23 @@ static int do_cmd_ioctl(comedi_device *dev,void *arg,void *file)
 		goto cleanup;
 	}
 
-	init_async_buf( async );
+	comedi_reset_async_buf( async );
 
 	async->cb_mask = COMEDI_CB_EOA | COMEDI_CB_BLOCK | COMEDI_CB_ERROR | COMEDI_CB_OVERFLOW;
 	if(async->cmd.flags & TRIG_WAKE_EOS){
 		async->cb_mask |= COMEDI_CB_EOS;
 	}
 
-	s->runflags=SRF_USER;
-
-	s->subdev_flags|=SDF_RUNNING;
+	comedi_set_subdevice_runflags(s, ~0, SRF_USER | SRF_RUNNING);
 
 #ifdef CONFIG_COMEDI_RT
 	if(async->cmd.flags&TRIG_RT){
 		comedi_switch_to_rt(dev);
-		s->runflags |= SRF_RT;
+		comedi_set_subdevice_runflags(s, SRF_RT, SRF_RT);
 	}
 #endif
 
 	ret=s->do_cmd(dev,s);
-
 	if(ret==0)return 0;
 
 cleanup:
@@ -1016,7 +1030,7 @@ static int do_cmdtest_ioctl(comedi_device *dev,void *arg,void *file)
 		return -EIO;
 	}
 
-	if(!s->do_cmd){
+	if(!s->do_cmd || !s->do_cmdtest){
 		DPRINTK("subdevice %i does not support commands\n", user_cmd.subdev);
 		return -EIO;
 	}
@@ -1182,6 +1196,8 @@ static int do_cancel_ioctl(comedi_device *dev,unsigned int arg,void *file)
 	if(arg>=dev->n_subdevices)
 		return -EINVAL;
 	s=dev->subdevices+arg;
+	if(s->async == NULL)
+		return -EINVAL;
 
 	if(s->lock && s->lock!=file)
 		return -EACCES;
@@ -1235,7 +1251,7 @@ static int do_cancel(comedi_device *dev,comedi_subdevice *s)
 {
 	int ret=0;
 
-	if((s->subdev_flags&SDF_RUNNING) && s->cancel)
+	if((comedi_get_subdevice_runflags(s) & SRF_RUNNING) && s->cancel)
 		ret=s->cancel(dev,s);
 
 	do_become_nonbusy(dev,s);
@@ -1244,8 +1260,6 @@ static int do_cancel(comedi_device *dev,comedi_subdevice *s)
 }
 
 
-#define RDEV_OF_FILE(x)        ((x)->f_dentry->d_inode->i_rdev)
-
 void comedi_unmap(struct vm_area_struct *area)
 {
 	comedi_async *async;
@@ -1261,25 +1275,29 @@ static struct vm_operations_struct comedi_vm_ops={
 
 static int comedi_mmap(struct file * file, struct vm_area_struct *vma)
 {
-	unsigned int minor=MINOR(RDEV_OF_FILE(file));
-	comedi_device *dev=comedi_get_device_by_minor(minor);
+	const unsigned minor = iminor(file->f_dentry->d_inode);
+	comedi_device *dev = comedi_get_device_by_minor(minor);
 	comedi_async *async = NULL;
 	unsigned long start = vma->vm_start;
 	unsigned long size;
 	int n_pages;
 	int i;
+	comedi_subdevice *s;
 
 	if(!dev->attached)
 	{
 		DPRINTK("no driver configured on comedi%i\n", dev->minor);
 		return -ENODEV;
 	}
-
-	if(vma->vm_flags & VM_WRITE){
-		async=dev->write_subdev->async;
-	}else{
-		async=dev->read_subdev->async;
+	if(vma->vm_flags & VM_WRITE)
+	{
+		s = comedi_get_write_subdevice(dev, minor);
+	}else
+	{
+		s = comedi_get_read_subdevice(dev, minor);
 	}
+	if(s == NULL) return -EINVAL;
+	async = s->async;
 	if(async==NULL){
 		return -EINVAL;
 	}
@@ -1296,8 +1314,8 @@ static int comedi_mmap(struct file * file, struct vm_area_struct *vma)
 		return -EFAULT;
 
 	n_pages = size >> PAGE_SHIFT;
-	for(i=0;i<n_pages;i++){
-		if(remap_pfn_range(vma, start, __pa(async->buf_page_list[i]) >> PAGE_SHIFT,
+	for(i = 0; i < n_pages; ++i){
+		if(remap_pfn_range(vma, start, page_to_pfn(virt_to_page(async->buf_page_list[i].virt_addr)),
 				PAGE_SIZE, PAGE_SHARED)){
 			return -EAGAIN;
 		}
@@ -1315,12 +1333,11 @@ static int comedi_mmap(struct file * file, struct vm_area_struct *vma)
 
 static unsigned int comedi_poll(struct file *file, poll_table * wait)
 {
-	comedi_device *dev;
-	comedi_subdevice *s;
-	comedi_async *async;
 	unsigned int mask;
-
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	const unsigned minor = iminor(file->f_dentry->d_inode);
+	comedi_device *dev = comedi_get_device_by_minor(minor);
+	comedi_subdevice *read_subdev = comedi_get_read_subdevice(dev, minor);
+	comedi_subdevice *write_subdev = comedi_get_write_subdevice(dev, minor);
 
 	if(!dev->attached)
 	{
@@ -1328,24 +1345,20 @@ static unsigned int comedi_poll(struct file *file, poll_table * wait)
 		return -ENODEV;
 	}
 
-	poll_wait(file, &dev->read_wait, wait);
-	poll_wait(file, &dev->write_wait, wait);
 	mask = 0;
-	if(dev->read_subdev && dev->read_subdev->async){
-		s = dev->read_subdev;
-		async = s->async;
-		if(!s->busy
-		   || comedi_buf_read_n_available(s)>0
-		   || !(s->subdev_flags&SDF_RUNNING)){
+	if(read_subdev){
+		poll_wait(file, &read_subdev->async->wait_head, wait);
+		if(!read_subdev->busy
+			|| comedi_buf_read_n_available(read_subdev->async) > 0
+			|| !(comedi_get_subdevice_runflags(read_subdev) & SRF_RUNNING)){
 			mask |= POLLIN | POLLRDNORM;
 		}
 	}
-	if(dev->write_subdev && dev->write_subdev->async){
-		s = dev->write_subdev;
-		async = s->async;
-		if(!s->busy
-		   || !(s->subdev_flags&SDF_RUNNING)
-		   || comedi_buf_write_n_available(s)>0){
+	if(write_subdev){
+		poll_wait(file, &write_subdev->async->wait_head, wait);
+		if(!write_subdev->busy
+			|| !(comedi_get_subdevice_runflags(write_subdev) & SRF_RUNNING)
+			|| comedi_buf_write_n_available(write_subdev->async) > 0){
 			mask |= POLLOUT | POLLWRNORM;
 		}
 	}
@@ -1360,30 +1373,32 @@ static ssize_t comedi_write(struct file *file,const char *buf,size_t nbytes,loff
 	comedi_async *async;
 	int n,m,count=0,retval=0;
 	DECLARE_WAITQUEUE(wait,current);
+	const unsigned minor = iminor(file->f_dentry->d_inode);
 
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
-
+	dev = comedi_get_device_by_minor(minor);
 	if(!dev->attached)
 	{
 		DPRINTK("no driver configured on comedi%i\n", dev->minor);
 		return -ENODEV;
 	}
 
-	if(dev->write_subdev == NULL)return -EIO;
-	s = dev->write_subdev;
+	s = comedi_get_write_subdevice(dev, minor);
+	if(s == NULL)
+		return -EIO;
 	async = s->async;
 
 	if(!nbytes)return 0;
 
 	if(!s->busy)
+	{
 		return 0;
+	}
 
 	if(s->busy != file)
 		return -EACCES;
-
-	add_wait_queue(&dev->write_wait,&wait);
+	add_wait_queue(&async->wait_head, &wait);
 	while(nbytes>0 && !retval){
-		current->state=TASK_INTERRUPTIBLE;
+		set_current_state(TASK_INTERRUPTIBLE);
 
 		n=nbytes;
 
@@ -1391,8 +1406,11 @@ static ssize_t comedi_write(struct file *file,const char *buf,size_t nbytes,loff
 		if(async->buf_write_ptr + m > async->prealloc_bufsz){
 			m = async->prealloc_bufsz - async->buf_write_ptr;
 		}
-		m = comedi_buf_write_alloc(async, m);
-
+		comedi_buf_write_alloc(async, async->prealloc_bufsz);
+		if(m > comedi_buf_write_n_allocated(async))
+		{
+			m = comedi_buf_write_n_allocated(async);
+		}
 		if(m < n) n = m;
 
 		if(n==0){
@@ -1400,12 +1418,8 @@ static ssize_t comedi_write(struct file *file,const char *buf,size_t nbytes,loff
 				retval=-EAGAIN;
 				break;
 			}
-			if(signal_pending(current)){
-				retval=-ERESTARTSYS;
-				break;
-			}
-			if(!(s->subdev_flags&SDF_RUNNING)){
-				if(s->runflags & SRF_ERROR){
+			if(!(comedi_get_subdevice_runflags(s) & SRF_RUNNING)){
+				if(comedi_get_subdevice_runflags(s) & SRF_ERROR){
 					retval = -EPIPE;
 				}else{
 					retval = 0;
@@ -1413,6 +1427,10 @@ static ssize_t comedi_write(struct file *file,const char *buf,size_t nbytes,loff
 				do_become_nonbusy(dev,s);
 				break;
 			}
+			if(signal_pending(current)){
+				retval=-ERESTARTSYS;
+				break;
+			}
 			schedule();
 			continue;
 		}
@@ -1423,7 +1441,6 @@ static ssize_t comedi_write(struct file *file,const char *buf,size_t nbytes,loff
 			n -= m;
 			retval = -EFAULT;
 		}
-		comedi_buf_munge(dev, s, async->buf_write_alloc_count - async->munge_count);
 		comedi_buf_write_free(async, n);
 
 		count+=n;
@@ -1432,8 +1449,8 @@ static ssize_t comedi_write(struct file *file,const char *buf,size_t nbytes,loff
 		buf+=n;
 		break;	/* makes device work like a pipe */
 	}
-	current->state=TASK_RUNNING;
-	remove_wait_queue(&dev->write_wait,&wait);
+	set_current_state(TASK_RUNNING);
+	remove_wait_queue(&async->wait_head, &wait);
 
 	return (count ? count : retval);
 }
@@ -1441,13 +1458,12 @@ static ssize_t comedi_write(struct file *file,const char *buf,size_t nbytes,loff
 
 static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *offset)
 {
-	comedi_device *dev;
 	comedi_subdevice *s;
 	comedi_async *async;
 	int n,m,count=0,retval=0;
 	DECLARE_WAITQUEUE(wait,current);
-
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	const unsigned minor = iminor(file->f_dentry->d_inode);
+	comedi_device *dev = comedi_get_device_by_minor(minor);
 
 	if(!dev->attached)
 	{
@@ -1455,10 +1471,9 @@ static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *of
 		return -ENODEV;
 	}
 
-	s = dev->read_subdev;
-	if(s == NULL)return -EIO;
+	s = comedi_get_read_subdevice(dev, minor);
+	if(s == NULL) return -EIO;
 	async = s->async;
-
 	if(!nbytes)return 0;
 
 	if(!s->busy)
@@ -1467,13 +1482,13 @@ static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *of
 	if(s->busy != file)
 		return -EACCES;
 
-	add_wait_queue(&dev->read_wait,&wait);
+	add_wait_queue(&async->wait_head, &wait);
 	while(nbytes>0 && !retval){
-		current->state=TASK_INTERRUPTIBLE;
+		set_current_state(TASK_INTERRUPTIBLE);
 
 		n=nbytes;
 
-		m = comedi_buf_read_n_available(s);
+		m = comedi_buf_read_n_available(async);
 //printk("%d available\n",m);
 		if(async->buf_read_ptr + m > async->prealloc_bufsz){
 			m = async->prealloc_bufsz - async->buf_read_ptr;
@@ -1482,19 +1497,19 @@ static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *of
 		if(m<n)n=m;
 
 		if(n==0){
-			if(!(s->subdev_flags&SDF_RUNNING)){
+			if(file->f_flags&O_NONBLOCK){
+				retval=-EAGAIN;
+				break;
+			}
+			if(!(comedi_get_subdevice_runflags(s) & SRF_RUNNING)){
 				do_become_nonbusy(dev,s);
-				if(s->runflags & SRF_ERROR){
+				if(comedi_get_subdevice_runflags(s) & SRF_ERROR){
 					retval = -EPIPE;
 				}else{
 					retval = 0;
 				}
 				break;
 			}
-			if(file->f_flags&O_NONBLOCK){
-				retval=-EAGAIN;
-				break;
-			}
 			if(signal_pending(current)){
 				retval=-ERESTARTSYS;
 				break;
@@ -1502,7 +1517,6 @@ static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *of
 			schedule();
 			continue;
 		}
-		comedi_buf_munge(dev, s, async->buf_write_count - async->munge_count);
 		m = copy_to_user(buf, async->prealloc_buf +
 			async->buf_read_ptr, n);
 		if(m){
@@ -1510,6 +1524,7 @@ static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *of
 			retval = -EFAULT;
 		}
 
+		comedi_buf_read_alloc(async, n);
 		comedi_buf_read_free(async, n);
 
 		count+=n;
@@ -1518,14 +1533,13 @@ static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *of
 		buf+=n;
 		break;	/* makes device work like a pipe */
 	}
-	if(!(s->subdev_flags&SDF_RUNNING) &&
-		!(s->runflags & SRF_ERROR) &&
+	if(!(comedi_get_subdevice_runflags(s) & (SRF_ERROR | SRF_RUNNING)) &&
 		async->buf_read_count - async->buf_write_count == 0)
 	{
 		do_become_nonbusy(dev,s);
 	}
-	current->state=TASK_RUNNING;
-	remove_wait_queue(&dev->read_wait,&wait);
+	set_current_state(TASK_RUNNING);
+	remove_wait_queue(&async->wait_head, &wait);
 
 	return (count ? count : retval);
 }
@@ -1536,18 +1550,17 @@ static ssize_t comedi_read(struct file * file,char *buf,size_t nbytes,loff_t *of
 void do_become_nonbusy(comedi_device *dev,comedi_subdevice *s)
 {
 	comedi_async *async = s->async;
-	/* we do this because it's useful for the non-standard cases */
-	s->subdev_flags &= ~SDF_RUNNING;
 
+	comedi_set_subdevice_runflags(s, SRF_RUNNING, 0);
 #ifdef CONFIG_COMEDI_RT
-	if(s->runflags&SRF_RT){
+	if(comedi_get_subdevice_runflags(s) & SRF_RT){
 		comedi_switch_to_non_rt(dev);
-		s->runflags &= ~SRF_RT;
+		comedi_set_subdevice_runflags(s, SRF_RT, 0);
 	}
 #endif
-
 	if(async){
-		init_async_buf( async );
+		comedi_reset_async_buf( async );
+		async->inttrig = NULL;
 	}else{
 		printk("BUG: (?) do_become_nonbusy called with async=0\n");
 	}
@@ -1555,48 +1568,16 @@ void do_become_nonbusy(comedi_device *dev,comedi_subdevice *s)
 	s->busy=NULL;
 }
 
-/* no chance that these will change soon */
-#define SEEK_SET 0
-#define SEEK_CUR 1
-#define SEEK_END 2
-
-static loff_t comedi_lseek(struct file *file,loff_t offset,int origin)
+static int comedi_open(struct inode *inode, struct file *file)
 {
-	comedi_device *dev;
-	loff_t new_offset;
-
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
-
-	switch(origin){
-	case SEEK_SET:
-		new_offset = offset;
-		break;
-	case SEEK_CUR:
-		new_offset = file->f_pos + offset;
-		break;
-	case SEEK_END:
-		new_offset = dev->n_subdevices + offset;
-		break;
-	default:
-		return -EINVAL;
-	}
-	if(new_offset<0 || new_offset >= dev->n_subdevices)
-		return -EINVAL;
-
-	return file->f_pos=new_offset;
-}
-
-static int comedi_open(struct inode *inode,struct file *file)
-{
-	unsigned int minor=MINOR(inode->i_rdev);
-	comedi_device *dev;
 	char mod[32];
-
-	if(minor>=COMEDI_NDEVICES){
+	const unsigned minor = iminor(inode);
+	comedi_device *dev = comedi_get_device_by_minor(minor);
+	if(dev == NULL)
+	{
 		DPRINTK("invalid minor number\n");
 		return -ENODEV;
 	}
-	dev=comedi_get_device_by_minor(minor);
 
 	/* This is slightly hacky, but we want module autoloading
 	 * to work for root.
@@ -1622,7 +1603,7 @@ static int comedi_open(struct inode *inode,struct file *file)
 
 	dev->in_request_module=1;
 
-	sprintf(mod,"char-major-%i-%i",COMEDI_MAJOR,minor);
+	sprintf(mod,"char-major-%i-%i", COMEDI_MAJOR, dev->minor);
 #ifdef CONFIG_KMOD
 	request_module(mod);
 #endif
@@ -1652,7 +1633,8 @@ ok:
 
 static int comedi_close(struct inode *inode,struct file *file)
 {
-	comedi_device *dev=comedi_get_device_by_minor(MINOR(inode->i_rdev));
+	const unsigned minor = iminor(inode);
+	comedi_device *dev = comedi_get_device_by_minor(minor);
 	comedi_subdevice *s = NULL;
 	int i;
 
@@ -1689,15 +1671,15 @@ static int comedi_close(struct inode *inode,struct file *file)
 
 static int comedi_fasync (int fd, struct file *file, int on)
 {
-	comedi_device *dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	const unsigned minor = iminor(file->f_dentry->d_inode);
+	comedi_device *dev = comedi_get_device_by_minor(minor);
 
 	return fasync_helper(fd,file,on,&dev->async_queue);
 }
 
 
-static struct file_operations comedi_fops={
+const struct file_operations comedi_fops={
 	owner		: THIS_MODULE,
-	llseek		: comedi_lseek,
 	ioctl		: comedi_ioctl,
 	open		: comedi_open,
 	release		: comedi_close,
@@ -1708,37 +1690,42 @@ static struct file_operations comedi_fops={
 	fasync		: comedi_fasync,
 };
 
-static struct class *comedi_class;
+struct class *comedi_class = NULL;
+static struct cdev comedi_cdev;
 
 static int __init comedi_init(void)
 {
 	int i;
+	int retval;
 
 	printk("comedi: version " COMEDI_RELEASE " - David Schleef <ds@schleef.org>\n");
 	spin_lock_init(&big_comedi_lock);
-	if(devfs_register_chrdev(COMEDI_MAJOR,"comedi",&comedi_fops)){
-		printk("comedi: unable to get major %d\n",COMEDI_MAJOR);
+	retval = register_chrdev_region(MKDEV(COMEDI_MAJOR, 0), COMEDI_NUM_MINORS, "comedi");
+	if(retval) return -EIO;
+	cdev_init(&comedi_cdev, &comedi_fops);
+	comedi_cdev.owner = THIS_MODULE;
+	kobject_set_name(&comedi_cdev.kobj, "comedi");
+	if(cdev_add(&comedi_cdev, MKDEV(COMEDI_MAJOR, 0), COMEDI_NUM_MINORS))
+	{
+		unregister_chrdev_region(MKDEV(COMEDI_MAJOR, 0), COMEDI_NUM_MINORS);
 		return -EIO;
 	}
 	comedi_class = class_create(THIS_MODULE, "comedi");
 	if(IS_ERR(comedi_class))
 	{
 		printk("comedi: failed to create class");
-		devfs_unregister_chrdev(COMEDI_MAJOR,"comedi");
+		unregister_chrdev_region(MKDEV(COMEDI_MAJOR, 0), COMEDI_NUM_MINORS);
+		cdev_del(&comedi_cdev);
 		return PTR_ERR(comedi_class);
 	}
-	comedi_devices=(comedi_device *)kmalloc(sizeof(comedi_device)*COMEDI_NDEVICES,GFP_KERNEL);
+	comedi_devices=kcalloc(COMEDI_NDEVICES,sizeof(comedi_device),GFP_KERNEL);
 	if(!comedi_devices)
 	{
+		unregister_chrdev_region(MKDEV(COMEDI_MAJOR, 0), COMEDI_NUM_MINORS);
+		cdev_del(&comedi_cdev);
 		class_destroy(comedi_class);
-		devfs_unregister_chrdev(COMEDI_MAJOR,"comedi");
 		return -ENOMEM;
 	}
-	memset(comedi_devices,0,sizeof(comedi_device)*COMEDI_NDEVICES);
-	for(i=0;i<COMEDI_NDEVICES;i++){
-		comedi_devices[i].minor=i;
-		spin_lock_init(&(comedi_devices[i].spinlock));
-	}
 
 	/* XXX requires /proc interface */
 	comedi_proc_init();
@@ -1746,9 +1733,10 @@ static int __init comedi_init(void)
 	for(i=0;i<COMEDI_NDEVICES;i++){
 		char name[20];
 		sprintf(name, "comedi%d", i);
-		devfs_register(NULL, name, DEVFS_FL_DEFAULT,
-			COMEDI_MAJOR, i, 0666 | S_IFCHR, &comedi_fops, NULL);
-		CLASS_DEVICE_CREATE(comedi_class, 0, MKDEV(COMEDI_MAJOR, i), NULL, "comedi%i", i);
+		comedi_devices[i].minor = i;
+		comedi_devices[i].class_dev = COMEDI_CLASS_DEVICE_CREATE(comedi_class, 0,
+			MKDEV(COMEDI_MAJOR, i), NULL, "comedi%i", i);
+		spin_lock_init(&comedi_devices[i].spinlock);
 	}
 
 	comedi_rt_init();
@@ -1763,25 +1751,25 @@ static void __exit comedi_cleanup(void)
 	if(MOD_IN_USE)
 		printk("comedi: module in use -- remove delayed\n");
 
-	for(i=0;i<COMEDI_NDEVICES;i++){
-		char name[20];
-		class_device_destroy(comedi_class, MKDEV(COMEDI_MAJOR, i));
-		sprintf(name, "comedi%d", i);
-		devfs_unregister(devfs_find_handle(NULL, name,
-			COMEDI_MAJOR, i, DEVFS_SPECIAL_CHR, 0));
-	}
-	class_destroy(comedi_class);
-	devfs_unregister_chrdev(COMEDI_MAJOR,"comedi");
-
-	comedi_proc_cleanup();
-
-	for(i=0;i<COMEDI_NDEVICES;i++){
+	for(i = 0; i < COMEDI_NDEVICES; i++){
 		comedi_device *dev;
 
-		dev=comedi_get_device_by_minor(i);
+		dev = comedi_devices + i;
 		if(dev->attached)
 			comedi_device_detach(dev);
 	}
+
+	for(i = 0; i < COMEDI_NDEVICES; i++){
+		char name[20];
+		class_device_destroy(comedi_class, MKDEV(COMEDI_MAJOR, i));
+		sprintf(name, "comedi%d", i);
+	}
+	class_destroy(comedi_class);
+	cdev_del(&comedi_cdev);
+	unregister_chrdev_region(MKDEV(COMEDI_MAJOR, 0), COMEDI_NUM_MINORS);
+
+	comedi_proc_cleanup();
+
 	kfree(comedi_devices);
 
 	comedi_rt_cleanup();
@@ -1792,55 +1780,56 @@ module_exit(comedi_cleanup);
 
 void comedi_error(const comedi_device *dev,const char *s)
 {
-	rt_printk("comedi%d: %s: %s\n",dev->minor,dev->driver->driver_name,s);
+	rt_printk("comedi%d: %s: %s\n", dev->minor, dev->driver->driver_name, s);
 }
 
-void comedi_event(comedi_device *dev,comedi_subdevice *s, unsigned int mask)
+void comedi_event(comedi_device *dev, comedi_subdevice *s)
 {
 	comedi_async *async = s->async;
+	unsigned runflags = 0;
+	unsigned runflags_mask = 0;
 
-	mask = s->async->events;
-	s->async->events = 0;
+	//DPRINTK("comedi_event 0x%x\n",mask);
 
-	//DPRINTK("comedi_event %x\n",mask);
-
-	if( (s->subdev_flags & SDF_RUNNING) == 0)
+	if((comedi_get_subdevice_runflags(s) & SRF_RUNNING) == 0)
 		return;
 
-	if(mask&(COMEDI_CB_EOA|COMEDI_CB_ERROR|COMEDI_CB_OVERFLOW)){
-		s->subdev_flags &= ~SDF_RUNNING;
+	if(s->async->events & (COMEDI_CB_EOA | COMEDI_CB_ERROR | COMEDI_CB_OVERFLOW)){
+		runflags_mask |= SRF_RUNNING;
 	}
-
 	/* remember if an error event has occured, so an error
 	 * can be returned the next time the user does a read() */
-	if(mask & (COMEDI_CB_ERROR|COMEDI_CB_OVERFLOW)){
-		s->runflags |= SRF_ERROR;
+	if(s->async->events & (COMEDI_CB_ERROR | COMEDI_CB_OVERFLOW)){
+		runflags_mask |= SRF_ERROR;
+		runflags |= SRF_ERROR;
 	}
-	if(async->cb_mask&mask){
-		if(s->runflags&SRF_USER){
+	if(runflags_mask)
+	{
+		/*sets SRF_ERROR and SRF_RUNNING together atomically */
+		comedi_set_subdevice_runflags(s, runflags_mask, runflags);
+	}
+
+	if(async->cb_mask & s->async->events){
+		if(comedi_get_subdevice_runflags(s) & SRF_USER){
 
 			if(dev->rt){
 #ifdef CONFIG_COMEDI_RT
 				// pend wake up
-				if(s==dev->read_subdev)
-					comedi_rt_pend_wakeup(&dev->read_wait);
-				if(s==dev->write_subdev)
-					comedi_rt_pend_wakeup(&dev->write_wait);
+				comedi_rt_pend_wakeup(&async->wait_head);
 #else
 				printk("BUG: comedi_event() code unreachable\n");
 #endif
 			}else{
-				if(s==dev->read_subdev){
-					wake_up_interruptible(&dev->read_wait);
+				wake_up_interruptible(&async->wait_head);
+				if(s->subdev_flags & SDF_CMD_READ){
 					kill_fasync(&dev->async_queue, SIGIO, POLL_IN);
 				}
-				if(s==dev->write_subdev){
-					wake_up_interruptible(&dev->write_wait);
+				if(s->subdev_flags & SDF_CMD_WRITE){
 					kill_fasync(&dev->async_queue, SIGIO, POLL_OUT);
 				}
 			}
 		}else{
-			if(async->cb_func)async->cb_func(mask,async->cb_arg);
+			if(async->cb_func)async->cb_func(s->async->events, async->cb_arg);
 			/* XXX bug here.  If subdevice A is rt, and
 			 * subdevice B tries to callback to a normal
 			 * linux kernel function, it will be at the
@@ -1848,23 +1837,26 @@ void comedi_event(comedi_device *dev,comedi_subdevice *s, unsigned int mask)
 			 * common, I'm not going to worry about it. */
 		}
 	}
+	s->async->events = 0;
 }
 
-static void init_async_buf( comedi_async *async )
+void comedi_set_subdevice_runflags(comedi_subdevice *s, unsigned mask, unsigned bits)
 {
-	async->buf_write_alloc_count = 0;
-	async->buf_write_count = 0;
-	async->buf_read_count = 0;
+	unsigned long flags;
 
-	async->buf_write_ptr = 0;
-	async->buf_read_ptr = 0;
-
-	async->cur_chan = 0;
-	async->scan_progress = 0;
-	async->munge_chan = 0;
-	async->munge_count = 0;
-	async->munge_ptr = 0;
-
-	async->events = 0;
+	comedi_spin_lock_irqsave(&s->runflags_lock, flags);
+	s->runflags &= ~mask;
+	s->runflags |= (bits & mask);
+	comedi_spin_unlock_irqrestore(&s->runflags_lock, flags);
 }
 
+unsigned comedi_get_subdevice_runflags(comedi_subdevice *s)
+{
+	unsigned long flags;
+	unsigned runflags;
+
+	comedi_spin_lock_irqsave(&s->runflags_lock, flags);
+	runflags = s->runflags;
+	comedi_spin_unlock_irqrestore(&s->runflags_lock, flags);
+	return runflags;
+}
