@@ -21,8 +21,11 @@
 
 */
 
+#define _GNU_SOURCE
 
 #define __NO_VERSION__
+#include "comedi_fops.h"
+#include <linux/device.h>
 #include <linux/module.h>
 #include <linux/errno.h>
 #include <linux/kernel.h>
@@ -36,6 +39,8 @@
 #include <linux/wrapper.h>
 #include <linux/highmem.h>  /* for SuSE brokenness */
 #include <linux/vmalloc.h>
+#include <linux/cdev.h>
+#include <linux/dma-mapping.h>
 
 #include <asm/io.h>
 #include <asm/system.h>
@@ -57,7 +62,7 @@ int comedi_modprobe(int minor)
 	return -EINVAL;
 }
 
-static void cleanup_device_allocations(comedi_device *dev)
+static void cleanup_device(comedi_device *dev)
 {
 	int i;
 	comedi_subdevice *s;
@@ -67,59 +72,67 @@ static void cleanup_device_allocations(comedi_device *dev)
 		for(i = 0; i < dev->n_subdevices; i++)
 		{
 			s = dev->subdevices + i;
+			if(s->class_dev)
+			{
+				unsigned minor = comedi_construct_minor_for_subdevice(dev, i);
+				dev_t devt = MKDEV(COMEDI_MAJOR, minor);
+				class_device_destroy(comedi_class, devt);
+			}
 			if(s->async)
 			{
 				comedi_buf_alloc(dev, s, 0);
-				if(s->buf_change) s->buf_change(dev, s, 0);
 				kfree(s->async);
 			}
 		}
 		kfree(dev->subdevices);
 		dev->subdevices = NULL;
+		dev->n_subdevices = 0;
 	}
 	if(dev->private)
 	{
 		kfree(dev->private);
 		dev->private = NULL;
 	}
+	module_put(dev->driver->module);
+	dev->driver = 0;
+	dev->board_name = NULL;
+	dev->board_ptr = NULL;
+	dev->iobase = 0;
+	dev->irq = 0;
+	dev->read_subdev = NULL;
+	dev->write_subdev = NULL;
+	dev->open = NULL;
+	dev->close = NULL;
+	comedi_set_hw_dev(dev, NULL);
+}
+
+static int __comedi_device_detach(comedi_device *dev)
+{
+	dev->attached = 0;
+	if(dev->driver){
+		dev->driver->detach(dev);
+	}else{
+		printk("BUG: dev->driver=NULL in comedi_device_detach()\n");
+	}
+	cleanup_device(dev);
+	return 0;
 }
 
 int comedi_device_detach(comedi_device *dev)
 {
 	if(!dev->attached)
 		return 0;
-
-	/* this is not correct for the kmod case? */
-	module_put(dev->driver->module);
-
-	dev->attached=0;
-
-	if(dev->driver){
-		dev->driver->detach(dev);
-	}else{
-		printk("BUG: dev->driver=NULL in comedi_device_detach()\n");
-	}
-
-	cleanup_device_allocations(dev);
-	return 0;
+	return __comedi_device_detach(dev);
 }
 
 int comedi_device_attach(comedi_device *dev,comedi_devconfig *it)
 {
 	comedi_driver *driv;
 	int ret;
-	int minor;
-	int use_count;
 
 	if(dev->attached)
 		return -EBUSY;
 
-	minor = dev->minor;
-	use_count = dev->use_count;
-	memset(dev,0,sizeof(comedi_device));
-	spin_lock_init(&dev->spinlock);
-	dev->minor=minor;
-	dev->use_count = use_count;
 	for(driv=comedi_drivers;driv;driv=driv->next){
 		if(!try_module_get( driv->module ))
 		{
@@ -127,7 +140,7 @@ int comedi_device_attach(comedi_device *dev,comedi_devconfig *it)
 			continue;
 		}
 		if(driv->num_names){
-			dev->board_ptr=comedi_recognize(driv, it->board_name);
+			dev->board_ptr = comedi_recognize(driv, it->board_name);
 			if(dev->board_ptr==NULL){
 				module_put( driv->module );
 				continue;
@@ -142,13 +155,10 @@ int comedi_device_attach(comedi_device *dev,comedi_devconfig *it)
 		dev->driver=driv;
 		ret=driv->attach(dev,it);
 		if(ret<0){
-			driv->detach(dev);
-			cleanup_device_allocations(dev);
-			module_put( driv->module );
+			__comedi_device_detach(dev);
 			return ret;
 		}
 		goto attached;
-		module_put( driv->module );
 	}
 
 	// recognize has failed if we get here
@@ -169,19 +179,15 @@ attached:
 	ret = postconfig(dev);
 	if(ret < 0)
 	{
-		driv->detach(dev);
-		cleanup_device_allocations(dev);
+		__comedi_device_detach(dev);
 		return ret;
 	}
 
-	init_waitqueue_head(&dev->read_wait);
-	init_waitqueue_head(&dev->write_wait);
-
 	if(!dev->board_name){
 		printk("BUG: dev->board_name=<%p>\n",dev->board_name);
 		dev->board_name="BUG";
 	}
-	mb();
+	smp_wmb();
 	dev->attached=1;
 
 	return 0;
@@ -204,7 +210,7 @@ int comedi_driver_unregister(comedi_driver *driver)
 	for(i=0;i<COMEDI_NDEVICES;i++){
 		comedi_device *dev;
 
-		dev=comedi_get_device_by_minor(i);
+		dev = comedi_devices + i;
 		if(dev->attached && dev->driver==driver){
 			if(dev->use_count)
 				printk("BUG! detaching device with use_count=%d\n",dev->use_count);
@@ -228,15 +234,7 @@ int comedi_driver_unregister(comedi_driver *driver)
 
 comedi_device *comedi_allocate_dev(comedi_driver *driver)
 {
-	comedi_device *dev;
-
-	// XXX we need to do actual allocation here.
-	
-	dev=comedi_get_device_by_minor(0);
-
-	dev->driver=driver;
-
-	return dev;
+	return NULL;
 }
 
 void comedi_deallocate_dev(comedi_device *dev)
@@ -261,13 +259,20 @@ static int postconfig(comedi_device *dev)
 			s->len_chanlist=1;
 
 		if(s->do_cmd){
-			async = kmalloc(sizeof(comedi_async), GFP_KERNEL);
+			unsigned minor;
+			dev_t devt;
+
+			BUG_ON((s->subdev_flags & (SDF_CMD_READ | SDF_CMD_WRITE)) == 0);
+			BUG_ON(!s->do_cmdtest);
+
+			async = kzalloc(sizeof(comedi_async), GFP_KERNEL);
 			if(async == NULL)
 			{
 				printk("failed to allocate async struct\n");
 				return -ENOMEM;
 			}
-			memset(async, 0, sizeof(comedi_async));
+			init_waitqueue_head(&async->wait_head);
+			async->subdevice = s;
 			s->async = async;
 
 #define DEFAULT_BUF_MAXSIZE (64*1024)
@@ -285,6 +290,10 @@ static int postconfig(comedi_device *dev)
 				ret = s->buf_change(dev,s,DEFAULT_BUF_SIZE);
 				if(ret < 0)return ret;
 			}
+			minor = comedi_construct_minor_for_subdevice(dev, i);
+			devt = MKDEV(COMEDI_MAJOR, minor);
+			s->class_dev = COMEDI_CLASS_DEVICE_CREATE(comedi_class, dev->class_dev,
+				devt, NULL, "comedi%i_sub%i", dev->minor, i);
 		}
 
 		if(!s->range_table && !s->range_table_list)
@@ -309,15 +318,13 @@ static int postconfig(comedi_device *dev)
 // generic recognize function for drivers that register their supported board names
 void *comedi_recognize(comedi_driver *driv, const char *name)
 {
-	unsigned int i = 0;
-	void *name_ptr;
-
-	name_ptr=driv->board_name;
+	unsigned i;
+	const char * const *name_ptr = driv->board_name;
 	for(i = 0; i < driv->num_names; i++)
 	{
-		if(strcmp(*(char **)name_ptr, name) == 0)
-			return name_ptr;
-		name_ptr += driv->offset;
+		if(strcmp(*name_ptr, name) == 0)
+			return (void*)name_ptr;
+		name_ptr = (const char * const *)((const char *)name_ptr + driv->offset);
 	}
 
 	return NULL;
@@ -326,15 +333,15 @@ void *comedi_recognize(comedi_driver *driv, const char *name)
 void comedi_report_boards(comedi_driver *driv)
 {
 	unsigned int i;
-	void *name_ptr;
+	const char * const *name_ptr;
 
 	printk("comedi: valid board names for %s driver are:\n", driv->driver_name);
 
-	name_ptr=driv->board_name;
+	name_ptr = driv->board_name;
 	for(i = 0; i < driv->num_names; i++)
 	{
-		printk(" %s\n", *(char **)name_ptr);
-		name_ptr += driv->offset;
+		printk(" %s\n", *name_ptr);
+		name_ptr = (const char **)((char *)name_ptr + driv->offset);
 	}
 
 	if(driv->num_names == 0)
@@ -356,32 +363,32 @@ static int insn_rw_emulate_bits(comedi_device *dev,comedi_subdevice *s,
 {
 	comedi_insn new_insn;
 	int ret;
+	static const unsigned channels_per_bitfield = 32;
+
+	unsigned chan = CR_CHAN(insn->chanspec);
+	const unsigned base_bitfield_channel = (chan < channels_per_bitfield) ? 0 : chan;
 	lsampl_t new_data[2];
-	unsigned int chan;
-
-	chan = CR_CHAN(insn->chanspec);
-
-	memset(&new_insn,0,sizeof(new_insn));
+	memset(new_data, 0, sizeof(new_data));
+	memset(&new_insn, 0, sizeof(new_insn));
 	new_insn.insn = INSN_BITS;
+	new_insn.chanspec = base_bitfield_channel;
 	new_insn.n = 2;
 	new_insn.data = new_data;
 	new_insn.subdev = insn->subdev;
 
-	if(insn->insn == INSN_WRITE){
+	if(insn->insn == INSN_WRITE)
+	{
 		if(!(s->subdev_flags & SDF_WRITABLE))
 			return -EINVAL;
-		new_data[0] = 1<<chan; /* mask */
-		new_data[1] = data[0]?(1<<chan):0; /* bits */
-	}else {
-		new_data[0] = 0;
-		new_data[1] = 0;
+		new_data[0] = 1 << (chan - base_bitfield_channel); /* mask */
+		new_data[1] = data[0] ? (1 << (chan - base_bitfield_channel)) : 0; /* bits */
 	}
 
 	ret = s->insn_bits(dev,s,&new_insn,new_data);
-	if(ret<0)return ret;
+	if(ret < 0) return ret;
 
 	if(insn->insn == INSN_READ){
-		data[0] = (new_data[1]>>chan)&1;
+		data[0] = (new_data[1] >> (chan - base_bitfield_channel)) & 1;
 	}
 
 	return 1;
@@ -394,7 +401,7 @@ static inline unsigned long uvirt_to_kva(pgd_t *pgd, unsigned long adr)
 	pmd_t *pmd;
 	pte_t *ptep, pte;
 	pud_t *pud;
-	
+
 	if(!pgd_none(*pgd)){
 		pud = pud_offset(pgd, adr);
 		pmd = pmd_offset(pud, adr);
@@ -426,47 +433,93 @@ int comedi_buf_alloc(comedi_device *dev, comedi_subdevice *s,
 {
 	comedi_async *async = s->async;
 
+	/* Round up new_size to multiple of PAGE_SIZE */
+	new_size = (new_size + PAGE_SIZE - 1) & PAGE_MASK;
+
 	/* if no change is required, do nothing */
 	if(async->prealloc_buf && async->prealloc_bufsz == new_size){
 		return 0;
 	}
-
-	if(async->prealloc_bufsz){
-		int i;
-		int n_pages = async->prealloc_bufsz >> PAGE_SHIFT;
-
-		for(i=0;i<n_pages;i++){
-			mem_map_unreserve(virt_to_page(__va(__pa(async->buf_page_list[i]))));
-		}
-
-		vfree(async->prealloc_buf);
+	// deallocate old buffer
+	if(async->prealloc_buf)
+	{
+		vunmap(async->prealloc_buf);
 		async->prealloc_buf = NULL;
-		kfree(async->buf_page_list);
-		async->buf_page_list = NULL;
+		async->prealloc_bufsz = 0;
 	}
-
+	if(async->buf_page_list)
+	{
+		unsigned i;
+		for(i = 0; i < async->n_buf_pages; ++i)
+		{
+			if(async->buf_page_list[i].virt_addr){
+				mem_map_unreserve(virt_to_page(async->buf_page_list[i].virt_addr));
+				if(s->async_dma_dir != DMA_NONE){
+					dma_free_coherent(dev->hw_dev, PAGE_SIZE,
+						async->buf_page_list[i].virt_addr, async->buf_page_list[i].dma_addr);
+				}else{
+					free_page((unsigned long)async->buf_page_list[i].virt_addr);
+				}
+			}
+		}
+		vfree(async->buf_page_list);
+		async->buf_page_list = NULL;
+		async->n_buf_pages = 0;
+	}
+	// allocate new buffer
 	if(new_size){
-		unsigned long adr;
-		int n_pages = new_size >> PAGE_SHIFT;
-		int i;
+		unsigned i = 0;
+		unsigned n_pages = new_size >> PAGE_SHIFT;
+		struct page** pages = NULL;
 
-		async->buf_page_list = kmalloc(sizeof(unsigned long)*n_pages, GFP_KERNEL);
-		async->prealloc_buf = vmalloc_32(new_size);
+		async->buf_page_list = vmalloc(sizeof(struct comedi_buf_page) * n_pages);
+		if(async->buf_page_list){
+			memset(async->buf_page_list, 0, sizeof(struct comedi_buf_page) * n_pages);
+			pages = vmalloc(sizeof(struct page*) * n_pages);
+		}
+		if(pages){
+			for(i = 0; i < n_pages; i++){
+				if(s->async_dma_dir != DMA_NONE){
+					async->buf_page_list[i].virt_addr = dma_alloc_coherent(dev->hw_dev,
+						PAGE_SIZE, &async->buf_page_list[i].dma_addr, GFP_KERNEL | __GFP_COMP);
+				}else{
+					async->buf_page_list[i].virt_addr = (void*)__get_free_page(GFP_KERNEL);
+				}
+				if(async->buf_page_list[i].virt_addr == NULL){
+					break;
+				}
+				mem_map_reserve(virt_to_page(async->buf_page_list[i].virt_addr));
+				pages[i] = virt_to_page(async->buf_page_list[i].virt_addr);
+			}
+		}
+		if(i == n_pages){
+			async->prealloc_buf = vmap(pages, n_pages, VM_MAP, PAGE_KERNEL_NOCACHE);
+		}
+		if(pages){
+			vfree(pages);
+		}
 		if(async->prealloc_buf == NULL){
-			async->prealloc_bufsz = 0;
-			kfree(async->buf_page_list);
+			/* Some allocation failed above. */
+			if(async->buf_page_list){
+				for(i = 0; i < n_pages; i++){
+					if(async->buf_page_list[i].virt_addr == NULL){
+						break;
+					}
+					mem_map_unreserve(virt_to_page(async->buf_page_list[i].virt_addr));
+					if(s->async_dma_dir != DMA_NONE){
+						dma_free_coherent(dev->hw_dev, PAGE_SIZE,
+							async->buf_page_list[i].virt_addr, async->buf_page_list[i].dma_addr);
+					}else{
+						free_page((unsigned long)async->buf_page_list[i].virt_addr);
+					}
+				}
+				vfree(async->buf_page_list);
+				async->buf_page_list = NULL;
+			}
 			return -ENOMEM;
 		}
-		memset(async->prealloc_buf,0,new_size);
-
-		adr = (unsigned long)async->prealloc_buf;
-		for(i=0;i<n_pages;i++){
-			async->buf_page_list[i] = kvirt_to_kva(adr);
-			mem_map_reserve(virt_to_page(__va(__pa(async->buf_page_list[i]))));
-			adr += PAGE_SIZE;
-		}
+		async->n_buf_pages = n_pages;
 	}
-
 	async->prealloc_bufsz = new_size;
 
 	return 0;
@@ -474,15 +527,21 @@ int comedi_buf_alloc(comedi_device *dev, comedi_subdevice *s,
 
 /* munging is applied to data by core as it passes between user
  * and kernel space */
-unsigned int comedi_buf_munge( comedi_device *dev, comedi_subdevice *s,
-	unsigned int num_bytes )
+unsigned int comedi_buf_munge(comedi_async *async,
+	unsigned int num_bytes)
 {
+	comedi_subdevice *s = async->subdevice;
 	unsigned int count = 0;
+	const unsigned num_sample_bytes = bytes_per_sample(s);
 
-	if( s->munge == NULL || ( s->async->cmd.flags & CMDF_RAWDATA ) )
-		return count;
+	if( s->munge == NULL || ( async->cmd.flags & CMDF_RAWDATA ) )
+	{
+		async->munge_count += num_bytes;
+		if((int)(async->munge_count - async->buf_write_count) > 0) BUG();
+		return num_bytes;
+	}
 	/* don't munge partial samples */
-	num_bytes -= num_bytes % bytes_per_sample(s);
+	num_bytes -= num_bytes % num_sample_bytes;
 	while( count < num_bytes )
 	{
 		int block_size;
@@ -493,33 +552,44 @@ unsigned int comedi_buf_munge( comedi_device *dev, comedi_subdevice *s,
 			rt_printk("%s: %s: bug! block_size is negative\n", __FILE__, __FUNCTION__);
 			break;
 		}
-		if( (int)(s->async->munge_ptr + block_size - s->async->prealloc_bufsz) > 0 )
-			block_size = s->async->prealloc_bufsz - s->async->munge_ptr;
+		if( (int)(async->munge_ptr + block_size - async->prealloc_bufsz) > 0 )
+			block_size = async->prealloc_bufsz - async->munge_ptr;
 
-		s->munge( dev, s, s->async->prealloc_buf + s->async->munge_ptr,
-			block_size, s->async->munge_chan );
+		s->munge(s->device, s, async->prealloc_buf + async->munge_ptr,
+			block_size, async->munge_chan );
 
-		s->async->munge_chan += block_size / bytes_per_sample( s );
-		s->async->munge_chan %= s->async->cmd.chanlist_len;
-		s->async->munge_count += block_size;
-		s->async->munge_ptr += block_size;
-		s->async->munge_ptr %= s->async->prealloc_bufsz;
+		smp_wmb(); //barrier insures data is munged in buffer before munge_count is incremented
+
+		async->munge_chan += block_size / num_sample_bytes;
+		async->munge_chan %= async->cmd.chanlist_len;
+		async->munge_count += block_size;
+		async->munge_ptr += block_size;
+		async->munge_ptr %= async->prealloc_bufsz;
 		count += block_size;
 	}
+	if((int)(async->munge_count - async->buf_write_count) > 0) BUG();
 	return count;
 }
 
-unsigned int comedi_buf_write_n_available(comedi_subdevice *s)
+unsigned int comedi_buf_write_n_available(comedi_async *async)
 {
-	comedi_async *async=s->async;
-	unsigned int free_end = async->buf_read_count + async->prealloc_bufsz;
+	unsigned int free_end;
 	unsigned int nbytes;
 
+	if(async == NULL) return 0;
+
+	free_end = async->buf_read_count + async->prealloc_bufsz;
 	nbytes = free_end - async->buf_write_alloc_count;
-	nbytes -= nbytes % bytes_per_sample(s);
+	nbytes -= nbytes % bytes_per_sample(async->subdevice);
+	/* barrier insures the read of buf_read_count in this
+	query occurs before any following writes to the buffer which
+	might be based on the return value from this query.
+	*/
+	smp_mb();
 	return nbytes;
 }
 
+/* allocates chunk for the writer from free buffer space */
 unsigned int comedi_buf_write_alloc(comedi_async *async, unsigned int nbytes)
 {
 	unsigned int free_end = async->buf_read_count + async->prealloc_bufsz;
@@ -527,12 +597,14 @@ unsigned int comedi_buf_write_alloc(comedi_async *async, unsigned int nbytes)
 	if((int)(async->buf_write_alloc_count + nbytes - free_end) > 0){
 		nbytes = free_end - async->buf_write_alloc_count;
 	}
-
 	async->buf_write_alloc_count += nbytes;
-
+	/* barrier insures the read of buf_read_count above occurs before
+	we write data to the write-alloc'ed buffer space */
+	smp_mb();
 	return nbytes;
 }
 
+/* allocates nothing unless it can completely fulfill the request */
 unsigned int comedi_buf_write_alloc_strict(comedi_async *async,
 	unsigned int nbytes)
 {
@@ -541,29 +613,58 @@ unsigned int comedi_buf_write_alloc_strict(comedi_async *async,
 	if((int)(async->buf_write_alloc_count + nbytes - free_end) > 0){
 		nbytes = 0;
 	}
-
 	async->buf_write_alloc_count += nbytes;
-
+	/* barrier insures the read of buf_read_count above occurs before
+	we write data to the write-alloc'ed buffer space */
+	smp_mb();
 	return nbytes;
 }
 
-/* transfers control of a chunk from writer to reader */
-void comedi_buf_write_free(comedi_async *async, unsigned int nbytes)
+/* transfers a chunk from writer to filled buffer space */
+unsigned comedi_buf_write_free(comedi_async *async, unsigned int nbytes)
 {
+	if((int)(async->buf_write_count + nbytes - async->buf_write_alloc_count) > 0)
+	{
+		rt_printk("comedi: attempted to write-free more bytes than have been write-allocated.\n");
+		nbytes = async->buf_write_alloc_count - async->buf_write_count;
+	}
 	async->buf_write_count += nbytes;
 	async->buf_write_ptr += nbytes;
+	comedi_buf_munge(async, async->buf_write_count - async->munge_count);
 	if(async->buf_write_ptr >= async->prealloc_bufsz){
 		async->buf_write_ptr %= async->prealloc_bufsz;
-		async->events |= COMEDI_CB_EOBUF;
 	}
+	return nbytes;
 }
 
-/* transfers control of a chunk from reader to free area */
-void comedi_buf_read_free(comedi_async *async, unsigned int nbytes)
+/* allocates a chunk for the reader from filled (and munged) buffer space */
+unsigned comedi_buf_read_alloc(comedi_async *async, unsigned nbytes)
 {
+	if((int)(async->buf_read_alloc_count + nbytes - async->munge_count) > 0)
+	{
+		nbytes = async->munge_count - async->buf_read_alloc_count;
+	}
+	async->buf_read_alloc_count += nbytes;
+	/* barrier insures read of munge_count occurs before we actually read
+	data out of buffer */
+	smp_rmb();
+	return nbytes;
+}
+
+/* transfers control of a chunk from reader to free buffer space */
+unsigned comedi_buf_read_free(comedi_async *async, unsigned int nbytes)
+{
+	// barrier insures data has been read out of buffer before read count is incremented
+	smp_mb();
+	if((int)(async->buf_read_count + nbytes - async->buf_read_alloc_count) > 0)
+	{
+		rt_printk("comedi: attempted to read-free more bytes than have been read-allocated.\n");
+		nbytes = async->buf_read_alloc_count - async->buf_read_count;
+	}
 	async->buf_read_count += nbytes;
 	async->buf_read_ptr += nbytes;
 	async->buf_read_ptr %= async->prealloc_bufsz;
+	return nbytes;
 }
 
 void comedi_buf_memcpy_to( comedi_async *async, unsigned int offset, const void *data,
@@ -590,7 +691,6 @@ void comedi_buf_memcpy_to( comedi_async *async, unsigned int offset, const void
 
 		write_ptr = 0;
 	}
-	barrier();
 }
 
 void comedi_buf_memcpy_from(comedi_async *async, unsigned int offset,
@@ -602,7 +702,6 @@ void comedi_buf_memcpy_from(comedi_async *async, unsigned int offset,
 	if( read_ptr >= async->prealloc_bufsz )
 		read_ptr %= async->prealloc_bufsz;
 
-	barrier();
 	while( nbytes )
 	{
 		unsigned int block_size;
@@ -621,29 +720,27 @@ void comedi_buf_memcpy_from(comedi_async *async, unsigned int offset,
 	}
 }
 
-static inline unsigned int _comedi_buf_read_n_available(comedi_async *async)
+unsigned int comedi_buf_read_n_available(comedi_async *async)
 {
-	return async->buf_write_count - async->buf_read_count;
-}
-
-unsigned int comedi_buf_read_n_available(comedi_subdevice *s)
-{
-	comedi_async *async = s->async;
-	unsigned int nbytes;
+	unsigned num_bytes;
 
 	if(async == NULL)
 		return 0;
-
-	nbytes = _comedi_buf_read_n_available(async);
-	nbytes -= nbytes % bytes_per_sample(s);
-	return nbytes;
+	num_bytes = async->munge_count - async->buf_read_count;
+	/* barrier insures the read of munge_count in this
+	query occurs before any following reads of the buffer which
+	might be based on the return value from this query.
+	*/
+	smp_rmb();
+	return num_bytes;
 }
 
 int comedi_buf_get(comedi_async *async, sampl_t *x)
 {
-	unsigned int n = _comedi_buf_read_n_available(async);
+	unsigned int n = comedi_buf_read_n_available(async);
 
-	if(n<sizeof(sampl_t))return 0;
+	if(n < sizeof(sampl_t)) return 0;
+	comedi_buf_read_alloc(async, sizeof(sampl_t));
 	*x = *(sampl_t *)(async->prealloc_buf + async->buf_read_ptr);
 	comedi_buf_read_free(async, sizeof(sampl_t));
 	return 1;
@@ -662,3 +759,22 @@ int comedi_buf_put(comedi_async *async, sampl_t x)
 	return 1;
 }
 
+void comedi_reset_async_buf(comedi_async *async)
+{
+	async->buf_write_alloc_count = 0;
+	async->buf_write_count = 0;
+	async->buf_read_alloc_count = 0;
+	async->buf_read_count = 0;
+
+	async->buf_write_ptr = 0;
+	async->buf_read_ptr = 0;
+
+	async->cur_chan = 0;
+	async->scan_progress = 0;
+	async->munge_chan = 0;
+	async->munge_count = 0;
+	async->munge_ptr = 0;
+
+	async->events = 0;
+}
+
