@@ -1,9 +1,10 @@
 /*
+ * comedi/drivers/acl7225b.c
  * Driver for Adlink NuDAQ ACL-7225b and clones
  * José Luis Sánchez
  */
 /*
-Driver: acl7225b.o
+Driver: acl7225b
 Description: Adlink NuDAQ ACL-7225b & compatibles
 Author: José Luis Sánchez (jsanchezv@teleline.es)
 Status: testing
@@ -25,24 +26,24 @@ static int acl7225b_attach(comedi_device *dev,comedi_devconfig *it);
 static int acl7225b_detach(comedi_device *dev);
 
 typedef struct {
-	char	*name;		// driver name
+	const char	*name;		// driver name
 	int		io_range;	// len of I/O space
 } boardtype;
 
-static boardtype boardtypes[] =
+static const boardtype boardtypes[] =
 {
 	{ "acl7225b",  ACL7225_SIZE, },
 	{ "p16r16dio", P16R16DIO_SIZE, },
 };
 #define n_boardtypes (sizeof(boardtypes)/sizeof(boardtype))
-#define this_board ((boardtype *)dev->board_ptr)
+#define this_board ((const boardtype *)dev->board_ptr)
 
 static comedi_driver driver_acl7225b = {
 	driver_name:	"acl7225b",
 	module:		THIS_MODULE,
 	attach:		acl7225b_attach,
 	detach:		acl7225b_detach,
-	board_name: boardtypes,
+	board_name: &boardtypes[0].name,
 	num_names:	n_boardtypes,
 	offset:		sizeof(boardtype),
 };
@@ -63,7 +64,7 @@ static int acl7225b_do_insn(comedi_device *dev,comedi_subdevice *s,
         outb(s->state & 0xff, dev->iobase+(unsigned long)s->private);
     if( data[0] & 0xff00 )
         outb((s->state >> 8), dev->iobase+(unsigned long)s->private+1);
-	
+
 	data[1]=s->state;
 
 	return 2;
@@ -142,6 +143,6 @@ static int acl7225b_detach(comedi_device *dev)
 
 	if(dev->iobase)
         release_region(dev->iobase, this_board->io_range);
-	
+
 	return 0;
 }
