@@ -69,7 +69,7 @@ You shoud also find the complete GPL in the COPYING file accompanying this sourc
 
 #define VOID void
 #define UINT unsigned int
-#define ULONG unsigned long
+#define ULONG unsigned int	/* 32-bit */
 #define USHORT unsigned short
 #define PUSHORT unsigned short *
 #define INT int
@@ -87,7 +87,8 @@ You shoud also find the complete GPL in the COPYING file accompanying this sourc
 #define HIWORD(W)                                (USHORT       )(((W)>>16)&0xFFFF)
 #define MAKEDWORD(H,L)  						(UINT         )((L)|( (H)<<16) )
 
-#define DWORD         unsigned long
+#define ULONG_PTR     unsigned long
+#define DWORD         unsigned int	/* 32-bit */
 #define WORD          unsigned short
 
 #define ADDI_ENABLE   1 
@@ -101,11 +102,16 @@ You shoud also find the complete GPL in the COPYING file accompanying this sourc
 #define ADDIDATA_S5933        "S5933"
 #define ADDIDATA_9054         "9054"
 
+//ADDIDATA Enable Disable
+#define ADDIDATA_ENABLE                            1
+#define ADDIDATA_DISABLE                           0
+
+
 // Structures
 // structure for the boardtype
 typedef struct {
 
-		PCHAR  		pc_DriverName; // driver name
+		const PCHAR  		pc_DriverName; // driver name
 		INT 		i_VendorId;   //PCI vendor a device ID of card
 		INT			i_DeviceId;
                 INT         i_IorangeBase0;
@@ -137,7 +143,7 @@ typedef struct {
 		UINT        ui_MinDelaytimeNs;       // Minimum Delay in Nano secs
 
 // interrupt and reset
- void (*v_hwdrv_Interrupt)(int irq, void *d, struct pt_regs *regs);
+ void (*v_hwdrv_Interrupt)(int irq, void *d);
  int (*i_hwdrv_Reset)(comedi_device *dev);
 
 //Subdevice functions 
@@ -385,7 +391,7 @@ typedef struct{
  		INT      i_IobaseAmcc;  // base+size for AMCC chip
 		INT      i_IobaseAddon; //addon base address
                 INT      i_IobaseReserved;
-                DWORD    dw_AiBase;
+                ULONG_PTR dw_AiBase;
    		struct pcilst_struct	*amcc;		// ptr too AMCC data
 		UINT		        master;		// master capable
 		BYTE         		allocated;	// we have blocked card
@@ -496,7 +502,7 @@ static int i_ADDI_Attach(comedi_device *dev,comedi_devconfig *it);
 static int i_ADDI_Detach(comedi_device *dev);
 static int i_ADDI_Reset(comedi_device *dev);
 
-static irqreturn_t v_ADDI_Interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t v_ADDI_Interrupt(int irq, void *d  PT_REGS_ARG);
 static int i_ADDIDATA_InsnReadEeprom(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 
 
