@@ -25,12 +25,12 @@
 
 */
 /*
-Driver: amplc_dio200.o
-Description: Amplicon PC272E, PCI272
+Driver: amplc_dio200
+Description: Amplicon 200 Series Digital I/O
 Author: Ian Abbott <abbotti@mev.co.uk>
 Devices: [Amplicon] PC212E (pc212e), PC214E (pc214e), PC215E (pc215e),
   PCI215 (pci215), PC218E (pc218e), PC272E (pc272e), PCI272 (pci272)
-Updated: Fri, 07 Oct 2005 16:59:59 +0100
+Updated: Mon, 06 Aug 2007 11:06:10 +0100
 Status: works
 
 Configuration options - PC212E, PC214E, PC215E, PC218E, PC272E:
@@ -61,7 +61,7 @@ SUBDEVICES
 
                     PC214E      PC272E/PCI272
                  -------------  -------------
-  Subdevices           4              4 
+  Subdevices           4              4
    0                 PPI-X          PPI-X
    1                 PPI-Y          PPI-Y
    2                 CTR-Z1*        PPI-Z
@@ -110,10 +110,11 @@ instructions are supported:
         channels on the chip.
 
   INSN_CONFIG_GET_CLOCK_SRC.  Returns the counter channel's current
-    clock source in data[1].
+    clock source in data[1].  For internal clock sources, data[2] is set
+    to the period in ns.
 
   INSN_CONFIG_SET_GATE_SRC.  Sets the counter channel's gate source as
-    specified in data[1] (this is a hardware-specific value).  Not
+    specified in data[2] (this is a hardware-specific value).  Not
     supported on PC214E.  For the other boards, valid gate sources are 0
     to 7 as follows:
 
@@ -130,7 +131,7 @@ instructions are supported:
       7.  Reserved.
 
   INSN_CONFIG_GET_GATE_SRC.  Returns the counter channel's current gate
-    source in data[1].
+    source in data[2].
 
 Clock and gate interconnection notes:
 
@@ -141,7 +142,7 @@ Clock and gate interconnection notes:
   2.  Gate source /OUT n-2 is the inverted output of channel 0 on the
   same counter subdevice if n = 2, or the inverted output of channel n+1
   on the preceding counter subdevice (see note 3) if n < 2.
-  
+
   3.  The counter subdevices are connected in a ring, so the highest
   counter subdevice precedes the lowest.
 
@@ -242,6 +243,20 @@ order they appear in the channel list.
 #define CLK_SCE(which, chan, source) (((which) << 5) | ((chan) << 3) | (source))
 #define GAT_SCE(which, chan, source) (((which) << 5) | ((chan) << 3) | (source))
 
+/*
+ * Periods of the internal clock sources in nanoseconds.
+ */
+static const unsigned clock_period[8] = {
+	0,		/* dedicated clock input/output pin */
+	100,		/* 10 MHz */
+	1000,		/* 1 MHz */
+	10000,		/* 100 kHz */
+	100000,		/* 10 kHz */
+	1000000,	/* 1 kHz */
+	0,		/* OUT N-1 */
+	0		/* group clock input pin */
+};
+
 /*
  * Board descriptions.
  */
@@ -265,13 +280,13 @@ enum dio200_layout {
 };
 
 typedef struct dio200_board_struct {
-	char *name;
+	const char *name;
 	enum dio200_bustype bustype;
 	enum dio200_model model;
 	enum dio200_layout layout;
 } dio200_board;
 
-static dio200_board dio200_boards[] = {
+static const dio200_board dio200_boards[] = {
 	{
 	name:		"pc212e",
 	bustype:	isa_bustype,
@@ -334,7 +349,7 @@ typedef struct dio200_layout_struct {
 	char has_clk_gat_sce;		/* has clock/gate selection registers */
 } dio200_layout;
 
-static dio200_layout dio200_layouts[] = {
+static const dio200_layout dio200_layouts[] = {
 	[pc212_layout] = {
 		n_subdevs:	6,
 		sdtype:		{ sd_8255, sd_8254, sd_8254, sd_8254, sd_8254, sd_intr },
@@ -388,7 +403,7 @@ MODULE_DEVICE_TABLE(pci, dio200_pci_table);
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((dio200_board *)dev->board_ptr)
+#define thisboard ((const dio200_board *)dev->board_ptr)
 #define thislayout (&dio200_layouts[((dio200_board *)dev->board_ptr)->layout])
 
 /* this structure is for data unique to this hardware driver.  If
@@ -436,7 +451,7 @@ static comedi_driver driver_amplc_dio200 = {
 	module:		THIS_MODULE,
 	attach:		dio200_attach,
 	detach:		dio200_detach,
-	board_name:	dio200_boards,
+	board_name:	&dio200_boards[0].name,
 	offset:		sizeof(dio200_board),
 	num_names:	sizeof(dio200_boards) / sizeof(dio200_board),
 };
@@ -468,7 +483,7 @@ dio200_find_pci(comedi_device *dev, int bus, int slot,
 
 	/* Look for matching PCI device. */
 	for(pci_dev = pci_get_device(pci_id->vendor, pci_id->device, NULL);
-			pci_dev != NULL ; 
+			pci_dev != NULL ;
 			pci_dev = pci_get_device(pci_id->vendor,
 				pci_id->device, pci_dev)) {
 		/* If bus/slot specified, check them. */
@@ -616,7 +631,7 @@ dio200_inttrig_start_intr(comedi_device *dev, comedi_subdevice *s,
 	comedi_spin_unlock_irqrestore(&subpriv->spinlock, flags);
 
 	if (event) {
-		comedi_event(dev, s, s->async->events);
+		comedi_event(dev, s);
 	}
 
 	return 1;
@@ -665,7 +680,7 @@ dio200_handle_read_intr(comedi_device *dev, comedi_subdevice *s)
 		 */
 		triggered = subpriv->enabled_isns;
 	}
-	
+
 	if (triggered) {
 		/*
 		 * Some interrupt sources have triggered and have been
@@ -677,7 +692,7 @@ dio200_handle_read_intr(comedi_device *dev, comedi_subdevice *s)
 		if (subpriv->has_int_sce) {
 			outb(cur_enabled, subpriv->iobase);
 		}
-		
+
 		if (subpriv->active) {
 			/*
 			 * The command is still active.
@@ -726,7 +741,7 @@ dio200_handle_read_intr(comedi_device *dev, comedi_subdevice *s)
 	comedi_spin_unlock_irqrestore(&subpriv->spinlock, flags);
 
 	if (oldevents != s->async->events) {
-		comedi_event(dev, s, s->async->events);
+		comedi_event(dev, s);
 	}
 
 	return (triggered != 0);
@@ -775,7 +790,7 @@ dio200_subdev_intr_cmdtest(comedi_device *dev, comedi_subdevice *s,
 	if (!cmd->convert_src || tmp != cmd->convert_src) err++;
 
 	tmp = cmd->scan_end_src;
-	cmd->scan_end_src &= TRIG_COUNT;  
+	cmd->scan_end_src &= TRIG_COUNT;
 	if (!cmd->scan_end_src || tmp != cmd->scan_end_src) err++;
 
 	tmp = cmd->stop_src;
@@ -884,7 +899,7 @@ dio200_subdev_intr_cmd(comedi_device *dev, comedi_subdevice *s)
 	comedi_spin_unlock_irqrestore(&subpriv->spinlock, flags);
 
 	if (event) {
-		comedi_event(dev, s, s->async->events);
+		comedi_event(dev, s);
 	}
 
 	return 0;
@@ -899,12 +914,11 @@ dio200_subdev_intr_init(comedi_device *dev, comedi_subdevice *s,
 {
 	dio200_subdev_intr *subpriv;
 
-	subpriv = kmalloc(sizeof(*subpriv), GFP_KERNEL);
+	subpriv = kzalloc(sizeof(*subpriv), GFP_KERNEL);
 	if (!subpriv) {
 		printk(KERN_ERR "comedi%d: error! out of memory!\n", dev->minor);
 		return -ENOMEM;
 	}
-	memset(subpriv, 0, sizeof(*subpriv));
 	subpriv->iobase = iobase;
 	subpriv->has_int_sce = has_int_sce;
 	subpriv->valid_isns = valid_isns;
@@ -916,7 +930,7 @@ dio200_subdev_intr_init(comedi_device *dev, comedi_subdevice *s,
 
 	s->private = subpriv;
 	s->type = COMEDI_SUBD_DI;
-	s->subdev_flags = SDF_READABLE;
+	s->subdev_flags = SDF_READABLE | SDF_CMD_READ;
 	if (has_int_sce) {
 		s->n_chan = DIO200_MAX_ISNS;
 		s->len_chanlist = DIO200_MAX_ISNS;
@@ -952,11 +966,15 @@ dio200_subdev_intr_cleanup(comedi_device *dev, comedi_subdevice *s)
  * Interrupt service routine.
  */
 static irqreturn_t
-dio200_interrupt(int irq, void *d, struct pt_regs *regs)
+dio200_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev=d;
 	int handled;
 
+	if (!dev->attached) {
+		return IRQ_NONE;
+	}
+
 	if (devpriv->intr_sd >= 0) {
 		handled = dio200_handle_read_intr(dev,
 				dev->subdevices + devpriv->intr_sd);
@@ -977,7 +995,7 @@ dio200_subdev_8254_read(comedi_device *dev, comedi_subdevice *s,
 	dio200_subdev_8254 *subpriv = s->private;
 	int chan = CR_CHAN(insn->chanspec);
 
-	data[0] = i8254_read(subpriv->iobase, chan);
+	data[0] = i8254_read(subpriv->iobase, 0, chan);
 
 	return 1;
 }
@@ -992,7 +1010,7 @@ dio200_subdev_8254_write(comedi_device *dev, comedi_subdevice *s,
 	dio200_subdev_8254 *subpriv = s->private;
 	int chan = CR_CHAN(insn->chanspec);
 
-	i8254_write(subpriv->iobase, chan, data[0]);
+	i8254_write(subpriv->iobase, 0, chan, data[0]);
 
 	return 1;
 }
@@ -1053,12 +1071,17 @@ dio200_set_clock_src(dio200_subdev_8254 *subpriv, unsigned int counter_number,
  * Get clock source for an '8254' counter subdevice channel.
  */
 static int
-dio200_get_clock_src(dio200_subdev_8254 *subpriv, unsigned int counter_number)
+dio200_get_clock_src(dio200_subdev_8254 *subpriv, unsigned int counter_number,
+		lsampl_t *period_ns)
 {
+	unsigned clock_src;
+
 	if (!subpriv->has_clk_gat_sce) return -1;
 	if (counter_number > 2) return -1;
 
-	return subpriv->clock_src[counter_number];
+	clock_src = subpriv->clock_src[counter_number];
+	*period_ns = clock_period[clock_src];
+	return clock_src;
 }
 
 /*
@@ -1072,31 +1095,29 @@ dio200_subdev_8254_config(comedi_device *dev, comedi_subdevice *s,
 	int ret;
 	int chan = CR_CHAN(insn->chanspec);
 
-	if (insn->n != 2) return -EINVAL;
-
 	switch (data[0]) {
 	case INSN_CONFIG_8254_SET_MODE:
-		ret = i8254_set_mode(subpriv->iobase, chan, data[1]);
+		ret = i8254_set_mode(subpriv->iobase, 0, chan, data[1]);
 		if (ret < 0) return -EINVAL;
 		break;
 	case INSN_CONFIG_8254_READ_STATUS:
-		data[1] = i8254_status(subpriv->iobase, chan);
+		data[1] = i8254_status(subpriv->iobase, 0, chan);
 		break;
 	case INSN_CONFIG_SET_GATE_SRC:
-		ret = dio200_set_gate_src(subpriv, chan, data[1]);
+		ret = dio200_set_gate_src(subpriv, chan, data[2]);
 		if (ret < 0) return -EINVAL;
 		break;
 	case INSN_CONFIG_GET_GATE_SRC:
 		ret = dio200_get_gate_src(subpriv, chan);
 		if (ret < 0) return -EINVAL;
-		data[1] = ret;
+		data[2] = ret;
 		break;
 	case INSN_CONFIG_SET_CLOCK_SRC:
 		ret = dio200_set_clock_src(subpriv, chan, data[1]);
 		if (ret < 0) return -EINVAL;
 		break;
 	case INSN_CONFIG_GET_CLOCK_SRC:
-		ret = dio200_get_clock_src(subpriv, chan);
+		ret = dio200_get_clock_src(subpriv, chan, &data[2]);
 		if (ret < 0) return -EINVAL;
 		data[1] = ret;
 		break;
@@ -1104,7 +1125,7 @@ dio200_subdev_8254_config(comedi_device *dev, comedi_subdevice *s,
 		return -EINVAL;
 		break;
 	}
-	return 2;
+	return insn->n;
 }
 
 /*
@@ -1120,12 +1141,11 @@ dio200_subdev_8254_init(comedi_device *dev, comedi_subdevice *s,
 	dio200_subdev_8254 *subpriv;
 	unsigned int chan;
 
-	subpriv = kmalloc(sizeof(*subpriv), GFP_KERNEL);
+	subpriv = kzalloc(sizeof(*subpriv), GFP_KERNEL);
 	if (!subpriv) {
 		printk(KERN_ERR "comedi%d: error! out of memory!\n", dev->minor);
 		return -ENOMEM;
 	}
-	memset(subpriv, 0, sizeof(*subpriv));
 
 	s->private = subpriv;
 	s->type = COMEDI_SUBD_COUNTER;
@@ -1150,7 +1170,7 @@ dio200_subdev_8254_init(comedi_device *dev, comedi_subdevice *s,
 
 	/* Initialize channels. */
 	for (chan = 0; chan < 3; chan++) {
-		i8254_set_mode(subpriv->iobase, chan,
+		i8254_set_mode(subpriv->iobase, 0, chan,
 				I8254_MODE0 | I8254_BINARY);
 		if (subpriv->has_clk_gat_sce) {
 			/* Gate source 0 is VCC (logic 1). */
@@ -1190,7 +1210,7 @@ dio200_attach(comedi_device *dev,comedi_devconfig *it)
 	unsigned long iobase = 0;
 	unsigned int irq = 0;
 	int bus = 0, slot = 0;
-	dio200_layout *layout;
+	const dio200_layout *layout;
 	int share_irq = 0;
 	int sdx;
 	unsigned n;
@@ -1308,7 +1328,7 @@ dio200_attach(comedi_device *dev,comedi_devconfig *it)
 	dev->board_name = thisboard->name;
 
 	if (irq) {
-		unsigned long flags = share_irq ? SA_SHIRQ : 0;
+		unsigned long flags = share_irq ? IRQF_SHARED : 0;
 
 		if (comedi_request_irq(irq, dio200_interrupt, flags,
 					DIO200_DRIVER_NAME, dev) >= 0) {
@@ -1330,7 +1350,7 @@ dio200_attach(comedi_device *dev,comedi_devconfig *it)
 	} else {
 		printk("(no irq) ");
 	}
-	
+
 	printk("attached\n");
 
 	return 1;
@@ -1338,7 +1358,7 @@ dio200_attach(comedi_device *dev,comedi_devconfig *it)
 
 /*
  * _detach is called to deconfigure a device.  It should deallocate
- * resources.  
+ * resources.
  * This function is also called when _attach() fails, so it should be
  * careful not to release resources that were not necessarily
  * allocated by _attach().  dev->private and dev->subdevices are
@@ -1347,7 +1367,7 @@ dio200_attach(comedi_device *dev,comedi_devconfig *it)
 static int
 dio200_detach(comedi_device *dev)
 {
-	dio200_layout *layout;
+	const dio200_layout *layout;
 	unsigned n;
 
 	printk(KERN_DEBUG "comedi%d: %s: detach\n", dev->minor,
@@ -1391,7 +1411,7 @@ dio200_detach(comedi_device *dev)
 		printk(KERN_INFO "comedi%d: %s removed\n",
 				dev->minor, dev->board_name);
 	}
-	
+
 	return 0;
 }
 
