@@ -23,7 +23,7 @@
 
 */
 /*
-Driver: amplc_pc236.o
+Driver: amplc_pc236
 Description: Amplicon PC36AT, PCI236
 Author: Ian Abbott <abbotti@mev.co.uk>
 Devices: [Amplicon] PC36AT (pc36at), PCI236 (pci236)
@@ -71,6 +71,23 @@ unused.
 #define PC236_IO_SIZE		4
 #define PC236_LCR_IO_SIZE	128
 
+/*
+ * INTCSR values for PCI236.
+ */
+/* Disable interrupt, also clear any interrupt there */
+#define PCI236_INTR_DISABLE ( PLX9052_INTCSR_LI1ENAB_DISABLED \
+        | PLX9052_INTCSR_LI1POL_HIGH \
+        | PLX9052_INTCSR_LI2POL_HIGH \
+        | PLX9052_INTCSR_PCIENAB_DISABLED \
+        | PLX9052_INTCSR_LI1SEL_EDGE \
+        | PLX9052_INTCSR_LI1CLRINT_ASSERTED )
+/* Enable interrupt, also clear any interrupt there. */
+#define PCI236_INTR_ENABLE ( PLX9052_INTCSR_LI1ENAB_ENABLED \
+        | PLX9052_INTCSR_LI1POL_HIGH \
+        | PLX9052_INTCSR_LI2POL_HIGH \
+        | PLX9052_INTCSR_PCIENAB_ENABLED \
+        | PLX9052_INTCSR_LI1SEL_EDGE \
+        | PLX9052_INTCSR_LI1CLRINT_ASSERTED )
 
 /*
  * Board descriptions for Amplicon PC36AT and PCI236.
@@ -80,12 +97,12 @@ enum pc236_bustype {isa_bustype, pci_bustype};
 enum pc236_model {pc36at_model, pci236_model};
 
 typedef struct pc236_board_struct{
-	char *name;
-	char *fancy_name;
+	const char *name;
+	const char *fancy_name;
 	enum pc236_bustype bustype;
 	enum pc236_model model;
 }pc236_board;
-static pc236_board pc236_boards[] = {
+static const pc236_board pc236_boards[] = {
 	{
 	name:		"pc36at",
 	fancy_name:	"PC36AT",
@@ -109,7 +126,7 @@ MODULE_DEVICE_TABLE(pci, pc236_pci_table);
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((pc236_board *)dev->board_ptr)
+#define thisboard ((const pc236_board *)dev->board_ptr)
 
 /* this structure is for data unique to this hardware driver.  If
    several hardware drivers keep similar information in this structure,
@@ -136,7 +153,7 @@ static comedi_driver driver_amplc_pc236={
 	module:		THIS_MODULE,
 	attach:		pc236_attach,
 	detach:		pc236_detach,
-	board_name:	pc236_boards,
+	board_name:	&pc236_boards[0].name,
 	offset:		sizeof(pc236_board),
 	num_names:	sizeof(pc236_boards) / sizeof(pc236_board),
 };
@@ -153,7 +170,7 @@ static int pc236_intr_cmdtest(comedi_device *dev,comedi_subdevice *s,
 	comedi_cmd *cmd);
 static int pc236_intr_cmd(comedi_device *dev,comedi_subdevice *s);
 static int pc236_intr_cancel(comedi_device *dev,comedi_subdevice *s);
-static irqreturn_t pc236_interrupt(int irq,void *d,struct pt_regs *regs);
+static irqreturn_t pc236_interrupt(int irq,void *d PT_REGS_ARG);
 
 /*
  * Attach is called by the Comedi core to configure the driver
@@ -205,7 +222,7 @@ static int pc236_attach(comedi_device *dev,comedi_devconfig *it)
 
 		/* Look for matching PCI device. */
 		for(pci_dev = pci_get_device(pci_id->vendor, pci_id->device,
-					NULL); pci_dev != NULL; 
+					NULL); pci_dev != NULL;
 				pci_dev = pci_get_device(pci_id->vendor,
 					pci_id->device, pci_dev)) {
 			/* If bus/slot specified, check them. */
@@ -287,13 +304,13 @@ static int pc236_attach(comedi_device *dev,comedi_devconfig *it)
 	s->type = COMEDI_SUBD_UNUSED;
 	pc236_intr_disable(dev);
 	if (irq) {
-		unsigned long flags = share_irq ? SA_SHIRQ : 0;
+		unsigned long flags = share_irq ? IRQF_SHARED : 0;
 
 		if (comedi_request_irq(irq, pc236_interrupt, flags,
 					PC236_DRIVER_NAME, dev) >= 0) {
 			dev->irq = irq;
 			s->type = COMEDI_SUBD_DI;
-			s->subdev_flags = SDF_READABLE;
+			s->subdev_flags = SDF_READABLE | SDF_CMD_READ;
 			s->n_chan = 1;
 			s->maxdata = 1;
 			s->range_table = &range_digital;
@@ -313,7 +330,7 @@ static int pc236_attach(comedi_device *dev,comedi_devconfig *it)
 	} else {
 		printk("(no irq) ");
 	}
-	
+
 	printk("attached\n");
 
 	return 1;
@@ -322,7 +339,7 @@ static int pc236_attach(comedi_device *dev,comedi_devconfig *it)
 
 /*
  * _detach is called to deconfigure a device.  It should deallocate
- * resources.  
+ * resources.
  * This function is also called when _attach() fails, so it should be
  * careful not to release resources that were not necessarily
  * allocated by _attach().  dev->private and dev->subdevices are
@@ -416,7 +433,7 @@ static int pc236_intr_check(comedi_device *dev)
 		if (devpriv->lcr_iobase) {
 			if ((inl(devpriv->lcr_iobase+PLX9052_INTCSR)
 						& PLX9052_INTCSR_LI1STAT_MASK)
-					== PLX9052_INTCSR_LI1STAT_SHIFTED_INACTIVE) {
+					== PLX9052_INTCSR_LI1STAT_INACTIVE) {
 				retval = 0;
 			} else {
 				/* Clear interrupt and keep it enabled. */
@@ -534,17 +551,17 @@ static int pc236_intr_cancel(comedi_device *dev,comedi_subdevice *s)
  * Interrupt service routine.
  * Based on the comedi_parport driver.
  */
-static irqreturn_t pc236_interrupt(int irq,void *d,struct pt_regs *regs)
+static irqreturn_t pc236_interrupt(int irq,void *d PT_REGS_ARG)
 {
 	comedi_device *dev=d;
 	comedi_subdevice *s=dev->subdevices+1;
 	int handled;
 
 	handled = pc236_intr_check(dev);
-	if (handled) {
+	if (dev->attached && handled) {
 		comedi_buf_put(s->async,0);
 		s->async->events |= COMEDI_CB_BLOCK | COMEDI_CB_EOS;
-		comedi_event(dev,s,s->async->events);
+		comedi_event(dev, s);
 	}
 	return IRQ_RETVAL(handled);
 }
