@@ -23,7 +23,7 @@
 
 */
 /*
-Driver: amplc_pci224.o
+Driver: amplc_pci224
 Description: Amplicon PCI224, PCI234
 Author: Ian Abbott <abbotti@mev.co.uk>
 Devices: [Amplicon] PCI224 (pci224), PCI234 (pci234)
@@ -272,7 +272,7 @@ Caveats:
  */
 
 /* The software selectable internal ranges for PCI224 (option[2] == 0). */
-static comedi_lrange range_pci224_internal = {
+static const comedi_lrange range_pci224_internal = {
 	8,
 	{
 		BIP_RANGE(10),
@@ -286,7 +286,7 @@ static comedi_lrange range_pci224_internal = {
 	}
 };
 
-static unsigned short hwrange_pci224_internal[8] = {
+static const unsigned short hwrange_pci224_internal[8] = {
 	PCI224_DACCON_POLAR_BI | PCI224_DACCON_VREF_10,
 	PCI224_DACCON_POLAR_BI | PCI224_DACCON_VREF_5,
 	PCI224_DACCON_POLAR_BI | PCI224_DACCON_VREF_2_5,
@@ -298,7 +298,7 @@ static unsigned short hwrange_pci224_internal[8] = {
 };
 
 /* The software selectable external ranges for PCI224 (option[2] == 1). */
-static comedi_lrange range_pci224_external = {
+static const comedi_lrange range_pci224_external = {
 	2,
 	{
 		RANGE_ext(-1, 1),	/* bipolar [-Vref,+Vref] */
@@ -306,14 +306,14 @@ static comedi_lrange range_pci224_external = {
 	}
 };
 
-static unsigned short hwrange_pci224_external[2] = {
+static const unsigned short hwrange_pci224_external[2] = {
 	PCI224_DACCON_POLAR_BI,
 	PCI224_DACCON_POLAR_UNI,
 };
 
 /* The hardware selectable Vref*2 external range for PCI234
  * (option[2] == 1, option[3+n] == 0). */
-static comedi_lrange range_pci234_ext2 = {
+static const comedi_lrange range_pci234_ext2 = {
 	1,
 	{
 		RANGE_ext(-2, 2),
@@ -322,7 +322,7 @@ static comedi_lrange range_pci234_ext2 = {
 
 /* The hardware selectable Vref external range for PCI234
  * (option[2] == 1, option[3+n] == 1). */
-static comedi_lrange range_pci234_ext = {
+static const comedi_lrange range_pci234_ext = {
 	1,
 	{
 		RANGE_ext(-1, 1),
@@ -330,7 +330,7 @@ static comedi_lrange range_pci234_ext = {
 };
 
 /* This serves for all the PCI234 ranges. */
-static unsigned short hwrange_pci234[1] = {
+static const unsigned short hwrange_pci234[1] = {
 	PCI224_DACCON_POLAR_BI,		/* bipolar - hardware ignores it! */
 };
 
@@ -342,13 +342,13 @@ static unsigned short hwrange_pci234[1] = {
 enum pci224_model	{ pci224_model, pci234_model };
 
 typedef struct pci224_board_struct {
-	char *name;
+	const char *name;
 	enum pci224_model model;
 	unsigned int ao_chans;
 	unsigned int ao_bits;
 } pci224_board;
 
-static pci224_board pci224_boards[] = {
+static const pci224_board pci224_boards[] = {
 	{
 	name:		"pci224",
 	model:		pci224_model,
@@ -386,7 +386,7 @@ MODULE_DEVICE_TABLE(pci, pci224_pci_table);
    feel free to suggest moving the variable to the comedi_device struct.  */
 typedef struct {
 	struct pci_dev *pci_dev;	/* PCI device */
-	unsigned short *hwrange;
+	const unsigned short *hwrange;
 	unsigned long iobase1;
 	spinlock_t ao_spinlock;
 	lsampl_t *ao_readback;
@@ -419,7 +419,7 @@ static comedi_driver driver_amplc_pci224 = {
 	module:		THIS_MODULE,
 	attach:		pci224_attach,
 	detach:		pci224_detach,
-	board_name:	pci224_boards,
+	board_name:	&pci224_boards[0].name,
 	offset:		sizeof(pci224_board),
 	num_names:	sizeof(pci224_boards) / sizeof(pci224_board),
 };
@@ -575,7 +575,7 @@ pci224_ao_start(comedi_device *dev, comedi_subdevice *s)
 		/* An empty acquisition! */
 		pci224_ao_stop(dev, s);
 		s->async->events |= COMEDI_CB_EOA;
-		comedi_event(dev, s, s->async->events);
+		comedi_event(dev, s);
 	} else {
 		/* Enable interrupts. */
 		comedi_spin_lock_irqsave(&devpriv->ao_spinlock, flags);
@@ -609,7 +609,7 @@ pci224_ao_handle_fifo(comedi_device *dev, comedi_subdevice *s)
 		bytes_per_scan = sizeof(sampl_t);
 	}
 	/* Determine number of scans available in buffer. */
-	num_scans = comedi_buf_read_n_available(s) / bytes_per_scan;
+	num_scans = comedi_buf_read_n_available(s->async) / bytes_per_scan;
 	if (!devpriv->ao_stop_continuous) {
 		/* Fixed number of scans. */
 		if (num_scans > devpriv->ao_stop_count) {
@@ -714,7 +714,7 @@ pci224_ao_handle_fifo(comedi_device *dev, comedi_subdevice *s)
 		}
 	}
 	if (s->async->events) {
-		comedi_event(dev, s, s->async->events);
+		comedi_event(dev, s);
 	}
 }
 
@@ -1106,14 +1106,14 @@ pci224_ao_cmd(comedi_device *dev, comedi_subdevice *s)
 			outb(CLK_CONFIG(2, CLK_10MHZ),
 					devpriv->iobase1 + PCI224_ZCLK_SCE);
 			/* Load Z2-2 mode (2) and counter (div1). */
-			i8254_load(devpriv->iobase1 + PCI224_Z2_CT0,
+			i8254_load(devpriv->iobase1 + PCI224_Z2_CT0, 0,
 					2, div1, 2);
 			/* Z2-0 is clocked from Z2-2's output. */
 			outb(CLK_CONFIG(0, CLK_OUTNM1),
 					devpriv->iobase1 + PCI224_ZCLK_SCE);
 		}
 		/* Load Z2-0 mode (2) and counter (div2). */
-		i8254_load(devpriv->iobase1 + PCI224_Z2_CT0, 0, div2, 2);
+		i8254_load(devpriv->iobase1 + PCI224_Z2_CT0, 0, 0, div2, 2);
 	}
 
 	/*
@@ -1201,7 +1201,7 @@ pci224_ao_munge(comedi_device *dev, comedi_subdevice *s, void *data,
  * Interrupt handler.
  */
 static irqreturn_t
-pci224_interrupt(int irq, void *d, struct pt_regs *regs)
+pci224_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = &dev->subdevices[0];
@@ -1274,7 +1274,7 @@ pci224_find_pci(comedi_device *dev, int bus, int slot,
 
 	/* Look for matching PCI device. */
 	for(pci_dev = pci_get_device(pci_id->vendor, pci_id->device, NULL);
-			pci_dev != NULL ; 
+			pci_dev != NULL ;
 			pci_dev = pci_get_device(pci_id->vendor,
 				pci_id->device, pci_dev)) {
 		/* If bus/slot specified, check them. */
@@ -1392,18 +1392,18 @@ pci224_attach(comedi_device *dev,comedi_devconfig *it)
 			PCI224_DACCON_FIFOENAB | PCI224_DACCON_FIFOINTR_EMPTY);
 	outw(devpriv->daccon | PCI224_DACCON_FIFORESET,
 			dev->iobase + PCI224_DACCON);
-	
+
 	/* Allocate subdevices.  There is only one!  */
 	if ((ret=alloc_subdevices(dev, 1)) < 0) {
 		printk(KERN_ERR "comedi%d: error! out of memory!\n",
 				dev->minor);
 		return ret;
 	}
-	
+
 	s = dev->subdevices + 0;
 	/* Analog output subdevice. */
 	s->type = COMEDI_SUBD_AO;
-	s->subdev_flags = SDF_WRITABLE | SDF_GROUND;
+	s->subdev_flags = SDF_WRITABLE | SDF_GROUND | SDF_CMD_WRITE;
 	s->n_chan = thisboard->ao_chans;
 	s->maxdata = (1 << thisboard->ao_bits) - 1;
 	s->insn_write = &pci224_ao_insn_write;
@@ -1419,7 +1419,9 @@ pci224_attach(comedi_device *dev,comedi_devconfig *it)
         /* Sort out channel range options. */
         if (thisboard->model == pci234_model) {
                 /* PCI234 range options. */
-                s->range_table_list = kmalloc(
+		const comedi_lrange **range_table_list;
+
+                s->range_table_list = range_table_list = kmalloc(
                                 sizeof(comedi_lrange *) * s->n_chan,
                                 GFP_KERNEL);
                 if (!s->range_table_list) {
@@ -1437,18 +1439,18 @@ pci224_attach(comedi_device *dev,comedi_devconfig *it)
                         if (n < COMEDI_NDEVCONFOPTS - 3 &&
                                         it->options[3+n] == 1) {
                                 if (it->options[2] == 1) {
-                                        s->range_table_list[n] =
+                                        range_table_list[n] =
                                                 &range_pci234_ext;
                                 } else {
-                                        s->range_table_list[n] =
+                                        range_table_list[n] =
                                                 &range_bipolar5;
                                 }
                         } else {
                                 if (it->options[2] == 1) {
-                                        s->range_table_list[n] =
+                                        range_table_list[n] =
                                                 &range_pci234_ext2;
                                 } else {
-                                        s->range_table_list[n] =
+                                        range_table_list[n] =
                                                 &range_bipolar10;
                                 }
                         }
@@ -1474,7 +1476,7 @@ pci224_attach(comedi_device *dev,comedi_devconfig *it)
 	dev->board_name = thisboard->name;
 
 	if (irq) {
-		ret = comedi_request_irq(irq, pci224_interrupt, SA_SHIRQ,
+		ret = comedi_request_irq(irq, pci224_interrupt, IRQF_SHARED,
 				DRIVER_NAME, dev);
 		if (ret < 0) {
 			printk(KERN_ERR "comedi%d: error! "
@@ -1493,7 +1495,7 @@ pci224_attach(comedi_device *dev,comedi_devconfig *it)
 	} else {
 		printk("(no irq) ");
 	}
-	
+
 	printk("attached\n");
 
 	return 1;
@@ -1501,7 +1503,7 @@ pci224_attach(comedi_device *dev,comedi_devconfig *it)
 
 /*
  * _detach is called to deconfigure a device.  It should deallocate
- * resources.  
+ * resources.
  * This function is also called when _attach() fails, so it should be
  * careful not to release resources that were not necessarily
  * allocated by _attach().  dev->private and dev->subdevices are
@@ -1547,7 +1549,7 @@ pci224_detach(comedi_device *dev)
 		printk(KERN_INFO "comedi%d: %s removed\n",
 				dev->minor, dev->board_name);
 	}
-	
+
 	return 0;
 }
 
