@@ -1,5 +1,5 @@
 /*
-    cb_pcidas.c
+    comedi/drivers/cb_pcidas.c
 
     Developed by Ivan Martinez and Frank Mori Hess, with valuable help from
     David Schleef and the rest of the Comedi developers comunity.
@@ -27,7 +27,7 @@
 ************************************************************************
 */
 /*
-Driver: cb_pcidas.o
+Driver: cb_pcidas
 Description: MeasurementComputing PCI-DAS series with the AMCC S5933 PCI controller
 Author: Ivan Martinez <imr@oersted.dtu.dk>,
   Frank Mori Hess <fmhess@users.sourceforge.net>
@@ -43,7 +43,7 @@ Status:
 
   The boards may be autocalibrated using the comedi_calibrate
   utility.
-	
+
 Configuration options:
   [0] - PCI bus of device (optional)
   [1] - PCI slot of device (optional)
@@ -196,7 +196,7 @@ static inline unsigned int DAC_DATA_REG( unsigned int channel )
 // bit in hexadecimal representation of range index that indicates unipolar input range
 #define IS_UNIPOLAR 0x4
 // analog input ranges for most boards
-static comedi_lrange cb_pcidas_ranges =
+static const comedi_lrange cb_pcidas_ranges =
 {
 	8,
 	{
@@ -212,7 +212,7 @@ static comedi_lrange cb_pcidas_ranges =
 };
 
 // pci-das1001 input ranges
-static comedi_lrange cb_pcidas_alt_ranges =
+static const comedi_lrange cb_pcidas_alt_ranges =
 {
 	8,
 	{
@@ -228,7 +228,7 @@ static comedi_lrange cb_pcidas_alt_ranges =
 };
 
 // analog output ranges
-static comedi_lrange cb_pcidas_ao_ranges =
+static const comedi_lrange cb_pcidas_ao_ranges =
 {
 	4,
 	{
@@ -247,7 +247,7 @@ enum trimpot_model
 
 typedef struct cb_pcidas_board_struct
 {
-	char *name;
+	const char *name;
 	unsigned short device_id;
 	int ai_se_chans;	// Inputs in single-ended mode
 	int ai_diff_chans;	// Inputs in differential mode
@@ -257,12 +257,12 @@ typedef struct cb_pcidas_board_struct
 	int has_ao_fifo;	// analog output has fifo
 	int ao_scan_speed;	// analog output speed for 1602 series (for a scan, not conversion)
 	int fifo_size;	// number of samples fifo can hold
-	comedi_lrange *ranges;
+	const comedi_lrange *ranges;
 	enum trimpot_model trimpot;
 	unsigned has_dac08 : 1;
 } cb_pcidas_board;
 
-static cb_pcidas_board cb_pcidas_boards[] =
+static const cb_pcidas_board cb_pcidas_boards[] =
 {
 	{
 		name:		"pci-das1602/16",
@@ -398,7 +398,7 @@ MODULE_DEVICE_TABLE(pci, cb_pcidas_pci_table);
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((cb_pcidas_board *)dev->board_ptr)
+#define thisboard ((const cb_pcidas_board *)dev->board_ptr)
 
 /* this structure is for data unique to this hardware driver.  If
    several hardware drivers keep similar information in this structure,
@@ -466,7 +466,7 @@ static int cb_pcidas_ao_cmd(comedi_device *dev,comedi_subdevice *s);
 static int cb_pcidas_ao_inttrig(comedi_device *dev, comedi_subdevice *subdev, unsigned int trig_num);
 static int cb_pcidas_ao_cmdtest(comedi_device *dev,comedi_subdevice *s,
 	comedi_cmd *cmd);
-static irqreturn_t cb_pcidas_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t cb_pcidas_interrupt(int irq, void *d PT_REGS_ARG);
 static void handle_ao_interrupt(comedi_device *dev, unsigned int status);
 static int cb_pcidas_cancel(comedi_device *dev, comedi_subdevice *s);
 static int cb_pcidas_ao_cancel(comedi_device *dev, comedi_subdevice *s);
@@ -522,8 +522,8 @@ static int cb_pcidas_attach(comedi_device *dev, comedi_devconfig *it)
  */
 	printk("\n");
 
-	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
-		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) 
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ;
+		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev))
 	{
 		// is it not a computer boards card?
 		if(pcidev->vendor != PCI_VENDOR_ID_CB)
@@ -575,17 +575,20 @@ found:
 	 * Initialize devpriv->control_status and devpriv->adc_fifo to point to
 	 * their base address.
 	 */
-	devpriv->s5933_config = pci_resource_start(devpriv->pci_dev, S5933_BADRINDEX); 
-	devpriv->control_status = pci_resource_start(devpriv->pci_dev, CONT_STAT_BADRINDEX); 
-	devpriv->adc_fifo = pci_resource_start(devpriv->pci_dev, ADC_FIFO_BADRINDEX); 
-	devpriv->pacer_counter_dio = pci_resource_start(devpriv->pci_dev, PACER_BADRINDEX); 
+	devpriv->s5933_config = pci_resource_start(devpriv->pci_dev, S5933_BADRINDEX);
+	devpriv->control_status = pci_resource_start(devpriv->pci_dev, CONT_STAT_BADRINDEX);
+	devpriv->adc_fifo = pci_resource_start(devpriv->pci_dev, ADC_FIFO_BADRINDEX);
+	devpriv->pacer_counter_dio = pci_resource_start(devpriv->pci_dev, PACER_BADRINDEX);
 	if(thisboard->ao_nchan)
 	{
-		devpriv->ao_registers = pci_resource_start(devpriv->pci_dev, AO_BADRINDEX); 
+		devpriv->ao_registers = pci_resource_start(devpriv->pci_dev, AO_BADRINDEX);
 	}
 
+	// disable and clear interrupts on amcc s5933
+	outl(INTCSR_INBOX_INTR_STATUS, devpriv->s5933_config + AMCC_OP_REG_INTCSR);
+
 	// get irq
-	if(comedi_request_irq(devpriv->pci_dev->irq, cb_pcidas_interrupt, SA_SHIRQ, "cb_pcidas", dev ))
+	if(comedi_request_irq(devpriv->pci_dev->irq, cb_pcidas_interrupt, IRQF_SHARED, "cb_pcidas", dev ))
 	{
 		printk(" unable to allocate irq %d\n", devpriv->pci_dev->irq);
 		return -EINVAL;
@@ -605,7 +608,7 @@ found:
 	/* analog input subdevice */
 	dev->read_subdev = s;
 	s->type = COMEDI_SUBD_AI;
-	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_DIFF;
+	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_DIFF | SDF_CMD_READ;
 	/* WARNING: Number of inputs in differential mode is ignored */
 	s->n_chan = thisboard->ai_se_chans;
 	s->len_chanlist = thisboard->ai_se_chans;
@@ -631,6 +634,7 @@ found:
 		if(thisboard->has_ao_fifo)
 		{
 			dev->write_subdev = s;
+			s->subdev_flags |= SDF_CMD_WRITE;
 			s->insn_write = cb_pcidas_ao_fifo_winsn;
 			s->do_cmdtest = cb_pcidas_ao_cmdtest;
 			s->do_cmd = cb_pcidas_ao_cmd;
@@ -1409,8 +1413,8 @@ static int cb_pcidas_ao_cmd(comedi_device *dev,comedi_subdevice *s)
 			cmd->flags);
 
 		/* Write the values of ctr1 and ctr2 into counters 1 and 2 */
-		i8254_load(devpriv->pacer_counter_dio + DAC8254, 1, devpriv->ao_divisor1, 2);
-		i8254_load(devpriv->pacer_counter_dio + DAC8254, 2, devpriv->ao_divisor2, 2);
+		i8254_load(devpriv->pacer_counter_dio + DAC8254, 0, 1, devpriv->ao_divisor1, 2);
+		i8254_load(devpriv->pacer_counter_dio + DAC8254, 0, 2, devpriv->ao_divisor2, 2);
 	}
 
 	// set number of conversions
@@ -1491,7 +1495,7 @@ static int cb_pcidas_ao_inttrig(comedi_device *dev, comedi_subdevice *s, unsigne
 	return 0;
 }
 
-static irqreturn_t cb_pcidas_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t cb_pcidas_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = (comedi_device*) d;
 	comedi_subdevice *s = dev->read_subdev;
@@ -1504,7 +1508,7 @@ static irqreturn_t cb_pcidas_interrupt(int irq, void *d, struct pt_regs *regs)
 
 	if(dev->attached == 0)
 	{
-		comedi_error(dev, "premature interrupt");
+		return IRQ_NONE;
 	}
 
 	async = s->async;
@@ -1603,7 +1607,7 @@ static irqreturn_t cb_pcidas_interrupt(int irq, void *d, struct pt_regs *regs)
 		async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 	}
 
-	comedi_event(dev, s, async->events);
+	comedi_event(dev, s);
 
 	return IRQ_HANDLED;
 }
@@ -1660,7 +1664,7 @@ static void handle_ao_interrupt(comedi_device *dev, unsigned int status)
 		comedi_spin_unlock_irqrestore( &dev->spinlock, flags );
 	}
 
-	comedi_event(dev, s, async->events);
+	comedi_event(dev, s);
 }
 
 // cancel analog input command
@@ -1707,8 +1711,8 @@ static void cb_pcidas_load_counters(comedi_device *dev, unsigned int *ns, int ro
 		&(devpriv->divisor2), ns, rounding_flags & TRIG_ROUND_MASK);
 
 	/* Write the values of ctr1 and ctr2 into counters 1 and 2 */
-	i8254_load(devpriv->pacer_counter_dio + ADC8254, 1, devpriv->divisor1, 2);
-	i8254_load(devpriv->pacer_counter_dio + ADC8254, 2, devpriv->divisor2, 2);
+	i8254_load(devpriv->pacer_counter_dio + ADC8254, 0, 1, devpriv->divisor1, 2);
+	i8254_load(devpriv->pacer_counter_dio + ADC8254, 0, 2, devpriv->divisor2, 2);
 }
 
 static void write_calibration_bitstream( comedi_device *dev, unsigned int register_bits,
