@@ -1,5 +1,5 @@
 /*
-    cb_pcidda.c
+    comedi/drivers/cb_pcidda.c
     This intends to be a driver for the ComputerBoards / MeasurementComputing
     PCI-DDA series.
 
@@ -25,7 +25,7 @@
 
 */
 /*
-Driver: cb_pcidda.o
+Driver: cb_pcidda
 Description: MeasurementComputing PCI-DDA series
 Author: Ivan Martinez <ivanmr@altavista.com>, Frank Mori Hess <fmhess@users.sourceforge.net>
 Status: Supports 08/16, 04/16, 02/16, 08/12, 04/12, and 02/12
@@ -112,7 +112,7 @@ Please report sucess/failure with other different cards to
 
 #define DADATA	8	// FIRST D/A DATA REGISTER (0)
 
-static comedi_lrange cb_pcidda_ranges =
+static const comedi_lrange cb_pcidda_ranges =
 {
 	6,
 	{
@@ -132,7 +132,7 @@ static comedi_lrange cb_pcidda_ranges =
  */
 typedef struct cb_pcidda_board_struct
 {
-	char *name;
+	const char *name;
 	char status;		// Driver status:
 				// 0 - tested
 				// 1 - manual read, not tested
@@ -140,9 +140,9 @@ typedef struct cb_pcidda_board_struct
 	unsigned short device_id;
 	int ao_chans;
 	int ao_bits;
-	comedi_lrange *ranges;
+	const comedi_lrange *ranges;
 } cb_pcidda_board;
-static cb_pcidda_board cb_pcidda_boards[] =
+static const cb_pcidda_board cb_pcidda_boards[] =
 {
 	{
 		name:		"pci-dda02/12",
@@ -208,7 +208,7 @@ MODULE_DEVICE_TABLE(pci, cb_pcidda_pci_table);
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((cb_pcidda_board *)dev->board_ptr)
+#define thisboard ((const cb_pcidda_board *)dev->board_ptr)
 
 /* this structure is for data unique to this hardware driver.  If
    several hardware drivers keep similar information in this structure,
@@ -240,9 +240,8 @@ static int cb_pcidda_detach(comedi_device *dev);
 //static int cb_pcidda_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 static int cb_pcidda_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 //static int cb_pcidda_ai_cmd(comedi_device *dev,comedi_subdevice *s);
-static int cb_pcidda_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,
-	comedi_cmd *cmd);
-static int cb_pcidda_ns_to_timer(unsigned int *ns,int round);
+//static int cb_pcidda_ai_cmdtest(comedi_device *dev,comedi_subdevice *s, comedi_cmd *cmd);
+//static int cb_pcidda_ns_to_timer(unsigned int *ns,int round);
 static unsigned int cb_pcidda_serial_in(comedi_device *dev);
 static void cb_pcidda_serial_out(comedi_device *dev, unsigned int value, unsigned int num_bits);
 static unsigned int cb_pcidda_read_eeprom(comedi_device *dev, unsigned int address);
@@ -284,7 +283,7 @@ static int cb_pcidda_attach(comedi_device *dev, comedi_devconfig *it)
  */
 	printk("\n");
 
-	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ;
 		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) {
 		if(pcidev->vendor==PCI_VENDOR_ID_CB){
 			if(it->options[0] || it->options[1]){
@@ -358,8 +357,9 @@ found:
 	s->maxdata = (1 << thisboard->ao_bits) - 1;
 	s->range_table = thisboard->ranges;
 	s->insn_write = cb_pcidda_ao_winsn;
-//	s->do_cmd = cb_pcidda_ai_cmd;
-	s->do_cmdtest = cb_pcidda_ai_cmdtest;
+// 	s->subdev_flags |= SDF_CMD_READ;
+// 	s->do_cmd = cb_pcidda_ai_cmd;
+// 	s->do_cmdtest = cb_pcidda_ai_cmdtest;
 
 	// two 8255 digital io subdevices
 	s = dev->subdevices + 1;
@@ -423,7 +423,8 @@ static int cb_pcidda_detach(comedi_device *dev)
 
 /*
  * I will program this later... ;-)
- *
+ */
+#if 0
 static int cb_pcidda_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 {
 	printk("cb_pcidda_ai_cmd\n");
@@ -440,8 +441,9 @@ static int cb_pcidda_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 	printk("stop_arg: %d\n", cmd->stop_arg);
 	printk("chanlist_len: %d\n", cmd->chanlist_len);
 }
-*/
+#endif
 
+#if 0
 static int cb_pcidda_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,
 	comedi_cmd *cmd)
 {
@@ -607,17 +609,20 @@ static int cb_pcidda_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,
 
 	return 0;
 }
+#endif
 
 /* This function doesn't require a particular form, this is just
  * what happens to be used in some of the drivers.  It should
  * convert ns nanoseconds to a counter value suitable for programming
  * the device.  Also, it should adjust ns so that it cooresponds to
  * the actual time that the device will use. */
+#if 0
 static int cb_pcidda_ns_to_timer(unsigned int *ns,int round)
 {
 	/* trivial timer */
 	return *ns;
 }
+#endif
 
 static int cb_pcidda_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data)
 {
