@@ -1,5 +1,5 @@
 /*
-    module/comedi_rt_timer.c
+    comedi/drivers/comedi_rt_timer.c
     virtual driver for using RTL timing sources
 
     Authors: David A. Schleef, Frank M. Hess
@@ -25,7 +25,7 @@
 **************************************************************************
 */
 /*
-Driver: comedi_rt_timer.o
+Driver: comedi_rt_timer
 Description: Command emulator using real-time tasks
 Author: ds, fmhess
 Devices:
@@ -331,7 +331,7 @@ static void scan_task_func(int d)
 				}
 			}
 			s->async->events |= COMEDI_CB_BLOCK;
-			comedi_event(dev,s,s->async->events);
+			comedi_event(dev,s);
 			s->async->events = 0;
 		}
 
@@ -339,7 +339,7 @@ cleanup:
 
 		comedi_unlock(devpriv->device,devpriv->subd);
 		async->events |= COMEDI_CB_EOA;
-		comedi_event(dev, s, async->events);
+		comedi_event(dev, s);
 		async->events = 0;
 		devpriv->scan_task_active = 0;
 		// suspend task until next comedi_cmd
@@ -634,12 +634,14 @@ static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 	case COMEDI_SUBD_AI:
 		s->insn_read=timer_insn;
 		dev->read_subdev = s;
+		s->subdev_flags |= SDF_CMD_READ;
 		devpriv->io_function = timer_data_read;
 		break;
 	case COMEDI_SUBD_AO:
 		s->insn_write=timer_insn;
 		s->insn_read=timer_insn;
 		dev->write_subdev = s;
+		s->subdev_flags |= SDF_CMD_WRITE;
 		devpriv->io_function = timer_data_write;
 		break;
 	case COMEDI_SUBD_DIO:
@@ -647,6 +649,7 @@ static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 		s->insn_read=timer_insn;
 		s->insn_bits=timer_insn;
 		dev->read_subdev = s;
+		s->subdev_flags |= SDF_CMD_READ;
 		devpriv->io_function = timer_dio_read;
 		break;
 	default:
@@ -658,8 +661,7 @@ static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 	start_rt_timer( 1 );
 	devpriv->timer_running = 1;
 
-	devpriv->rt_task = kmalloc(sizeof(RT_TASK),GFP_KERNEL);
-	memset(devpriv->rt_task,0,sizeof(RT_TASK));
+	devpriv->rt_task = kzalloc(sizeof(RT_TASK),GFP_KERNEL);
 
 	// initialize real-time tasks
 	ret = rt_task_init(devpriv->rt_task, timer_task_func,(int)dev, 3000,
@@ -671,8 +673,7 @@ static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 		return ret;
 	}
 
-	devpriv->scan_task = kmalloc(sizeof(RT_TASK),GFP_KERNEL);
-	memset(devpriv->scan_task,0,sizeof(RT_TASK));
+	devpriv->scan_task = kzalloc(sizeof(RT_TASK),GFP_KERNEL);
 
 	ret = rt_task_init(devpriv->scan_task, scan_task_func,
 		(int)dev, 3000, scan_priority, 0, 0);
@@ -703,7 +704,7 @@ static int timer_detach(comedi_device *dev)
 		if(devpriv->timer_running)
 			stop_rt_timer();
 		if(devpriv->device)
-			comedi_close(devpriv->device);	
+			comedi_close(devpriv->device);
 	}
 	return 0;
 }
