@@ -25,7 +25,7 @@
 
 */
 /*
-Driver: das08.o
+Driver: das08
 Description: DAS-08 compatible boards
 Author: Warren Jasper, ds, Frank Hess
 Devices: [Keithley Metrabyte] DAS08 (das08), [ComputerBoards] DAS08 (das08),
@@ -164,7 +164,7 @@ static int das08jr_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *
 static int das08ao_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 static void i8254_set_mode_low(unsigned int base, int channel, unsigned int mode);
 
-static comedi_lrange range_das08_pgl = { 9, {
+static const comedi_lrange range_das08_pgl = { 9, {
 	BIP_RANGE(10),
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
@@ -175,7 +175,7 @@ static comedi_lrange range_das08_pgl = { 9, {
 	UNI_RANGE(2.5),
 	UNI_RANGE(1.25)
 }};
-static comedi_lrange range_das08_pgh = { 12, {
+static const comedi_lrange range_das08_pgh = { 12, {
 	BIP_RANGE(10),
 	BIP_RANGE(5),
 	BIP_RANGE(1),
@@ -189,7 +189,7 @@ static comedi_lrange range_das08_pgh = { 12, {
 	UNI_RANGE(0.1),
 	UNI_RANGE(0.01),
 }};
-static comedi_lrange range_das08_pgm = { 9, {
+static const comedi_lrange range_das08_pgm = { 9, {
 	BIP_RANGE(10),
 	BIP_RANGE(5),
 	BIP_RANGE(0.5),
@@ -215,7 +215,7 @@ static comedi_lrange range_das08_pgm = { 9, {
 
 */
 
-static comedi_lrange *das08_ai_lranges[]={
+static const comedi_lrange * const das08_ai_lranges[]={
 	&range_unknown,
 	&range_bipolar5,
 	&range_das08_pgh,
@@ -223,11 +223,11 @@ static comedi_lrange *das08_ai_lranges[]={
 	&range_das08_pgm,
 };
 
-static int das08_pgh_gainlist[] = { 8, 0, 10, 2, 12, 4, 14, 6, 1, 3, 5, 7 };
-static int das08_pgl_gainlist[] = { 8, 0, 2, 4, 6, 1, 3, 5, 7 };
-static int das08_pgm_gainlist[] = { 8, 0, 10, 12, 14, 9, 11, 13, 15 };
+static const int das08_pgh_gainlist[] = { 8, 0, 10, 2, 12, 4, 14, 6, 1, 3, 5, 7 };
+static const int das08_pgl_gainlist[] = { 8, 0, 2, 4, 6, 1, 3, 5, 7 };
+static const int das08_pgm_gainlist[] = { 8, 0, 10, 12, 14, 9, 11, 13, 15 };
 
-static int *das08_gainlists[] = {
+static const int * const das08_gainlists[] = {
 	NULL,
 	NULL,
 	das08_pgh_gainlist,
@@ -235,7 +235,7 @@ static int *das08_gainlists[] = {
 	das08_pgm_gainlist,
 };
 
-static struct das08_board_struct das08_boards[]={
+static const struct das08_board_struct das08_boards[]={
 	{
 	name:		"das08",		// cio-das08.pdf
 	bustype:	isa,
@@ -489,7 +489,7 @@ static struct pci_device_id das08_pci_table[] __devinitdata = {
 MODULE_DEVICE_TABLE(pci, das08_pci_table);
 
 #define devpriv ((struct das08_private_struct *)dev->private)
-#define thisboard ((struct das08_board_struct *)dev->board_ptr)
+#define thisboard ((const struct das08_board_struct *)dev->board_ptr)
 
 #define TIMEOUT 100000
 
@@ -677,7 +677,7 @@ static unsigned int i8254_read_channel_low(unsigned int base, int chan)
         /* The following instructions must be in order.
            We must avoid other process reading the counter's value in the
            middle.
-           The spin_lock isn't needed since ioctl calls grab the big kernel 
+           The spin_lock isn't needed since ioctl calls grab the big kernel
            lock automatically */
         /*spin_lock(sp);*/
         outb(chan<<6,base+I8254_CTRL);
@@ -748,7 +748,7 @@ static unsigned int i8254_read_status_low(unsigned int base, int channel)
 static unsigned int i8254_read_status(struct i8254_struct *st, int channel)
 {
         int chan=st->logic2phys[channel];
-        
+
         return i8254_read_status_low(st->iobase,chan);
 }
 
@@ -759,7 +759,7 @@ static int das08_counter_read(comedi_device *dev,comedi_subdevice *s, comedi_ins
         //printk("Reading counter channel %d ",chan);
         data[0]=i8254_read_channel(&devpriv->i8254,chan);
         //printk("=> 0x%08X\n",data[0]);
-           
+
 	return 1;
 }
 
@@ -769,7 +769,7 @@ static int das08_counter_write(comedi_device *dev,comedi_subdevice *s, comedi_in
 
         //printk("Writing counter channel %d with 0x%04X\n",chan,data[0]);
         i8254_write_channel(&devpriv->i8254,chan,data[0]);
-           
+
 	return 1;
 }
 
@@ -780,7 +780,7 @@ static int das08_counter_config(comedi_device *dev,comedi_subdevice *s,comedi_in
 	if(insn->n != 2)return -EINVAL;
 
 	switch (data[0]){
-	case INSN_CONFIG_8254_SET_MODE:
+	case INSN_CONFIG_SET_COUNTER_MODE:
 		i8254_set_mode(&devpriv->i8254,chan,data[1]);
 		break;
 	case INSN_CONFIG_8254_READ_STATUS:
@@ -800,7 +800,7 @@ static comedi_driver driver_das08={
 	module:		THIS_MODULE,
 	attach:		das08_attach,
 	detach:		das08_common_detach,
-	board_name:	das08_boards,
+	board_name:	&das08_boards[0].name,
 	num_names:	sizeof(das08_boards)/sizeof(struct das08_board_struct),
 	offset:		sizeof(struct das08_board_struct),
 };
@@ -937,7 +937,7 @@ static int das08_attach(comedi_device *dev,comedi_devconfig *it)
 		}
 		printk("\n");
 		// find card
-		for(pdev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pdev != NULL ; 
+		for(pdev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pdev != NULL ;
 			pdev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pdev)) {
 			if(pdev->vendor == PCI_VENDOR_ID_COMPUTERBOARDS &&
 				pdev->device == PCI_DEVICE_ID_PCIDAS08){
