@@ -30,7 +30,7 @@ enum das08_ai_encoding {das08_encode12, das08_encode16, das08_pcm_encode12};
 enum das08_lrange {das08_pg_none, das08_bipolar5, das08_pgh, das08_pgl, das08_pgm};
 
 typedef struct das08_board_struct{
-	char		*name;
+	const char * name;
 	unsigned int	id;	// id for pci/pcmcia boards
 	enum das08_bustype	bustype;
 	void		*ai;
@@ -63,7 +63,7 @@ struct i8254_struct
 struct das08_private_struct{
 	unsigned int	do_mux_bits;	// bits for do/mux register on boards without seperate do register
 	unsigned int	do_bits;	// bits for do register on boards with register dedicated to digital out only
-	unsigned int	*pg_gainlist;
+	const unsigned int *pg_gainlist;
 	struct pci_dev *pdev;	// struct for pci-das08
 	unsigned int	pci_iobase;	// additional base address for pci-das08
         struct i8254_struct i8254;
