@@ -24,7 +24,7 @@
 ************************************************************************
 */
 /*
-Driver: das16.o
+Driver: das16
 Description: DAS16 compatible boards
 Author: Sam Moore, Warren Jasper, ds, Chris Baugher, Frank Hess, Roman Fietze
 Devices: [Keithley Metrabyte] DAS-16 (das-16), DAS-16G (das-16g),
@@ -236,31 +236,31 @@ static const int sample_size = 2;	// size in bytes of a sample from board
 #define   DAS1600_WS			0x02
 #define   DAS1600_CLK_10MHZ		0x01
 
-static comedi_lrange range_das1x01_bip = { 4, {
+static const comedi_lrange range_das1x01_bip = { 4, {
 	BIP_RANGE( 10 ),
 	BIP_RANGE( 1 ),
 	BIP_RANGE( 0.1 ),
 	BIP_RANGE( 0.01 ),
 }};
-static comedi_lrange range_das1x01_unip = { 4, {
+static const comedi_lrange range_das1x01_unip = { 4, {
 	UNI_RANGE( 10 ),
 	UNI_RANGE( 1 ),
 	UNI_RANGE( 0.1 ),
 	UNI_RANGE( 0.01 ),
 }};
-static comedi_lrange range_das1x02_bip = { 4, {
+static const comedi_lrange range_das1x02_bip = { 4, {
 	BIP_RANGE( 10 ),
 	BIP_RANGE( 5 ),
 	BIP_RANGE( 2.5 ),
 	BIP_RANGE( 1.25 ),
 }};
-static comedi_lrange range_das1x02_unip = { 4, {
+static const comedi_lrange range_das1x02_unip = { 4, {
 	UNI_RANGE( 10 ),
 	UNI_RANGE( 5 ),
 	UNI_RANGE( 2.5 ),
 	UNI_RANGE( 1.25 ),
 }};
-static comedi_lrange range_das16jr = { 9, {
+static const comedi_lrange range_das16jr = { 9, {
 	// also used by 16/330
 	BIP_RANGE( 10 ),
 	BIP_RANGE( 5 ),
@@ -272,7 +272,7 @@ static comedi_lrange range_das16jr = { 9, {
 	UNI_RANGE( 2.5 ),
 	UNI_RANGE( 1.25 ),
 }};
-static comedi_lrange range_das16jr_16 = { 8, {
+static const comedi_lrange range_das16jr_16 = { 8, {
 	BIP_RANGE( 10 ),
 	BIP_RANGE( 5 ),
 	BIP_RANGE( 2.5 ),
@@ -283,9 +283,9 @@ static comedi_lrange range_das16jr_16 = { 8, {
 	UNI_RANGE( 1.25 ),
 }};
 
-static int das16jr_gainlist[] = { 8, 0, 1, 2, 3, 4, 5, 6, 7 };
-static int das16jr_16_gainlist[] = { 0, 1, 2, 3, 4, 5, 6, 7 };
-static int das1600_gainlist[] = { 0, 1, 2, 3 };
+static const int das16jr_gainlist[] = { 8, 0, 1, 2, 3, 4, 5, 6, 7 };
+static const int das16jr_16_gainlist[] = { 0, 1, 2, 3, 4, 5, 6, 7 };
+static const int das1600_gainlist[] = { 0, 1, 2, 3 };
 enum {
 	das16_pg_none = 0,
 	das16_pg_16jr,
@@ -293,21 +293,21 @@ enum {
 	das16_pg_1601,
 	das16_pg_1602,
 };
-static int *das16_gainlists[] = {
+static const int * const das16_gainlists[] = {
 	NULL,
 	das16jr_gainlist,
 	das16jr_16_gainlist,
 	das1600_gainlist,
 	das1600_gainlist,
 };
-static comedi_lrange *das16_ai_uni_lranges[]={
+static const comedi_lrange * const das16_ai_uni_lranges[]={
 	&range_unknown,
 	&range_das16jr,
 	&range_das16jr_16,
 	&range_das1x01_unip,
 	&range_das1x02_unip,
 };
-static comedi_lrange *das16_ai_bip_lranges[]={
+static const comedi_lrange *const das16_ai_bip_lranges[]={
 	&range_unknown,
 	&range_das16jr,
 	&range_das16jr_16,
@@ -333,7 +333,7 @@ static void das16_ai_munge(comedi_device *dev, comedi_subdevice *s, void *array,
 	unsigned int num_bytes, unsigned int start_chan_index);
 
 static void das16_reset(comedi_device *dev);
-static irqreturn_t das16_dma_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t das16_dma_interrupt(int irq, void *d PT_REGS_ARG);
 static void das16_timer_interrupt(unsigned long arg);
 static void das16_interrupt(comedi_device *dev);
 
@@ -344,7 +344,7 @@ static unsigned int das16_suggest_transfer_size(comedi_device *dev, comedi_cmd c
 static void reg_dump(comedi_device *dev);
 
 typedef struct das16_board_struct{
-	char		*name;
+	const char		*name;
 	void		*ai;
 	unsigned int	ai_nbits;
 	unsigned int	ai_speed;	// max conversion speed in nanosec
@@ -361,7 +361,7 @@ typedef struct das16_board_struct{
 	unsigned int id;
 } das16_board;
 
-static struct das16_board_struct das16_boards[]={
+static const struct das16_board_struct das16_boards[]={
 	{
 	name:		"das-16",
 	ai:		das16_ai_rinsn,
@@ -701,7 +701,7 @@ static comedi_driver driver_das16={
 	module:		THIS_MODULE,
 	attach:		das16_attach,
 	detach:		das16_detach,
-	board_name:	das16_boards,
+	board_name:	&das16_boards[0].name,
 	num_names:	n_das16_boards,
 	offset:		sizeof(das16_boards[0]),
 };
@@ -1142,7 +1142,7 @@ static int das16_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *in
 }
 
 
-static irqreturn_t das16_dma_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t das16_dma_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	int status;
 	comedi_device *dev = d;
@@ -1296,8 +1296,8 @@ static unsigned int das16_set_pacer(comedi_device *dev, unsigned int ns, int rou
 		&(devpriv->divisor2), &ns, rounding_flags & TRIG_ROUND_MASK);
 
 	/* Write the values of ctr1 and ctr2 into counters 1 and 2 */
-	i8254_load(dev->iobase + DAS16_CNTR0_DATA, 1, devpriv->divisor1, 2);
-	i8254_load(dev->iobase + DAS16_CNTR0_DATA, 2, devpriv->divisor2, 2);
+	i8254_load(dev->iobase + DAS16_CNTR0_DATA, 0, 1, devpriv->divisor1, 2);
+	i8254_load(dev->iobase + DAS16_CNTR0_DATA, 0, 2, devpriv->divisor2, 2);
 
 	return ns;
 }
@@ -1557,7 +1557,7 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	/* ai */
 	if(thisboard->ai){
 		s->type = COMEDI_SUBD_AI;
-		s->subdev_flags = SDF_READABLE;
+		s->subdev_flags = SDF_READABLE | SDF_CMD_READ;
 		if(devpriv->ai_singleended){
 			s->n_chan = 16;
 			s->len_chanlist = 16;
