@@ -25,10 +25,10 @@
 ************************************************************************
 */
 /*
-Driver: das16m1.o
+Driver: das16m1
 Description: CIO-DAS16/M1
 Author: Frank Mori Hess <fmhess@users.sourceforge.net>
-Devices: [MeasurementComputing] CIO-DAS16/M1 (cio-das16/m1)
+Devices: [Measurement Computing] CIO-DAS16/M1 (cio-das16/m1)
 Status: works
 
 This driver supports a single board - the CIO-DAS16/M1.
@@ -117,7 +117,7 @@ irq can be omitted, although the cmd interface will not work without it.
 #define DAS16M1_82C55                  0x400
 #define DAS16M1_8254_THIRD             0x404
 
-static comedi_lrange range_das16m1 =
+static const comedi_lrange range_das16m1 =
 { 9,
 	{
 		BIP_RANGE( 5 ),
@@ -141,7 +141,7 @@ static int das16m1_cmd_exec(comedi_device *dev,comedi_subdevice *s);
 static int das16m1_cancel(comedi_device *dev, comedi_subdevice *s);
 
 static int das16m1_poll(comedi_device *dev, comedi_subdevice *s);
-static irqreturn_t das16m1_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t das16m1_interrupt(int irq, void *d PT_REGS_ARG);
 static void das16m1_handler(comedi_device *dev, unsigned int status);
 
 static unsigned int das16m1_set_pacer(comedi_device *dev, unsigned int ns, int round_flag);
@@ -149,11 +149,11 @@ static unsigned int das16m1_set_pacer(comedi_device *dev, unsigned int ns, int r
 static int das16m1_irq_bits(unsigned int irq);
 
 typedef struct das16m1_board_struct{
-	char *name;
+	const char *name;
 	unsigned int ai_speed;
 }das16m1_board;
 
-static das16m1_board das16m1_boards[]={
+static const das16m1_board das16m1_boards[]={
 	{
 	name:		"cio-das16/m1",	// CIO-DAS16_M1.pdf
 	ai_speed:	1000,		// 1MHz max speed
@@ -169,7 +169,7 @@ static comedi_driver driver_das16m1={
 	module:		THIS_MODULE,
 	attach:		das16m1_attach,
 	detach:		das16m1_detach,
-	board_name:	das16m1_boards,
+	board_name:	&das16m1_boards[0].name,
 	num_names:	das16m1_num_boards,
 	offset:		sizeof(das16m1_boards[0]),
 };
@@ -187,7 +187,7 @@ struct das16m1_private_struct {
 	unsigned int divisor2;	// divides master clock to obtain conversion speed
 };
 #define devpriv ((struct das16m1_private_struct *)(dev->private))
-#define thisboard ((struct das16m1_board_struct *)(dev->board_ptr))
+#define thisboard ((const das16m1_board *)(dev->board_ptr))
 
 COMEDI_INITCLEANUP(driver_das16m1);
 
@@ -332,10 +332,10 @@ static int das16m1_cmd_exec(comedi_device *dev,comedi_subdevice *s)
 	/* Initialize lower half of hardware counter, used to determine how
 	 * many samples are in fifo.  Value doesn't actually load into counter
 	 * until counter's next clock (the next a/d conversion) */
-	i8254_load(dev->iobase + DAS16M1_8254_FIRST, 1, 0, 2);
+	i8254_load(dev->iobase + DAS16M1_8254_FIRST, 0, 1, 0, 2);
 	/* remember current reading of counter so we know when counter has
 	 * actually been loaded */
-	devpriv->initial_hw_count = i8254_read(dev->iobase + DAS16M1_8254_FIRST, 1);
+	devpriv->initial_hw_count = i8254_read(dev->iobase + DAS16M1_8254_FIRST, 0, 1);
 	/* setup channel/gain queue */
 	for(i = 0; i < cmd->chanlist_len; i++)
 	{
@@ -464,7 +464,7 @@ static int das16m1_poll(comedi_device *dev, comedi_subdevice *s)
 	return s->async->buf_write_count - s->async->buf_read_count;
 }
 
-static irqreturn_t das16m1_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t das16m1_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	int status;
 	comedi_device *dev = d;
@@ -520,7 +520,7 @@ static void das16m1_handler(comedi_device *dev, unsigned int status)
 	cmd = &async->cmd;
 
 	// figure out how many samples are in fifo
-	hw_counter = i8254_read(dev->iobase + DAS16M1_8254_FIRST, 1);
+	hw_counter = i8254_read(dev->iobase + DAS16M1_8254_FIRST, 0, 1);
 	/* make sure hardware counter reading is not bogus due to initial value
 	 * not having been loaded yet */
 	if(devpriv->adc_count == 0 && hw_counter == devpriv->initial_hw_count)
@@ -568,7 +568,7 @@ static void das16m1_handler(comedi_device *dev, unsigned int status)
 		comedi_error(dev, "fifo overflow");
 	}
 
-	comedi_event(dev, s, async->events);
+	comedi_event(dev, s);
 
 }
 
@@ -581,8 +581,8 @@ static unsigned int das16m1_set_pacer(comedi_device *dev, unsigned int ns, int r
 		&(devpriv->divisor2), &ns, rounding_flags & TRIG_ROUND_MASK);
 
 	/* Write the values of ctr1 and ctr2 into counters 1 and 2 */
-	i8254_load(dev->iobase + DAS16M1_8254_SECOND, 1, devpriv->divisor1, 2);
-	i8254_load(dev->iobase + DAS16M1_8254_SECOND, 2, devpriv->divisor2, 2);
+	i8254_load(dev->iobase + DAS16M1_8254_SECOND, 0, 1, devpriv->divisor1, 2);
+	i8254_load(dev->iobase + DAS16M1_8254_SECOND, 0, 2, devpriv->divisor2, 2);
 
 	return ns;
 }
@@ -689,7 +689,7 @@ static int das16m1_attach(comedi_device *dev, comedi_devconfig *it)
 	dev->read_subdev = s;
 	/* ai */
 	s->type = COMEDI_SUBD_AI;
-	s->subdev_flags = SDF_READABLE;
+	s->subdev_flags = SDF_READABLE | SDF_CMD_READ;
 	s->n_chan = 8;
 	s->subdev_flags = SDF_DIFF;
 	s->len_chanlist = 256;
