@@ -1,5 +1,6 @@
 /*
-    das1800.c driver for Keitley das1700/das1800 series boards
+    comedi/drivers/das1800.c
+    Driver for Keitley das1700/das1800 series boards
     Copyright (C) 2000 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     COMEDI - Linux Control and Measurement Device Interface
@@ -22,7 +23,7 @@
 ************************************************************************
 */
 /*
-Driver: das1800.o
+Driver: das1800
 Description: Keithley Metrabyte DAS1800 (& compatibles)
 Author: Frank Mori Hess <fmhess@users.sourceforge.net>
 Devices: [Keithley Metrabyte] DAS-1701ST (das-1701st),
@@ -182,7 +183,7 @@ static int das1800_attach(comedi_device *dev, comedi_devconfig *it);
 static int das1800_detach(comedi_device *dev);
 static int das1800_probe(comedi_device *dev);
 static int das1800_cancel(comedi_device *dev, comedi_subdevice *s);
-static irqreturn_t das1800_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t das1800_interrupt(int irq, void *d PT_REGS_ARG);
 static int das1800_ai_poll(comedi_device *dev,comedi_subdevice *s);
 static void das1800_ai_handler(comedi_device *dev);
 static void das1800_handle_dma(comedi_device *dev, comedi_subdevice *s, unsigned int status);
@@ -202,7 +203,7 @@ static unsigned int burst_convert_arg(unsigned int convert_arg, int round_mode);
 static unsigned int suggest_transfer_size(comedi_cmd *cmd);
 
 // analog input ranges
-static comedi_lrange range_ai_das1801 = {
+static const comedi_lrange range_ai_das1801 = {
 	8,
 	{
 		RANGE( -5, 5 ),
@@ -216,7 +217,7 @@ static comedi_lrange range_ai_das1801 = {
 	}
 };
 
-static comedi_lrange range_ai_das1802 = {
+static const comedi_lrange range_ai_das1802 = {
 	8,
 	{
 		RANGE(-10, 10),
@@ -231,7 +232,7 @@ static comedi_lrange range_ai_das1802 = {
 };
 
 typedef struct das1800_board_struct{
-	char *name;
+	const char *name;
 	int ai_speed;	/* max conversion period in nanoseconds */
 	int resolution;	/* bits of ai resolution */
 	int qram_len;	/* length of card's channel / gain queue */
@@ -239,14 +240,14 @@ typedef struct das1800_board_struct{
 	int do_n_chan;	/* number of digital output channels */
 	int ao_ability;	/* 0 == no analog out, 1 == basic analog out, 2 == waveform analog out */
 	int ao_n_chan;	/* number of analog out channels */
-	comedi_lrange *range_ai;	/* available input ranges */
+	const comedi_lrange *range_ai;	/* available input ranges */
 }das1800_board;
 
 /* Warning: the maximum conversion speeds listed below are
  * not always achievable depending on board setup (see
  * user manual.)
  */
-static das1800_board das1800_boards[] =
+static const das1800_board das1800_boards[] =
 {
 	{
 		name:	"das-1701st",
@@ -450,7 +451,7 @@ static das1800_board das1800_boards[] =
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((das1800_board *)dev->board_ptr)
+#define thisboard ((const das1800_board *)dev->board_ptr)
 
 typedef struct{
 	volatile unsigned int count;  /* number of data points left to be taken */
@@ -475,7 +476,7 @@ typedef struct{
 #define devpriv ((das1800_private *)dev->private)
 
 // analog out range for boards with basic analog out
-static comedi_lrange range_ao_1 = {
+static const comedi_lrange range_ao_1 = {
 	1,
 	{
 		RANGE(-10, 10),
@@ -484,7 +485,7 @@ static comedi_lrange range_ao_1 = {
 
 // analog out range for 'ao' boards
 /*
-static comedi_lrange range_ao_2 = {
+static const comedi_lrange range_ao_2 = {
 	2,
 	{
 		RANGE(-10, 10),
@@ -499,7 +500,7 @@ static comedi_driver driver_das1800={
 	attach:		das1800_attach,
 	detach:		das1800_detach,
 	num_names:	sizeof(das1800_boards) / sizeof(das1800_board),
-	board_name:	das1800_boards,
+	board_name:	&das1800_boards[0].name,
 	offset:		sizeof(das1800_board),
 };
 
@@ -512,7 +513,7 @@ COMEDI_INITCLEANUP(driver_das1800);
 static int das1800_init_dma( comedi_device *dev, unsigned int dma0, unsigned int dma1 )
 {
 	unsigned long flags;
-	
+
 	// need an irq to do dma
 	if( dev->irq && dma0 )
 	{
@@ -705,7 +706,7 @@ static int das1800_attach(comedi_device *dev, comedi_devconfig *it)
 	s = dev->subdevices + 0;
 	dev->read_subdev = s;
 	s->type = COMEDI_SUBD_AI;
-	s->subdev_flags = SDF_READABLE | SDF_DIFF | SDF_GROUND;
+	s->subdev_flags = SDF_READABLE | SDF_DIFF | SDF_GROUND | SDF_CMD_READ;
 	if(thisboard->common)
 		s->subdev_flags |= SDF_COMMON;
 	s->n_chan = thisboard->qram_len;
@@ -883,7 +884,7 @@ static int das1800_ai_poll(comedi_device *dev,comedi_subdevice *s)
 	return s->async->buf_write_count - s->async->buf_read_count;
 }
 
-static irqreturn_t das1800_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t das1800_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = d;
 	unsigned int status;
@@ -947,7 +948,7 @@ static void das1800_ai_handler(comedi_device *dev)
 		comedi_error(dev, "DAS1800 FIFO overflow");
 		das1800_cancel(dev, s);
 		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
-		comedi_event(dev, s, async->events);
+		comedi_event(dev, s);
 		return;
 	}
 
@@ -970,7 +971,7 @@ static void das1800_ai_handler(comedi_device *dev)
 		async->events |= COMEDI_CB_EOA;
 	}
 
-	comedi_event(dev, s, async->events);
+	comedi_event(dev, s);
 
 	return;
 }
@@ -1421,7 +1422,7 @@ static int setup_counters(comedi_device *dev, comedi_cmd cmd)
 	if(cmd.stop_src == TRIG_EXT)
 	{
 		// load counter 0 in mode 0
-		i8254_load(dev->iobase + DAS1800_COUNTER, 0, 1, 0);
+		i8254_load(dev->iobase + DAS1800_COUNTER, 0, 0, 1, 0);
 	}
 
 	return 0;
@@ -1687,9 +1688,9 @@ static int das1800_set_frequency(comedi_device *dev)
 	int err = 0;
 
 	// counter 1, mode 2
-	if(i8254_load(dev->iobase + DAS1800_COUNTER, 1, devpriv->divisor1, 2)) err++;
+	if(i8254_load(dev->iobase + DAS1800_COUNTER, 0, 1, devpriv->divisor1, 2)) err++;
 	// counter 2, mode 2
-	if(i8254_load(dev->iobase + DAS1800_COUNTER, 2, devpriv->divisor2, 2)) err++;
+	if(i8254_load(dev->iobase + DAS1800_COUNTER, 0, 2, devpriv->divisor2, 2)) err++;
 	if(err)
 		return -1;
 
