@@ -1,5 +1,6 @@
 /*
-    das800.c driver for Keitley das800 series boards and compatibles
+    comedi/drivers/das800.c
+    Driver for Keitley das800 series boards and compatibles
     Copyright (C) 2000 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     COMEDI - Linux Control and Measurement Device Interface
@@ -22,7 +23,7 @@
 ************************************************************************
 */
 /*
-Driver: das800.o
+Driver: das800
 Description: Keithley Metrabyte DAS800 (& compatibles)
 Author: Frank Mori Hess <fmhess@users.sourceforge.net>
 Devices: [Keithley Metrabyte] DAS-800 (das-800), DAS-801 (das-801),
@@ -106,21 +107,21 @@ cmd triggers supported:
 #define DAS800_ID             7
 
 typedef struct das800_board_struct{
-	char *name;
+	const char *name;
 	int ai_speed;
-	comedi_lrange *ai_range;
+	const comedi_lrange *ai_range;
 	int resolution;
 }das800_board;
 
 //analog input ranges
-static comedi_lrange range_das800_ai = {
+static const comedi_lrange range_das800_ai = {
 	1,
 	{
 		RANGE( -5, 5 ),
 	}
 };
 
-static comedi_lrange range_das801_ai = {
+static const comedi_lrange range_das801_ai = {
 	9,
 	{
 		RANGE(-5, 5),
@@ -135,7 +136,7 @@ static comedi_lrange range_das801_ai = {
 	}
 };
 
-static comedi_lrange range_cio_das801_ai = {
+static const comedi_lrange range_cio_das801_ai = {
 	9,
 	{
 		RANGE(-5, 5),
@@ -150,7 +151,7 @@ static comedi_lrange range_cio_das801_ai = {
 	}
 };
 
-static comedi_lrange range_das802_ai = {
+static const comedi_lrange range_das802_ai = {
 	9,
 	{
 		RANGE(-5, 5),
@@ -165,7 +166,7 @@ static comedi_lrange range_das802_ai = {
 	}
 };
 
-static comedi_lrange range_das80216_ai = {
+static const comedi_lrange range_das80216_ai = {
 	8,
 	{
 		RANGE(-10, 10),
@@ -181,7 +182,7 @@ static comedi_lrange range_das80216_ai = {
 
 enum{das800, ciodas800, das801, ciodas801, das802, ciodas802, ciodas80216};
 
-static das800_board das800_boards[] =
+static const das800_board das800_boards[] =
 {
 	{
 		name:	"das-800",
@@ -229,7 +230,7 @@ static das800_board das800_boards[] =
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((das800_board *)dev->board_ptr)
+#define thisboard ((const das800_board *)dev->board_ptr)
 
 typedef struct{
 	volatile unsigned int count;  /* number of data points left to be taken */
@@ -251,11 +252,11 @@ static comedi_driver driver_das800={
 	attach:		das800_attach,
 	detach:		das800_detach,
 	num_names:	sizeof(das800_boards) / sizeof(das800_board),
-	board_name:	(char **)das800_boards,
+	board_name:	&das800_boards[0].name,
 	offset:		sizeof(das800_board),
 };
 
-static irqreturn_t das800_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t das800_interrupt(int irq, void *d PT_REGS_ARG);
 static void enable_das800(comedi_device *dev);
 static void disable_das800(comedi_device *dev);
 static int das800_ai_do_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *cmd);
@@ -345,7 +346,7 @@ static int das800_probe(comedi_device *dev)
 COMEDI_INITCLEANUP(driver_das800);
 
 /* interrupt service routine */
-static irqreturn_t das800_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t das800_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	short i;		/* loop index */
 	sampl_t dataPoint = 0;
@@ -428,7 +429,7 @@ static irqreturn_t das800_interrupt(int irq, void *d, struct pt_regs *regs)
 		comedi_error(dev, "DAS800 FIFO overflow");
 		das800_cancel(dev, dev->subdevices + 0);
 		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
-		comedi_event(dev, s, async->events);
+		comedi_event(dev, s);
 		async->events = 0;
 		return IRQ_HANDLED;
 	}
@@ -446,7 +447,7 @@ static irqreturn_t das800_interrupt(int irq, void *d, struct pt_regs *regs)
 		disable_das800(dev);		/* diable hardware triggered conversions */
 		async->events |= COMEDI_CB_EOA;
 	}
-	comedi_event(dev, s, async->events);
+	comedi_event(dev, s);
 	async->events = 0;
 	return IRQ_HANDLED;
 }
@@ -517,7 +518,7 @@ static int das800_attach(comedi_device *dev, comedi_devconfig *it)
 	s = dev->subdevices + 0;
 	dev->read_subdev = s;
 	s->type = COMEDI_SUBD_AI;
-	s->subdev_flags = SDF_READABLE | SDF_GROUND;
+	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_CMD_READ;
 	s->n_chan = 8;
 	s->len_chanlist = 8;
 	s->maxdata = (1 << thisboard->resolution) - 1;
@@ -901,8 +902,8 @@ static int das800_set_frequency(comedi_device *dev)
 {
 	int err = 0;
 
-	if(i8254_load(dev->iobase + DAS800_8254, 1, devpriv->divisor1, 2)) err++;
-	if(i8254_load(dev->iobase + DAS800_8254, 2, devpriv->divisor2, 2)) err++;
+	if(i8254_load(dev->iobase + DAS800_8254, 0, 1, devpriv->divisor1, 2)) err++;
+	if(i8254_load(dev->iobase + DAS800_8254, 0, 2, devpriv->divisor2, 2)) err++;
 	if(err)
 		return -1;
 
