@@ -1,6 +1,6 @@
 /*
-   module/dt2811.c
-   hardware driver for Data Translation DT2811
+   comedi/drivers/dt2811.c
+   Hardware driver for Data Translation DT2811
 
    COMEDI - Linux Control and Measurement Device Interface
    History:
@@ -24,7 +24,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 /*
-Driver: dt2811.o
+Driver: dt2811
 Description: Data Translation DT2811
 Author: ds
 Devices: [Data Translation] DT2811-PGL (dt2811-pgl), DT2811-PGH (dt2811-pgh)
@@ -49,39 +49,39 @@ Configuration options:
 
 #include <linux/ioport.h>
 
-static char *driver_name = "dt2811";
+static const char *driver_name = "dt2811";
 
-static comedi_lrange range_dt2811_pgh_ai_5_unipolar = { 4, {
+static const comedi_lrange range_dt2811_pgh_ai_5_unipolar = { 4, {
 	RANGE( 0,	5 ),
 	RANGE( 0,	2.5 ),
 	RANGE( 0,	1.25 ),
 	RANGE( 0,	0.625 )
 }};
-static comedi_lrange range_dt2811_pgh_ai_2_5_bipolar = { 4, {
+static const comedi_lrange range_dt2811_pgh_ai_2_5_bipolar = { 4, {
 	RANGE( -2.5,	2.5 ),
 	RANGE( -1.25,	1.25 ),
 	RANGE( -0.625,	0.625 ),
 	RANGE( -0.3125,	0.3125 )
 }};
-static comedi_lrange range_dt2811_pgh_ai_5_bipolar = { 4, {
+static const comedi_lrange range_dt2811_pgh_ai_5_bipolar = { 4, {
 	RANGE( -5,	5 ),
 	RANGE( -2.5,	2.5 ),
 	RANGE( -1.25,	1.25 ),
 	RANGE( -0.625,	0.625 )
 }};
-static comedi_lrange range_dt2811_pgl_ai_5_unipolar = { 4, {
+static const comedi_lrange range_dt2811_pgl_ai_5_unipolar = { 4, {
 	RANGE( 0,	5 ),
 	RANGE( 0,	0.5 ),
 	RANGE( 0,	0.05 ),
 	RANGE( 0,	0.01 )
 }};
-static comedi_lrange range_dt2811_pgl_ai_2_5_bipolar = { 4, {
+static const comedi_lrange range_dt2811_pgl_ai_2_5_bipolar = { 4, {
 	RANGE( -2.5,	2.5 ),
 	RANGE( -0.25,	0.25 ),
 	RANGE( -0.025,	0.025 ),
 	RANGE( -0.005,	0.005 )
 }};
-static comedi_lrange range_dt2811_pgl_ai_5_bipolar = { 4, {
+static const comedi_lrange range_dt2811_pgl_ai_5_bipolar = { 4, {
 	RANGE( -5,	5 ),
 	RANGE( -0.5,	0.5 ),
 	RANGE( -0.05,	0.05 ),
@@ -186,12 +186,12 @@ static comedi_lrange range_dt2811_pgl_ai_5_bipolar = { 4, {
 #define DT2811_ADMODE   0x03
 
 typedef struct {
-	char *name;
-	comedi_lrange *bip_5;
-	comedi_lrange *bip_2_5;
-	comedi_lrange *unip_5;
+	const char *name;
+	const comedi_lrange *bip_5;
+	const comedi_lrange *bip_2_5;
+	const comedi_lrange *unip_5;
 }boardtype;
-static boardtype boardtypes[]={
+static const boardtype boardtypes[]={
 	{ "dt2811-pgh",
 		&range_dt2811_pgh_ai_5_bipolar,
 		&range_dt2811_pgh_ai_2_5_bipolar,
@@ -203,7 +203,7 @@ static boardtype boardtypes[]={
 		&range_dt2811_pgl_ai_5_unipolar,
 	},
 };
-#define this_board ((boardtype *)dev->board_ptr)
+#define this_board ((const boardtype *)dev->board_ptr)
 
 static int dt2811_attach(comedi_device *dev,comedi_devconfig *it);
 static int dt2811_detach(comedi_device *dev);
@@ -212,7 +212,7 @@ static comedi_driver driver_dt2811={
 	module:		THIS_MODULE,
 	attach:		dt2811_attach,
 	detach:		dt2811_detach,
-	board_name:	boardtypes,
+	board_name:	&boardtypes[0].name,
 	num_names:	sizeof(boardtypes)/sizeof(boardtype),
 	offset:		sizeof(boardtype),
 };
@@ -239,13 +239,13 @@ typedef struct {
         enum {
 	  dac_bipolar_5, dac_bipolar_2_5, dac_unipolar_5
 	} dac_range[2];
-        comedi_lrange * range_type_list[2];
+        const comedi_lrange * range_type_list[2];
 	lsampl_t ao_readback[2];
 } dt2811_private;
 
 #define devpriv ((dt2811_private *)dev->private)
 
-static comedi_lrange *dac_range_types[] =
+static const comedi_lrange *dac_range_types[] =
 {
   &range_bipolar5,
   &range_bipolar2_5,
@@ -255,12 +255,18 @@ static comedi_lrange *dac_range_types[] =
 #define DT2811_TIMEOUT 5
 
 #if 0
-static void dt2811_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t dt2811_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	int lo, hi;
 	int data;
 	comedi_device *dev = d;
 
+	if (!dev->attached)
+	{
+		comedi_error(dev, "spurious interrupt");
+		return IRQ_HANDLED;
+	}
+
 	lo = inb(dev->iobase + DT2811_ADDATLO);
 	hi = inb(dev->iobase + DT2811_ADDATHI);
 
@@ -270,7 +276,8 @@ static void dt2811_interrupt(int irq, void *d, struct pt_regs *regs)
 		/* how to turn off acquisition */
 		s->async->events |= COMEDI_SB_EOA;
 	}
-	comedi_event(dev, s, s->async->events);
+	comedi_event(dev, s);
+	return IRQ_HANDLED;
 }
 #endif
 
