@@ -1,6 +1,6 @@
 /*
-   modules/dt282x.c
-   hardware driver for Data Translation DT2821 series
+   comedi/drivers/dt282x.c
+   Hardware driver for Data Translation DT2821 series
 
    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@schleef.org>
@@ -21,7 +21,7 @@
 
  */
 /*
-Driver: dt282x.o
+Driver: dt282x
 Description: Data Translation DT2821 series (including DT-EZ)
 Author: ds
 Devices: [Data Translation] DT2821 (dt2821),
@@ -154,37 +154,37 @@ Notes:
 #define DT2821_XCLK	0x0002	/* (R/W) external clock enable            */
 #define DT2821_BDINIT	0x0001	/* (W)   initialize board         */
 
-static comedi_lrange range_dt282x_ai_lo_bipolar = { 4, {
+static const comedi_lrange range_dt282x_ai_lo_bipolar = { 4, {
 	RANGE( -10,	10 ),
 	RANGE( -5,	5 ),
 	RANGE( -2.5,	2.5 ),
 	RANGE( -1.25,	1.25 )
 }};
-static comedi_lrange range_dt282x_ai_lo_unipolar = { 4, {
+static const comedi_lrange range_dt282x_ai_lo_unipolar = { 4, {
 	RANGE( 0,	10 ),
 	RANGE( 0,	5 ),
 	RANGE( 0,	2.5 ),
 	RANGE( 0,	1.25 )
 }};
-static comedi_lrange range_dt282x_ai_5_bipolar = { 4, {
+static const comedi_lrange range_dt282x_ai_5_bipolar = { 4, {
 	RANGE( -5,	5 ),
 	RANGE( -2.5,	2.5 ),
 	RANGE( -1.25,	1.25 ),
 	RANGE( -0.625,	0.625 ),
 }};
-static comedi_lrange range_dt282x_ai_5_unipolar = { 4, {
+static const comedi_lrange range_dt282x_ai_5_unipolar = { 4, {
 	RANGE( 0,	5 ),
 	RANGE( 0,	2.5 ),
 	RANGE( 0,	1.25 ),
 	RANGE( 0,	0.625 ),
 }};
-static comedi_lrange range_dt282x_ai_hi_bipolar = { 4, {
+static const comedi_lrange range_dt282x_ai_hi_bipolar = { 4, {
 	RANGE( -10,	10 ),
 	RANGE( -1,	1 ),
 	RANGE( -0.1,	0.1 ),
 	RANGE( -0.02,	0.02 )
 }};
-static comedi_lrange range_dt282x_ai_hi_unipolar = { 4, {
+static const comedi_lrange range_dt282x_ai_hi_unipolar = { 4, {
 	RANGE( 0,	10 ),
 	RANGE( 0,	1 ),
 	RANGE( 0,	0.1 ),
@@ -193,7 +193,7 @@ static comedi_lrange range_dt282x_ai_hi_unipolar = { 4, {
 
 
 typedef struct {
-	char *name;
+	const char *name;
 	int adbits;
 	int adchan_se;
 	int adchan_di;
@@ -203,7 +203,7 @@ typedef struct {
 	int dabits;
 } boardtype_t;
 
-static boardtype_t boardtypes[] =
+static const boardtype_t boardtypes[] =
 {
 	{	name:		"dt2821",
 		adbits:		12,
@@ -333,7 +333,7 @@ static boardtype_t boardtypes[] =
 	},
 };
 #define n_boardtypes sizeof(boardtypes)/sizeof(boardtype_t)
-#define this_board ((boardtype_t *)dev->board_ptr)
+#define this_board ((const boardtype_t *)dev->board_ptr)
 
 
 typedef struct {
@@ -341,7 +341,7 @@ typedef struct {
 	int da0_2scomp;		/* same, for DAC0               */
 	int da1_2scomp;		/* same, for DAC1               */
 
-	comedi_lrange *darangelist[2];
+	const comedi_lrange *darangelist[2];
 
 	sampl_t ao[2];
 
@@ -364,7 +364,7 @@ typedef struct {
 } dt282x_private;
 
 #define devpriv ((dt282x_private *)dev->private)
-#define boardtype (*(boardtype_t *)dev->board_ptr)
+#define boardtype (*(const boardtype_t *)dev->board_ptr)
 
 /*
  *    Some useless abstractions
@@ -397,7 +397,7 @@ static comedi_driver driver_dt282x={
 	module:		THIS_MODULE,
 	attach:		dt282x_attach,
 	detach:		dt282x_detach,
-	board_name:	boardtypes,
+	board_name:	&boardtypes[0].name,
 	num_names:	n_boardtypes,
 	offset:		sizeof(boardtype_t),
 };
@@ -457,16 +457,13 @@ static void dt282x_ao_dma_interrupt(comedi_device * dev)
 
 	devpriv->current_dma_index=1-i;
 
-	size = comedi_buf_read_n_available(s);
-	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
+	size = cfc_read_array_from_buffer(s, ptr, devpriv->dma_maxsize);
 	if( size == 0){
 		rt_printk("dt282x: AO underrun\n");
 		dt282x_ao_cancel(dev,s);
 		s->async->events |= COMEDI_CB_OVERFLOW;
 		return;
 	}
-	comedi_buf_memcpy_from(s->async, 0, ptr, size);
-	comedi_buf_read_free(s->async, size);
 	prep_ao_dma(dev,i,size);
 	return;
 }
@@ -578,14 +575,22 @@ static int prep_ao_dma(comedi_device * dev,int dma_index,int n)
 	return n;
 }
 
-static irqreturn_t dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t dt282x_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = d;
-	comedi_subdevice *s = dev->subdevices+0;
-	comedi_subdevice *s_ao = dev->subdevices+1;
+	comedi_subdevice *s;
+	comedi_subdevice *s_ao;
 	unsigned int supcsr, adcsr, dacsr;
 	int handled = 0;
 
+	if (!dev->attached)
+	{
+		comedi_error(dev, "spurious interrupt");
+		return IRQ_HANDLED;
+	}
+
+	s = dev->subdevices+0;
+	s_ao = dev->subdevices+1;
 	adcsr = inw(dev->iobase + DT2821_ADCSR);
 	dacsr = inw(dev->iobase + DT2821_DACSR);
 	supcsr = inw(dev->iobase + DT2821_SUPCSR);
@@ -642,7 +647,7 @@ static irqreturn_t dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 		handled = 1;
 	}
 #endif
-	comedi_event(dev, s, s->async->events);
+	comedi_event(dev, s);
 	/* printk("adcsr=0x%02x dacsr-0x%02x supcsr=0x%02x\n", adcsr, dacsr, supcsr); */
 	return IRQ_RETVAL(handled);
 }
@@ -1049,24 +1054,18 @@ static int dt282x_ao_inttrig(comedi_device *dev,comedi_subdevice *s,
 
 	if(x!=0)return -EINVAL;
 
-	size = comedi_buf_read_n_available(s);
-	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
+	size = cfc_read_array_from_buffer(s, devpriv->dma[0].buf, devpriv->dma_maxsize);
 	if( size == 0){
 		rt_printk("dt282x: AO underrun\n");
 		return -EPIPE;
 	}
-	comedi_buf_memcpy_from(s->async, 0, devpriv->dma[0].buf, size);
-	comedi_buf_read_free(s->async, size);
 	prep_ao_dma(dev,0,size);
 
-	size = comedi_buf_read_n_available(s);
-	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
+	size = cfc_read_array_from_buffer(s, devpriv->dma[1].buf, devpriv->dma_maxsize);
 	if( size == 0){
 		rt_printk("dt282x: AO underrun\n");
 		return -EPIPE;
 	}
-	comedi_buf_memcpy_from(s->async, 0, devpriv->dma[1].buf, size);
-	comedi_buf_read_free(s->async, size);
 	prep_ao_dma(dev,1,size);
 
 	update_supcsr(DT2821_STRIG);
@@ -1154,15 +1153,15 @@ static int dt282x_dio_insn_config(comedi_device *dev,comedi_subdevice *s,
 }
 
 
-static comedi_lrange *ai_range_table[]={
+static const comedi_lrange * const ai_range_table[]={
 	&range_dt282x_ai_lo_bipolar,
 	&range_dt282x_ai_lo_unipolar,
 	&range_dt282x_ai_5_bipolar,
 	&range_dt282x_ai_5_unipolar };
-static comedi_lrange *ai_range_pgl_table[]={
+static const comedi_lrange * const ai_range_pgl_table[]={
 	&range_dt282x_ai_hi_bipolar,
 	&range_dt282x_ai_hi_unipolar };
-static comedi_lrange *opt_ai_range_lkup(int ispgl,int x)
+static const comedi_lrange *opt_ai_range_lkup(int ispgl,int x)
 {
 	if(ispgl){
 		if(x<0 || x>=2)x=0;
@@ -1172,13 +1171,13 @@ static comedi_lrange *opt_ai_range_lkup(int ispgl,int x)
 		return ai_range_table[x];
 	}
 }
-static comedi_lrange *ao_range_table[]={
+static const comedi_lrange * const ao_range_table[]={
 	&range_bipolar10,
 	&range_unipolar10,
 	&range_bipolar5,
 	&range_unipolar5,
 	&range_bipolar2_5 };
-static comedi_lrange *opt_ao_range_lkup(int x)
+static const comedi_lrange *opt_ao_range_lkup(int x)
 {
 	if(x<0 || x>=5)x=0;
 	return ao_range_table[x];
@@ -1256,7 +1255,7 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 	if (irq < 0) {
 		unsigned long flags;
 		int irqs;
-		
+
 		save_flags(flags);
 		sti();
 		irqs = probe_irq_on();
@@ -1305,7 +1304,8 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 	dev->read_subdev=s;
 	/* ai subdevice */
 	s->type=COMEDI_SUBD_AI;
-	s->subdev_flags=SDF_READABLE|((it->options[opt_diff])?SDF_DIFF:SDF_COMMON);
+	s->subdev_flags=SDF_READABLE | SDF_CMD_READ |
+		((it->options[opt_diff])?SDF_DIFF:SDF_COMMON);
 	s->n_chan=(it->options[opt_diff])?boardtype.adchan_di:boardtype.adchan_se;
 	s->insn_read=dt282x_ai_insn_read;
 	s->do_cmdtest=dt282x_ai_cmdtest;
@@ -1321,7 +1321,7 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 		/* ao subsystem */
 		s->type=COMEDI_SUBD_AO;
 		dev->write_subdev=s;
-		s->subdev_flags=SDF_WRITABLE;
+		s->subdev_flags = SDF_WRITABLE | SDF_CMD_WRITE;
 		s->insn_read=dt282x_ao_insn_read;
 		s->insn_write=dt282x_ao_insn_write;
 		s->do_cmdtest=dt282x_ao_cmdtest;
