@@ -1,5 +1,5 @@
 /*
-    module/dt3000.c
+    comedi/drivers/dt3000.c
     Data Translation DT3000 series driver
 
     COMEDI - Linux Control and Measurement Device Interface
@@ -21,7 +21,7 @@
 
 */
 /*
-Driver: dt3000.o
+Driver: dt3000
 Description: Data Translation DT3000 series
 Author: ds
 Devices: [Data Translation] DT3001 (dt3000), DT3001-PGL, DT3002, DT3003,
@@ -60,13 +60,13 @@ AO commands are not supported.
 
 #define PCI_VENDOR_ID_DT	0x1116
 
-static comedi_lrange range_dt3000_ai = { 4, {
+static const comedi_lrange range_dt3000_ai = { 4, {
 	RANGE( -10,	10 ),
 	RANGE( -5,	5 ),
 	RANGE( -2.5,	2.5 ),
 	RANGE( -1.25,	1.25 )
 }};
-static comedi_lrange range_dt3000_ai_pgl = { 4, {
+static const comedi_lrange range_dt3000_ai_pgl = { 4, {
 	RANGE( -10,	10 ),
 	RANGE( -1,	1 ),
 	RANGE( -0.1,	0.1 ),
@@ -74,17 +74,17 @@ static comedi_lrange range_dt3000_ai_pgl = { 4, {
 }};
 
 typedef struct{
-	char *name;
+	const char *name;
 	unsigned int device_id;
 	int adchan;
 	int adbits;
 	int ai_speed;
-	comedi_lrange *adrange;
+	const comedi_lrange *adrange;
 	int dachan;
 	int dabits;
 }dt3k_boardtype;
 
-static dt3k_boardtype dt3k_boardtypes[]={
+static const dt3k_boardtype dt3k_boardtypes[]={
 	{	name:		"dt3001",
 		device_id:	0x22,
 		adchan:		16,
@@ -150,7 +150,7 @@ static dt3k_boardtype dt3k_boardtypes[]={
 	},
 };
 #define n_dt3k_boards sizeof(dt3k_boardtypes)/sizeof(dt3k_boardtype)
-#define this_board ((dt3k_boardtype *)dev->board_ptr)
+#define this_board ((const dt3k_boardtype *)dev->board_ptr)
 
 static struct pci_device_id dt3k_pci_table[] __devinitdata = {
 	{ PCI_VENDOR_ID_DT, 0x0022, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
@@ -290,7 +290,7 @@ static int dt3k_send_cmd(comedi_device *dev,unsigned int cmd)
 	unsigned int status = 0;
 
 	writew(cmd,devpriv->io_addr+DPR_Command_Mbx);
-	
+
 	for(i=0;i<TIMEOUT;i++){
 		status=readw(devpriv->io_addr+DPR_Command_Mbx);
 		if((status&DT3000_COMPLETION_MASK)!=DT3000_NOTPROCESSED)
@@ -302,7 +302,7 @@ static int dt3k_send_cmd(comedi_device *dev,unsigned int cmd)
 	}
 
 	printk("dt3k_send_cmd() timeout/error status=0x%04x\n",status);
-	
+
 	return -ETIME;
 }
 
@@ -310,10 +310,10 @@ static unsigned int dt3k_readsingle(comedi_device *dev,unsigned int subsys,
 	unsigned int chan,unsigned int gain)
 {
 	writew(subsys,devpriv->io_addr+DPR_SubSys);
-	
+
 	writew(chan,devpriv->io_addr+DPR_Params(0));
 	writew(gain,devpriv->io_addr+DPR_Params(1));
-	
+
 	dt3k_send_cmd(dev,CMD_READSINGLE);
 
 	return readw(devpriv->io_addr+DPR_Params(2));
@@ -323,24 +323,34 @@ static void dt3k_writesingle(comedi_device *dev,unsigned int subsys,
 	unsigned int chan,unsigned int data)
 {
 	writew(subsys,devpriv->io_addr+DPR_SubSys);
-	
+
 	writew(chan,devpriv->io_addr+DPR_Params(0));
 	writew(0,devpriv->io_addr+DPR_Params(1));
 	writew(data,devpriv->io_addr+DPR_Params(2));
-	
+
 	dt3k_send_cmd(dev,CMD_WRITESINGLE);
 }
 
 static int debug_n_ints = 0;
 
-static irqreturn_t dt3k_interrupt(int irq, void *d, struct pt_regs *regs)
+// FIXME! Assumes shared interrupt is for this card.
+// What's this debug_n_ints stuff? Obviously needs some work...
+static irqreturn_t dt3k_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = d;
-	comedi_subdevice *s = dev->subdevices + 0;
+	comedi_subdevice *s;
 	unsigned int status;
 
+	if (!dev->attached)
+	{
+		return IRQ_NONE;
+	}
+
+	s = dev->subdevices + 0;
 	status = readw(devpriv->io_addr+DPR_Intr_Flag);
+#ifdef DEBUG
 	debug_intr_flags(status);
+#endif
 
 	if(status & DT3000_ADFULL){
 		dt3k_ai_empty_fifo(dev,s);
@@ -357,7 +367,7 @@ static irqreturn_t dt3k_interrupt(int irq, void *d, struct pt_regs *regs)
 		s->async->events |= COMEDI_CB_EOA;
 	}
 
-	comedi_event(dev,s,s->async->events);
+	comedi_event(dev, s);
 	return IRQ_HANDLED;
 }
 
@@ -401,7 +411,7 @@ printk("reading %d samples\n",count);
 		rear++;
 		if(rear>=AI_FIFO_DEPTH)rear = 0;
 	}
-	
+
 	devpriv->ai_rear = rear;
 	writew(rear,devpriv->io_addr + DPR_AD_Buf_Rear);
 }
@@ -549,7 +559,7 @@ static int dt3k_ns_to_timer(unsigned int timer_base, unsigned int *nanosec,
 			return (prescale<<16)|(divider);
 		}
 	}
-	
+
 	prescale = 15;
 	base = timer_base * (1<<prescale);
 	divider = 65535;
@@ -572,11 +582,11 @@ printk("dt3k_ai_cmd:\n");
 	for(i=0;i<cmd->chanlist_len;i++){
 		chan=CR_CHAN(cmd->chanlist[i]);
 		range=CR_RANGE(cmd->chanlist[i]);
-		
+
 		writew((range<<6)|chan,devpriv->io_addr+DPR_ADC_buffer+i);
 	}
 	aref=CR_AREF(cmd->chanlist[0]);
-	
+
 	writew(cmd->scan_end_arg,devpriv->io_addr+DPR_Params(0));
 printk("param[0]=0x%04x\n",cmd->scan_end_arg);
 
@@ -601,7 +611,7 @@ printk("param[4]=0x%04x\n",tscandiv&0xffff);
 	}else{
 		/* not supported */
 	}
-	
+
 	mode = DT3000_AD_RETRIG_INTERNAL | 0 | 0;
 	writew(mode,devpriv->io_addr+DPR_Params(5));
 printk("param[5]=0x%04x\n",mode);
@@ -610,7 +620,7 @@ printk("param[6]=0x%04x\n",aref==AREF_DIFF);
 
 	writew(AI_FIFO_DEPTH/2,devpriv->io_addr+DPR_Params(7));
 printk("param[7]=0x%04x\n",AI_FIFO_DEPTH/2);
-	
+
 	writew(SUBS_AI,devpriv->io_addr+DPR_SubSys);
 	ret = dt3k_send_cmd(dev,CMD_CONFIG);
 
@@ -688,14 +698,14 @@ static void dt3k_dio_config(comedi_device *dev,int bits)
 {
 	/* XXX */
 	writew(SUBS_DOUT,devpriv->io_addr+DPR_SubSys);
-	
+
 	writew(bits,devpriv->io_addr+DPR_Params(0));
 #if 0
 	/* don't know */
 	writew(0,devpriv->io_addr+DPR_Params(1));
 	writew(0,devpriv->io_addr+DPR_Params(2));
 #endif
-	
+
 	dt3k_send_cmd(dev,CMD_CONFIG);
 }
 
@@ -705,7 +715,7 @@ static int dt3k_dio_insn_config(comedi_device *dev,comedi_subdevice *s,
 	int mask;
 
 	mask=(CR_CHAN(insn->chanspec)<4)?0x0f:0xf0;
-	
+
 	switch(data[0])
 	{
 	case INSN_CONFIG_DIO_OUTPUT:
@@ -721,7 +731,7 @@ static int dt3k_dio_insn_config(comedi_device *dev,comedi_subdevice *s,
 	default:
 		return -EINVAL;
 		break;
-	}	
+	}
 	mask=(s->io_bits&0x01)|((s->io_bits&0x10)>>3);
 	dt3k_dio_config(dev,mask);
 
@@ -758,7 +768,7 @@ static int dt3k_mem_insn_read(comedi_device *dev,comedi_subdevice *s,
 
 		data[i]=readw(devpriv->io_addr+DPR_Params(2));
 	}
-	
+
 	return i;
 }
 
@@ -768,9 +778,9 @@ static int dt3000_attach(comedi_device *dev,comedi_devconfig *it)
 {
 	comedi_subdevice *s;
 	int ret=0;
-	
+
 	printk("dt3000:");
-	
+
 	if((ret=alloc_private(dev,sizeof(dt3k_private)))<0)
 		return ret;
 
@@ -784,7 +794,7 @@ static int dt3000_attach(comedi_device *dev,comedi_devconfig *it)
 	dev->board_name = this_board->name;
 
 	if(comedi_request_irq(devpriv->pci_dev->irq, dt3k_interrupt,
-			SA_SHIRQ, "dt3000", dev)){
+			IRQF_SHARED, "dt3000", dev)){
 		printk(" unable to allocate IRQ %u\n", devpriv->pci_dev->irq);
 		return -EINVAL;
 	}
@@ -798,7 +808,7 @@ static int dt3000_attach(comedi_device *dev,comedi_devconfig *it)
 
 	/* ai subdevice */
 	s->type=COMEDI_SUBD_AI;
-	s->subdev_flags=SDF_READABLE|SDF_GROUND|SDF_DIFF;
+	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_DIFF | SDF_CMD_READ;
 	s->n_chan=this_board->adchan;
 	s->insn_read=dt3k_ai_insn;
 	s->maxdata=(1<<this_board->adbits)-1;
@@ -865,7 +875,7 @@ static int dt3000_detach(comedi_device *dev)
 			pci_dev_put(devpriv->pci_dev);
 		}
 		if(devpriv->io_addr) iounmap(devpriv->io_addr);
-	}	
+	}
 	/* XXX */
 
 	return 0;
@@ -918,7 +928,7 @@ static int setup_pci(comedi_device *dev)
 static struct pci_dev *dt_pci_find_device(struct pci_dev *from,int *board)
 {
 	int i;
-	
+
 	for(from=pci_get_device(PCI_VENDOR_ID_DT,PCI_ANY_ID,from); from!=NULL;
 			from=pci_get_device(PCI_VENDOR_ID_DT,PCI_ANY_ID,from)){
 		for(i=0;i<n_dt3k_boards;i++){
