@@ -23,7 +23,7 @@
  */
 
 /*
-Driver: dt9812.o
+Driver: dt9812
 Description: Data Translation DT9812 USB module
 Author: anders.blomdell@control.lth.se (Anders Blomdell)
 Status: in development
@@ -99,22 +99,22 @@ typedef struct slot_dt9812 {
   comedi_dt9812_t *comedi;
 } slot_dt9812_t;
 
-static comedi_lrange dt9812_10_ain_range={ 1, {
+static const comedi_lrange dt9812_10_ain_range={ 1, {
     BIP_RANGE(10),
   }
 };
 
-static comedi_lrange dt9812_2pt5_ain_range={ 1, {
+static const comedi_lrange dt9812_2pt5_ain_range={ 1, {
     UNI_RANGE(2.5),
   }
 };
 
-static comedi_lrange dt9812_10_aout_range={ 1, {
+static const comedi_lrange dt9812_10_aout_range={ 1, {
     BIP_RANGE(10),
   }
 };
 
-static comedi_lrange dt9812_2pt5_aout_range={ 1, {
+static const comedi_lrange dt9812_2pt5_aout_range={ 1, {
     UNI_RANGE(2.5),
   }
 };
@@ -489,12 +489,11 @@ static int dt9812_probe(struct usb_interface *interface,
   u8 fw;
 
   // allocate memory for our device state and initialize it
-  dev = kmalloc(sizeof(*dev), GFP_KERNEL);
+  dev = kzalloc(sizeof(*dev), GFP_KERNEL);
   if (dev == NULL) {
     err("Out of memory");
     goto error;
   }
-  memset(dev, 0x00, sizeof(*dev));
   KREF_INIT(&dev->kref, dt9812_delete);
   
   dev->udev = usb_get_dev(interface_to_usbdev(interface));
