@@ -1,5 +1,5 @@
 /*
-    gsc_hpdi.c
+    comedi/drivers/gsc_hpdi.c
     This is a driver for the General Standards Corporation High
     Speed Parallel Digital Interface rs485 boards.
 
@@ -27,7 +27,7 @@
 
 /*
 
-Driver: gsc_hpdi.o
+Driver: gsc_hpdi
 Description: General Standards Corporation High
     Speed Parallel Digital Interface rs485 boards
 Author: Frank Mori Hess <fmhess@users.sourceforge.net>
@@ -58,7 +58,7 @@ void abort_dma( comedi_device *dev, unsigned int channel );
 static int hpdi_cmd( comedi_device *dev, comedi_subdevice *s );
 static int hpdi_cmd_test( comedi_device *dev, comedi_subdevice *s, comedi_cmd *cmd );
 static int hpdi_cancel( comedi_device *dev, comedi_subdevice *s );
-static irqreturn_t handle_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t handle_interrupt(int irq, void *d PT_REGS_ARG);
 static int dio_config_block_size( comedi_device *dev, lsampl_t *data );
 
 #undef HPDI_DEBUG	// disable debugging messages
@@ -440,7 +440,7 @@ static int setup_subdevices(comedi_device *dev)
 	dev->read_subdev = s;
 /*	dev->write_subdev = s; */
 	s->type = COMEDI_SUBD_DIO;
-	s->subdev_flags = SDF_READABLE | SDF_WRITEABLE | SDF_LSAMPL;
+	s->subdev_flags = SDF_READABLE | SDF_WRITEABLE | SDF_LSAMPL | SDF_CMD_READ;
 	s->n_chan = 32;
 	s->len_chanlist = 32;
 	s->maxdata = 1;
@@ -470,10 +470,11 @@ static int init_hpdi( comedi_device *dev )
 	priv(dev)->rx_fifo_size = fifo_size( readl( priv(dev)->hpdi_iobase +
 		RX_FIFO_SIZE_REG ) );
 
+	writel( 0, priv(dev)->hpdi_iobase + INTERRUPT_CONTROL_REG );
+
 	// enable interrupts
 	plx_intcsr_bits = ICS_AERR | ICS_PERR | ICS_PIE | ICS_PLIE | ICS_PAIE | ICS_LIE | ICS_DMA0_E;
 	writel( plx_intcsr_bits, priv(dev)->plx9080_iobase + PLX_INTRCS_REG );
-	writel( 0, priv(dev)->hpdi_iobase + INTERRUPT_CONTROL_REG );
 
 	return 0;
 }
@@ -609,8 +610,10 @@ static int hpdi_attach(comedi_device *dev, comedi_devconfig *it)
 	DEBUG_PRINT(" plx9080 remapped to 0x%p\n", priv(dev)->plx9080_iobase);
 	DEBUG_PRINT(" hpdi remapped to 0x%p\n", priv(dev)->hpdi_iobase);
 
+	init_plx9080(dev);
+
 	// get irq
-	if( comedi_request_irq( pcidev->irq, handle_interrupt, SA_SHIRQ, driver_hpdi.driver_name,
+	if( comedi_request_irq( pcidev->irq, handle_interrupt, IRQF_SHARED, driver_hpdi.driver_name,
 		 dev ) )
 	{
 		printk( " unable to allocate irq %u\n", pcidev->irq );
@@ -620,8 +623,6 @@ static int hpdi_attach(comedi_device *dev, comedi_devconfig *it)
 
 	printk(" irq %u\n", dev->irq);
 
-	init_plx9080(dev);
-
 	// alocate pci dma buffers
 	for( i = 0; i < NUM_DMA_BUFFERS; i++ )
 	{
@@ -917,7 +918,7 @@ static void drain_dma_buffers(comedi_device *dev, unsigned int channel)
 	// XXX check for buffer overrun somehow
 }
 
-static irqreturn_t handle_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t handle_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = dev->read_subdev;
@@ -928,7 +929,17 @@ static irqreturn_t handle_interrupt(int irq, void *d, struct pt_regs *regs)
 	uint8_t dma0_status, dma1_status;
 	unsigned long flags;
 
+	if (!dev->attached)
+	{
+		return IRQ_NONE;
+	}
+
 	plx_status = readl( priv(dev)->plx9080_iobase + PLX_INTRCS_REG );
+	if( ( plx_status & ( ICS_DMA0_A | ICS_DMA1_A | ICS_LIA ) ) == 0 )
+	{
+		return IRQ_NONE;
+	}
+
 	hpdi_intr_status = readl( priv(dev)->hpdi_iobase + INTERRUPT_STATUS_REG );
 	hpdi_board_status = readl( priv(dev)->hpdi_iobase + BOARD_STATUS_REG );
 
@@ -938,9 +949,6 @@ static irqreturn_t handle_interrupt(int irq, void *d, struct pt_regs *regs)
 	{
 		DEBUG_PRINT("hpdi: intr status 0x%x, ", hpdi_intr_status);
 		writel( hpdi_intr_status, priv(dev)->hpdi_iobase + INTERRUPT_STATUS_REG );
-	}else if( ( plx_status & ( ICS_DMA0_A | ICS_DMA1_A | ICS_LIA ) ) == 0 )
-	{
-		return IRQ_NONE;
 	}
 
 	// spin lock makes sure noone else changes plx dma control reg
