@@ -22,7 +22,7 @@
 
 
 /*
-Driver: icp_multi.o
+Driver: icp_multi
 Description: Inova ICP_MULTI
 Author: Anne Smorthit <anne.smorthit@sfwte.ch>
 Devices: [Inova] ICP_MULTI (icp_multi)
@@ -46,9 +46,9 @@ There are 4 x 12-bit Analogue Outputs.  Ranges : 5V, 10V, +/-5V, +/-10V
 4 x 16-bit counters
 
 Options:
- [0] - PCI bus number - if bus number and slot number are 0, 
+ [0] - PCI bus number - if bus number and slot number are 0,
                         then driver search for first unused card
- [1] - PCI slot number 
+ [1] - PCI slot number
 */
 
 #include <linux/comedidev.h>
@@ -66,7 +66,7 @@ Options:
 // Hardware types of the cards
 #define TYPE_ICP_MULTI	0
 
-#define IORANGE_ICP_MULTI 	32		
+#define IORANGE_ICP_MULTI 	32
 
 #define ICP_MULTI_ADC_CSR	0	/* R/W:	ADC command/status register */
 #define ICP_MULTI_AI		2	/* R:	Analogue input data */
@@ -111,7 +111,7 @@ Options:
 
 
 // Define analogue range
-static comedi_lrange range_analog={ 4, {
+static const comedi_lrange range_analog={ 4, {
 	UNI_RANGE(5),
 	UNI_RANGE(10),
 	BIP_RANGE(5),
@@ -119,7 +119,7 @@ static comedi_lrange range_analog={ 4, {
 	}
 };
 
-static char range_codes_analog[]={0x00, 0x20, 0x10, 0x30};
+static const char range_codes_analog[]={0x00, 0x20, 0x10, 0x30};
 
 
 /*
@@ -139,7 +139,7 @@ static int icp_multi_detach(comedi_device *dev);
 static unsigned short	pci_list_builded=0;	/*>0 list of card is known */
 
 typedef struct {
-	char 		*name;		// driver name
+	const char 		*name;		// driver name
 	int		device_id;
 	int		iorange;	// I/O range len
 	char		have_irq;	// 1=card support IRQ
@@ -152,12 +152,12 @@ typedef struct {
 	int		n_ctrs;		// num of counters
 	int		ai_maxdata;	// resolution of A/D
 	int		ao_maxdata;	// resolution of D/A
-	comedi_lrange	*rangelist_ai;	// rangelist for A/D
-	char		*rangecode;	// range codes for programming
-	comedi_lrange	*rangelist_ao;	// rangelist for D/A
+	const comedi_lrange *rangelist_ai; // rangelist for A/D
+	const char	*rangecode;	// range codes for programming
+	const comedi_lrange *rangelist_ao; // rangelist for D/A
 } boardtype;
 
-static boardtype boardtypes[] =
+static const boardtype boardtypes[] =
 {
 	{"icp_multi",		// Driver name
 	 DEVICE_ID,		// PCI device ID
@@ -185,8 +185,8 @@ static comedi_driver driver_icp_multi={
 	attach:		icp_multi_attach,
 	detach:		icp_multi_detach,
 	num_names:	n_boardtypes,
-	board_name:	boardtypes,
-	offset:		sizeof(boardtype),	
+	board_name:	&boardtypes[0].name,
+	offset:		sizeof(boardtype),
 };
 COMEDI_INITCLEANUP(driver_icp_multi);
 
@@ -210,9 +210,9 @@ typedef struct{
 } icp_multi_private;
 
 #define devpriv ((icp_multi_private *)dev->private)
-#define this_board ((boardtype *)dev->board_ptr)
+#define this_board ((const boardtype *)dev->board_ptr)
 
-/* 
+/*
 ==============================================================================
 	More forward declarations
 ==============================================================================
@@ -225,7 +225,7 @@ static void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsign
 static int icp_multi_reset(comedi_device *dev);
 
 
-/* 
+/*
 ==============================================================================
 	Functions
 ==============================================================================
@@ -260,7 +260,7 @@ static int icp_multi_insn_read_ai(comedi_device * dev, comedi_subdevice * s, com
 	// Disable A/D conversion ready interrupt
 	devpriv->IntEnable &= ~ADC_READY;
 	writew(devpriv->IntEnable,devpriv->io_addr + ICP_MULTI_INT_EN);
-	
+
 	// Clear interrupt status
 	devpriv->IntStatus |= ADC_READY;
 	writew(devpriv->IntStatus,devpriv->io_addr + ICP_MULTI_INT_STAT);
@@ -324,7 +324,7 @@ static int icp_multi_insn_read_ai(comedi_device * dev, comedi_subdevice * s, com
 conv_finish:
 		data[n] = (readw(devpriv->io_addr+ICP_MULTI_AI) >> 4 ) & 0x0fff;
 	}
-	
+
 	// Disable interrupt
 	devpriv->IntEnable &= ~ADC_READY;
 	writew(devpriv->IntEnable,devpriv->io_addr + ICP_MULTI_INT_EN);
@@ -341,7 +341,7 @@ conv_finish:
 
 /*
 ==============================================================================
-	
+
 	Name:	icp_multi_insn_write_ao
 
 	Description:
@@ -367,7 +367,7 @@ static int icp_multi_insn_write_ao(comedi_device * dev, comedi_subdevice * s, co
 	// Disable D/A conversion ready interrupt
 	devpriv->IntEnable &= ~DAC_READY;
 	writew(devpriv->IntEnable,devpriv->io_addr + ICP_MULTI_INT_EN);
-	
+
 	// Clear interrupt status
 	devpriv->IntStatus |= DAC_READY;
 	writew(devpriv->IntStatus,devpriv->io_addr + ICP_MULTI_INT_STAT);
@@ -384,7 +384,7 @@ static int icp_multi_insn_write_ao(comedi_device * dev, comedi_subdevice * s, co
 	devpriv->DacCmdStatus &= 0xfccf;
 	devpriv->DacCmdStatus |= this_board->rangecode[range];
 	devpriv->DacCmdStatus |= (chan << 8);
-	
+
 	writew(devpriv->DacCmdStatus, devpriv->io_addr+ICP_MULTI_DAC_CSR);
 
 	for (n=0; n<insn->n; n++) {
@@ -442,7 +442,7 @@ dac_ready:
 
 /*
 ==============================================================================
-	
+
 	Name:	icp_multi_insn_read_ao
 
 	Description:
@@ -462,11 +462,11 @@ static int icp_multi_insn_read_ao(comedi_device * dev, comedi_subdevice * s, com
 {
 	int n,chan;
 
-	// Get channel number	
+	// Get channel number
 	chan = CR_CHAN(insn->chanspec);
 
 	// Read analogue outputs
-	for (n=0; n<insn->n; n++) 
+	for (n=0; n<insn->n; n++)
 		data[n]=devpriv->ao_data[chan];
 
 	return n;
@@ -474,7 +474,7 @@ static int icp_multi_insn_read_ao(comedi_device * dev, comedi_subdevice * s, com
 
 /*
 ==============================================================================
-	
+
 	Name:	icp_multi_insn_bits_di
 
 	Description:
@@ -499,7 +499,7 @@ static int icp_multi_insn_bits_di(comedi_device *dev,comedi_subdevice *s, comedi
 
 /*
 ==============================================================================
-	
+
 	Name:	icp_multi_insn_bits_do
 
 	Description:
@@ -600,11 +600,10 @@ static int icp_multi_insn_write_ctr(comedi_device * dev, comedi_subdevice * s, c
 	Parameters:
 		int irq
 		void *d			Pointer to current device
-		struct pt_regs *regs	Pointer to
 
 ==============================================================================
 */
-static irqreturn_t interrupt_service_icp_multi(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_service_icp_multi(int irq, void *d PT_REGS_ARG)
 {
         comedi_device *dev = d;
 	int	int_no;
@@ -678,7 +677,7 @@ static irqreturn_t interrupt_service_icp_multi(int irq, void *d, struct pt_regs
 static int check_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist, unsigned int n_chan)
 {
         unsigned int i;
-    
+
 #ifdef ICP_MULTI_EXTDEBUG
 	printk("icp multi EDBG:  check_channel_list(...,%d)\n",n_chan);
 #endif
@@ -792,7 +791,7 @@ static void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsign
 
 	Description:
 		This function resets the icp multi device to a 'safe' state
-	
+
 	Parameters:
 		comedi_device *dev	Pointer to current sevice structure
 
@@ -827,11 +826,11 @@ static int icp_multi_reset(comedi_device *dev)
 
 			// Output to command / status register
 			writew(devpriv->DacCmdStatus, devpriv->io_addr+ICP_MULTI_DAC_CSR);
-			
+
 			// Delay to allow DAC time to recover
 			comedi_udelay(1);
 		}
-	
+
 	// Digital outputs to 0
 	writew(0, devpriv->io_addr + ICP_MULTI_DO);
 
@@ -845,7 +844,7 @@ static int icp_multi_reset(comedi_device *dev)
 ==============================================================================
 
 	Name:	icp_multi_attach
-	
+
 	Description:
 		This function sets up all the appropriate data for the current
 		device.
@@ -927,9 +926,11 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
     		return ret;
 	}
 
+	icp_multi_reset(dev);
+
 	if (this_board->have_irq) {
 		if (irq)  {
-			if (comedi_request_irq(irq, interrupt_service_icp_multi, SA_SHIRQ, "Inova Icp Multi", dev)) {
+			if (comedi_request_irq(irq, interrupt_service_icp_multi, IRQF_SHARED, "Inova Icp Multi", dev)) {
 				printk(", unable to allocate IRQ %u, DISABLING IT", irq);
 				irq=0; /* Can't use IRQ */
 			}
@@ -941,13 +942,13 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 	}
 	else
 		irq=0;
-	
+
         dev->irq = irq;
 
 	printk(".\n");
 
 	subdev=0;
-	
+
 	if (this_board->n_aichan) {
 		s = dev->subdevices + subdev;
 		dev->read_subdev = s;
@@ -962,7 +963,7 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 		s->insn_read=icp_multi_insn_read_ai;
 		subdev++;
 	}
-	
+
 	if (this_board->n_aochan) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_AO;
@@ -1015,10 +1016,8 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 		s->insn_write=icp_multi_insn_write_ctr;
 		subdev++;
 	}
-	
-	devpriv->valid = 1;
 
-	icp_multi_reset(dev);
+	devpriv->valid = 1;
 
 #ifdef ICP_MULTI_EXTDEBUG
 	printk("icp multi EDBG: END: icp_multi_attach(...)\n");
@@ -1031,11 +1030,11 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 ==============================================================================
 
 	Name:	icp_multi_detach
-	
+
 	Description:
 		This function releases all the resources used by the current
 		device.
-	
+
 	Parameters:
 		comedi_device *dev	Pointer to current device structure
 
@@ -1046,10 +1045,10 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 static int icp_multi_detach(comedi_device *dev)
 {
 
-	if (dev->private) 
+	if (dev->private)
 		if (devpriv->valid)
 			icp_multi_reset(dev);
-	
+
 	if (dev->irq)
 		comedi_free_irq(dev->irq,dev);
 
