@@ -1,7 +1,7 @@
 /*
- *	ii_pci20kc.c
- *	driver for Intelligent Instruments PCI-20001C carrier board
- *	and modules
+ *	comedi/drivers/ii_pci20kc.c
+ *	Driver for Intelligent Instruments PCI-20001C carrier board
+ *	and modules.
  *
  *	Copyright (C) 2000 Markus Kempf <kempf@matsci.uni-sb.de>
  *	with suggestions from David Schleef   
@@ -39,7 +39,7 @@
  *
  */
 /*
-Driver: ii_pci20kc.o
+Driver: ii_pci20kc
 Description: Intelligent Instruments PCI-20001C carrier board
 Author: Markus Kempf <kempf@matsci.uni-sb.de>
 Devices: [Intelligent Instrumentation] PCI-20001C (ii_pci20kc)
@@ -140,7 +140,7 @@ typedef union {
 	void *iobase;
 	struct {
 		void *iobase;
-		comedi_lrange *ao_range_list[2];	/* range of channels of ao module */
+		const comedi_lrange *ao_range_list[2];	/* range of channels of ao module */
 		lsampl_t last_data[2];
 	}pci20006;
 	struct {
@@ -215,7 +215,7 @@ static int pci20xxx_attach(comedi_device * dev, comedi_devconfig * it)
 	if ((ret = alloc_private(dev, sizeof(pci20xxx_private))) < 0)
 		return ret;
 
-	devpriv->ioaddr = (void*) it->options[0];
+	devpriv->ioaddr = (void*) (unsigned long) it->options[0];
 	dev->board_name = "pci20kc";
 
 	/* Check PCI-20001 C-2A Carrier Board ID */
@@ -275,7 +275,7 @@ static int pci20006_insn_read(comedi_device * dev, comedi_subdevice * s,
 static int pci20006_insn_write(comedi_device * dev, comedi_subdevice * s,
 	comedi_insn *insn, lsampl_t *data);
 
-static comedi_lrange *pci20006_range_list[] = {
+static const comedi_lrange *pci20006_range_list[] = {
 	&range_bipolar10,
 	&range_unipolar10,
 	&range_bipolar5,
@@ -350,14 +350,14 @@ static int pci20006_insn_write(comedi_device * dev, comedi_subdevice * s,
 static int pci20341_insn_read(comedi_device * dev, comedi_subdevice * s,
 	comedi_insn *insn, lsampl_t *data);
 
-static int pci20341_timebase[] = { 0x00, 0x00, 0x00, 0x04 };
-static int pci20341_settling_time[] = { 0x58, 0x58, 0x93, 0x99 };
+static const int pci20341_timebase[] = { 0x00, 0x00, 0x00, 0x04 };
+static const int pci20341_settling_time[] = { 0x58, 0x58, 0x93, 0x99 };
 
-static comedi_lrange range_bipolar0_5 = { 1, { BIP_RANGE(0.5) }};
-static comedi_lrange range_bipolar0_05 = { 1, { BIP_RANGE(0.05) }};
-static comedi_lrange range_bipolar0_025 = { 1, { BIP_RANGE(0.025) }};
+static const comedi_lrange range_bipolar0_5 = { 1, { BIP_RANGE(0.5) }};
+static const comedi_lrange range_bipolar0_05 = { 1, { BIP_RANGE(0.05) }};
+static const comedi_lrange range_bipolar0_025 = { 1, { BIP_RANGE(0.025) }};
 
-static comedi_lrange *pci20341_ranges[] = {
+static const comedi_lrange * const pci20341_ranges[] = {
 	&range_bipolar5,
 	&range_bipolar0_5,
 	&range_bipolar0_05,
