@@ -24,7 +24,7 @@
 */
 
 /*
-Driver: me_daq.o
+Driver: me_daq
 Description: Meilhaus PCI data acquisition cards
 Author: Michael Hillmann <hillmann@syscongroup.de>
 Devices: [Meilhaus] ME-2600i (me_daq), ME-2000i
@@ -43,10 +43,10 @@ Configuration options:
     device will be used.
 
 The 2600 requires a firmware upload, which can be accomplished
-using the -i or --init-data option of comedi_config. 
+using the -i or --init-data option of comedi_config.
 The firmware can be
 found in the comedi_nonfree_firmware tarball available
-from http://www.comedi.org 
+from http://www.comedi.org
 
 */
 
@@ -150,7 +150,7 @@ from http://www.comedi.org
 static int me_attach(comedi_device *dev, comedi_devconfig *it);
 static int me_detach(comedi_device *dev);
 
-static comedi_lrange me2000_ai_range=
+static const comedi_lrange me2000_ai_range=
 {
   8,
   {
@@ -165,7 +165,7 @@ static comedi_lrange me2000_ai_range=
   }
 };
 
-static comedi_lrange me2600_ai_range=
+static const comedi_lrange me2600_ai_range=
 {
   8,
   {
@@ -180,7 +180,7 @@ static comedi_lrange me2600_ai_range=
   }
 };
 
-static comedi_lrange me2600_ao_range=
+static const comedi_lrange me2600_ao_range=
 {
   3,
   {
@@ -204,20 +204,20 @@ MODULE_DEVICE_TABLE(pci, me_pci_table);
 
 typedef struct
 {
-  char          *name;              // driver name
+  const char          *name;              // driver name
   int           device_id;
   int           ao_channel_nbr;     // DA config
   int           ao_resolution;
   int           ao_resolution_mask;
-  comedi_lrange *ao_range_list;
+  const comedi_lrange *ao_range_list;
   int           ai_channel_nbr;     // AD config
   int           ai_resolution;
   int           ai_resolution_mask;
-  comedi_lrange *ai_range_list;
+  const comedi_lrange *ai_range_list;
   int           dio_channel_nbr;    // DIO config
 } me_board_struct;
 
-static me_board_struct me_boards[] =
+static const me_board_struct me_boards[] =
 {
   {                                                     // -- ME-2600i --
     name:                         ME_DRIVER_NAME,
@@ -256,7 +256,7 @@ static comedi_driver me_driver=
   attach:      me_attach,
   detach:      me_detach,
   num_names:   me_board_nbr,
-  board_name:  me_boards,
+  board_name:  &me_boards[0].name,
   offset:      sizeof(me_board_struct),
 };
 COMEDI_INITCLEANUP(me_driver);
@@ -697,7 +697,7 @@ static int me_attach(comedi_device *dev,comedi_devconfig *it)
 //
 // Probe the device to determine what device in the series it is.
 //
-	for(pci_device = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pci_device != NULL ; 
+	for(pci_device = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pci_device != NULL ;
 		pci_device = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pci_device))
   {
     if(pci_device->vendor == PCI_VENDOR_ID_MEILHAUS)
@@ -844,7 +844,7 @@ found:
 
   subdevice = dev->subdevices + 0;
   subdevice->type         = COMEDI_SUBD_AI;
-  subdevice->subdev_flags = SDF_READABLE | SDF_COMMON;
+  subdevice->subdev_flags = SDF_READABLE | SDF_COMMON | SDF_CMD_READ;
   subdevice->n_chan       = board->ai_channel_nbr;
   subdevice->maxdata      = board->ai_resolution_mask;
   subdevice->len_chanlist = board->ai_channel_nbr;
