@@ -1,5 +1,5 @@
 /*
-    module/mite.c
+    comedi/drivers/mite.c
     Hardware driver for NI Mite PCI interface chip
 
     COMEDI - Linux Control and Measurement Device Interface
@@ -27,13 +27,13 @@
 
 
 	References for specifications:
-	
+
 	   321747b.pdf  Register Level Programmer Manual (obsolete)
 	   321747c.pdf  Register Level Programmer Manual (new)
 	   DAQ-STC reference manual
 
 	Other possibly relevant info:
-	
+
 	   320517c.pdf  User manual (obsolete)
 	   320517f.pdf  User manual (new)
 	   320889a.pdf  delete
@@ -50,12 +50,14 @@
 
 //#define USE_KMALLOC
 
+#include "mite.h"
+
+#include "comedi_fc.h"
 #include <linux/comedidev.h>
 
 #include <linux/pci.h>
 #include <asm/system.h>
 
-#include "mite.h"
 
 #define PCI_MITE_SIZE		4096
 #define PCI_DAQ_SIZE		4096
@@ -73,19 +75,25 @@ void mite_init(void)
 	struct pci_dev *pcidev;
 	struct mite_struct *mite;
 
-	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ;
 		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) {
 		if(pcidev->vendor==PCI_VENDOR_ID_NATINST){
-			mite=kmalloc(sizeof(*mite),GFP_KERNEL);
+			unsigned i;
+
+			mite=kzalloc(sizeof(*mite),GFP_KERNEL);
 			if(!mite){
 				printk("mite: allocation failed\n");
 				pci_dev_put(pcidev);
 				return;
 			}
-			memset(mite,0,sizeof(*mite));
-
+			spin_lock_init(&mite->lock);
 			mite->pcidev=pci_dev_get(pcidev);
-
+			for(i = 0; i < MAX_MITE_DMA_CHANNELS; ++i)
+			{
+				mite->channels[i].mite = mite;
+				mite->channels[i].channel = i;
+				mite->channels[i].done = 1;
+			}
 			mite->next=mite_devices;
 			mite_devices=mite;
 		}
@@ -97,18 +105,19 @@ static void dump_chip_signature(u32 csigr_bits)
 	printk("mite: version = %i, type = %i, mite mode = %i, interface mode = %i\n",
 		mite_csigr_version(csigr_bits), mite_csigr_type(csigr_bits),
 		mite_csigr_mmode(csigr_bits), mite_csigr_imode(csigr_bits));
-	printk("mite: num channels = %i, write post fifo depth = %i, wins = %i, iowins = %i\n", 
+	printk("mite: num channels = %i, write post fifo depth = %i, wins = %i, iowins = %i\n",
 		mite_csigr_dmac(csigr_bits), mite_csigr_wpdep(csigr_bits),
 		mite_csigr_wins(csigr_bits), mite_csigr_iowins(csigr_bits));
 }
 
-int mite_setup(struct mite_struct *mite)
+int mite_setup2(struct mite_struct *mite, unsigned use_iodwbsr_1)
 {
 	unsigned long length;
 	resource_size_t addr;
 	int i;
 	u32 csigr_bits;
-	
+	unsigned unknown_dma_burst_bits;
+
 	if(pci_enable_device(mite->pcidev)){
 		printk("error enabling mite\n");
 		return -EIO;
@@ -130,13 +139,8 @@ int mite_setup(struct mite_struct *mite)
 
 	addr=pci_resource_start(mite->pcidev, 1);
 	mite->daq_phys_addr=addr;
-
+	length = pci_resource_len(mite->pcidev, 1);
 	// In case of a 660x board, DAQ size is 8k instead of 4k (see as shown by lspci output)
-	if ((mite->pcidev->device == 0x1310) || (mite->pcidev->device == 0x2c60)){
-	        length = PCI_DAQ_SIZE_660X;
-		printk("mite: detected NI660X board, using PCI DAQ SIZE of 8k\n");
-	}
-	else length = PCI_DAQ_SIZE;
 	mite->daq_io_addr = ioremap(mite->daq_phys_addr, length);
 	if( !mite->daq_io_addr ) {
 		printk("failed to remap daq io memory address\n");
@@ -144,14 +148,25 @@ int mite_setup(struct mite_struct *mite)
 	}
 	printk("DAQ:0x%08llx mapped to %p\n",(unsigned long long)mite->daq_phys_addr,mite->daq_io_addr);
 
-	// The 6602 board needs different initalisation, see the
-	// _updated_ (nov 2002) reg. Level Manual (filename 370505b.pdf) p. 3.55
-	if (mite->pcidev->device == 0x1310 ){
-		printk("mite: detected NI6602, using other I/O Window Base Size register\n");
-		writel((mite->daq_phys_addr & 0xffffff00L) | WENAB_6602 , mite->mite_io_addr + MITE_IODWBSR_1);
-		writel(0 , mite->mite_io_addr + MITE_IODWCR_1);
+	if(use_iodwbsr_1)
+	{
+		writel(0, mite->mite_io_addr + MITE_IODWBSR);
+		printk("mite: using I/O Window Base Size register 1\n");
+		writel(mite->daq_phys_addr | WENAB | MITE_IODWBSR_1_WSIZE_bits(length), mite->mite_io_addr + MITE_IODWBSR_1);
+		writel(0, mite->mite_io_addr + MITE_IODWCR_1);
+	}else
+	{
+		writel(mite->daq_phys_addr | WENAB, mite->mite_io_addr + MITE_IODWBSR);
 	}
-	else writel(mite->daq_phys_addr | WENAB , mite->mite_io_addr + MITE_IODWBSR);
+	/* make sure dma bursts work.  I got this from running a bus analyzer
+	on a pxi-6281 and a pxi-6713.  6713 powered up with register value
+	of 0x61f and bursts worked.  6281 powered up with register value of
+	0x1f and bursts didn't work.  The NI windows driver reads the register,
+	then does a bitwise-or of 0x600 with it and writes it back.
+	*/
+	unknown_dma_burst_bits = readl(mite->mite_io_addr + MITE_UNKNOWN_DMA_BURST_REG);
+	unknown_dma_burst_bits |= UNKNOWN_DMA_BURST_ENABLE_BITS;
+	writel(unknown_dma_burst_bits, mite->mite_io_addr + MITE_UNKNOWN_DMA_BURST_REG);
 
 	csigr_bits = readl(mite->mite_io_addr + MITE_CSIGR);
 	mite->num_channels = mite_csigr_dmac(csigr_bits);
@@ -165,13 +180,19 @@ int mite_setup(struct mite_struct *mite)
 	for( i = 0; i < mite->num_channels; i++ ) {
 		writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR(i));
 		/* disable interrupts */
-		writel(0, mite->mite_io_addr + MITE_CHCR(i));
+		writel(CHCR_CLR_DMA_IE | CHCR_CLR_LINKP_IE | CHCR_CLR_SAR_IE | CHCR_CLR_DONE_IE |
+			CHCR_CLR_MRDY_IE | CHCR_CLR_DRDY_IE | CHCR_CLR_LC_IE | CHCR_CLR_CONT_RB_IE,
+			mite->mite_io_addr + MITE_CHCR(i));
 	}
 	mite->used = 1;
 
 	return 0;
 }
 
+int mite_setup(struct mite_struct *mite)
+{
+	return mite_setup2(mite, 0);
+}
 
 void mite_cleanup(void)
 {
@@ -222,88 +243,130 @@ void mite_list_devices(void)
 
 }
 
-void mite_dma_arm( struct mite_struct *mite, unsigned int channel )
+struct mite_channel* mite_request_channel_in_range(struct mite_struct *mite, struct mite_dma_descriptor_ring *ring,
+	unsigned min_channel, unsigned max_channel)
 {
+	int i;
+	unsigned long flags;
+	struct mite_channel *channel = NULL;
+
+	// spin lock so mite_release_channel can be called safely from interrupts
+	comedi_spin_lock_irqsave(&mite->lock, flags);
+	for(i = min_channel; i <= max_channel; ++i)
+	{
+		if(mite->channel_allocated[i] == 0)
+		{
+			mite->channel_allocated[i] = 1;
+			channel = &mite->channels[i];
+			channel->ring = ring;
+			break;
+		}
+	}
+	comedi_spin_unlock_irqrestore(&mite->lock, flags);
+	return channel;
+}
+
+void mite_release_channel(struct mite_channel *mite_chan)
+{
+	struct mite_struct *mite = mite_chan->mite;
+	unsigned long flags;
+
+	// spin lock to prevent races with mite_request_channel
+	comedi_spin_lock_irqsave(&mite->lock, flags);
+	if(mite->channel_allocated[mite_chan->channel])
+	{
+		// disable all channel's interrupts
+		writel(CHCR_CLR_DMA_IE | CHCR_CLR_LINKP_IE |
+			CHCR_CLR_SAR_IE | CHCR_CLR_DONE_IE |
+			CHCR_CLR_MRDY_IE | CHCR_CLR_DRDY_IE |
+			CHCR_CLR_LC_IE | CHCR_CLR_CONT_RB_IE,
+			mite->mite_io_addr + MITE_CHCR(mite_chan->channel));
+		mite_dma_disarm(mite_chan);
+		mite_dma_reset(mite_chan);
+		mite->channel_allocated[mite_chan->channel] = 0;
+		mite_chan->ring = NULL;
+		mmiowb();
+	}
+	comedi_spin_unlock_irqrestore(&mite->lock, flags);
+}
+
+void mite_dma_arm(struct mite_channel *mite_chan)
+{
+	struct mite_struct *mite = mite_chan->mite;
 	int chor;
+	unsigned long flags;
 
 	MDPRINTK("mite_dma_arm ch%i\n", channel);
+	/* memory barrier is intended to insure any twiddling with the buffer
+	is done before writing to the mite to arm dma transfer */
+	smp_mb();
 	/* arm */
 	chor = CHOR_START;
-	writel(chor, mite->mite_io_addr + MITE_CHOR(channel));
-	mite_dma_tcr(mite, channel);
+	comedi_spin_lock_irqsave(&mite->lock, flags);
+	mite_chan->done = 0;
+	writel(chor, mite->mite_io_addr + MITE_CHOR(mite_chan->channel));
+	mmiowb();
+	comedi_spin_unlock_irqrestore(&mite->lock, flags);
+// 	mite_dma_tcr(mite, channel);
 }
 
 
 /**************************************/
 
-int mite_load_buffer(struct mite_struct *mite, unsigned int channel, comedi_async *async)
+int mite_buf_change(struct mite_dma_descriptor_ring *ring, comedi_async *async)
 {
 	unsigned int n_links;
 	int i;
-	struct mite_channel *mite_chan = &mite->channels[ channel ];
 
-	MDPRINTK("mite_load_buffer ch%i\n", channel);
-
-	if(mite_chan->ring){
-		kfree(mite_chan->ring);
-		mite_chan->ring = NULL;
+	if(ring->descriptors){
+		dma_free_coherent(ring->hw_dev, ring->n_links * sizeof(struct mite_dma_descriptor),
+			ring->descriptors, ring->descriptors_dma_addr);
 	}
+	ring->descriptors = NULL;
+	ring->descriptors_dma_addr = 0;
+	ring->n_links = 0;
 
 	if(async->prealloc_bufsz==0){
 		return 0;
 	}
-
 	n_links = async->prealloc_bufsz >> PAGE_SHIFT;
 
-	MDPRINTK("buf=%p buf(bus)=%08lx bufsz=0x%08x n_links=0x%04x\n",
-		async->prealloc_buf, virt_to_bus(async->prealloc_buf), async->prealloc_bufsz, n_links);
+	MDPRINTK("ring->hw_dev=%p, n_links=0x%04x\n", ring->hw_dev, n_links);
 
-	mite_chan->ring = kmalloc(n_links * sizeof(struct mite_dma_chain), GFP_KERNEL);
-	if(!mite_chan->ring){
+	ring->descriptors = dma_alloc_coherent(ring->hw_dev, n_links * sizeof(struct mite_dma_descriptor),
+		&ring->descriptors_dma_addr, GFP_KERNEL);
+	if(!ring->descriptors){
 		printk("mite: ring buffer allocation failed\n");
 		return -ENOMEM;
 	}
-	mite_chan->n_links = n_links;
-	mite_chan->current_link = 0;
+	ring->n_links = n_links;
 
-	for(i=0;i<n_links;i++){
-		mite_chan->ring[i].count = cpu_to_le32(PAGE_SIZE);
-		mite_chan->ring[i].addr = cpu_to_le32(virt_to_bus(
-				(void *)async->buf_page_list[i]));
-		mite_chan->ring[i].next = cpu_to_le32(virt_to_bus(
-				mite_chan->ring+i+1));
+	for(i = 0; i < n_links; i++){
+		ring->descriptors[i].count = cpu_to_le32(PAGE_SIZE);
+		ring->descriptors[i].addr = cpu_to_le32(async->buf_page_list[i].dma_addr);
+		ring->descriptors[i].next = cpu_to_le32(ring->descriptors_dma_addr + (i + 1) * sizeof(struct mite_dma_descriptor));
 	}
-
-	mite_chan->ring[n_links-1].next = cpu_to_le32(virt_to_bus(
-				mite_chan->ring));
-
+	ring->descriptors[n_links - 1].next = cpu_to_le32(ring->descriptors_dma_addr);
+	/* barrier is meant to insure that all the writes to the dma descriptors
+	have completed before the dma controller is commanded to read them */
+	smp_wmb();
 	return 0;
 }
 
-int mite_buf_change(struct mite_struct *mite, unsigned int channel,
-	comedi_async *async, unsigned long new_size)
-{
-	MDPRINTK("mite_buf_change ch%i\n", channel);
-
-	mite_load_buffer(mite, channel, async);
-
-	return 0;
-}
-
-void mite_prep_dma( struct mite_struct *mite, unsigned int channel,
+void mite_prep_dma(struct mite_channel *mite_chan,
 	unsigned int num_device_bits, unsigned int num_memory_bits )
 {
 	unsigned int chor,chcr,mcr,dcr,lkcr;
-	struct mite_channel *mite_chan = &mite->channels[ channel ];
+	struct mite_struct *mite = mite_chan->mite;
 
-	MDPRINTK("mite_prep_dma ch%i\n", channel );
+	MDPRINTK("mite_prep_dma ch%i\n", mite_chan->channel);
 
 	/* reset DMA and FIFO */
 	chor = CHOR_DMARESET | CHOR_FRESET;
-	writel(chor, mite->mite_io_addr + MITE_CHOR(channel));
+	writel(chor, mite->mite_io_addr + MITE_CHOR(mite_chan->channel));
 
 	/* short link chaining mode */
-	chcr = CHCR_SET_DMA_IE| CHCR_LINKSHORT | CHCR_SET_DONE_IE | CHCR_BURSTEN;
+	chcr = CHCR_SET_DMA_IE | CHCR_LINKSHORT | CHCR_SET_DONE_IE | CHCR_BURSTEN;
 	/*
 	 * Link Complete Interrupt: interrupt every time a link
 	 * in MITE_RING is completed. This can generate a lot of
@@ -313,11 +376,18 @@ void mite_prep_dma( struct mite_struct *mite, unsigned int channel,
 	 * "read()" to calculate the number of bytes available.
 	 */
 	chcr |= CHCR_SET_LC_IE;
-
+	if(num_memory_bits == 32 && num_device_bits == 16)
+	{
+		/* Doing a combined 32 and 16 bit byteswap gets the 16 bit samples into the fifo in the right order.
+		 Tested doing 32 bit memory to 16 bit device transfers to the analog out of a pxi-6281,
+		 which has mite version = 1, type = 4.  This also works for dma reads from the counters
+		 on e-series boards.  */
+		chcr |= CHCR_BYTE_SWAP_DEVICE | CHCR_BYTE_SWAP_MEMORY;
+	}
 	if(mite_chan->dir == COMEDI_INPUT){
 		chcr |= CHCR_DEV_TO_MEM;
 	}
-	writel(chcr, mite->mite_io_addr + MITE_CHCR(channel));
+	writel(chcr, mite->mite_io_addr + MITE_CHCR(mite_chan->channel));
 
 	/* to/from memory */
 	mcr = CR_RL(64) | CR_ASEQUP;
@@ -335,11 +405,11 @@ void mite_prep_dma( struct mite_struct *mite, unsigned int channel,
 			rt_printk( "mite: bug! invalid mem bit width for dma transfer\n" );
 			break;
 	}
-	writel(mcr, mite->mite_io_addr + MITE_MCR(channel));
+	writel(mcr, mite->mite_io_addr + MITE_MCR(mite_chan->channel));
 
 	/* from/to device */
-	dcr = CR_RL(64) |  CR_ASEQUP;
-	dcr |= CR_PORTIO | CR_AMDEVICE | CR_REQSDRQ(channel);
+	dcr = CR_RL(64) | CR_ASEQUP;
+	dcr |= CR_PORTIO | CR_AMDEVICE | CR_REQSDRQ(mite_chan->channel);
 	switch( num_device_bits ){
 		case 8:
 			dcr |= CR_PSIZE8;
@@ -354,91 +424,196 @@ void mite_prep_dma( struct mite_struct *mite, unsigned int channel,
 			rt_printk( "mite: bug! invalid dev bit width for dma transfer\n" );
 			break;
 	}
-	writel(dcr, mite->mite_io_addr + MITE_DCR(channel));
+	writel(dcr, mite->mite_io_addr + MITE_DCR(mite_chan->channel));
 
 	/* reset the DAR */
-	writel(0, mite->mite_io_addr + MITE_DAR(channel));
+	writel(0, mite->mite_io_addr + MITE_DAR(mite_chan->channel));
 
 	/* the link is 32bits */
 	lkcr = CR_RL(64) | CR_ASEQUP | CR_PSIZE32;
-	writel(lkcr, mite->mite_io_addr + MITE_LKCR(channel));
+	writel(lkcr, mite->mite_io_addr + MITE_LKCR(mite_chan->channel));
 
 	/* starting address for link chaining */
-	writel(virt_to_bus(mite_chan->ring),
-		mite->mite_io_addr + MITE_LKAR(channel));
+	writel(mite_chan->ring->descriptors_dma_addr,
+		mite->mite_io_addr + MITE_LKAR(mite_chan->channel));
 
 	MDPRINTK("exit mite_prep_dma\n");
 }
 
-u32 mite_device_bytes_transferred(struct mite_struct *mite, unsigned int chan)
+u32 mite_device_bytes_transferred(struct mite_channel *mite_chan)
 {
-	return readl(mite->mite_io_addr + MITE_DAR(chan));
+	struct mite_struct *mite = mite_chan->mite;
+	return readl(mite->mite_io_addr + MITE_DAR(mite_chan->channel));
 }
 
-u32 mite_bytes_in_transit(struct mite_struct *mite, unsigned int chan)
+u32 mite_bytes_in_transit(struct mite_channel *mite_chan)
 {
-	return readl(mite->mite_io_addr + MITE_FCR(chan)) & 0x000000FF;
+	struct mite_struct *mite = mite_chan->mite;
+	return readl(mite->mite_io_addr + MITE_FCR(mite_chan->channel)) & 0x000000FF;
 }
 
 // returns lower bound for number of bytes transferred from device to memory
-u32 mite_bytes_written_to_memory_lb(struct mite_struct *mite, unsigned int chan)
+u32 mite_bytes_written_to_memory_lb(struct mite_channel *mite_chan)
 {
 	u32 device_byte_count;
 
-	device_byte_count = mite_device_bytes_transferred(mite, chan);
-	rmb();
-	return device_byte_count - mite_bytes_in_transit(mite, chan);
+	device_byte_count = mite_device_bytes_transferred(mite_chan);
+	return device_byte_count - mite_bytes_in_transit(mite_chan);
 }
 
 // returns upper bound for number of bytes transferred from device to memory
-u32 mite_bytes_written_to_memory_ub(struct mite_struct *mite, unsigned int chan)
+u32 mite_bytes_written_to_memory_ub(struct mite_channel *mite_chan)
 {
 	u32 in_transit_count;
 
-	in_transit_count = mite_bytes_in_transit(mite, chan);
-	rmb();
-	return mite_device_bytes_transferred(mite, chan) - in_transit_count;
+	in_transit_count = mite_bytes_in_transit(mite_chan);
+	return mite_device_bytes_transferred(mite_chan) - in_transit_count;
 }
 
 // returns lower bound for number of bytes read from memory for transfer to device
-u32 mite_bytes_read_from_memory_lb(struct mite_struct *mite, unsigned int chan)
+u32 mite_bytes_read_from_memory_lb(struct mite_channel *mite_chan)
 {
 	u32 device_byte_count;
 
-	device_byte_count = mite_device_bytes_transferred(mite, chan);
-	rmb();
-	return device_byte_count + mite_bytes_in_transit(mite, chan);
+	device_byte_count = mite_device_bytes_transferred(mite_chan);
+	return device_byte_count + mite_bytes_in_transit(mite_chan);
 }
 
 // returns upper bound for number of bytes read from memory for transfer to device
-u32 mite_bytes_read_from_memory_ub(struct mite_struct *mite, unsigned int chan)
+u32 mite_bytes_read_from_memory_ub(struct mite_channel *mite_chan)
 {
 	u32 in_transit_count;
 
-	in_transit_count = mite_bytes_in_transit(mite, chan);
-	rmb();
-	return mite_device_bytes_transferred(mite, chan) + in_transit_count;
+	in_transit_count = mite_bytes_in_transit(mite_chan);
+	return mite_device_bytes_transferred(mite_chan) + in_transit_count;
 }
 
-int mite_dma_tcr(struct mite_struct *mite, unsigned int channel)
+unsigned mite_dma_tcr(struct mite_channel *mite_chan)
 {
+	struct mite_struct *mite = mite_chan->mite;
 	int tcr;
 	int lkar;
 
-	lkar = readl(mite->mite_io_addr + MITE_LKAR(channel));
-	tcr = readl(mite->mite_io_addr + MITE_TCR(channel));
-	MDPRINTK("mite_dma_tcr ch%i, lkar=0x%08x tcr=%d\n", channel, lkar, tcr);
+	lkar = readl(mite->mite_io_addr + MITE_LKAR(mite_chan->channel));
+	tcr = readl(mite->mite_io_addr + MITE_TCR(mite_chan->channel));
+	MDPRINTK("mite_dma_tcr ch%i, lkar=0x%08x tcr=%d\n", mite_chan->channel, lkar, tcr);
 
 	return tcr;
 }
 
-void mite_dma_disarm(struct mite_struct *mite, unsigned int channel)
+void mite_dma_disarm(struct mite_channel *mite_chan)
 {
-	int chor;
+	struct mite_struct *mite = mite_chan->mite;
+	unsigned chor;
 
 	/* disarm */
 	chor = CHOR_ABORT;
-	writel(chor, mite->mite_io_addr + MITE_CHOR(channel));
+	writel(chor, mite->mite_io_addr + MITE_CHOR(mite_chan->channel));
+}
+
+int mite_sync_input_dma(struct mite_channel *mite_chan, comedi_async *async)
+{
+	int count;
+	unsigned int nbytes, old_alloc_count;
+	const unsigned bytes_per_scan = cfc_bytes_per_scan(async->subdevice);
+
+	old_alloc_count = async->buf_write_alloc_count;
+	// write alloc as much as we can
+	comedi_buf_write_alloc(async, async->prealloc_bufsz);
+
+	nbytes = mite_bytes_written_to_memory_lb(mite_chan);
+	if((int)(mite_bytes_written_to_memory_ub(mite_chan) - old_alloc_count) > 0)
+	{
+		rt_printk("mite: DMA overwrite of free area\n");
+		async->events |= COMEDI_CB_OVERFLOW;
+		return -1;
+	}
+
+	count = nbytes - async->buf_write_count;
+	/* it's possible count will be negative due to
+	 * conservative value returned by mite_bytes_written_to_memory_lb */
+	if( count <= 0 )
+	{
+		return 0;
+	}
+	comedi_buf_write_free(async, count);
+
+	async->scan_progress += count;
+	if(async->scan_progress >= bytes_per_scan)
+	{
+		async->scan_progress %= bytes_per_scan;
+		async->events |= COMEDI_CB_EOS;
+	}
+	async->events |= COMEDI_CB_BLOCK;
+	return 0;
+}
+
+int mite_sync_output_dma(struct mite_channel *mite_chan, comedi_async *async)
+{
+	int count;
+	u32 nbytes_ub, nbytes_lb;
+	unsigned int old_alloc_count;
+	u32 stop_count = async->cmd.stop_arg * cfc_bytes_per_scan(async->subdevice);
+
+	old_alloc_count = async->buf_read_alloc_count;
+	// read alloc as much as we can
+	comedi_buf_read_alloc(async, async->prealloc_bufsz);
+	nbytes_lb = mite_bytes_read_from_memory_lb(mite_chan);
+	if(async->cmd.stop_src == TRIG_COUNT &&
+		(int) (nbytes_lb - stop_count) > 0)
+		nbytes_lb = stop_count;
+	nbytes_ub = mite_bytes_read_from_memory_ub(mite_chan);
+	if(async->cmd.stop_src == TRIG_COUNT &&
+		(int) (nbytes_ub - stop_count) > 0)
+		nbytes_ub = stop_count;
+	if((int)(nbytes_ub - old_alloc_count) > 0)
+	{
+		rt_printk("mite: DMA underrun\n");
+		async->events |= COMEDI_CB_OVERFLOW;
+		return -1;
+	}
+	count = nbytes_lb - async->buf_read_count;
+	if(count <= 0)
+	{
+		return 0;
+	}
+	if(count)
+	{
+		comedi_buf_read_free(async, count);
+		async->events |= COMEDI_CB_BLOCK;
+	}
+	return 0;
+}
+
+unsigned mite_get_status(struct mite_channel *mite_chan)
+{
+	struct mite_struct *mite = mite_chan->mite;
+	unsigned status;
+	unsigned long flags;
+
+	comedi_spin_lock_irqsave(&mite->lock, flags);
+	status = readl(mite->mite_io_addr + MITE_CHSR(mite_chan->channel));
+	if(status & CHSR_DONE)
+	{
+		mite_chan->done = 1;
+		writel(CHOR_CLRDONE, mite->mite_io_addr + MITE_CHOR(mite_chan->channel));
+	}
+	mmiowb();
+	comedi_spin_unlock_irqrestore(&mite->lock, flags);
+	return status;
+}
+
+int mite_done(struct mite_channel *mite_chan)
+{
+	struct mite_struct *mite = mite_chan->mite;
+	unsigned long flags;
+	int done;
+
+	mite_get_status(mite_chan);
+	comedi_spin_lock_irqsave(&mite->lock, flags);
+	done = mite_chan->done;
+	comedi_spin_unlock_irqrestore(&mite->lock, flags);
+	return done;
 }
 
 #ifdef DEBUG_MITE
@@ -447,7 +622,7 @@ static void mite_decode(char **bit_str, unsigned int bits);
 
 /* names of bits in mite registers */
 
-static char *mite_CHOR_strings[] = {
+static const char * const mite_CHOR_strings[] = {
 	"start", "cont", "stop", "abort",
 	"freset", "clrlc", "clrrb", "clrdone",
 	"clr_lpause", "set_lpause", "clr_send_tc",
@@ -459,7 +634,7 @@ static char *mite_CHOR_strings[] = {
 	"dmareset",
 };
 
-static char *mite_CHCR_strings[] = {
+static const char * const mite_CHCR_strings[] = {
 	"continue", "ringbuff", "2", "3",
 	"4", "5", "6", "7",
 	"8", "9", "10", "11",
@@ -470,7 +645,7 @@ static char *mite_CHCR_strings[] = {
 	"clr_linkp_ie", "set_linkp_ie", "clr_dma_ie", "set_dma_ie",
 };
 
-static char *mite_MCR_strings[] = {
+static const char * const mite_MCR_strings[] = {
 	"amdevice", "1", "2", "3",
 	"4", "5", "portio", "portvxi",
 	"psizebyte", "psizehalf (byte & half = word)", "aseqxp1", "11",
@@ -481,7 +656,7 @@ static char *mite_MCR_strings[] = {
 	"28", "29", "30", "stopen",
 };
 
-static char *mite_DCR_strings[] = {
+static const char * const mite_DCR_strings[] = {
 	"amdevice", "1", "2", "3",
 	"4", "5", "portio", "portvxi",
 	"psizebyte", "psizehalf (byte & half = word)", "aseqxp1", "aseqxp2",
@@ -492,7 +667,7 @@ static char *mite_DCR_strings[] = {
 	"28", "wsdevc", "wsdevs", "rwdevpack",
 };
 
-static char *mite_LKCR_strings[] = {
+static const char * const mite_LKCR_strings[] = {
 	"amdevice", "1", "2", "3",
 	"4", "5", "portio", "portvxi",
 	"psizebyte", "psizehalf (byte & half = word)", "asequp", "aseqdown",
@@ -504,7 +679,7 @@ static char *mite_LKCR_strings[] = {
 };
 
 
-static char *mite_CHSR_strings[] = {
+static const char * const mite_CHSR_strings[] = {
 	"d.err0", "d.err1", "m.err0", "m.err1",
 	"l.err0", "l.err1", "drq0", "drq1",
 	"end", "xferr", "operr0", "operr1",
@@ -515,14 +690,14 @@ static char *mite_CHSR_strings[] = {
 	"28", "lpauses", "30", "int",
 };
 
-void mite_dump_regs(struct mite_struct *mite, int channel)
+void mite_dump_regs(struct mite_channel *mite_chan)
 {
-	unsigned long mite_io_addr = (unsigned long) mite->mite_io_addr;
+	unsigned long mite_io_addr = (unsigned long) mite_chan->mite->mite_io_addr;
 	unsigned long addr=0;
 	unsigned long temp=0;
 
-	printk("mite_dump_regs ch%i\n", channel);
-	printk("mite address is  =0x%08lx\n",mite_io_addr);
+	printk("mite_dump_regs ch%i\n", mite_chan->channel);
+	printk("mite address is  =0x%08lx\n", mite_io_addr);
 
 	addr = mite_io_addr+MITE_CHOR(channel);
 	printk("mite status[CHOR]at 0x%08lx =0x%08lx\n",addr, temp=readl(addr));
@@ -587,7 +762,10 @@ void __exit cleanup_module(void)
 EXPORT_SYMBOL(mite_dma_tcr);
 EXPORT_SYMBOL(mite_dma_arm);
 EXPORT_SYMBOL(mite_dma_disarm);
+EXPORT_SYMBOL(mite_sync_input_dma);
+EXPORT_SYMBOL(mite_sync_output_dma);
 EXPORT_SYMBOL(mite_setup);
+EXPORT_SYMBOL(mite_setup2);
 EXPORT_SYMBOL(mite_unsetup);
 #if 0
 EXPORT_SYMBOL(mite_kvmem_segment_load);
@@ -596,6 +774,8 @@ EXPORT_SYMBOL(mite_setregs);
 #endif
 EXPORT_SYMBOL(mite_devices);
 EXPORT_SYMBOL(mite_list_devices);
+EXPORT_SYMBOL(mite_request_channel_in_range);
+EXPORT_SYMBOL(mite_release_channel);
 EXPORT_SYMBOL(mite_prep_dma);
 EXPORT_SYMBOL(mite_buf_change);
 EXPORT_SYMBOL(mite_bytes_written_to_memory_lb);
@@ -603,6 +783,8 @@ EXPORT_SYMBOL(mite_bytes_written_to_memory_ub);
 EXPORT_SYMBOL(mite_bytes_read_from_memory_lb);
 EXPORT_SYMBOL(mite_bytes_read_from_memory_ub);
 EXPORT_SYMBOL(mite_bytes_in_transit);
+EXPORT_SYMBOL(mite_get_status);
+EXPORT_SYMBOL(mite_done);
 #ifdef DEBUG_MITE
 EXPORT_SYMBOL(mite_decode);
 EXPORT_SYMBOL(mite_dump_regs);
