@@ -30,7 +30,7 @@
 
 #define PCI_VENDOR_ID_NATINST		0x1093
 
-//#define DEBUG_MITE
+// #define DEBUG_MITE
 #define PCIMIO_COMPAT
 
 #ifdef DEBUG_MITE
@@ -41,21 +41,27 @@
 
 #define MAX_MITE_DMA_CHANNELS 8
 
-struct mite_dma_chain{
+struct mite_dma_descriptor{
 	u32 count;
 	u32 addr;
 	u32 next;
 	u32 dar;
 };
 
-struct mite_channel{
-	int DMA_CheckNearEnd;
-
-	int dir;
-	unsigned int current_link;
+struct mite_dma_descriptor_ring
+{
+	struct device *hw_dev;
 	unsigned int n_links;
+	struct mite_dma_descriptor *descriptors;
+	dma_addr_t descriptors_dma_addr;
+};
 
-	struct mite_dma_chain *ring;
+struct mite_channel{
+	struct mite_struct *mite;
+	unsigned channel;
+	int dir;
+	int done;
+	struct mite_dma_descriptor_ring *ring;
 };
 
 struct mite_struct{
@@ -69,7 +75,39 @@ struct mite_struct{
 	void *daq_io_addr;
 
 	struct mite_channel channels[MAX_MITE_DMA_CHANNELS];
+	short channel_allocated[MAX_MITE_DMA_CHANNELS];
 	int num_channels;
+	spinlock_t lock;
+};
+
+static inline struct mite_dma_descriptor_ring* mite_alloc_ring(struct mite_struct *mite)
+{
+	struct mite_dma_descriptor_ring *ring = kmalloc(sizeof(struct mite_dma_descriptor_ring), GFP_KERNEL);
+	if(ring == NULL) return ring;
+	ring->hw_dev = get_device(&mite->pcidev->dev);
+	if(ring->hw_dev == NULL)
+	{
+		kfree(ring);
+		return NULL;
+	}
+	ring->n_links = 0;
+	ring->descriptors = NULL;
+	ring->descriptors_dma_addr = 0;
+	return ring;
+};
+
+static inline void mite_free_ring(struct mite_dma_descriptor_ring *ring)
+{
+	if(ring)
+	{
+		if(ring->descriptors)
+		{
+			dma_free_coherent(ring->hw_dev, ring->n_links * sizeof(struct mite_dma_descriptor),
+				ring->descriptors, ring->descriptors_dma_addr);
+		}
+		put_device(ring->hw_dev);
+		kfree(ring);
+	}
 };
 
 extern struct mite_struct *mite_devices;
@@ -86,31 +124,44 @@ static inline unsigned int mite_device_id(struct mite_struct *mite)
 void mite_init(void);
 void mite_cleanup(void);
 int mite_setup(struct mite_struct *mite);
+int mite_setup2(struct mite_struct *mite, unsigned use_iodwbsr_1);
 void mite_unsetup(struct mite_struct *mite);
 void mite_list_devices(void);
+struct mite_channel* mite_request_channel_in_range(
+	struct mite_struct *mite, struct mite_dma_descriptor_ring *ring,
+	unsigned min_channel, unsigned max_channel);
+static inline struct mite_channel* mite_request_channel(
+	struct mite_struct *mite, struct mite_dma_descriptor_ring *ring)
+{
+	return mite_request_channel_in_range(mite, ring, 0, mite->num_channels - 1);
+}
+void mite_release_channel(struct mite_channel *mite_chan);
 
-int mite_dma_tcr(struct mite_struct *mite, unsigned int channel );
-void mite_dma_arm(struct mite_struct *mite, unsigned int channel );
-void mite_dma_disarm(struct mite_struct *mite, unsigned int channel );
-u32 mite_bytes_written_to_memory_lb(struct mite_struct *mite, unsigned int chan);
-u32 mite_bytes_written_to_memory_ub(struct mite_struct *mite, unsigned int chan);
-u32 mite_bytes_read_from_memory_lb(struct mite_struct *mite, unsigned int chan);
-u32 mite_bytes_read_from_memory_ub(struct mite_struct *mite, unsigned int chan);
-u32 mite_bytes_in_transit(struct mite_struct *mite, unsigned int chan);
+unsigned mite_dma_tcr(struct mite_channel *mite_chan );
+void mite_dma_arm(struct mite_channel *mite_chan );
+void mite_dma_disarm(struct mite_channel *mite_chan );
+int mite_sync_input_dma(struct mite_channel *mite_chan, comedi_async *async);
+int mite_sync_output_dma(struct mite_channel *mite_chan, comedi_async *async);
+u32 mite_bytes_written_to_memory_lb(struct mite_channel *mite_chan);
+u32 mite_bytes_written_to_memory_ub(struct mite_channel *mite_chan);
+u32 mite_bytes_read_from_memory_lb(struct mite_channel *mite_chan);
+u32 mite_bytes_read_from_memory_ub(struct mite_channel *mite_chan);
+u32 mite_bytes_in_transit(struct mite_channel *mite_chan);
+unsigned mite_get_status(struct mite_channel *mite_chan);
+int mite_done(struct mite_channel *mite_chan);
 
 #if 0
 unsigned long mite_ll_from_kvmem(struct mite_struct *mite,comedi_async *async,int len);
 void mite_setregs(struct mite_struct *mite,unsigned long ll_start,int chan, int dir);
 #endif
 
-void mite_prep_dma(struct mite_struct *mite, unsigned int channel,
+void mite_prep_dma(struct mite_channel *mite_chan,
 	unsigned int num_device_bits, unsigned int num_memory_bits );
-int mite_buf_change(struct mite_struct *mite, unsigned int channel,
-	comedi_async *async, unsigned long new_size);
+int mite_buf_change(struct mite_dma_descriptor_ring *ring, comedi_async *async);
 
 #ifdef DEBUG_MITE
 void mite_print_chsr(unsigned int chsr);
-void mite_dump_regs(struct mite_struct *mite, int channel);
+void mite_dump_regs(struct mite_channel *mite_chan);
 #endif
 
 static inline int CHAN_OFFSET(int channel)
@@ -120,10 +171,15 @@ static inline int CHAN_OFFSET(int channel)
 
 enum mite_registers
 {
+	/* The bits 0x90180700 in MITE_UNKNOWN_DMA_BURST_REG can be
+	written and read back.  The bits 0x1f always read as 1.
+	The rest always read as zero. */
+	MITE_UNKNOWN_DMA_BURST_REG = 0x28,
 	MITE_IODWBSR = 0xc0, //IO Device Window Base Size Register
-	MITE_CSIGR = 0x460,	//chip signature
-	MITE_IODWBSR_1 = 0xc4, // IO Device Window Base Size Register 1 (used by 6602 boards)
-	MITE_IODWCR_1 = 0xf4
+	MITE_IODWBSR_1 = 0xc4, // IO Device Window Base Size Register 1
+	MITE_IODWCR_1 = 0xf4,
+	MITE_PCI_CONFIG_OFFSET = 0x300,
+	MITE_CSIGR = 0x460	//chip signature
 };
 static inline int MITE_CHOR(int channel)	// channel operation
 {
@@ -161,7 +217,7 @@ static inline int MITE_LKAR(int channel)	// link address
 {
 	return CHAN_OFFSET(channel) + 0x20;
 };
-static inline int MITE_LLKAR(int channel)	// ?
+static inline int MITE_LLKAR(int channel)	// see mite section of tnt5002 manual
 {
 	return CHAN_OFFSET(channel) + 0x24;
 };
@@ -197,7 +253,19 @@ static inline int MITE_FCR(int channel)	// fifo count
 enum MITE_IODWBSR_bits
 {
 	WENAB = 0x80,	// window enable
-	WENAB_6602 = 0x8c // window enable for 6602 boards
+};
+
+static inline unsigned MITE_IODWBSR_1_WSIZE_bits(unsigned size)
+{
+	unsigned order = 0;
+	while(size >>= 1) ++order;
+	BUG_ON(order < 1);
+	return (order - 1) & 0x1f;
+}
+
+enum MITE_UNKNOWN_DMA_BURST_bits
+{
+	UNKNOWN_DMA_BURST_ENABLE_BITS = 0x600
 };
 
 static inline int mite_csigr_version(u32 csigr_bits)
@@ -222,7 +290,7 @@ static inline int mite_csigr_dmac(u32 csigr_bits)
 };
 static inline int mite_csigr_wpdep(u32 csigr_bits)
 {	// write post fifo depth
-	unsigned int wpdep_bits = (csigr_bits >> 16) & 0xf;
+	unsigned int wpdep_bits = (csigr_bits >> 20) & 0x7;
 	if(wpdep_bits == 0) return 0;
 	else return 1 << (wpdep_bits - 1);
 };
@@ -257,8 +325,8 @@ enum MITE_CHOR_bits
 	CHOR_CLRRB			= (1<<6),
 	CHOR_CLRLC			= (1<<5),
 	CHOR_FRESET			= (1<<4),
-	CHOR_ABORT			= (1<<3),
-	CHOR_STOP			= (1<<2),
+	CHOR_ABORT			= (1<<3),	/* stop without emptying fifo */
+	CHOR_STOP			= (1<<2),	/* stop after emptying fifo */
 	CHOR_CONT			= (1<<1),
 	CHOR_START			= (1<<0),
 	CHOR_PON		= (CHOR_CLR_SEND_TC|CHOR_CLR_LPAUSE),
@@ -286,8 +354,8 @@ enum MITE_CHCR_bits
 	CHCR_FIFO_ON			= 0,
 	CHCR_BURSTEN			= (1<<14),
 	CHCR_NO_BURSTEN			= 0,
-	CHCR_CHEND1			= (1<<5),
-	CHCR_CHEND0			= (1<<4),
+	CHCR_BYTE_SWAP_DEVICE	= (1 << 6),
+	CHCR_BYTE_SWAP_MEMORY	= (1 << 4),
 	CHCR_DIR			= (1<<3),
 	CHCR_DEV_TO_MEM		= CHCR_DIR,
 	CHCR_MEM_TO_DEV		= 0,
@@ -317,11 +385,13 @@ enum ConfigRegister_bits
 };
 static inline int CR_REQS(int source)
 {
-	return (source & 0x7) <<16;
+	return (source & 0x7) << 16;
 };
-static inline int CR_REQSDRQ(int drq_line)
+static inline int CR_REQSDRQ(unsigned drq_line)
 {
-	return CR_REQS(drq_line + 0x4);
+	/* This also works on m-series when
+	using channels (drq_line) 4 or 5. */
+	return CR_REQS((drq_line & 0x3) | 0x4);
 }
 static inline int CR_RL(unsigned int retry_limit)
 {
@@ -372,5 +442,10 @@ enum CHSR_bits
 	CHSR_DOERR		= (3<<0),
 };
 
+static inline void mite_dma_reset(struct mite_channel *mite_chan)
+{
+	writel(CHOR_DMARESET | CHOR_FRESET, mite_chan->mite->mite_io_addr + MITE_CHOR(mite_chan->channel));
+};
+
 #endif
 
