@@ -1,5 +1,6 @@
 /*
-    ni_at_a2150.c driver for National Instruments AT-A2150 boards
+    comedi/drivers/ni_at_a2150.c
+    Driver for National Instruments AT-A2150 boards
     Copyright (C) 2001, 2002 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     COMEDI - Linux Control and Measurement Device Interface
@@ -22,7 +23,7 @@
 ************************************************************************
 */
 /*
-Driver: ni_at_a2150.o
+Driver: ni_at_a2150
 Description: National Instruments AT-A2150
 Author: Frank Mori Hess
 Status: works
@@ -119,14 +120,14 @@ TRIG_WAKE_EOS
 #define   HW_COUNT_DISABLE		0x30	// disable hardware counting of conversions
 
 typedef struct a2150_board_struct{
-	char *name;
+	const char *name;
 	int clock[4];	// master clock periods, in nanoseconds
 	int num_clocks;	// number of available master clock speeds
 	int ai_speed;	// maximum conversion rate in nanoseconds
 }a2150_board;
 
 //analog input range
-static comedi_lrange range_a2150 = {
+static const comedi_lrange range_a2150 = {
 	1,
 	{
 		RANGE( -2.828, 2.828 ),
@@ -135,7 +136,7 @@ static comedi_lrange range_a2150 = {
 
 // enum must match board indices
 enum{a2150_c, a2150_s};
-static a2150_board a2150_boards[] =
+static const a2150_board a2150_boards[] =
 {
 	{
 		name:	"at-a2150c",
@@ -153,7 +154,7 @@ static a2150_board a2150_boards[] =
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((a2150_board *)dev->board_ptr)
+#define thisboard ((const a2150_board *)dev->board_ptr)
 
 typedef struct{
 	volatile unsigned int count;  /* number of data points left to be taken */
@@ -177,7 +178,7 @@ static comedi_driver driver_a2150={
 	detach:		a2150_detach,
 };
 
-static irqreturn_t a2150_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t a2150_interrupt(int irq, void *d PT_REGS_ARG);
 static int a2150_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *cmd);
 static int a2150_ai_cmd(comedi_device *dev, comedi_subdevice *s);
 static int a2150_ai_rinsn(comedi_device *dev, comedi_subdevice *s, comedi_insn *insn, lsampl_t *data);
@@ -202,7 +203,7 @@ static void ni_dump_regs(comedi_device *dev)
 #endif
 
 /* interrupt service routine */
-static irqreturn_t a2150_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t a2150_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	int i;
 	int status;
@@ -245,7 +246,7 @@ static irqreturn_t a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 		comedi_error(dev, "caught non-dma interrupt?  Aborting.");
 		a2150_cancel(dev, s);
 		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
-		comedi_event(dev, s, async->events);
+		comedi_event(dev, s);
 		return IRQ_HANDLED;
 	}
 
@@ -313,7 +314,7 @@ static irqreturn_t a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 
 	async->events |= COMEDI_CB_BLOCK;
 
-	comedi_event(dev, s, async->events);
+	comedi_event(dev, s);
 
 	/* clear interrupt */
 	outw(0x00, dev->iobase + DMA_TC_CLEAR_REG);
@@ -424,7 +425,7 @@ static int a2150_attach(comedi_device *dev, comedi_devconfig *it)
 	s = dev->subdevices + 0;
 	dev->read_subdev = s;
 	s->type = COMEDI_SUBD_AI;
-	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_OTHER;
+	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_OTHER | SDF_CMD_READ;
 	s->n_chan = 4;
 	s->len_chanlist = 4;
 	s->maxdata = 0xffff;
@@ -708,7 +709,7 @@ static int a2150_ai_cmd(comedi_device *dev, comedi_subdevice *s)
 	outw(devpriv->irq_dma_bits, dev->iobase + IRQ_DMA_CNTRL_REG);
 
 	// may need to wait 72 sampling periods if timing was changed
-	i8254_load(dev->iobase + I8253_BASE_REG, 2, 72, 0);
+	i8254_load(dev->iobase + I8253_BASE_REG, 0, 2, 72, 0);
 
 	// setup start triggering
 	trigger_bits = 0;
