@@ -20,7 +20,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /*
-Driver: ni_atmio.o
+Driver: ni_atmio
 Description: National Instruments AT-MIO-E series
 Author: ds
 Devices: [National Instruments] AT-MIO-16E-1 (ni_atmio),
@@ -115,7 +115,7 @@ are not supported.
 
 #define MAX_N_CALDACS 32
 
-static ni_board ni_boards[]={
+static const ni_board ni_boards[]={
 	{	device_id:	44,
 		isapnp_id:	0x0000,	/* XXX unknown */
 		name:		"at-mio-16e-1",
@@ -130,7 +130,9 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:	2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
+		ao_speed:	1000,
 		has_8255:	0,
+		.num_p0_dio_channels = 8,
 		caldac:		{mb88341},
 	},
 	{	device_id:	25,
@@ -147,7 +149,9 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:	2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
+		ao_speed:	1000,
 		has_8255:	0,
+		.num_p0_dio_channels = 8,
 		caldac:		{mb88341},
 	},
 	{	device_id:	36,
@@ -164,6 +168,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:	0,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
+		ao_speed:	10000,
+		.num_p0_dio_channels = 8,
 		caldac:		{ad8804_debug},
 		has_8255:	0,
 	},
@@ -181,6 +187,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:	0,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
+		ao_speed:	10000,
+		.num_p0_dio_channels = 8,
 		caldac:		{ad8804_debug},
 		has_8255:	1,
 	},
@@ -198,7 +206,9 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:	2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
+		ao_speed:	1000,
 		has_8255:	0,
+		.num_p0_dio_channels = 8,
 		caldac:		{ad8804_debug},
 	},
 	{	device_id:	39,
@@ -215,6 +225,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:	0,
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:	0,
+		ao_speed:	50000,
+		.num_p0_dio_channels = 8,
 		caldac:		{dac8800,dac8043},
 		has_8255:	0,
 	},
@@ -232,6 +244,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:	2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		caldac:		{dac8800,dac8043,ad8522},
 		has_8255:	0,
 	},
@@ -248,12 +262,13 @@ static ni_board ni_boards[]={
 		aobits:		0,
 		ao_fifo_depth:	0,
 		ao_unipolar:	0,
+		.num_p0_dio_channels = 8,
 		caldac:		{dac8800,dac8043,ad8522},
 		has_8255:	0,
 	}
 };
 
-static int ni_irqpin[]={-1,-1,-1,0,1,2,-1,3,-1,-1,4,5,6,-1,-1,7};
+static const int ni_irqpin[]={-1,-1,-1,0,1,2,-1,3,-1,-1,4,5,6,-1,-1,7};
 
 #define interrupt_pin(a)	(ni_irqpin[(a)])
 
@@ -358,7 +373,7 @@ static int ni_isapnp_find_board( struct pnp_dev **dev )
 {
 	struct pnp_dev *isapnp_dev = NULL;
 	int i;
-	
+
 	for( i = 0; i < n_ni_boards; i++ )
 	{
 		isapnp_dev = pnp_find_dev(NULL,
@@ -405,6 +420,7 @@ static int ni_atmio_attach(comedi_device *dev,comedi_devconfig *it)
 	devpriv->stc_writew = &ni_atmio_win_out;
 	devpriv->stc_readw = &ni_atmio_win_in;
 	devpriv->stc_writel = &win_out2;
+	devpriv->stc_readl = &win_in2;
 
 	iobase=it->options[0];
 	irq=it->options[1];
