@@ -1,5 +1,6 @@
 /*
-    ni_labpc.c driver for National Instruments Lab-PC series boards and compatibles
+    comedi/drivers/ni_labpc.c
+    Driver for National Instruments Lab-PC series boards and compatibles
     Copyright (C) 2001, 2002, 2003 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     This program is free software; you can redistribute it and/or modify
@@ -19,7 +20,7 @@
 ************************************************************************
 */
 /*
-Driver: ni_labpc.o
+Driver: ni_labpc
 Description: National Instruments Lab-PC (& compatibles)
 Author: Frank Mori Hess <fmhess@users.sourceforge.net>
 Devices: [National Instruments] Lab-PC-1200 (labpc-1200),
@@ -163,7 +164,7 @@ NI manuals:
 
 static int labpc_attach(comedi_device *dev,comedi_devconfig *it);
 static int labpc_cancel(comedi_device *dev, comedi_subdevice *s);
-static irqreturn_t labpc_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t labpc_interrupt(int irq, void *d PT_REGS_ARG);
 static int labpc_drain_fifo(comedi_device *dev);
 static void labpc_drain_dma(comedi_device *dev);
 static void handle_isa_dma(comedi_device *dev);
@@ -199,7 +200,7 @@ enum scan_mode
 //analog input ranges
 #define NUM_LABPC_PLUS_AI_RANGES 16
 // indicates unipolar ranges
-static int labpc_plus_is_unipolar[NUM_LABPC_PLUS_AI_RANGES] =
+static const int labpc_plus_is_unipolar[NUM_LABPC_PLUS_AI_RANGES] =
 {
 	0,
 	0,
@@ -219,7 +220,7 @@ static int labpc_plus_is_unipolar[NUM_LABPC_PLUS_AI_RANGES] =
 	1,
 };
 // map range index to gain bits
-static int labpc_plus_ai_gain_bits[NUM_LABPC_PLUS_AI_RANGES] =
+static const int labpc_plus_ai_gain_bits[NUM_LABPC_PLUS_AI_RANGES] =
 {
 	0x00,
 	0x10,
@@ -238,7 +239,7 @@ static int labpc_plus_ai_gain_bits[NUM_LABPC_PLUS_AI_RANGES] =
 	0x60,
 	0x70,
 };
-static comedi_lrange range_labpc_plus_ai = {
+static const comedi_lrange range_labpc_plus_ai = {
 	NUM_LABPC_PLUS_AI_RANGES,
 	{
 		BIP_RANGE(5),
@@ -262,7 +263,7 @@ static comedi_lrange range_labpc_plus_ai = {
 
 #define NUM_LABPC_1200_AI_RANGES 14
 // indicates unipolar ranges
-static int labpc_1200_is_unipolar[NUM_LABPC_1200_AI_RANGES] =
+static const int labpc_1200_is_unipolar[NUM_LABPC_1200_AI_RANGES] =
 {
 	0,
 	0,
@@ -280,7 +281,7 @@ static int labpc_1200_is_unipolar[NUM_LABPC_1200_AI_RANGES] =
 	1,
 };
 // map range index to gain bits
-static int labpc_1200_ai_gain_bits[NUM_LABPC_1200_AI_RANGES] =
+static const int labpc_1200_ai_gain_bits[NUM_LABPC_1200_AI_RANGES] =
 {
 	0x00,
 	0x20,
@@ -297,7 +298,7 @@ static int labpc_1200_ai_gain_bits[NUM_LABPC_1200_AI_RANGES] =
 	0x60,
 	0x70,
 };
-static comedi_lrange range_labpc_1200_ai = {
+static const comedi_lrange range_labpc_1200_ai = {
 	NUM_LABPC_1200_AI_RANGES,
 	{
 		BIP_RANGE(5),
@@ -319,7 +320,7 @@ static comedi_lrange range_labpc_1200_ai = {
 
 //analog output ranges
 #define AO_RANGE_IS_UNIPOLAR 0x1
-comedi_lrange range_labpc_ao = {
+static const comedi_lrange range_labpc_ao = {
 	2,
 	{
 		BIP_RANGE(5),
@@ -346,7 +347,7 @@ static inline void labpc_writeb(unsigned int byte, unsigned long address)
 	writeb(byte, (void*) address);
 }
 
-static labpc_board labpc_boards[] =
+static const labpc_board labpc_boards[] =
 {
 	{
 		name:	"lab-pc-1200",
@@ -399,7 +400,7 @@ static labpc_board labpc_boards[] =
 	},
 };
 
-labpc_board labpc_cs_boards[NUM_LABPC_CS_BOARDS] =
+const labpc_board labpc_cs_boards[NUM_LABPC_CS_BOARDS] =
 {
 	{
 		name:	"daqcard-1200",
@@ -446,7 +447,7 @@ static comedi_driver driver_labpc={
 	attach:		labpc_attach,
 	detach:		labpc_common_detach,
 	num_names:	sizeof(labpc_boards) / sizeof(labpc_board),
-	board_name:	(char **)labpc_boards,
+	board_name: &labpc_boards[0].name,
 	offset:		sizeof(labpc_board),
 };
 
@@ -460,9 +461,9 @@ static inline int labpc_counter_load(comedi_device *dev, unsigned long base_addr
 	unsigned int counter_number, unsigned int count, unsigned int mode)
 {
 	if(thisboard->memory_mapped_io)
-		return i8254_mm_load((void*)base_address, counter_number, count, mode);
+		return i8254_mm_load((void*)base_address, 0, counter_number, count, mode);
 	else
-		return i8254_load(base_address, counter_number, count, mode);
+		return i8254_load(base_address, 0, counter_number, count, mode);
 }
 
 int labpc_common_attach( comedi_device *dev, unsigned long iobase,
@@ -527,7 +528,7 @@ int labpc_common_attach( comedi_device *dev, unsigned long iobase,
 	{
 		isr_flags = 0;
 		if(thisboard->bustype == pci_bustype)
-			isr_flags |= SA_SHIRQ;
+			isr_flags |= IRQF_SHARED;
 		if(comedi_request_irq( irq, labpc_interrupt, isr_flags, driver_labpc.driver_name, dev))
 		{
 			printk( "unable to allocate irq %u\n", irq);
@@ -571,7 +572,7 @@ int labpc_common_attach( comedi_device *dev, unsigned long iobase,
 	s = dev->subdevices + 0;
 	dev->read_subdev = s;
 	s->type = COMEDI_SUBD_AI;
-	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_COMMON | SDF_DIFF;
+	s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_COMMON | SDF_DIFF | SDF_CMD_READ;
 	s->n_chan = 8;
 	s->len_chanlist = 8;
 	s->maxdata = (1 << 12) - 1;	// 12 bit resolution
@@ -1317,7 +1318,7 @@ static int labpc_ai_cmd(comedi_device *dev, comedi_subdevice *s)
 }
 
 /* interrupt service routine */
-static irqreturn_t labpc_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t labpc_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = dev->read_subdev;
@@ -1351,7 +1352,7 @@ static irqreturn_t labpc_interrupt(int irq, void *d, struct pt_regs *regs)
 		// clear error interrupt
 		devpriv->write_byte(0x1, dev->iobase + ADC_CLEAR_REG);
 		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
-		comedi_event(dev, s, async->events);
+		comedi_event(dev, s);
 		comedi_error(dev, "overrun");
 		return IRQ_HANDLED;
 	}
@@ -1378,7 +1379,7 @@ static irqreturn_t labpc_interrupt(int irq, void *d, struct pt_regs *regs)
 		// clear error interrupt
 		devpriv->write_byte(0x1, dev->iobase + ADC_CLEAR_REG);
 		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
-		comedi_event(dev, s, async->events);
+		comedi_event(dev, s);
 		comedi_error(dev, "overflow");
 		return IRQ_HANDLED;
 	}
@@ -1404,7 +1405,7 @@ static irqreturn_t labpc_interrupt(int irq, void *d, struct pt_regs *regs)
 		}
 	}
 
-	comedi_event(dev, s, async->events);
+	comedi_event(dev, s);
 	return IRQ_HANDLED;
 }
 
