@@ -23,7 +23,7 @@
 
 */
 /*
-Driver: ni_pcidio.o
+Driver: ni_pcidio
 Description: National Instruments PCI-DIO32HS, PCI-DIO96, PCI-6533, PCI-6503
 Author: ds
 Status: works
@@ -205,8 +205,17 @@ comedi_nonfree_firmware tarball available from http://www.comedi.org
 #define Interrupt_Control		75
   /* bits same as flags */
 
-#define DMA_Line_Control		76
-  #define DMAChannel(x)		((x)&0xf)
+#define DMA_Line_Control_Group1		76
+#define DMA_Line_Control_Group2		108
+// channel zero is none
+static inline unsigned primary_DMAChannel_bits(unsigned channel)
+{
+	return channel & 0x3;
+}
+static inline unsigned secondary_DMAChannel_bits(unsigned channel)
+{
+	return (channel << 2) & 0xc;
+}
 
 #define Transfer_Size_Control		77
   #define TransferWidth(x)	((x)&3)
@@ -282,13 +291,10 @@ enum FPGA_Control_Bits
 #define IntEn (TransferReady|CountExpired|Waited|PrimaryTC|SecondaryTC)
 #endif
 
-enum mite_dma_channels
-{
-	DI_DMA_CHAN = 1,
-};
-
 static int nidio_attach(comedi_device *dev,comedi_devconfig *it);
 static int nidio_detach(comedi_device *dev);
+static int ni_pcidio_cancel(comedi_device *dev, comedi_subdevice *s);
+
 static comedi_driver driver_pcidio={
 	driver_name:	"ni_pcidio",
 	module:		THIS_MODULE,
@@ -299,76 +305,76 @@ COMEDI_INITCLEANUP(driver_pcidio);
 
 typedef struct{
 	int dev_id;
-	char *name;
+	const char *name;
 	int n_8255;
 	unsigned int is_diodaq : 1;
 	unsigned int uses_firmware : 1;
 }nidio_board;
-static nidio_board nidio_boards[]={
+static const nidio_board nidio_boards[]={
 	{
-	dev_id:		0x1150,
-	name:		"pci-dio-32hs",
-	n_8255:		0,
-	is_diodaq:	1,
+		dev_id:		0x1150,
+		name:		"pci-dio-32hs",
+		n_8255:		0,
+		is_diodaq:	1,
 	},
 	{
-	dev_id:		0x1320,
-	name:		"pxi-6533",
-	n_8255:		0,
-	is_diodaq:	1,
+		dev_id:		0x1320,
+		name:		"pxi-6533",
+		n_8255:		0,
+		is_diodaq:	1,
 	},
 	{
-	dev_id:		0x12b0,
-	name:		"pci-6534",
-	n_8255:		0,
-	is_diodaq:	1,
-	uses_firmware:	1,
+		dev_id:		0x12b0,
+		name:		"pci-6534",
+		n_8255:		0,
+		is_diodaq:	1,
+		uses_firmware:	1,
 	},
 	{
-	dev_id:		0x0160,
-	name:		"pci-dio-96",
-	n_8255:		4,
-	is_diodaq:	0,
+		dev_id:		0x0160,
+		name:		"pci-dio-96",
+		n_8255:		4,
+		is_diodaq:	0,
 	},
 	{
-	dev_id:		0x1630,
-	name:		"pci-dio-96b",
-	n_8255:		4,
-	is_diodaq:	0,
+		dev_id:		0x1630,
+		name:		"pci-dio-96b",
+		n_8255:		4,
+		is_diodaq:	0,
 	},
 	{
-	dev_id:		0x13c0,
-	name:		"pxi-6508",
-	n_8255:		4,
-	is_diodaq:	0,
+		dev_id:		0x13c0,
+		name:		"pxi-6508",
+		n_8255:		4,
+		is_diodaq:	0,
 	},
 	{
-	dev_id:		0x0400,
-	name:		"pci-6503",
-	n_8255:		1,
-	is_diodaq:	0,
+		dev_id:		0x0400,
+		name:		"pci-6503",
+		n_8255:		1,
+		is_diodaq:	0,
 	},
 	{
-	dev_id:		0x1250,
-	name:		"pci-6503b",
-	n_8255:		1,
-	is_diodaq:	0,
+		dev_id:		0x1250,
+		name:		"pci-6503b",
+		n_8255:		1,
+		is_diodaq:	0,
 	},
 	{
-	dev_id:		0x17d0,
-	name:		"pci-6503x",
-	n_8255:		1,
-	is_diodaq:	0,
+		dev_id:		0x17d0,
+		name:		"pci-6503x",
+		n_8255:		1,
+		is_diodaq:	0,
 	},
 	{
-	dev_id:		0x1800,
-	name:		"pxi-6503",
-	n_8255:		1,
-	is_diodaq:	0,
+		dev_id:		0x1800,
+		name:		"pxi-6503",
+		n_8255:		1,
+		is_diodaq:	0,
 	},
 };
 #define n_nidio_boards (sizeof(nidio_boards)/sizeof(nidio_boards[0]))
-#define this_board ((nidio_board *)dev->board_ptr)
+#define this_board ((const nidio_board *)dev->board_ptr)
 
 static struct pci_device_id ni_pcidio_pci_table[] __devinitdata = {
 	{ PCI_VENDOR_ID_NATINST, 0x1150, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
@@ -390,6 +396,9 @@ typedef struct{
 	int boardtype;
 	int dio;
 	unsigned short OpModeBits;
+	struct mite_channel *di_mite_chan;
+	struct mite_dma_descriptor_ring *di_mite_ring;
+	spinlock_t mite_channel_lock;
 }nidio96_private;
 #define devpriv ((nidio96_private *)dev->private)
 
@@ -400,7 +409,7 @@ static int ni_pcidio_inttrig(comedi_device *dev, comedi_subdevice *s,
 	unsigned int trignum);
 static int nidio_find_device(comedi_device *dev,int bus,int slot);
 static int ni_pcidio_ns_to_timer(int *nanosec, int round_mode);
-static void setup_mite_dma(comedi_device *dev,comedi_subdevice *s);
+static int setup_mite_dma(comedi_device *dev,comedi_subdevice *s);
 
 #ifdef DEBUG_FLAGS
 static void ni_pcidio_print_flags(unsigned int flags);
@@ -410,6 +419,45 @@ static void ni_pcidio_print_status(unsigned int status);
 #define ni_pcidio_print_status(x)
 #endif
 
+static int ni_pcidio_request_di_mite_channel(comedi_device *dev)
+{
+	unsigned long flags;
+
+	comedi_spin_lock_irqsave(&devpriv->mite_channel_lock, flags);
+	BUG_ON(devpriv->di_mite_chan);
+	devpriv->di_mite_chan = mite_request_channel_in_range(devpriv->mite, devpriv->di_mite_ring, 1, 2);
+	if(devpriv->di_mite_chan == NULL)
+	{
+		comedi_spin_unlock_irqrestore(&devpriv->mite_channel_lock, flags);
+		comedi_error(dev, "failed to reserve mite dma channel.");
+		return -EBUSY;
+	}
+	writeb(primary_DMAChannel_bits(devpriv->di_mite_chan->channel) |
+		secondary_DMAChannel_bits(devpriv->di_mite_chan->channel),
+		devpriv->mite->daq_io_addr + DMA_Line_Control_Group1);
+	mmiowb();
+	comedi_spin_unlock_irqrestore(&devpriv->mite_channel_lock, flags);
+	return 0;
+}
+
+static void ni_pcidio_release_di_mite_channel(comedi_device *dev)
+{
+	unsigned long flags;
+
+	comedi_spin_lock_irqsave(&devpriv->mite_channel_lock, flags);
+	if(devpriv->di_mite_chan)
+	{
+		mite_dma_disarm(devpriv->di_mite_chan);
+		mite_dma_reset(devpriv->di_mite_chan);
+		mite_release_channel(devpriv->di_mite_chan);
+		writeb(primary_DMAChannel_bits(0) |
+			secondary_DMAChannel_bits(0),
+			devpriv->mite->daq_io_addr + DMA_Line_Control_Group1);
+		mmiowb();
+	}
+	comedi_spin_unlock_irqrestore(&devpriv->mite_channel_lock, flags);
+}
+
 static int nidio96_8255_cb(int dir,int port,int data,unsigned long iobase)
 {
 	if(dir){
@@ -420,13 +468,22 @@ static int nidio96_8255_cb(int dir,int port,int data,unsigned long iobase)
 	}
 }
 
-static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
+void ni_pcidio_event(comedi_device *dev, comedi_subdevice *s)
+{
+	if(s->async->events & (COMEDI_CB_EOA | COMEDI_CB_ERROR | COMEDI_CB_OVERFLOW))
+	{
+		ni_pcidio_cancel(dev, s);
+	}
+	comedi_event(dev, s);
+}
+
+static irqreturn_t nidio_interrupt(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev=d;
 	comedi_subdevice *s = dev->subdevices;
 	comedi_async *async = s->async;
 	struct mite_struct *mite = devpriv->mite;
-	struct mite_channel *mite_chan = &mite->channels[ DI_DMA_CHAN ];
+
 	//int i, j;
 	long int AuxData = 0;
 	sampl_t data1 = 0;
@@ -434,60 +491,47 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 	int flags;
 	int status;
 	int work = 0;
-	unsigned int m_status;
-
-	status = readb(devpriv->mite->daq_io_addr+Interrupt_And_Window_Status);
-	flags = readb(devpriv->mite->daq_io_addr+Group_1_Flags);
-	m_status = readl(mite->mite_io_addr + MITE_CHSR(DI_DMA_CHAN));
+	unsigned int m_status = 0;
+	unsigned long irq_flags;
 
 	//interrupcions parasites
 	if(dev->attached == 0){
-		comedi_error(dev,"premature interrupt");
-		async->events |= COMEDI_CB_ERROR|COMEDI_CB_EOA;
+		// assume it's from another card
+		return IRQ_NONE;
 	}
 
-	DPRINTK("ni_pcidio_interrupt: status=0x%02x,flags=0x%02x,m_status=0x%08x\n",
-		status,flags,m_status);
+	status = readb(devpriv->mite->daq_io_addr+Interrupt_And_Window_Status);
+	flags = readb(devpriv->mite->daq_io_addr+Group_1_Flags);
+
+	DPRINTK("ni_pcidio_interrupt: status=0x%02x,flags=0x%02x\n",
+		status,flags);
 	ni_pcidio_print_flags(flags);
 	ni_pcidio_print_status(status);
+
+	//printk("buf[0]=%08x\n",*(unsigned int *)async->prealloc_buf);
+	//printk("buf[4096]=%08x\n",*(unsigned int *)(async->prealloc_buf+4096));
+
+	comedi_spin_lock_irqsave(&devpriv->mite_channel_lock, irq_flags);
+	if(devpriv->di_mite_chan)
+		m_status = mite_get_status(devpriv->di_mite_chan);
 #ifdef MITE_DEBUG
 	mite_print_chsr(m_status);
 #endif
 	//printk("mite_bytes_transferred: %d\n",mite_bytes_transferred(mite,DI_DMA_CHAN));
 	//mite_dump_regs(mite);
-
-	//printk("buf[0]=%08x\n",*(unsigned int *)async->prealloc_buf);
-	//printk("buf[4096]=%08x\n",*(unsigned int *)(async->prealloc_buf+4096));
-
 	if(m_status & CHSR_INT){
 		if(m_status & CHSR_LINKC){
-			unsigned int count;
-
-			writel(CHOR_CLRLC, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
-			count = le32_to_cpu(mite_chan->ring[mite_chan->current_link].count);
-
+			writel(CHOR_CLRLC, mite->mite_io_addr + MITE_CHOR(devpriv->di_mite_chan->channel));
+			mite_sync_input_dma(devpriv->di_mite_chan, s->async);
 			/* XXX need to byteswap */
-
-			async->buf_write_count += count;
-			async->buf_write_ptr += count;
-			if(async->buf_write_ptr >= async->prealloc_bufsz){
-				async->buf_write_ptr -= async->prealloc_bufsz;
-			}
-			mite_chan->current_link++;
-			if(mite_chan->current_link >= mite_chan->n_links){
-				mite_chan->current_link=0;
-			}
-		}
-		if(m_status & CHSR_DONE){
-			writel(CHOR_CLRDONE, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
 		}
 		if(m_status & ~(CHSR_INT | CHSR_LINKC | CHSR_DONE | CHSR_DRDY | CHSR_DRQ1 | CHSR_MRDY)){
 			DPRINTK("unknown mite interrupt, disabling IRQ\n");
-			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
+			async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 			disable_irq(dev->irq);
 		}
-		async->events |= COMEDI_CB_BLOCK;
 	}
+	comedi_spin_unlock_irqrestore(&devpriv->mite_channel_lock, irq_flags);
 
 	while(status&DataLeft){
 		work++;
@@ -529,32 +573,20 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 			async->events |= COMEDI_CB_EOA;
 
 			writeb(0x00,devpriv->mite->daq_io_addr+OpMode);
-			writeb(0x00,devpriv->mite->daq_io_addr+Master_DMA_And_Interrupt_Control);
-#ifdef USE_DMA
-			mite_dma_disarm(mite, DI_DMA_CHAN);
-			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
-#endif
 			break;
 		}else if(flags & Waited){
 			DPRINTK("Waited\n");
 			writeb(ClearWaited,devpriv->mite->daq_io_addr+Group_1_First_Clear);
-			writeb(0x00,devpriv->mite->daq_io_addr+Master_DMA_And_Interrupt_Control);
 			async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
-#ifdef USE_DMA
-			mite_dma_disarm(mite, DI_DMA_CHAN);
-			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
-#endif
 			break;
 		}else if(flags & PrimaryTC){
 			DPRINTK("PrimaryTC\n");
 			writeb(ClearPrimaryTC,devpriv->mite->daq_io_addr+Group_1_First_Clear);
 			async->events |= COMEDI_CB_EOA;
-			writeb(0x00,devpriv->mite->daq_io_addr+Master_DMA_And_Interrupt_Control);
 		}else if(flags & SecondaryTC){
 			DPRINTK("SecondaryTC\n");
 			writeb(ClearSecondaryTC,devpriv->mite->daq_io_addr+Group_1_First_Clear);
 			async->events |= COMEDI_CB_EOA;
-			writeb(0x00,devpriv->mite->daq_io_addr+Master_DMA_And_Interrupt_Control);
 		}
 #if 0
 		else{
@@ -572,8 +604,7 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 	}
 
 out:
-	comedi_event(dev,s,async->events);
-
+	ni_pcidio_event(dev, s);
 #if 0
 	if(!tag){
 		writeb(0x03,devpriv->mite->daq_io_addr+Master_DMA_And_Interrupt_Control);
@@ -583,7 +614,7 @@ out:
 }
 
 #ifdef DEBUG_FLAGS
-static char *flags_strings[] = {
+static const char * const flags_strings[] = {
 	"TransferReady", "CountExpired", "2", "3",
 	"4", "Waited", "PrimaryTC", "SecondaryTC",
 };
@@ -705,7 +736,7 @@ static int ni_pcidio_cmdtest(comedi_device *dev,comedi_subdevice *s,
 	if(!cmd->convert_src || tmp!=cmd->convert_src)err++;
 
 	tmp=cmd->scan_end_src;
-	cmd->scan_end_src &= TRIG_COUNT;  
+	cmd->scan_end_src &= TRIG_COUNT;
 	if(!cmd->scan_end_src || tmp!=cmd->scan_end_src)err++;
 
 	tmp=cmd->stop_src;
@@ -770,7 +801,7 @@ static int ni_pcidio_cmdtest(comedi_device *dev,comedi_subdevice *s,
 	if(err)return 3;
 
 	/* step 4: fix up any arguments */
-	
+
 	if(cmd->scan_begin_src == TRIG_TIMER){
 		tmp = cmd->scan_begin_arg;
 		ni_pcidio_ns_to_timer(&cmd->scan_begin_arg,
@@ -807,7 +838,6 @@ static int ni_pcidio_ns_to_timer(int *nanosec, int round_mode)
 	return divider;
 }
 
-
 static int ni_pcidio_cmd(comedi_device *dev,comedi_subdevice *s)
 {
 	comedi_cmd *cmd = &s->async->cmd;
@@ -876,14 +906,18 @@ static int ni_pcidio_cmd(comedi_device *dev,comedi_subdevice *s)
 	}
 
 #ifdef USE_DMA
-	writeb(0x05,devpriv->mite->daq_io_addr+DMA_Line_Control);
-	writeb(0x30,devpriv->mite->daq_io_addr+Group_1_First_Clear);
+	writeb(ClearPrimaryTC | ClearSecondaryTC,
+		devpriv->mite->daq_io_addr + Group_1_First_Clear);
 
-	setup_mite_dma(dev,s);
+	{
+		int retval = setup_mite_dma(dev,s);
+		if(retval) return retval;
+	}
 #else
-	writeb(0x00,devpriv->mite->daq_io_addr+DMA_Line_Control);
+	writeb(0x00, devpriv->mite->daq_io_addr + DMA_Line_Control_Group1);
 #endif
-  
+	writeb(0x00, devpriv->mite->daq_io_addr + DMA_Line_Control_Group2);
+
 	/* clear and enable interrupts */
 	writeb(0xff,devpriv->mite->daq_io_addr+Group_1_First_Clear);
 	//writeb(ClearExpired,devpriv->mite->daq_io_addr+Group_1_Second_Clear);
@@ -909,19 +943,19 @@ static int ni_pcidio_cmd(comedi_device *dev,comedi_subdevice *s)
 	return 0;
 }
 
-
-static void setup_mite_dma(comedi_device *dev,comedi_subdevice *s)
+static int setup_mite_dma(comedi_device *dev,comedi_subdevice *s)
 {
-	struct mite_struct *mite = devpriv->mite;
-	struct mite_channel *mite_chan = &mite->channels[ DI_DMA_CHAN ];
+	int retval;
 
-	mite_chan->current_link = 0;
+	retval = ni_pcidio_request_di_mite_channel(dev);
+	if(retval) return retval;
 
-	mite_chan->dir = COMEDI_INPUT;
+	devpriv->di_mite_chan->dir = COMEDI_INPUT;
 
-	mite_prep_dma(mite, DI_DMA_CHAN, 32, 32);
+	mite_prep_dma(devpriv->di_mite_chan, 32, 32);
 
-	mite_dma_arm(mite, DI_DMA_CHAN);
+	mite_dma_arm(devpriv->di_mite_chan);
+	return 0;
 }
 
 
@@ -939,7 +973,8 @@ static int ni_pcidio_inttrig(comedi_device *dev, comedi_subdevice *s,
 
 static int ni_pcidio_cancel(comedi_device *dev, comedi_subdevice *s)
 {
-	writeb(0x00,devpriv->mite->daq_io_addr+Master_DMA_And_Interrupt_Control);
+	writeb(0x00, devpriv->mite->daq_io_addr + Master_DMA_And_Interrupt_Control);
+	ni_pcidio_release_di_mite_channel(dev);
 
 	return 0;
 }
@@ -950,7 +985,7 @@ static int ni_pcidio_change(comedi_device *dev, comedi_subdevice *s,
 {
 	int ret;
 
-	ret = mite_buf_change(devpriv->mite, DI_DMA_CHAN, s->async, new_size);
+	ret = mite_buf_change(devpriv->di_mite_ring, s->async);
 	if(ret<0)return ret;
 
 	memset(s->async->prealloc_buf, 0xaa, s->async->prealloc_bufsz);
@@ -1023,7 +1058,7 @@ static int pci_6534_reset_fpgas(comedi_device *dev)
 	return ret;
 }
 
-static void pci_6534_init_main_fpga(comedi_device *dev)	
+static void pci_6534_init_main_fpga(comedi_device *dev)
 {
 	writel(0, devpriv->mite->daq_io_addr + FPGA_Control1_Register);
 	writel(0, devpriv->mite->daq_io_addr + FPGA_Control2_Register);
@@ -1038,7 +1073,7 @@ static int pci_6534_upload_firmware(comedi_device *dev, int options[])
 	int ret;
 	void *main_fpga_data, *scarab_a_data, *scarab_b_data;
 	int main_fpga_data_len, scarab_a_data_len, scarab_b_data_len;
-	
+
 	if(options[COMEDI_DEVCONF_AUX_DATA_LENGTH] == 0) return 0;
 	ret = pci_6534_reset_fpgas(dev);
 	if(ret < 0) return ret;
@@ -1048,13 +1083,13 @@ static int pci_6534_upload_firmware(comedi_device *dev, int options[])
 	if(ret < 0) return ret;
 	pci_6534_init_main_fpga(dev);
 	scarab_a_data = comedi_aux_data(options, 1);
-	scarab_a_data_len = options[COMEDI_DEVCONF_AUX_DATA1_LENGTH];	
+	scarab_a_data_len = options[COMEDI_DEVCONF_AUX_DATA1_LENGTH];
 	ret = pci_6534_load_fpga(dev, 0, scarab_a_data, scarab_a_data_len);
 	if(ret < 0) return ret;
 	scarab_b_data = comedi_aux_data(options, 2);
-	scarab_b_data_len = options[COMEDI_DEVCONF_AUX_DATA2_LENGTH];	
+	scarab_b_data_len = options[COMEDI_DEVCONF_AUX_DATA2_LENGTH];
 	ret = pci_6534_load_fpga(dev, 1, scarab_b_data, scarab_b_data_len);
-	if(ret < 0) return ret;	
+	if(ret < 0) return ret;
 	return 0;
 }
 
@@ -1070,6 +1105,7 @@ static int nidio_attach(comedi_device *dev,comedi_devconfig *it)
 
 	if((ret=alloc_private(dev,sizeof(nidio96_private)))<0)
 		return ret;
+	spin_lock_init(&devpriv->mite_channel_lock);
 
 	ret=nidio_find_device(dev,it->options[0],it->options[1]);
 	if(ret<0)return ret;
@@ -1080,6 +1116,9 @@ static int nidio_attach(comedi_device *dev,comedi_devconfig *it)
 		printk("error setting up mite\n");
 		return ret;
 	}
+	comedi_set_hw_dev(dev, &devpriv->mite->pcidev->dev);
+	devpriv->di_mite_ring = mite_alloc_ring(devpriv->mite);
+	if(devpriv->di_mite_ring == NULL) return -ENOMEM;
 
 	dev->board_name=this_board->name;
 	irq=mite_irq(devpriv->mite);
@@ -1111,17 +1150,18 @@ static int nidio_attach(comedi_device *dev,comedi_devconfig *it)
 
 		dev->read_subdev = s;
 		s->type=COMEDI_SUBD_DIO;
-		s->subdev_flags=SDF_READABLE|SDF_WRITABLE|SDF_LSAMPL;
+		s->subdev_flags = SDF_READABLE | SDF_WRITABLE | SDF_LSAMPL | SDF_PACKED | SDF_CMD_READ;
 		s->n_chan=32;
 		s->range_table=&range_digital;
 		s->maxdata=1;
-		s->insn_config = ni_pcidio_insn_config;
-		s->insn_bits = ni_pcidio_insn_bits;
-		s->do_cmd = ni_pcidio_cmd;
-		s->do_cmdtest = ni_pcidio_cmdtest;
-		s->cancel = ni_pcidio_cancel;
+		s->insn_config = &ni_pcidio_insn_config;
+		s->insn_bits = &ni_pcidio_insn_bits;
+		s->do_cmd = &ni_pcidio_cmd;
+		s->do_cmdtest = &ni_pcidio_cmdtest;
+		s->cancel = &ni_pcidio_cancel;
 		s->len_chanlist=32;		/* XXX */
-		s->buf_change = ni_pcidio_change;
+		s->buf_change = &ni_pcidio_change;
+		s->async_dma_dir = DMA_BIDIRECTIONAL;
 
 		writel(0,devpriv->mite->daq_io_addr+Port_IO(0));
 		writel(0,devpriv->mite->daq_io_addr+Port_Pin_Directions(0));
@@ -1130,7 +1170,7 @@ static int nidio_attach(comedi_device *dev,comedi_devconfig *it)
 		/* disable interrupts on board */
 		writeb(0x00,devpriv->mite->daq_io_addr+Master_DMA_And_Interrupt_Control);
 
-		ret=comedi_request_irq(irq,nidio_interrupt,SA_SHIRQ,"ni_pcidio",dev);
+		ret=comedi_request_irq(irq,nidio_interrupt,IRQF_SHARED,"ni_pcidio",dev);
 		if(ret<0){
 			printk(" irq not available");
 		}
@@ -1155,9 +1195,16 @@ static int nidio_detach(comedi_device *dev)
 	if(dev->irq)
 		comedi_free_irq(dev->irq,dev);
 
-	if(devpriv && devpriv->mite)
-		mite_unsetup(devpriv->mite);
-
+	if(devpriv)
+	{
+		if(devpriv->di_mite_ring)
+		{
+			mite_free_ring(devpriv->di_mite_ring);
+			devpriv->di_mite_ring = NULL;
+		}
+		if(devpriv->mite)
+			mite_unsetup(devpriv->mite);
+	}
 	return 0;
 }
 
@@ -1166,7 +1213,7 @@ static int nidio_find_device(comedi_device *dev,int bus,int slot)
 {
 	struct mite_struct *mite;
 	int i;
-	
+
 	for(mite=mite_devices;mite;mite=mite->next){
 		if(mite->used)continue;
 		if(bus || slot){
