@@ -1,5 +1,5 @@
 /*
-    module/ni_pcimio.c
+    comedi/drivers/ni_pcimio.c
     Hardware driver for NI PCI-MIO E series cards
 
     COMEDI - Linux Control and Measurement Device Interface
@@ -20,30 +20,36 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /*
-Driver: ni_pcimio.o
-Description: National Instruments PCI-MIO-E series (all boards)
+Driver: ni_pcimio
+Description: National Instruments PCI-MIO-E series and M series (all boards)
 Author: ds, John Hallen, Frank Mori Hess, Rolf Mueller, Herbert Peremans,
   Herman Bruyninckx, Terry Barnaby
 Status: works
 Devices: [National Instruments] PCI-MIO-16XE-50 (ni_pcimio),
   PCI-MIO-16XE-10, PXI-6030E, PCI-MIO-16E-1, PCI-MIO-16E-4, PCI-6014, PCI-6040E,
-  PXI-6040E, PCI-6031E, PCI-6032E, PCI-6033E, PCI-6071E, PCI-6023E,
+  PXI-6040E, PCI-6030E, PCI-6031E, PCI-6032E, PCI-6033E, PCI-6071E, PCI-6023E,
   PCI-6024E, PCI-6025E, PXI-6025E, PCI-6034E, PCI-6035E, PCI-6052E,
   PCI-6110, PCI-6111, PCI-6220, PCI-6221, PCI-6224, PCI-6225, PCI-6229,
-  PCI-6250, PCI-6251, PCI-6254, PCI-6259,
-  PCI-6280, PCI-6281, PCI-6284, PCI-6289,
+  PCI-6250, PCI-6251, PCIe-6251, PCI-6254, PCI-6259, PCIe-6259,
+  PCI-6280, PCI-6281, PXI-6281, PCI-6284, PCI-6289,
   PCI-6711, PXI-6711, PCI-6713, PXI-6713,
-  PXI-6071E, PXI-6070E,
-  PXI-6052E, PCI-6036E, PCI-6731, PCI-6733, PXI-6733
-  PCI-6143
-Updated: Tue Mar 21 17:40:10 EST 2006
+  PXI-6071E, PCI-6070E, PXI-6070E,
+  PXI-6052E, PCI-6036E, PCI-6731, PCI-6733, PXI-6733,
+  PCI-6143, PXI-6143
+Updated: Wed Nov 29 10:30:36 EST 2006
 
 These boards are almost identical to the AT-MIO E series, except that
 they use the PCI bus instead of ISA (i.e., AT).  See the notes for
 the ni_atmio.o driver for additional information about these boards.
 
 Autocalibration is supported on many of the devices, using the
-calibration utility in Comedilib.
+comedi_calibrate (or comedi_soft_calibrate for m-series) utility.
+M-Series boards do analog input and analog output calibration entirely
+in software. The software calibration corrects
+the analog input for offset, gain and
+nonlinearity.  The analog outputs are corrected for offset and gain.
+See the comedilib documentation on comedi_get_softcal_converter() for
+more information.
 
 By default, the driver uses DMA to transfer analog input data to
 memory.  When DMA is enabled, not all triggering features are
@@ -56,19 +62,18 @@ With this board all of the convertors perform one simultaineous sample during
 a scan interval. The period for a scan is used for the convert time in a
 Comedi cmd. The convert trigger source is normally set to TRIG_NOW by default.
 
-Basic support for the RTSI trigger bus is supported on these cards on
+The RTSI trigger bus is supported on these cards on
 subdevice 10. See the comedilib documentation for details.
 
 Information (number of channels, bits, etc.) for some devices may be
 incorrect.  Please check this and submit a bug if there are problems
 for your device.
 
-The support for the M-Series boards is still under development.
+SCXI is probably broken for m-series boards.
 
 Bugs:
- - When DMA is enabled, COMEDI_EV_SCAN_END and COMEDI_EV_CONVERT do
+ - When DMA is enabled, COMEDI_EV_CONVERT does
    not work correctly.
- - There are reported problems with the 67xx boards.
 
 */
 /*
@@ -166,10 +171,14 @@ static struct pci_device_id ni_pci_table[] __devinitdata = {
 	{ PCI_VENDOR_ID_NATINST, 0x70b8, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ PCI_VENDOR_ID_NATINST, 0x70bc, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ PCI_VENDOR_ID_NATINST, 0x70bd, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x70bf, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x70c0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ PCI_VENDOR_ID_NATINST, 0x70f2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x710d, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ PCI_VENDOR_ID_NATINST, 0x716c, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x717f, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ PCI_VENDOR_ID_NATINST, 0x71bc, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ PCI_VENDOR_ID_NATINST, 0x70C0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x717d, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ 0 }
 };
 MODULE_DEVICE_TABLE(pci, ni_pci_table);
@@ -181,7 +190,7 @@ MODULE_DEVICE_TABLE(pci, ni_pci_table);
  63 different possibilities.  An AO channel
  can not act as it's own OFFSET or REFERENCE.
 */
-static comedi_lrange range_ni_M_628x_ao = { 8, {
+static const comedi_lrange range_ni_M_628x_ao = { 8, {
 	RANGE(-10, 10),
 	RANGE(-5, 5),
 	RANGE(-2, 2),
@@ -192,16 +201,16 @@ static comedi_lrange range_ni_M_628x_ao = { 8, {
 	RANGE(4, 6),
 	RANGE_ext(-1, 1)
 }};
-static comedi_lrange range_ni_M_625x_ao = { 3, {
+static const comedi_lrange range_ni_M_625x_ao = { 3, {
 	RANGE(-10, 10),
 	RANGE(-5, 5),
 	RANGE_ext(-1, 1)
 }};
-static comedi_lrange range_ni_M_622x_ao = { 1, {
+static const comedi_lrange range_ni_M_622x_ao = { 1, {
 	RANGE(-10, 10),
 }};
 
-static ni_board ni_boards[]={
+static const ni_board ni_boards[]={
 	{       device_id:      0x0162, // NI also says 0x1620.  typo?
 		name:           "pci-mio-16xe-50",
 		n_adchan:       16,
@@ -215,11 +224,13 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  0,
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
+		ao_speed:	50000,
+		.num_p0_dio_channels = 8,
 		caldac:         {dac8800,dac8043},
 		has_8255:       0,
 	},
 	{       device_id:      0x1170,
-		name:           "pci-mio-16xe-10",
+		name:           "pci-mio-16xe-10", // aka pci-6030E
 		n_adchan:       16,
 		adbits:         16,
 		ai_fifo_depth:  512,
@@ -231,6 +242,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	10000,
+		.num_p0_dio_channels = 8,
 		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
@@ -247,6 +260,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  0,
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
+		ao_speed:	100000,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug},
 		has_8255:       0,
 	},
@@ -263,12 +278,14 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	10000,
+		.num_p0_dio_channels = 8,
 		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
 
 	{       device_id:      0x1180,
-		name:           "pci-mio-16e-1",
+		name:           "pci-mio-16e-1",	/* aka pci-6070e */
 		n_adchan:       16,
 		adbits:         12,
 		ai_fifo_depth:  512,
@@ -280,6 +297,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		caldac:         {mb88341},
 		has_8255:       0,
 	},
@@ -298,6 +317,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  512,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug}, // doc says mb88341
 		has_8255:       0,
 	},
@@ -314,6 +335,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  512,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		caldac:         {mb88341},
 		has_8255:       0,
 	},
@@ -331,6 +354,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	10000,
+		.num_p0_dio_channels = 8,
 		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
@@ -346,6 +371,7 @@ static ni_board ni_boards[]={
 		aobits:         0,
 		ao_fifo_depth:  0,
 		ao_unipolar:    0,
+		.num_p0_dio_channels = 8,
 		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
@@ -361,6 +387,7 @@ static ni_board ni_boards[]={
 		aobits:         0,
 		ao_fifo_depth:  0,
 		ao_unipolar:    0,
+		.num_p0_dio_channels = 8,
 		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
@@ -377,6 +404,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug},
 		has_8255:       0,
 	},
@@ -391,7 +420,8 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_unipolar:    0,
-		caldac:         {ad8804}, /* manual is wrong */
+		.num_p0_dio_channels = 8,
+		caldac:         {ad8804_debug}, /* manual is wrong */
 		has_8255:	0,
 	},
 	{       device_id:      0x2a70,
@@ -407,7 +437,9 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  0,
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
-		caldac:         {ad8804}, /* manual is wrong */
+		ao_speed:	100000,
+		.num_p0_dio_channels = 8,
+		caldac:         {ad8804_debug}, /* manual is wrong */
 		has_8255:	0,
 	},
 	{       device_id:      0x2a80,
@@ -423,7 +455,9 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  0,
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
-		caldac:         {ad8804}, /* manual is wrong */
+		ao_speed:	100000,
+		.num_p0_dio_channels = 8,
+		caldac:         {ad8804_debug}, /* manual is wrong */
 		has_8255:	1,
 	},
 	{       device_id:      0x2ab0,
@@ -439,7 +473,9 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  0,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
-		caldac:         {ad8804}, /* manual is wrong */
+		ao_speed:	100000,
+		.num_p0_dio_channels = 8,
+		caldac:         {ad8804_debug}, /* manual is wrong */
 		has_8255:	1,
 	},
 
@@ -455,6 +491,7 @@ static ni_board ni_boards[]={
 		aobits:         0,
 		ao_fifo_depth:  0,
 		ao_unipolar:    0,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug},
 		has_8255:	0,
 	},
@@ -471,6 +508,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  0,
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
+		ao_speed:	100000,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug},
 		has_8255:	0,
 	},
@@ -487,25 +526,10 @@ static ni_board ni_boards[]={
 		ao_unipolar:    1,
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
+		ao_speed:	3000,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug,ad8804_debug,ad8522}, /* manual is wrong */
 	},
-#if 0
-	{       device_id:      0x0000, /* unknown */
-		name:           "pci-6053e",
-		n_adchan:       64,
-		adbits:         16,
-		ai_fifo_depth:  512,
-		alwaysdither:   1,
-		gainlkup:       ai_gain_16,
-		ai_speed:	3000,
-		n_aochan:       2,
-		aobits:         16,
-		ao_unipolar:    1,
-		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
-		caldac:         {ad8804,mb88341,ad8522}, /* manual is wrong */
-	},
-#endif
 	{       device_id:      0x14e0,
 		name:           "pci-6110",
 		n_adchan:       4,
@@ -520,6 +544,8 @@ static ni_board ni_boards[]={
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
+		ao_speed:	250,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804,ad8804},
 	},
 	{       device_id:      0x14f0,
@@ -536,6 +562,8 @@ static ni_board ni_boards[]={
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
+		ao_speed:	250,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804,ad8804},
 	},
 #if 0
@@ -553,6 +581,8 @@ static ni_board ni_boards[]={
 		ao_671x:	1,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
+		ao_speed:	250,
+		.num_p0_dio_channels = 8,
 		reg_611x:	1,
 		caldac:         {ad8804_debug,ad8804_debug,ad8804_debug},/* XXX */
 	},
@@ -571,7 +601,9 @@ static ni_board ni_boards[]={
 		ao_671x:	1,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
+		ao_speed:	250,
 		reg_611x:	1,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug,ad8804_debug,ad8804_debug},/* XXX */
 	},
 #endif
@@ -583,6 +615,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384, /* data sheet says 8192, but fifo really holds 16384 samples */
 		.ao_range_table = &range_bipolar10,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
 		caldac:         {ad8804_debug},
 	},
@@ -594,6 +628,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
 		.ao_range_table = &range_bipolar10,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
 		caldac:         {ad8804_debug},
 	},
@@ -605,6 +641,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
 		.ao_range_table = &range_bipolar10,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
 		caldac:         {ad8804_debug,ad8804_debug},
 	},
@@ -616,6 +654,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
 		.ao_range_table = &range_bipolar10,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
 		caldac:         {ad8804_debug,ad8804_debug},
 	},
@@ -627,6 +667,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  8192,
 		.ao_range_table = &range_bipolar10,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
 		caldac:         {ad8804_debug},
 	},
@@ -639,6 +681,7 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  8192,
 		.ao_range_table = &range_bipolar10,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
 		caldac:         {ad8804_debug},
 	},
@@ -651,6 +694,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
 		.ao_range_table = &range_bipolar10,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
 		caldac:         {ad8804_debug,ad8804_debug},
 	},
@@ -662,6 +707,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
 		.ao_range_table = &range_bipolar10,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
 		caldac:         {ad8804_debug,ad8804_debug},
 	},
@@ -678,6 +725,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug},
 		has_8255:       0,
 	},
@@ -694,7 +743,9 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
-		caldac:         {mb88341},
+		ao_speed:	1000,
+		.num_p0_dio_channels = 8,
+		caldac:         {ad8804_debug},
 		has_8255:       0,
 	},
  	{	device_id:      0x18c0,
@@ -710,6 +761,8 @@ static ni_board ni_boards[]={
 		ao_unipolar:    1,
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
+		ao_speed:	3000,
+		.num_p0_dio_channels = 8,
 		caldac:         {mb88341,mb88341,ad8522},
 	},
 	{	device_id:      0x1580,
@@ -725,6 +778,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  2048,
 		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:    1,
+		ao_speed:	10000,
+		.num_p0_dio_channels = 8,
 		caldac:         {dac8800,dac8043,ad8522},
 	},
 	{	device_id:      0x2890,
@@ -740,6 +795,8 @@ static ni_board ni_boards[]={
 		ao_fifo_depth:  0,
 		.ao_range_table = &range_bipolar10,
 		ao_unipolar:    0,
+		ao_speed:	100000,
+		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug},
 		has_8255:	0,
 	},
@@ -753,7 +810,8 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_fifo_depth:  0,
-		reg_type:	ni_reg_m_series,
+		.num_p0_dio_channels = 8,
+		reg_type:	ni_reg_622x,
 		ao_unipolar:    0,
 		.caldac = {caldac_none},
 		has_8255:	0,
@@ -769,9 +827,10 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_622x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_622x,
 		ao_unipolar:    0,
-// 		ao_speed:	1200,
+		ao_speed:	1200,
+		.num_p0_dio_channels = 8,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -786,9 +845,10 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_622x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_622x,
 		ao_unipolar:    0,
-// 		ao_speed:	1200,
+		ao_speed:	1200,
+		.num_p0_dio_channels = 8,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -802,9 +862,9 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_fifo_depth:  0,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_622x,
 		ao_unipolar:    0,
-// 		ao_speed:	0,
+		.num_p0_dio_channels = 32,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -819,9 +879,10 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_622x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_622x,
 		ao_unipolar:    0,
-// 		ao_speed:	1200,
+		ao_speed:	1200,
+		.num_p0_dio_channels = 32,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -836,9 +897,10 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_622x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_622x,
 		ao_unipolar:    0,
-// 		ao_speed:	1200,
+		ao_speed:	1200,
+		.num_p0_dio_channels = 32,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -852,8 +914,9 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_fifo_depth:  0,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_625x,
 		ao_unipolar:    0,
+		.num_p0_dio_channels = 8,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -868,9 +931,28 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_625x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_625x,
 		ao_unipolar:    0,
-// 		ao_speed:	357,
+		ao_speed:	357,
+		.num_p0_dio_channels = 8,
+		.caldac = {caldac_none},
+		has_8255:	0,
+	},
+	{	device_id:      0x717d,
+		name:           "pcie-6251",
+		n_adchan:       16,
+		adbits:         16,
+		ai_fifo_depth:  4095,
+		.gainlkup = ai_gain_628x,
+		ai_speed:	800,
+		n_aochan:       2,
+		aobits:         16,
+		ao_fifo_depth:  8191,
+		.ao_range_table = &range_ni_M_625x_ao,
+		reg_type:	ni_reg_625x,
+		ao_unipolar:    0,
+		ao_speed:	357,
+		.num_p0_dio_channels = 8,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -884,8 +966,9 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_fifo_depth:  0,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_625x,
 		ao_unipolar:    0,
+		.num_p0_dio_channels = 32,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -900,9 +983,28 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_625x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_625x,
 		ao_unipolar:    0,
-// 		ao_speed:	357,
+		ao_speed:	357,
+		.num_p0_dio_channels = 32,
+		.caldac = {caldac_none},
+		has_8255:	0,
+	},
+	{	device_id:      0x717f,
+		name:           "pcie-6259",
+		n_adchan:       32,
+		adbits:         16,
+		ai_fifo_depth:  4095,
+		.gainlkup = ai_gain_628x,
+		ai_speed:	800,
+		n_aochan:       4,
+		aobits:         16,
+		ao_fifo_depth:  8191,
+		.ao_range_table = &range_ni_M_625x_ao,
+		reg_type:	ni_reg_625x,
+		ao_unipolar:    0,
+		ao_speed:	357,
+		.num_p0_dio_channels = 32,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -916,8 +1018,9 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_fifo_depth:  8191,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_628x,
 		ao_unipolar:    0,
+		.num_p0_dio_channels = 8,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -932,9 +1035,28 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_628x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_628x,
 		ao_unipolar:    1,
-// 		ao_speed:	357,
+		ao_speed:	357,
+		.num_p0_dio_channels = 8,
+		.caldac = {caldac_none},
+		has_8255:	0,
+	},
+	{	device_id:      0x70bf,
+		name:           "pxi-6281",
+		n_adchan:       16,
+		adbits:         18,
+		ai_fifo_depth:  2047,
+		.gainlkup = ai_gain_628x,
+		ai_speed:	1600,
+		n_aochan:       2,
+		aobits:         16,
+		ao_fifo_depth:  8191,
+		.ao_range_table = &range_ni_M_628x_ao,
+		reg_type:	ni_reg_628x,
+		ao_unipolar:    1,
+		ao_speed:	357,
+		.num_p0_dio_channels = 8,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -948,8 +1070,9 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_fifo_depth:  0,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_628x,
 		ao_unipolar:    0,
+		.num_p0_dio_channels = 32,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
@@ -964,13 +1087,14 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  8191,
 		.ao_range_table = &range_ni_M_628x_ao,
-		reg_type:	ni_reg_m_series,
+		reg_type:	ni_reg_628x,
 		ao_unipolar:    1,
-// 		ao_speed:	357,
+		ao_speed:	357,
+		.num_p0_dio_channels = 32,
 		.caldac = {caldac_none},
 		has_8255:	0,
 	},
-	{       device_id:      0x70C0,
+	{	device_id:      0x70C0,
 		name:           "pci-6143",
 		n_adchan:       8,
 		adbits:         16,
@@ -983,7 +1107,24 @@ static ni_board ni_boards[]={
 		reg_type:	ni_reg_6143,
 		ao_unipolar:    0,
 		ao_fifo_depth:  0,
-		caldac:         {ad8804,ad8804},
+		.num_p0_dio_channels = 8,
+		.caldac = {ad8804_debug,ad8804_debug},
+	},
+	{	device_id:      0x710D,
+		name:           "pxi-6143",
+		n_adchan:       8,
+		adbits:         16,
+		ai_fifo_depth:  1024,
+		alwaysdither:   0,
+		gainlkup:       ai_gain_6143,
+		ai_speed:	4000,
+		n_aochan:       0,
+		aobits:         0,
+		reg_type:	ni_reg_6143,
+		ao_unipolar:    0,
+		ao_fifo_depth:  0,
+		.num_p0_dio_channels = 8,
+		.caldac = {ad8804_debug,ad8804_debug},
 	},
 };
 #define n_pcimio_boards ((sizeof(ni_boards)/sizeof(ni_boards[0])))
@@ -999,11 +1140,7 @@ static comedi_driver driver_pcimio={
 COMEDI_INITCLEANUP(driver_pcimio);
 
 typedef struct{
-	struct mite_struct *mite;
-
 	NI_PRIVATE_COMMON
-
-	unsigned int last_buf_write_count;
 }ni_private;
 #define devpriv ((ni_private *)dev->private)
 
@@ -1094,6 +1231,9 @@ static void m_series_stc_writew(comedi_device *dev, uint16_t data, int reg)
 	case AI_Trigger_Select_Register:
 		offset = M_Offset_AI_Trigger_Select;
 		break;
+	case Analog_Trigger_Etc_Register:
+		offset = M_Offset_Analog_Trigger_Etc;
+		break;
 	case AO_Command_1_Register:
 		offset = M_Offset_AO_Command_1;
 		break;
@@ -1130,24 +1270,34 @@ static void m_series_stc_writew(comedi_device *dev, uint16_t data, int reg)
 	case DAC_FIFO_Clear:
 		offset = M_Offset_AO_FIFO_Clear;
 		break;
+	case DIO_Control_Register:
+		rt_printk("%s: FIXME: register 0x%x does not map cleanly on to m-series boards.\n", __FUNCTION__, reg);
+		return;
+		break;
 	case G_Autoincrement_Register(0):
 		offset = M_Offset_G0_Autoincrement;
 		break;
 	case G_Autoincrement_Register(1):
 		offset = M_Offset_G1_Autoincrement;
 		break;
+	case G_Command_Register(0):
+		offset = M_Offset_G0_Command;
+		break;
+	case G_Command_Register(1):
+		offset = M_Offset_G1_Command;
+		break;
 	case G_Input_Select_Register(0):
 		offset = M_Offset_G0_Input_Select;
-		break;	
+		break;
 	case G_Input_Select_Register(1):
 		offset = M_Offset_G1_Input_Select;
-		break;	
+		break;
 	case G_Mode_Register(0):
 		offset = M_Offset_G0_Mode;
-		break;	
+		break;
 	case G_Mode_Register(1):
 		offset = M_Offset_G1_Mode;
-		break;	
+		break;
 	case Interrupt_A_Ack_Register:
 		offset = M_Offset_Interrupt_A_Ack;
 		break;
@@ -1169,17 +1319,20 @@ static void m_series_stc_writew(comedi_device *dev, uint16_t data, int reg)
 	case Joint_Reset_Register:
 		offset = M_Offset_Joint_Reset;
 		break;
-	case Analog_Trigger_Etc_Register:
-		offset = M_Offset_Analog_Trigger_Etc;
+	case RTSI_Trig_A_Output_Register:
+		offset = M_Offset_RTSI_Trig_A_Output;
 		break;
-	case DIO_Control_Register:
-		rt_printk("%s: FIXME: register 0x%x does not map cleanly on to m-series boards.\n", __FUNCTION__, reg);
-		return;
+	case RTSI_Trig_B_Output_Register:
+		offset = M_Offset_RTSI_Trig_B_Output;
+		break;
+	case RTSI_Trig_Direction_Register:
+		offset = M_Offset_RTSI_Trig_Direction;
 		break;
 	/* FIXME: DIO_Output_Register (16 bit reg) is replaced by M_Offset_Static_Digital_Output (32 bit)
 	 and M_Offset_SCXI_Serial_Data_Out (8 bit) */
 	default:
 		rt_printk("%s: bug! unhandled register=0x%x in switch.\n", __FUNCTION__, reg);
+		BUG();
 		return;
 		break;
 	}
@@ -1209,9 +1362,12 @@ static uint16_t m_series_stc_readw(comedi_device *dev, int reg)
 	case Joint_Status_2_Register:
 		offset = M_Offset_Joint_Status_2;
 		break;
-	/* FIXME: DIO_Parallel_Input_Register (16 bit reg) is replaced by M_Offset_Static_Digital_Input (32 bit) */
+	case G_Status_Register:
+		offset = M_Offset_G01_Status;
+		break;
 	default:
 		rt_printk("%s: bug! unhandled register=0x%x in switch.\n", __FUNCTION__, reg);
+		BUG();
 		return 0;
 		break;
 	}
@@ -1238,18 +1394,57 @@ static void m_series_stc_writel(comedi_device *dev, uint32_t data, int reg)
 	case AO_UI_Load_A_Register:
 		offset = M_Offset_AO_UI_Load_A;
 		break;
+	case G_Load_A_Register(0):
+		offset = M_Offset_G0_Load_A;
+		break;
+	case G_Load_A_Register(1):
+		offset = M_Offset_G1_Load_A;
+		break;
+	case G_Load_B_Register(0):
+		offset = M_Offset_G0_Load_B;
+		break;
+	case G_Load_B_Register(1):
+		offset = M_Offset_G1_Load_B;
+		break;
 	default:
 		rt_printk("%s: bug! unhandled register=0x%x in switch.\n", __FUNCTION__, reg);
+		BUG();
 		return;
 		break;
 	}
 	ni_writel(data, offset);
 }
 
+static uint32_t m_series_stc_readl(comedi_device *dev, int reg)
+{
+	unsigned offset;
+	switch(reg)
+	{
+	case G_HW_Save_Register(0):
+		offset = M_Offset_G0_HW_Save;
+		break;
+	case G_HW_Save_Register(1):
+		offset = M_Offset_G1_HW_Save;
+		break;
+	case G_Save_Register(0):
+		offset = M_Offset_G0_Save;
+		break;
+	case G_Save_Register(1):
+		offset = M_Offset_G1_Save;
+		break;
+	default:
+		rt_printk("%s: bug! unhandled register=0x%x in switch.\n", __FUNCTION__, reg);
+		BUG();
+		return 0;
+		break;
+	}
+	return ni_readl(offset);
+}
+
 #define interrupt_pin(a)	0
 #define IRQ_POLARITY 1
 
-#define NI_E_IRQ_FLAGS		SA_SHIRQ
+#define NI_E_IRQ_FLAGS		IRQF_SHARED
 
 #include "ni_mio_common.c"
 
@@ -1259,6 +1454,12 @@ static int pcimio_ai_change(comedi_device *dev, comedi_subdevice *s,
 	unsigned long new_size);
 static int pcimio_ao_change(comedi_device *dev, comedi_subdevice *s,
 	unsigned long new_size);
+static int pcimio_gpct0_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size);
+static int pcimio_gpct1_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size);
+static int pcimio_dio_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size);
 
 static void m_series_init_eeprom_buffer(comedi_device *dev)
 {
@@ -1268,7 +1469,7 @@ static void m_series_init_eeprom_buffer(comedi_device *dev)
 	unsigned old_iodwbsr1_bits;
 	unsigned old_iodwcr1_bits;
 	int i;
-	
+
 	old_iodwbsr_bits = readl(devpriv->mite->mite_io_addr + MITE_IODWBSR);
 	old_iodwbsr1_bits = readl(devpriv->mite->mite_io_addr + MITE_IODWBSR_1);
 	old_iodwcr1_bits = readl(devpriv->mite->mite_io_addr + MITE_IODWCR_1);
@@ -1276,12 +1477,12 @@ static void m_series_init_eeprom_buffer(comedi_device *dev)
 	writel(((0x80 | window_size) | devpriv->mite->daq_phys_addr), devpriv->mite->mite_io_addr + MITE_IODWBSR_1);
 	writel(0x0, devpriv->mite->mite_io_addr + MITE_IODWCR_1);
 	writel(0xf, devpriv->mite->mite_io_addr + 0x30);
-	
+
 	for(i = 0; i < M_SERIES_EEPROM_SIZE; ++i)
 	{
 		devpriv->eeprom_buffer[i] = ni_readb(Start_Cal_EEPROM + i);
 	}
-    
+
 	writel(old_iodwbsr1_bits, devpriv->mite->mite_io_addr + MITE_IODWBSR_1);
 	writel(old_iodwbsr_bits, devpriv->mite->mite_io_addr + MITE_IODWBSR);
 	writel(old_iodwcr1_bits, devpriv->mite->mite_io_addr + MITE_IODWCR_1);
@@ -1297,7 +1498,7 @@ static void init_6143(comedi_device *dev)
 	ni_writeb(0x00, Magic_6143);		// Set G0,G1 DMA mode to E series version
 	ni_writeb(0x80, PipelineDelay_6143);	// Set EOCMode, ADCMode and pipelinedelay
 	ni_writeb(0x00, EOC_Set_6143);		// Set EOC Delay
-	
+
 	ni_writel(boardtype.ai_fifo_depth / 2, AIFIFO_Flag_6143);	// Set the FIFO half full level
 
 	// Strobe Relay disable bit
@@ -1310,12 +1511,18 @@ static void init_6143(comedi_device *dev)
 static int pcimio_detach(comedi_device *dev)
 {
 	mio_common_detach(dev);
-
-	if(dev->private && devpriv->mite)
-		mite_unsetup(devpriv->mite);
-	
 	if(dev->irq){
-		comedi_free_irq(dev->irq,dev);
+		comedi_free_irq(dev->irq, dev);
+	}
+	if(dev->private)
+	{
+		mite_free_ring(devpriv->ai_mite_ring);
+		mite_free_ring(devpriv->ao_mite_ring);
+		mite_free_ring(devpriv->cdo_mite_ring);
+		mite_free_ring(devpriv->gpct_mite_ring[0]);
+		mite_free_ring(devpriv->gpct_mite_ring[1]);
+		if(devpriv->mite)
+			mite_unsetup(devpriv->mite);
 	}
 
 	return 0;
@@ -1329,23 +1536,25 @@ static int pcimio_attach(comedi_device *dev,comedi_devconfig *it)
 
 	ret=ni_alloc_private(dev);
 	if(ret<0)return ret;
-	
+
 	ret=pcimio_find_device(dev,it->options[0],it->options[1]);
 	if(ret<0)return ret;
 
 	printk(" %s",boardtype.name);
 	dev->board_name=boardtype.name;
-	
-	if(boardtype.reg_type == ni_reg_m_series)
+
+	if(boardtype.reg_type & ni_reg_m_series_mask)
 	{
 		devpriv->stc_writew = &m_series_stc_writew;
 		devpriv->stc_readw = &m_series_stc_readw;
 		devpriv->stc_writel = &m_series_stc_writel;
+		devpriv->stc_readl = &m_series_stc_readl;
 	}else
 	{
 		devpriv->stc_writew = &e_series_win_out;
 		devpriv->stc_readw = &e_series_win_in;
 		devpriv->stc_writel = &win_out2;
+		devpriv->stc_readl = &win_in2;
 	}
 
 	ret = mite_setup(devpriv->mite);
@@ -1354,8 +1563,19 @@ static int pcimio_attach(comedi_device *dev,comedi_devconfig *it)
 		printk(" error setting up mite\n");
 		return ret;
 	}
+	comedi_set_hw_dev(dev, &devpriv->mite->pcidev->dev);
+	devpriv->ai_mite_ring = mite_alloc_ring(devpriv->mite);
+	if(devpriv->ai_mite_ring == NULL) return -ENOMEM;
+	devpriv->ao_mite_ring = mite_alloc_ring(devpriv->mite);
+	if(devpriv->ao_mite_ring == NULL) return -ENOMEM;
+	devpriv->cdo_mite_ring = mite_alloc_ring(devpriv->mite);
+	if(devpriv->cdo_mite_ring == NULL) return -ENOMEM;
+	devpriv->gpct_mite_ring[0] = mite_alloc_ring(devpriv->mite);
+	if(devpriv->gpct_mite_ring[0] == NULL) return -ENOMEM;
+	devpriv->gpct_mite_ring[1] = mite_alloc_ring(devpriv->mite);
+	if(devpriv->gpct_mite_ring[1] == NULL) return -ENOMEM;
 
-	if(boardtype.reg_type == ni_reg_m_series)
+	if(boardtype.reg_type & ni_reg_m_series_mask)
 		m_series_init_eeprom_buffer(dev);
 	if(boardtype.reg_type == ni_reg_6143)
 		init_6143(dev);
@@ -1375,8 +1595,11 @@ static int pcimio_attach(comedi_device *dev,comedi_devconfig *it)
 	ret = ni_E_init(dev,it);
 	if(ret<0)return ret;
 
-	dev->subdevices[0].buf_change = pcimio_ai_change;
-	dev->subdevices[1].buf_change = pcimio_ao_change;
+	dev->subdevices[NI_AI_SUBDEV].buf_change = &pcimio_ai_change;
+	dev->subdevices[NI_AO_SUBDEV].buf_change = &pcimio_ao_change;
+	dev->subdevices[NI_GPCT_SUBDEV(0)].buf_change = &pcimio_gpct0_change;
+	dev->subdevices[NI_GPCT_SUBDEV(1)].buf_change = &pcimio_gpct1_change;
+	dev->subdevices[NI_DIO_SUBDEV].buf_change = &pcimio_dio_change;
 
 	return ret;
 }
@@ -1414,8 +1637,8 @@ static int pcimio_ai_change(comedi_device *dev, comedi_subdevice *s,
 {
 	int ret;
 
-	ret = mite_buf_change(devpriv->mite, AI_DMA_CHAN, s->async, new_size);
-	if(ret<0)return ret;
+	ret = mite_buf_change(devpriv->ai_mite_ring, s->async);
+	if(ret < 0) return ret;
 
 	return 0;
 }
@@ -1425,8 +1648,41 @@ static int pcimio_ao_change(comedi_device *dev, comedi_subdevice *s,
 {
 	int ret;
 
-	ret = mite_buf_change(devpriv->mite, AO_DMA_CHAN, s->async, new_size);
-	if(ret<0)return ret;
+	ret = mite_buf_change(devpriv->ao_mite_ring, s->async);
+	if(ret < 0) return ret;
+
+	return 0;
+}
+
+static int pcimio_gpct0_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size)
+{
+	int ret;
+
+	ret = mite_buf_change(devpriv->gpct_mite_ring[0], s->async);
+	if(ret < 0) return ret;
+
+	return 0;
+}
+
+static int pcimio_gpct1_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size)
+{
+	int ret;
+
+	ret = mite_buf_change(devpriv->gpct_mite_ring[1], s->async);
+	if(ret < 0) return ret;
+
+	return 0;
+}
+
+static int pcimio_dio_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size)
+{
+	int ret;
+
+	ret = mite_buf_change(devpriv->cdo_mite_ring, s->async);
+	if(ret < 0) return ret;
 
 	return 0;
 }
