@@ -10,13 +10,13 @@
  *  card:   ACL-8112DG, ACL-8112HG, ACL-8112PG, ACL-8113, ACL-8216
  *  driver: acl8112dg,  acl8112hg,  acl8112pg,  acl8113,  acl8216
  * and for ICP DAS cards
- *  card:   ISO-813, A-821PGH, A-821PGL, A-821PGL-NDA, A-822PGH, A-822PGL, 
+ *  card:   ISO-813, A-821PGH, A-821PGL, A-821PGL-NDA, A-822PGH, A-822PGL,
  *  driver: iso813,  a821pgh,  a-821pgl, a-821pglnda,  a822pgh,  a822pgl,
- *  card:   A-823PGH, A-823PGL, A-826PG 
+ *  card:   A-823PGH, A-823PGL, A-826PG
  * driver:  a823pgh,  a823pgl,  a826pg
  */
 /*
-Driver: pcl812.o
+Driver: pcl812
 Description: Advantech PCL-812/PG, PCL-813/B,
              ADLink ACL-8112DG/HG/PG, ACL-8113, ACL-8216,
              ICP DAS A-821PGH/PGL/PGL-NDA, A-822PGH/PGL, A-823PGH/PGL, A-826PG,
@@ -28,6 +28,7 @@ Devices: [Advantech] PCL-812 (pcl812), PCL-812PG (pcl812pg),
   [ICP] ISO-813 (iso813), A-821PGH (a821pgh), A-821PGL (a821pgl),
   A-821PGL-NDA (a821pclnda), A-822PGH (a822pgh), A-822PGL (a822pgl),
   A-823PGH (a823pgh), A-823PGL (a823pgl), A-826PG (a826pg)
+Updated: Mon, 06 Aug 2007 12:03:15 +0100
 Status: works (I hope. My board fire up under my hands
                and I cann't test all features.)
 
@@ -35,80 +36,76 @@ This driver supports insn and cmd interfaces. Some boards support only insn
 becouse their hardware don't allow more (PCL-813/B, ACL-8113, ISO-813).
 Data transfer over DMA is supported only when you measure only one
 channel, this is too hardware limitation of these boards.
-See the head of the source file pcl812.c for configuration options.
+
+Options for PCL-812:
+  [0] - IO Base
+  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7; 10, 11, 12, 14, 15)
+  [2] - DMA  (0=disable, 1, 3)
+  [3] - 0=trigger source is internal 8253 with 2MHz clock
+        1=trigger source is external
+  [4] - 0=A/D input range is +/-10V
+        1=A/D input range is +/-5V
+        2=A/D input range is +/-2.5V
+        3=A/D input range is +/-1.25V
+        4=A/D input range is +/-0.625V
+        5=A/D input range is +/-0.3125V
+  [5] - 0=D/A outputs 0-5V  (internal reference -5V)
+        1=D/A outputs 0-10V (internal reference -10V)
+        2=D/A outputs unknow (external reference)
+
+Options for PCL-812PG, ACL-8112PG:
+  [0] - IO Base
+  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7; 10, 11, 12, 14, 15)
+  [2] - DMA  (0=disable, 1, 3)
+  [3] - 0=trigger source is internal 8253 with 2MHz clock
+        1=trigger source is external
+  [4] - 0=A/D have max +/-5V input
+        1=A/D have max +/-10V input
+  [5] - 0=D/A outputs 0-5V  (internal reference -5V)
+        1=D/A outputs 0-10V (internal reference -10V)
+        2=D/A outputs unknow (external reference)
+
+Options for ACL-8112DG/HG, A-822PGL/PGH, A-823PGL/PGH, ACL-8216, A-826PG:
+  [0] - IO Base
+  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7; 10, 11, 12, 14, 15)
+  [2] - DMA  (0=disable, 1, 3)
+  [3] - 0=trigger source is internal 8253 with 2MHz clock
+        1=trigger source is external
+  [4] - 0=A/D channels are S.E.
+        1=A/D channels are DIFF
+  [5] - 0=D/A outputs 0-5V  (internal reference -5V)
+        1=D/A outputs 0-10V (internal reference -10V)
+        2=D/A outputs unknow (external reference)
+
+Options for A-821PGL/PGH:
+  [0] - IO Base
+  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7)
+  [2] - 0=A/D channels are S.E.
+        1=A/D channels are DIFF
+  [3] - 0=D/A output 0-5V  (internal reference -5V)
+        1=D/A output 0-10V (internal reference -10V)
+
+Options for A-821PGL-NDA:
+  [0] - IO Base
+  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7)
+  [2] - 0=A/D channels are S.E.
+        1=A/D channels are DIFF
+
+Options for PCL-813:
+  [0] - IO Base
+
+Options for PCL-813B:
+  [0] - IO Base
+  [1] - 0= bipolar inputs
+        1= unipolar inputs
+
+Options for ACL-8113, ISO-813:
+  [0] - IO Base
+  [1] - 0= 10V bipolar inputs
+        1= 10V unipolar inputs
+        2= 20V bipolar inputs
+        3= 20V unipolar inputs
 */
-/*
- *
- * Options for PCL-812:
- *  [0] - IO Base
- *  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7; 10, 11, 12, 14, 15)
- *  [2] - DMA  (0=disable, 1, 3)
- *  [3] - 0=trigger source is internal 8253 with 2MHz clock
- *        1=trigger source is external 
- *  [4] - 0=A/D input range is +/-10V
- *        1=A/D input range is +/-5V
- *        2=A/D input range is +/-2.5V
- *        3=A/D input range is +/-1.25V
- *        4=A/D input range is +/-0.625V
- *        5=A/D input range is +/-0.3125V
- *  [5] - 0=D/A outputs 0-5V  (internal reference -5V)
- *        1=D/A outputs 0-10V (internal reference -10V)
- *        2=D/A outputs unknow (external reference)
- *
- * Options for PCL-812PG, ACL-8112PG:
- *  [0] - IO Base
- *  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7; 10, 11, 12, 14, 15)
- *  [2] - DMA  (0=disable, 1, 3)
- *  [3] - 0=trigger source is internal 8253 with 2MHz clock
- *        1=trigger source is external 
- *  [4] - 0=A/D have max +/-5V input
- *        1=A/D have max +/-10V input
- *  [5] - 0=D/A outputs 0-5V  (internal reference -5V)
- *        1=D/A outputs 0-10V (internal reference -10V)
- *        2=D/A outputs unknow (external reference)
- *
- * Options for ACL-8112DG/HG, A-822PGL/PGH, A-823PGL/PGH, ACL-8216, A-826PG:
- *  [0] - IO Base
- *  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7; 10, 11, 12, 14, 15)
- *  [2] - DMA  (0=disable, 1, 3)
- *  [3] - 0=trigger source is internal 8253 with 2MHz clock
- *        1=trigger source is external 
- *  [4] - 0=A/D channels are S.E.
- *        1=A/D channels are DIFF
- *  [5] - 0=D/A outputs 0-5V  (internal reference -5V)
- *        1=D/A outputs 0-10V (internal reference -10V)
- *        2=D/A outputs unknow (external reference)
- *
- * Options for A-821PGL/PGH:
- *  [0] - IO Base
- *  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7)
- *  [2] - 0=A/D channels are S.E.
- *        1=A/D channels are DIFF
- *  [3] - 0=D/A output 0-5V  (internal reference -5V) 
- *        1=D/A output 0-10V (internal reference -10V)
- *
- * Options for A-821PGL-NDA:
- *  [0] - IO Base
- *  [1] - IRQ  (0=disable, 2, 3, 4, 5, 6, 7)
- *  [2] - 0=A/D channels are S.E.
- *        1=A/D channels are DIFF
- *
- * Options for PCL-813:
- *  [0] - IO Base
- *
- * Options for PCL-813B:
- *  [0] - IO Base
- *  [1] - 0= bipolar inputs
- *        1= unipolar inputs
- *     
- * Options for ACL-8113, ISO-813:
- *  [0] - IO Base
- *  [1] - 0= 10V bipolar inputs
- *        1= 10V unipolar inputs
- *        2= 20V bipolar inputs
- *        3= 20V unipolar inputs
- *     
- */
 
 #include <linux/comedidev.h>
 
@@ -164,90 +161,90 @@ See the head of the source file pcl812.c for configuration options.
 
 #define MAX_CHANLIST_LEN	256	/* length of scan list */
 
-static comedi_lrange range_pcl812pg_ai = { 5, {
+static const comedi_lrange range_pcl812pg_ai = { 5, {
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
 	BIP_RANGE(0.625),
 	BIP_RANGE(0.3125),
 }};
-static comedi_lrange range_pcl812pg2_ai = { 5, {
+static const comedi_lrange range_pcl812pg2_ai = { 5, {
 	BIP_RANGE(10),
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
 	BIP_RANGE(0.625),
 }};
-static comedi_lrange range812_bipolar1_25 = { 1, {
+static const comedi_lrange range812_bipolar1_25 = { 1, {
 	BIP_RANGE(1.25),
 }};
-static comedi_lrange range812_bipolar0_625 = { 1, {
+static const comedi_lrange range812_bipolar0_625 = { 1, {
 	BIP_RANGE(0.625),
 }};
-static comedi_lrange range812_bipolar0_3125 = { 1, {
+static const comedi_lrange range812_bipolar0_3125 = { 1, {
 	BIP_RANGE(0.3125),
 }};
-static comedi_lrange range_pcl813b_ai = { 4, {
+static const comedi_lrange range_pcl813b_ai = { 4, {
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
 	BIP_RANGE(0.625),
 }};
-static comedi_lrange range_pcl813b2_ai = { 4, {
+static const comedi_lrange range_pcl813b2_ai = { 4, {
 	UNI_RANGE(10),
 	UNI_RANGE(5),
 	UNI_RANGE(2.5),
 	UNI_RANGE(1.25),
 }};
-static comedi_lrange range_iso813_1_ai = { 5, {
+static const comedi_lrange range_iso813_1_ai = { 5, {
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
 	BIP_RANGE(0.625),
 	BIP_RANGE(0.3125),
 }};
-static comedi_lrange range_iso813_1_2_ai = { 5, {
+static const comedi_lrange range_iso813_1_2_ai = { 5, {
 	UNI_RANGE(10),
 	UNI_RANGE(5),
 	UNI_RANGE(2.5),
 	UNI_RANGE(1.25),
 	UNI_RANGE(0.625),
 }};
-static comedi_lrange range_iso813_2_ai = { 4, {
+static const comedi_lrange range_iso813_2_ai = { 4, {
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
 	BIP_RANGE(0.625),
 }};
-static comedi_lrange range_iso813_2_2_ai = { 4, {
+static const comedi_lrange range_iso813_2_2_ai = { 4, {
 	UNI_RANGE(10),
 	UNI_RANGE(5),
 	UNI_RANGE(2.5),
 	UNI_RANGE(1.25),
 }};
-static comedi_lrange range_acl8113_1_ai = { 4, {
+static const comedi_lrange range_acl8113_1_ai = { 4, {
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
 	BIP_RANGE(0.625),
 }};
-static comedi_lrange range_acl8113_1_2_ai = { 4, {
+static const comedi_lrange range_acl8113_1_2_ai = { 4, {
 	UNI_RANGE(10),
 	UNI_RANGE(5),
 	UNI_RANGE(2.5),
 	UNI_RANGE(1.25),
 }};
-static comedi_lrange range_acl8113_2_ai = { 3, {
+static const comedi_lrange range_acl8113_2_ai = { 3, {
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
 }};
-static comedi_lrange range_acl8113_2_2_ai = { 3, {
+static const comedi_lrange range_acl8113_2_2_ai = { 3, {
 	UNI_RANGE(10),
 	UNI_RANGE(5),
 	UNI_RANGE(2.5),
 }};
-static comedi_lrange range_acl8112dg_ai = { 9, {
+static const comedi_lrange range_acl8112dg_ai = { 9, {
 	BIP_RANGE(5),
 	BIP_RANGE(2.5),
 	BIP_RANGE(1.25),
@@ -258,7 +255,7 @@ static comedi_lrange range_acl8112dg_ai = { 9, {
 	UNI_RANGE(1.25),
 	BIP_RANGE(10),
 }};
-static comedi_lrange range_acl8112hg_ai = { 12, {
+static const comedi_lrange range_acl8112hg_ai = { 12, {
 	BIP_RANGE(5),
 	BIP_RANGE(0.5),
 	BIP_RANGE(0.05),
@@ -272,7 +269,7 @@ static comedi_lrange range_acl8112hg_ai = { 12, {
 	BIP_RANGE(0.1),
 	BIP_RANGE(0.01),
 }};
-static comedi_lrange range_a821pgh_ai = { 4, {
+static const comedi_lrange range_a821pgh_ai = { 4, {
 	BIP_RANGE(5),
 	BIP_RANGE(0.5),
 	BIP_RANGE(0.05),
@@ -284,7 +281,7 @@ static int pcl812_detach(comedi_device *dev);
 
 
 typedef struct {
-	char 		*name;		// driver name
+	const char 		*name;		// board name
 	int 		board_type;	// type of this board
 	int 		n_aichan;	// num of AI chans in S.E.
 	int 		n_aichan_diff;	// DIFF num of chans
@@ -294,15 +291,15 @@ typedef struct {
 	int 		ai_maxdata;	// AI resolution
 	unsigned int	ai_ns_min;	// max sample speed of card v ns
 	unsigned int	i8254_osc_base;	// clock base
-	comedi_lrange	*rangelist_ai;	// rangelist for A/D
-	comedi_lrange	*rangelist_ao;	// rangelist for D/A
+	const comedi_lrange *rangelist_ai; // rangelist for A/D
+	const comedi_lrange *rangelist_ao; // rangelist for D/A
 	unsigned int	IRQbits;	// allowed IRQ
 	unsigned char 	DMAbits;	// allowed DMA chans
 	unsigned char	io_range;	// iorange for this board
 	unsigned char	haveMPC508;	// 1=board use MPC508A multiplexor
 } boardtype;
 
-static boardtype boardtypes[] =
+static const boardtype boardtypes[] =
 {
 	{"pcl812", boardPCL812, 16, 0, 2, 16, 16, 0x0fff,
 	 33000, 500, &range_bipolar10, &range_unipolar5,
@@ -361,14 +358,14 @@ static boardtype boardtypes[] =
 };
 
 #define n_boardtypes (sizeof(boardtypes)/sizeof(boardtype))
-#define this_board ((boardtype *)dev->board_ptr)
+#define this_board ((const boardtype *)dev->board_ptr)
 
 static comedi_driver driver_pcl812={
 	driver_name:	"pcl812",
 	module:		THIS_MODULE,
 	attach:		pcl812_attach,
 	detach:		pcl812_detach,
-	board_name:	boardtypes,
+	board_name:	&boardtypes[0].name,
 	num_names:	n_boardtypes,
 	offset:		sizeof(boardtype),
 };
@@ -391,7 +388,7 @@ typedef struct {
 	unsigned int	ai_scans;		// len of scanlist
 	unsigned int	ai_act_scan;		// how many scans we finished
 	unsigned int 	ai_chanlist[MAX_CHANLIST_LEN];// our copy of channel/range list
-	unsigned int	ai_n_chan;		// how many channels is measured	
+	unsigned int	ai_n_chan;		// how many channels is measured
 	unsigned int	ai_flags;		// flaglist
 	unsigned int	ai_data_len;		// len of data buffer
 	sampl_t		*ai_data;		// data buffer
@@ -410,14 +407,14 @@ typedef struct {
 
 #define devpriv ((pcl812_private *)dev->private)
 
-/* 
+/*
 ==============================================================================
 */
 static void start_pacer(comedi_device * dev, int mode, unsigned int divisor1, unsigned int divisor2);
-static void setup_range_channel(comedi_device * dev, comedi_subdevice * s, 
+static void setup_range_channel(comedi_device * dev, comedi_subdevice * s,
 	unsigned int rangechan, char wait);
 static int pcl812_ai_cancel(comedi_device * dev, comedi_subdevice * s);
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
@@ -449,7 +446,7 @@ static int pcl812_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 	return n;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int acl8216_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
@@ -480,7 +477,7 @@ static int acl8216_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 	return n;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_ao_insn_write(comedi_device *dev,comedi_subdevice *s,
@@ -498,7 +495,7 @@ static int pcl812_ao_insn_write(comedi_device *dev,comedi_subdevice *s,
 	return i;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_ao_insn_read(comedi_device *dev,comedi_subdevice *s,
@@ -514,7 +511,7 @@ static int pcl812_ao_insn_read(comedi_device *dev,comedi_subdevice *s,
 	return i;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_di_insn_bits(comedi_device *dev,comedi_subdevice *s,
@@ -528,7 +525,7 @@ static int pcl812_di_insn_bits(comedi_device *dev,comedi_subdevice *s,
 	return 2;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_do_insn_bits(comedi_device *dev,comedi_subdevice *s,
@@ -548,7 +545,7 @@ static int pcl812_do_insn_bits(comedi_device *dev,comedi_subdevice *s,
 }
 
 #ifdef PCL812_EXTDEBUG
-/* 
+/*
 ==============================================================================
 */
 static void pcl812_cmdtest_out(int e,comedi_cmd *cmd) {
@@ -559,7 +556,7 @@ static void pcl812_cmdtest_out(int e,comedi_cmd *cmd) {
 }
 #endif
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *cmd)
@@ -718,14 +715,14 @@ static int pcl812_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *
 	return 0;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 {
 	unsigned int	divisor1=0, divisor2=0, i, dma_flags, bytes;
 	comedi_cmd 	*cmd=&s->async->cmd;
-	
+
 #ifdef PCL812_EXTDEBUG
 	rt_printk("pcl812 EDBG: BGN: pcl812_ai_cmd(...)\n");
 #endif
@@ -743,8 +740,8 @@ static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 
 	if (cmd->convert_src==TRIG_TIMER) {
 		if(cmd->convert_arg<this_board->ai_ns_min) cmd->convert_arg=this_board->ai_ns_min;
-		i8253_cascade_ns_to_timer(this_board->i8254_osc_base, 
-			&divisor1, &divisor2, &cmd->convert_arg, 
+		i8253_cascade_ns_to_timer(this_board->i8254_osc_base,
+			&divisor1, &divisor2, &cmd->convert_arg,
 			cmd->flags&TRIG_ROUND_MASK);
 	}
 
@@ -756,13 +753,13 @@ static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 
 	if (devpriv->dma) { // check if we can use DMA transfer
 		devpriv->ai_dma=1;
-		for (i=1; i<devpriv->ai_n_chan; i++) 
+		for (i=1; i<devpriv->ai_n_chan; i++)
 			if (devpriv->ai_chanlist[0]!=devpriv->ai_chanlist[i]) {
 				devpriv->ai_dma=0;	// we cann't use DMA :-(
 				break;
 			}
 	} else devpriv->ai_dma=0;
-		
+
 	devpriv->ai_flags=cmd->flags;
 	devpriv->ai_data_len=s->async->prealloc_bufsz;
 	devpriv->ai_data=s->async->prealloc_buf;
@@ -773,31 +770,31 @@ static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 	devpriv->ai_poll_ptr=0;
         s->async->cur_chan=0;
 
-	if ((devpriv->ai_flags & TRIG_WAKE_EOS)) { 	// don't we want wake up every scan?		
+	if ((devpriv->ai_flags & TRIG_WAKE_EOS)) { 	// don't we want wake up every scan?
 		devpriv->ai_eos=1;
 		if (devpriv->ai_n_chan==1)
 			devpriv->ai_dma=0;	// DMA is useless for this situation
 	}
 
-	if (devpriv->ai_dma) {	
+	if (devpriv->ai_dma) {
 		if (devpriv->ai_eos) {  // we use EOS, so adapt DMA buffer to one scan
 			devpriv->dmabytestomove[0]=devpriv->ai_n_chan*sizeof(sampl_t);
 			devpriv->dmabytestomove[1]=devpriv->ai_n_chan*sizeof(sampl_t);
-			devpriv->dma_runs_to_end=1; 
+			devpriv->dma_runs_to_end=1;
 		} else {
-			devpriv->dmabytestomove[0]=devpriv->hwdmasize[0]; 
+			devpriv->dmabytestomove[0]=devpriv->hwdmasize[0];
 			devpriv->dmabytestomove[1]=devpriv->hwdmasize[1];
 			if (devpriv->ai_data_len<devpriv->hwdmasize[0])
 				devpriv->dmabytestomove[0]=devpriv->ai_data_len;
 			if (devpriv->ai_data_len<devpriv->hwdmasize[1])
 				devpriv->dmabytestomove[1]=devpriv->ai_data_len;
 			if (devpriv->ai_neverending) {
-				devpriv->dma_runs_to_end=1; 
+				devpriv->dma_runs_to_end=1;
 			} else {
 				bytes=devpriv->ai_n_chan*devpriv->ai_scans*sizeof(sampl_t); // how many samples we must transfer?
 				devpriv->dma_runs_to_end=bytes / devpriv->dmabytestomove[0]; // how many DMA pages we must fill
 				devpriv->last_dma_run=bytes % devpriv->dmabytestomove[0]; //on last dma transfer must be moved
-				if (devpriv->dma_runs_to_end==0) 
+				if (devpriv->dma_runs_to_end==0)
 					devpriv->dmabytestomove[0]=devpriv->last_dma_run;
 				devpriv->dma_runs_to_end--;
 			}
@@ -825,14 +822,14 @@ static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 			devpriv->ai_eos);
 #endif
 	}
-	
+
 	switch (cmd->convert_src) {
 	case TRIG_TIMER:
 		start_pacer(dev, 1, divisor1, divisor2);
 		break;
 	}
-	
-	if (devpriv->ai_dma) {	
+
+	if (devpriv->ai_dma) {
 		outb(devpriv->mode_reg_int|2, dev->iobase + PCL812_MODE);	// let's go!
 	} else {
 		outb(devpriv->mode_reg_int|6, dev->iobase + PCL812_MODE);	// let's go!
@@ -845,10 +842,10 @@ static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
         return 0;
 }
 
-/* 
+/*
 ==============================================================================
 */
-static irqreturn_t interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl812_ai_int(int irq, void *d)
 {
 	char err=1;
 	unsigned int mask, timeout;
@@ -880,7 +877,7 @@ static irqreturn_t interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *reg
 		rt_printk("comedi%d: pcl812: (%s at 0x%lx) A/D cmd IRQ without DRDY!\n", dev->minor, dev->board_name, dev->iobase);
 		pcl812_ai_cancel(dev,s);
 		s->async->events |= COMEDI_CB_EOA|COMEDI_CB_ERROR;
-		comedi_event(dev,s,s->async->events);
+		comedi_event(dev, s);
 		return IRQ_HANDLED;
 	}
 
@@ -898,7 +895,7 @@ static irqreturn_t interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *reg
 			}
 	}
 
-	comedi_event(dev,s,s->async->events);
+	comedi_event(dev, s);
 	return IRQ_HANDLED;
 }
 
@@ -909,7 +906,7 @@ static void transfer_from_dma_buf(comedi_device *dev,comedi_subdevice *s,
 	sampl_t *ptr, unsigned int bufptr, unsigned int len)
 {
 	unsigned int i;
-	
+
 	s->async->events = 0;
         for (i=len; i; i--) {
 		comedi_buf_put( s->async, ptr[bufptr++] ); // get one sample
@@ -925,13 +922,13 @@ static void transfer_from_dma_buf(comedi_device *dev,comedi_subdevice *s,
 		}
 	}
 
-	comedi_event(dev,s,s->async->events);
+	comedi_event(dev, s);
 }
 
-/* 
+/*
 ==============================================================================
 */
-static irqreturn_t interrupt_pcl812_ai_dma(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl812_ai_dma(int irq, void *d)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = dev->subdevices + 0;
@@ -976,12 +973,16 @@ static irqreturn_t interrupt_pcl812_ai_dma(int irq, void *d, struct pt_regs *reg
 /*
 ==============================================================================
 */
-static irqreturn_t interrupt_pcl812(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl812(int irq, void *d PT_REGS_ARG)
 {
 	comedi_device *dev = d;
 
-	if (devpriv->ai_dma) { return interrupt_pcl812_ai_dma(irq, d, regs); }
-			else { return interrupt_pcl812_ai_int(irq, d, regs); };
+	if (!dev->attached) {
+		comedi_error(dev, "spurious interrupt");
+		return IRQ_HANDLED;
+	}
+	if (devpriv->ai_dma) { return interrupt_pcl812_ai_dma(irq, d); }
+			else { return interrupt_pcl812_ai_int(irq, d); };
 }
 
 /*
@@ -1028,21 +1029,21 @@ static int pcl812_ai_poll(comedi_device *dev,comedi_subdevice *s)
 /*
 ==============================================================================
 */
-static void setup_range_channel(comedi_device * dev, comedi_subdevice * s, 
+static void setup_range_channel(comedi_device * dev, comedi_subdevice * s,
 	unsigned int rangechan, char wait)
 {
 	unsigned char chan_reg=CR_CHAN(rangechan); // normal board
 	unsigned char gain_reg=CR_RANGE(rangechan)+devpriv->range_correction; // gain index
-	
-	if ((chan_reg==devpriv->old_chan_reg)&&(gain_reg==devpriv->old_gain_reg)) 
+
+	if ((chan_reg==devpriv->old_chan_reg)&&(gain_reg==devpriv->old_gain_reg))
 		return;	// we can return, no change
-		
+
 	devpriv->old_chan_reg=chan_reg;
 	devpriv->old_gain_reg=gain_reg;
-	
+
 	if (devpriv->use_MPC) {
 		if (devpriv->use_diff) {
-			chan_reg=chan_reg | 0x30; // DIFF inputs 
+			chan_reg=chan_reg | 0x30; // DIFF inputs
 		} else {
 			if (chan_reg&0x80) {
 				chan_reg=chan_reg | 0x20; // SE inputs 8-15
@@ -1051,7 +1052,7 @@ static void setup_range_channel(comedi_device * dev, comedi_subdevice * s,
 			}
 		}
 	}
-	
+
 	outb(chan_reg, dev->iobase + PCL812_MUX); /* select channel */
 	outb(gain_reg, dev->iobase + PCL812_GAIN); /* select gain */
 
@@ -1064,7 +1065,7 @@ static void setup_range_channel(comedi_device * dev, comedi_subdevice * s,
 /*
 ==============================================================================
 */
-static void start_pacer(comedi_device * dev, int mode, unsigned int divisor1, unsigned int divisor2) 
+static void start_pacer(comedi_device * dev, int mode, unsigned int divisor1, unsigned int divisor2)
 {
 #ifdef PCL812_EXTDEBUG
 	rt_printk("pcl812 EDBG: BGN: start_pacer(%d,%u,%u)\n",mode,divisor1,divisor2);
@@ -1072,7 +1073,7 @@ static void start_pacer(comedi_device * dev, int mode, unsigned int divisor1, un
         outb(0xb4, dev->iobase + PCL812_CTRCTL);
         outb(0x74, dev->iobase + PCL812_CTRCTL);
         comedi_udelay(1);
-  
+
         if (mode==1) {
 		outb(divisor2 & 0xff, dev->iobase + PCL812_CTR2);
 		outb((divisor2 >> 8) & 0xff, dev->iobase + PCL812_CTR2);
@@ -1084,7 +1085,7 @@ static void start_pacer(comedi_device * dev, int mode, unsigned int divisor1, un
 #endif
 }
 
-/* 
+/*
 ==============================================================================
 */
 static void free_resources(comedi_device * dev)
@@ -1104,7 +1105,7 @@ static void free_resources(comedi_device * dev)
 		release_region(dev->iobase, this_board->io_range);
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_ai_cancel(comedi_device * dev, comedi_subdevice * s)
@@ -1123,7 +1124,7 @@ static int pcl812_ai_cancel(comedi_device * dev, comedi_subdevice * s)
 	return 0;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static void pcl812_reset(comedi_device * dev)
@@ -1272,7 +1273,6 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 	/* analog input */
 	if (this_board->n_aichan>0) {
 		s = dev->subdevices + subdev;
-		dev->read_subdev = s;
 		s->type = COMEDI_SUBD_AI;
 		s->subdev_flags = SDF_READABLE;
 		switch (this_board->board_type) {
@@ -1285,7 +1285,7 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 				s->n_chan = this_board->n_aichan;
 				s->subdev_flags |= SDF_GROUND;
 			}
-			break;		
+			break;
 		case boardACL8112:
 		case boardACL8216:
 			if (it->options[4] == 1) {
@@ -1296,11 +1296,11 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 				s->n_chan = this_board->n_aichan;
 				s->subdev_flags |= SDF_GROUND;
 			}
-			break;		
+			break;
 		default:
 			s->n_chan = this_board->n_aichan;
 			s->subdev_flags |= SDF_GROUND;
-			break;		
+			break;
 		}
 		s->maxdata = this_board->ai_maxdata;
 		s->len_chanlist = MAX_CHANLIST_LEN;
@@ -1313,6 +1313,8 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 		devpriv->use_MPC = this_board->haveMPC508;
 		s->cancel = pcl812_ai_cancel;
 		if (dev->irq) {
+			dev->read_subdev = s;
+			s->subdev_flags |= SDF_CMD_READ;
 			s->do_cmdtest = pcl812_ai_cmdtest;
 			s->do_cmd = pcl812_ai_cmd;
 			s->poll = pcl812_ai_poll;
@@ -1443,7 +1445,7 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 		devpriv->max_812_ai_mode0_rangewait = 5;	/* maybe there must by greatest timeout */
 		break;
 	}
-	
+
 	printk("\n");
 	devpriv->valid=1;
 
