@@ -20,9 +20,9 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /*
-Driver: pcmuio.o
+Driver: pcmuio
 Description: A driver for the PCM-UIO48A and PCM-UIO96A boards from Winsystems.
-Devices: (Winsystems) PCM-UIO48A [pcmuio48], (Winsystems) PCM-UIO96A [pcmuio96]
+Devices: [Winsystems] PCM-UIO48A (pcmuio48), PCM-UIO96A (pcmuio96)
 Author: Calin Culianu <calin@ajvar.org>
 Updated: Fri, 13 Jan 2006 12:01:01 -0500
 Status: works
@@ -41,14 +41,14 @@ the board.  You need to pass that information to this driver as the
 first and second comedi_config option, respectively.  Note that the
 48-channel version uses 16 bytes of IO memory and the 96-channel
 version uses 32-bytes (in case you are worried about conflicts).  The
-48-channel board is split into two 24-channel comedi subdevices. 
-The 96-channel board is split into 4 24-channel DIO subdevices.  
+48-channel board is split into two 24-channel comedi subdevices.
+The 96-channel board is split into 4 24-channel DIO subdevices.
 
 Note that IRQ support has been added, but it is untested.
 
-To use edge-detection IRQ support, pass the IRQs of both ASICS 
-(for the 96 channel version) or just 1 ASIC (for 48-channel version).  
-Then, use use comedi_commands with TRIG_NOW.  
+To use edge-detection IRQ support, pass the IRQs of both ASICS
+(for the 96 channel version) or just 1 ASIC (for 48-channel version).
+Then, use use comedi_commands with TRIG_NOW.
 Your callback will be called each time an edge is triggered, and the data
 values will be two sample_t's, which should be concatenated to form one
 32-bit unsigned int.  This value is the mask of channels that had
@@ -56,7 +56,7 @@ edges detected from your channel list.  Note that the bits positions
 in the mask correspond to positions in your chanlist when you specified
 the command and *not* channel id's!
 
-To set the polarity of the edge-detection interrupts pass a nonzero value for 
+To set the polarity of the edge-detection interrupts pass a nonzero value for
 either CR_RANGE or CR_AREF for edge-up polarity, or a zero value for both
 CR_RANGE and CR_AREF if you want edge-down polarity.
 
@@ -106,7 +106,7 @@ Configuration Options:
   --------------------------------------------------------------
   REG_PORTx         All          R/W         Read/Write/Configure IO
   REG_INT_PENDING   All          ReadOnly    Quickly see which INT_IDx has int.
-  REG_PAGELOCK      All          WriteOnly   Select a page 
+  REG_PAGELOCK      All          WriteOnly   Select a page
   REG_POLx          Pg. 1 only   WriteOnly   Select edge-detection polarity
   REG_ENABx         Pg. 2 only   WriteOnly   Enable/Disable edge-detect. int.
   REG_INT_IDx       Pg. 3 only   R/W         See which ports/bits have ints.
@@ -121,10 +121,10 @@ Configuration Options:
 #define REG_PAGELOCK 0x7 /* page selector register, upper 2 bits select a page
                             and bits 0-5 are used to 'lock down' a particular
                             port above to make it readonly.  */
-#define REG_POL0 0x8 
+#define REG_POL0 0x8
 #define REG_POL1 0x9
 #define REG_POL2 0xA
-#define REG_ENAB0 0x8 
+#define REG_ENAB0 0x8
 #define REG_ENAB1 0x9
 #define REG_ENAB2 0xA
 #define REG_INT_ID0 0x8
@@ -149,13 +149,13 @@ Configuration Options:
  */
 typedef struct pcmuio_board_struct
 {
-	char * const name;
+	const char *name;
 	const int num_asics;
 	const int num_channels_per_port;
 	const int num_ports;
 } pcmuio_board;
 
-static pcmuio_board pcmuio_boards[] = 
+static const pcmuio_board pcmuio_boards[] =
 {
 	{
 		name:                   "pcmuio48",
@@ -172,18 +172,18 @@ static pcmuio_board pcmuio_boards[] =
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((pcmuio_board *)dev->board_ptr)
+#define thisboard ((const pcmuio_board *)dev->board_ptr)
 
 /* this structure is for data unique to this subdevice.  */
-typedef struct 
+typedef struct
 {
     /* mapping of halfwords (bytes) in port/chanarray to iobase */
     unsigned long iobases[PORTS_PER_SUBDEV];
-  
+
     /* The below is only used for intr subdevices */
     struct {
       int asic; /* if non-negative, this subdev has an interrupt asic */
-      int first_chan; /* if nonnegative, the first channel id for 
+      int first_chan; /* if nonnegative, the first channel id for
                          interrupts. */
       int num_asic_chans; /* the number of asic channels in this subdev
                              that have interrutps */
@@ -202,8 +202,8 @@ typedef struct
    several hardware drivers keep similar information in this structure,
    feel free to suggest moving the variable to the comedi_device struct.  */
 typedef struct
-{  
-  struct 
+{
+  struct
   {
     unsigned char pagelock; /* current page and lock*/
     unsigned char pol [NUM_PAGED_REGS]; /* shadow of POLx registers */
@@ -212,7 +212,7 @@ typedef struct
     unsigned long iobase;
     unsigned int irq;
     spinlock_t spinlock;
-  } asics[MAX_ASICS];  
+  } asics[MAX_ASICS];
   pcmuio_subdev_private *sprivs;
 } pcmuio_private;
 
@@ -231,7 +231,7 @@ typedef struct
 static int pcmuio_attach(comedi_device *dev, comedi_devconfig *it);
 static int pcmuio_detach(comedi_device *dev);
 
-static comedi_driver driver = 
+static comedi_driver driver =
 {
 	driver_name:	"pcmuio",
 	module:		THIS_MODULE,
@@ -255,7 +255,7 @@ static comedi_driver driver =
 	 * the type of board in software.  ISA PnP, PCI, and PCMCIA
 	 * devices are such boards.
 	 */
-	board_name:	pcmuio_boards,
+	board_name:	&pcmuio_boards[0].name,
 	offset:		sizeof(pcmuio_board),
 	num_names:	sizeof(pcmuio_boards) / sizeof(pcmuio_board),
 };
@@ -265,7 +265,7 @@ static int pcmuio_dio_insn_bits(comedi_device *dev,comedi_subdevice *s,
 static int pcmuio_dio_insn_config(comedi_device *dev,comedi_subdevice *s,
                                   comedi_insn *insn,lsampl_t *data);
 
-static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t interrupt_pcmuio(int irq, void *d PT_REGS_ARG);
 static void pcmuio_stop_intr(comedi_device *, comedi_subdevice *);
 static int  pcmuio_cancel(comedi_device *dev, comedi_subdevice *s);
 static int pcmuio_cmd(comedi_device *dev, comedi_subdevice *s);
@@ -295,16 +295,16 @@ static int pcmuio_attach(comedi_device *dev, comedi_devconfig *it)
     irq[1] = it->options[2];
 
     printk("comedi%d: %s: io: %lx ", dev->minor, driver.driver_name, iobase);
-	
+
     dev->iobase = iobase;
-    
-    if ( !iobase || !request_region(iobase, 
-                                    thisboard->num_asics*ASIC_IOSIZE, 
+
+    if ( !iobase || !request_region(iobase,
+                                    thisboard->num_asics*ASIC_IOSIZE,
                                     driver.driver_name) ) {
       printk("I/O port conflict\n");
       return -EIO;
     }
-        
+
 /*
  * Initialize dev->board_name.  Note that we can use the "thisboard"
  * macro now, since we just initialized it in the last line.
@@ -323,25 +323,24 @@ static int pcmuio_attach(comedi_device *dev, comedi_devconfig *it)
     for (asic = 0; asic < MAX_ASICS; ++asic) {
       devpriv->asics[asic].num = asic;
       devpriv->asics[asic].iobase = dev->iobase + asic*ASIC_IOSIZE;
-      devpriv->asics[asic].irq = 0; /* this gets actually set at the end of 
-                                       this function when we 
+      devpriv->asics[asic].irq = 0; /* this gets actually set at the end of
+                                       this function when we
                                        comedi_request_irqs */
       spin_lock_init(&devpriv->asics[asic].spinlock);
     }
-    
-    chans_left = CHANS_PER_ASIC * thisboard->num_asics;    
+
+    chans_left = CHANS_PER_ASIC * thisboard->num_asics;
     n_subdevs = CALC_N_SUBDEVS(chans_left);
-    devpriv->sprivs = (pcmuio_subdev_private *)kmalloc(sizeof(pcmuio_subdev_private) * n_subdevs, GFP_KERNEL);
+    devpriv->sprivs = kcalloc(n_subdevs, sizeof(pcmuio_subdev_private), GFP_KERNEL);
     if (!devpriv->sprivs) {
         printk("cannot allocate subdevice private data structures\n");
         return -ENOMEM;
     }
-    memset(devpriv->sprivs, 0, sizeof(pcmuio_subdev_private) * n_subdevs);
       /*
        * Allocate the subdevice structures.  alloc_subdevice() is a
        * convenient macro defined in comedidev.h.
        *
-       * Allocate 2 subdevs (32 + 16 DIO lines) or 3 32 DIO subdevs for the 
+       * Allocate 2 subdevs (32 + 16 DIO lines) or 3 32 DIO subdevs for the
        * 96-channel version of the board.
        */
     if ( alloc_subdevices(dev, n_subdevs ) < 0 ) {
@@ -349,18 +348,16 @@ static int pcmuio_attach(comedi_device *dev, comedi_devconfig *it)
         return -ENOMEM;
     }
 
-    memset(dev->subdevices, 0, sizeof(*dev->subdevices) * n_subdevs);
-
     port = 0;
-    asic = 0;    
+    asic = 0;
     for (sdev_no = 0; sdev_no < (int)dev->n_subdevices; ++sdev_no)
-    {      
+    {
       int byte_no;
 
       s = dev->subdevices + sdev_no;
       s->private = devpriv->sprivs + sdev_no;
       s->maxdata = 1;
-      s->range_table = &range_digital;      
+      s->range_table = &range_digital;
       s->subdev_flags = SDF_READABLE|SDF_WRITABLE;
       s->type = COMEDI_SUBD_DIO;
       s->insn_bits = pcmuio_dio_insn_bits;
@@ -371,10 +368,10 @@ static int pcmuio_attach(comedi_device *dev, comedi_devconfig *it)
       subpriv->intr.asic_chan = -1;
       subpriv->intr.num_asic_chans = -1;
       subpriv->intr.active = 0;
-      s->len_chanlist = 1; 
+      s->len_chanlist = 1;
 
-      /* save the ioport address for each 'port' of 8 channels in the 
-         subdevice */         
+      /* save the ioport address for each 'port' of 8 channels in the
+         subdevice */
       for (byte_no = 0; byte_no < PORTS_PER_SUBDEV; ++byte_no, ++port) {
           if (port >= PORTS_PER_ASIC) {
             port = 0;
@@ -392,10 +389,12 @@ static int pcmuio_attach(comedi_device *dev, comedi_devconfig *it)
             subpriv->intr.first_chan = byte_no * 8;
             subpriv->intr.asic_chan = thisasic_chanct;
             subpriv->intr.num_asic_chans = s->n_chan - subpriv->intr.first_chan;
+            dev->read_subdev = s;
+            s->subdev_flags |= SDF_CMD_READ;
             s->cancel = pcmuio_cancel;
             s->do_cmd = pcmuio_cmd;
             s->do_cmdtest = pcmuio_cmdtest;
-            s->len_chanlist = subpriv->intr.num_asic_chans; 
+            s->len_chanlist = subpriv->intr.num_asic_chans;
           }
           thisasic_chanct += CHANS_PER_PORT;
       }
@@ -407,44 +406,44 @@ static int pcmuio_attach(comedi_device *dev, comedi_devconfig *it)
         asic = 0; /* reset the asic to our first asic, to do intr subdevs */
         port = 0;
       }
-      
+
     }
 
     init_asics(dev); /* clear out all the registers, basically */
 
     for (asic = 0; irq[0] && asic < MAX_ASICS; ++asic) {
-      if (irq[asic] && comedi_request_irq(irq[asic], interrupt_pcmuio, SA_SHIRQ, thisboard->name, dev)) {
+      if (irq[asic] && comedi_request_irq(irq[asic], interrupt_pcmuio, IRQF_SHARED, thisboard->name, dev)) {
         int i;
         /* unroll the allocated irqs.. */
         for (i = asic-1; i >= 0; --i) {
-          comedi_free_irq(irq[i], dev);          
+          comedi_free_irq(irq[i], dev);
           devpriv->asics[i].irq = irq[i] = 0;
         }
         irq[asic] = 0;
       }
       devpriv->asics[asic].irq = irq[asic];
     }
-    
-    dev->irq = irq[0]; /* grr.. wish comedi dev struct supported multiple 
+
+    dev->irq = irq[0]; /* grr.. wish comedi dev struct supported multiple
                           irqs.. */
 
     if (irq[0]) {
       printk("irq: %u ", irq[0]);
-      if (irq[1] && thisboard->num_asics == 2) 
+      if (irq[1] && thisboard->num_asics == 2)
         printk("second ASIC irq: %u ", irq[1]);
     } else {
       printk("(IRQ mode disabled) ");
     }
 
     printk("attached\n");
-    
+
     return 1;
 }
 
 
 /*
  * _detach is called to deconfigure a device.  It should deallocate
- * resources.  
+ * resources.
  * This function is also called when _attach() fails, so it should be
  * careful not to release resources that were not necessarily
  * allocated by _attach().  dev->private and dev->subdevices are
@@ -459,10 +458,10 @@ static int pcmuio_detach(comedi_device *dev)
       release_region(dev->iobase, ASIC_IOSIZE * thisboard->num_asics);
 
     for (i = 0; i < MAX_ASICS; ++i) {
-      if (devpriv->asics[i].irq) 
+      if (devpriv->asics[i].irq)
         comedi_free_irq(devpriv->asics[i].irq, dev);
     }
-    
+
     if (devpriv && devpriv->sprivs)
       kfree(devpriv->sprivs);
 
@@ -482,9 +481,9 @@ static int pcmuio_dio_insn_bits(comedi_device *dev, comedi_subdevice *s,
     if (insn->n != 2) return -EINVAL;
 
      /* NOTE:
-        reading a 0 means this channel was high 
+        reading a 0 means this channel was high
         writine a 0 sets the channel high
-        reading a 1 means this channel was low 
+        reading a 1 means this channel was low
         writing a 1 means set this channel low
 
         Therefore everything is always inverted. */
@@ -501,13 +500,13 @@ static int pcmuio_dio_insn_bits(comedi_device *dev, comedi_subdevice *s,
 
     for (byte_no = 0; byte_no < s->n_chan/CHANS_PER_PORT; ++byte_no) {
            /* address of 8-bit port */
-        unsigned long ioaddr = subpriv->iobases[byte_no], 
+        unsigned long ioaddr = subpriv->iobases[byte_no],
            /* bit offset of port in 32-bit doubleword */
-            offset = byte_no * 8; 
+            offset = byte_no * 8;
                       /* this 8-bit port's data */
-        unsigned char byte = 0,   
+        unsigned char byte = 0,
                       /* The write mask for this port (if any) */
-                      write_mask_byte = (data[0] >> offset) & 0xff, 
+                      write_mask_byte = (data[0] >> offset) & 0xff,
                       /* The data byte for this port */
                       data_byte = (data[1] >> offset) & 0xff;
 
@@ -518,7 +517,7 @@ static int pcmuio_dio_insn_bits(comedi_device *dev, comedi_subdevice *s,
         printk("byte %d wmb %02x db %02x offset %02d io %04x, data_in %02x ", byte_no, (unsigned)write_mask_byte, (unsigned)data_byte, offset, ioaddr, (unsigned)byte);
 #endif
 
-        if ( write_mask_byte ) { 
+        if ( write_mask_byte ) {
           /* this byte has some write_bits -- so set the output lines */
           byte &= ~write_mask_byte; /* clear bits for write mask */
           byte |= ~data_byte & write_mask_byte; /* set to inverted data_byte */
@@ -531,17 +530,17 @@ static int pcmuio_dio_insn_bits(comedi_device *dev, comedi_subdevice *s,
         printk("data_out_byte %02x\n", (unsigned)byte);
 #endif
         /* save the digital input lines for this byte.. */
-        s->state |= ((unsigned int)byte) << offset; 
+        s->state |= ((unsigned int)byte) << offset;
     }
 
     /* now return the DIO lines to data[1] - note they came inverted! */
-    data[1] = ~s->state; 
+    data[1] = ~s->state;
 
 #ifdef DAMMIT_ITS_BROKEN
     /* DEBUG */
     printk("s->state %08x data_out %08x\n", s->state, data[1]);
 #endif
-    
+
     return 2;
 }
 
@@ -558,23 +557,23 @@ static int pcmuio_dio_insn_config(comedi_device *dev, comedi_subdevice *s,
 
     /* Compute ioaddr for this channel */
     ioaddr = subpriv->iobases[byte_no];
-    
+
      /* NOTE:
-        writing a 0 an IO channel's bit sets the channel to INPUT 
+        writing a 0 an IO channel's bit sets the channel to INPUT
         and pulls the line high as well
-        
+
         writing a 1 to an IO channel's  bit pulls the line low
 
-        All channels are implicitly always in OUTPUT mode -- but when 
+        All channels are implicitly always in OUTPUT mode -- but when
         they are high they can be considered to be in INPUT mode..
 
-        Thus, we only force channels low if the config request was INPUT, 
+        Thus, we only force channels low if the config request was INPUT,
         otherwise we do nothing to the hardware.    */
 
 	switch(data[0])
 	{
 	case INSN_CONFIG_DIO_OUTPUT:
-          /* save to io_bits -- don't actually do anything since 
+          /* save to io_bits -- don't actually do anything since
              all input channels are also output channels... */
           s->io_bits |= 1<<chan;
           break;
@@ -584,11 +583,11 @@ static int pcmuio_dio_insn_config(comedi_device *dev, comedi_subdevice *s,
           byte = inb(ioaddr);
           byte &= ~(1<<bit_no); /**< set input channel to '0' */
 
-          /* write out byte -- this is the only time we actually affect the 
-             hardware as all channels are implicitly output -- but input 
+          /* write out byte -- this is the only time we actually affect the
+             hardware as all channels are implicitly output -- but input
              channels are set to float-high */
           outb(byte, ioaddr);
-          
+
           /* save to io_bits */
           s->io_bits &= ~(1<<chan);
           break;
@@ -602,16 +601,16 @@ static int pcmuio_dio_insn_config(comedi_device *dev, comedi_subdevice *s,
 	default:
           return -EINVAL;
           break;
-	}    
+	}
 
-	return insn->n;    
+	return insn->n;
 }
 
-static void init_asics(comedi_device *dev) /* sets up an 
+static void init_asics(comedi_device *dev) /* sets up an
                                               ASIC chip to defaults */
 {
   int asic;
-  
+
   for (asic = 0; asic < thisboard->num_asics; ++asic)
   {
     int port, page;
@@ -620,7 +619,7 @@ static void init_asics(comedi_device *dev) /* sets up an
     switch_page(dev, asic,  0); /* switch back to page 0 */
 
     /* first, clear all the DIO port bits */
-    for (port = 0; port < PORTS_PER_ASIC; ++port) 
+    for (port = 0; port < PORTS_PER_ASIC; ++port)
       outb(0, baseaddr + REG_PORT0 + port);
 
     /* Next, clear all the paged registers for each page */
@@ -641,7 +640,7 @@ static void init_asics(comedi_device *dev) /* sets up an
     /* END DEBUG */
 
     switch_page(dev, asic,  0); /* switch back to default page 0 */
-    
+
   }
 }
 
@@ -655,7 +654,7 @@ static void switch_page(comedi_device *dev, int asic, int page)
   devpriv->asics[asic].pagelock |= page<<REG_PAGE_BITOFFSET;
 
   /* now write out the shadow register */
-  outb(devpriv->asics[asic].pagelock, 
+  outb(devpriv->asics[asic].pagelock,
        dev->iobase + ASIC_IOSIZE*asic + REG_PAGELOCK);
 }
 
@@ -664,7 +663,7 @@ static void lock_port(comedi_device *dev, int asic, int port)
   if (asic < 0 || asic >= thisboard->num_asics) return; /* paranoia */
   if (port < 0 || port >= PORTS_PER_ASIC) return; /* more paranoia */
 
-  devpriv->asics[asic].pagelock |= 0x1<<port;  
+  devpriv->asics[asic].pagelock |= 0x1<<port;
   /* now write out the shadow register */
   outb(devpriv->asics[asic].pagelock, dev->iobase + ASIC_IOSIZE*asic + REG_PAGELOCK);
   return;
@@ -675,20 +674,20 @@ static void unlock_port(comedi_device *dev, int asic, int port)
 {
   if (asic < 0 || asic >= thisboard->num_asics) return; /* paranoia */
   if (port < 0 || port >= PORTS_PER_ASIC) return; /* more paranoia */
-  devpriv->asics[asic].pagelock &= ~(0x1<<port) | REG_LOCK_MASK;  
+  devpriv->asics[asic].pagelock &= ~(0x1<<port) | REG_LOCK_MASK;
   /* now write out the shadow register */
   outb(devpriv->asics[asic].pagelock, dev->iobase + ASIC_IOSIZE*asic + REG_PAGELOCK);
   (void)unlock_port(dev, asic, port); /* not reached, suppress compiler warnings*/
 }
 
-static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcmuio(int irq, void *d PT_REGS_ARG)
 {
   int asic, got1 = 0;
   comedi_device *dev = (comedi_device *)d;
-  
+
   for (asic = 0; asic < MAX_ASICS; ++asic) {
     if (irq == devpriv->asics[asic].irq) {
-      unsigned long flags;    
+      unsigned long flags;
       unsigned triggered = 0;
       unsigned long iobase = devpriv->asics[asic].iobase;
       /* it is an interrupt for ASIC #asic */
@@ -705,8 +704,8 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
             unsigned char io_lines_with_edges = 0;
             switch_page(dev, asic, PAGE_INT_ID);
             io_lines_with_edges = inb(iobase + REG_INT_ID0 + port);
-           
-            if (io_lines_with_edges) 
+
+            if (io_lines_with_edges)
               /* clear pending interrupt */
               outb(0, iobase + REG_INT_ID0 + port);
 
@@ -719,7 +718,7 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
 
       comedi_spin_unlock_irqrestore(&devpriv->asics[asic].spinlock, flags);
 
-      if (triggered) {        
+      if (triggered) {
         comedi_subdevice *s;
         /* TODO here: dispatch io lines to subdevs with commands.. */
         printk("PCMUIO DEBUG: got edge detect interrupt %d asic %d which_chans: %06x\n", irq, asic, triggered);
@@ -727,9 +726,9 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
           if (subpriv->intr.asic == asic) { /* this is an interrupt subdev, and it matches this asic! */
             unsigned long flags;
             unsigned oldevents;
-            
+
             comedi_spin_lock_irqsave(&subpriv->intr.spinlock, flags);
-            
+
             oldevents = s->async->events;
 
             if (subpriv->intr.active) {
@@ -737,7 +736,7 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
               if (mytrig & subpriv->intr.enabled_mask) {
                 lsampl_t val = 0;
                 unsigned int n, ch, len;
-                
+
                 len = s->async->cmd.chanlist_len;
                 for (n = 0; n < len; n++) {
                   ch = CR_CHAN(s->async->cmd.chanlist[n]);
@@ -752,9 +751,9 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
                 } else {
                   /* Overflow! Stop acquisition!! */
                   /* TODO: STOP_ACQUISITION_CALL_HERE!! */
-                  pcmuio_stop_intr(dev, s); 
+                  pcmuio_stop_intr(dev, s);
                 }
-                
+
                 /* Check for end of acquisition. */
                 if (!subpriv->intr.continuous) {
                   /* stop_src == TRIG_COUNT */
@@ -763,7 +762,7 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
                     if (subpriv->intr.stop_count == 0) {
                       s->async->events |= COMEDI_CB_EOA;
                       /* TODO: STOP_ACQUISITION_CALL_HERE!! */
-                      pcmuio_stop_intr(dev, s); 
+                      pcmuio_stop_intr(dev, s);
                     }
                   }
                 }
@@ -771,13 +770,13 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
             }
 
             comedi_spin_unlock_irqrestore(&subpriv->intr.spinlock, flags);
-            
+
             if (oldevents != s->async->events) {
-              comedi_event(dev, s, s->async->events);
+              comedi_event(dev, s);
             }
 
           }
-          
+
         }
       }
 
@@ -791,9 +790,9 @@ static irqreturn_t interrupt_pcmuio(int irq, void *d, struct pt_regs *regs)
 static void pcmuio_stop_intr(comedi_device *dev, comedi_subdevice *s)
 {
   int nports, firstport, asic, port;
-  
+
   if ( (asic = subpriv->intr.asic) < 0 ) return; /* not an interrupt subdev */
-  
+
   subpriv->intr.enabled_mask = 0;
   subpriv->intr.active = 0;
   s->async->inttrig = 0;
@@ -816,9 +815,9 @@ static int pcmuio_start_intr(comedi_device *dev, comedi_subdevice *s)
   } else {
     unsigned bits = 0, pol_bits = 0, n;
     int nports, firstport, asic, port;
-    comedi_cmd *cmd = &s->async->cmd;  
-    
-    if ( (asic = subpriv->intr.asic) < 0 ) return 1; /* not an interrupt 
+    comedi_cmd *cmd = &s->async->cmd;
+
+    if ( (asic = subpriv->intr.asic) < 0 ) return 1; /* not an interrupt
                                                         subdev */
     subpriv->intr.enabled_mask = 0;
     subpriv->intr.active = 1;
@@ -827,7 +826,7 @@ static int pcmuio_start_intr(comedi_device *dev, comedi_subdevice *s)
     if (cmd->chanlist) {
       for (n = 0; n < cmd->chanlist_len; n++) {
         bits |= (1U << CR_CHAN(cmd->chanlist[n]));
-        pol_bits |= 
+        pol_bits |=
           (CR_AREF(cmd->chanlist[n]) || CR_RANGE(cmd->chanlist[n]) ? 1U : 0U)
             << CR_CHAN(cmd->chanlist[n]);
       }
@@ -837,7 +836,7 @@ static int pcmuio_start_intr(comedi_device *dev, comedi_subdevice *s)
 
     switch_page(dev, asic, PAGE_ENAB);
     for (port = firstport; port < firstport+nports; ++port) {
-      unsigned enab = bits >> (subpriv->intr.first_chan + (port-firstport)*8) & 0xff, 
+      unsigned enab = bits >> (subpriv->intr.first_chan + (port-firstport)*8) & 0xff,
                pol = pol_bits >> (subpriv->intr.first_chan + (port-firstport)*8) & 0xff ;
       /* set enab intrs for this subdev.. */
       outb(enab, devpriv->asics[asic].iobase + REG_ENAB0 + port);
@@ -856,7 +855,7 @@ static int  pcmuio_cancel(comedi_device *dev, comedi_subdevice *s)
   if (subpriv->intr.active)  pcmuio_stop_intr(dev, s);
   comedi_spin_unlock_irqrestore(&subpriv->intr.spinlock, flags);
 
-  return 0;  
+  return 0;
 }
 
 /*
@@ -878,7 +877,7 @@ pcmuio_inttrig_start_intr(comedi_device *dev, comedi_subdevice *s, unsigned int
 	comedi_spin_unlock_irqrestore(&subpriv->intr.spinlock, flags);
 
 	if (event) {
-		comedi_event(dev, s, s->async->events);
+		comedi_event(dev, s);
 	}
 
 	return 1;
@@ -924,7 +923,7 @@ pcmuio_cmd(comedi_device *dev, comedi_subdevice *s)
 	comedi_spin_unlock_irqrestore(&subpriv->intr.spinlock, flags);
 
 	if (event) {
-		comedi_event(dev, s, s->async->events);
+		comedi_event(dev, s);
 	}
 
 	return 0;
@@ -955,7 +954,7 @@ pcmuio_cmdtest(comedi_device *dev, comedi_subdevice *s,
 	if (!cmd->convert_src || tmp != cmd->convert_src) err++;
 
 	tmp = cmd->scan_end_src;
-	cmd->scan_end_src &= TRIG_COUNT;  
+	cmd->scan_end_src &= TRIG_COUNT;
 	if (!cmd->scan_end_src || tmp != cmd->scan_end_src) err++;
 
 	tmp = cmd->stop_src;
