@@ -19,7 +19,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /*
-Driver: poc.o
+Driver: poc
 Description: Generic driver for very simple devices
 Author: ds
 Devices: [Keithley Metrabyte] DAC-02 (dac02), [Advantech] PCL-733 (pcl733),
@@ -52,7 +52,7 @@ static int pcl734_insn_bits(comedi_device *dev,comedi_subdevice *s,
 	comedi_insn *insn,lsampl_t *data);
 
 struct boarddef_struct{
-	char *name;
+	const char *name;
 	unsigned int iosize;
 	int (*setup)(comedi_device *);
 	int type;
@@ -61,9 +61,9 @@ struct boarddef_struct{
 	int (*winsn)(comedi_device *,comedi_subdevice *,comedi_insn *,lsampl_t *);
 	int (*rinsn)(comedi_device *,comedi_subdevice *,comedi_insn *,lsampl_t *);
 	int (*insnbits)(comedi_device *,comedi_subdevice *,comedi_insn *,lsampl_t *);
-	comedi_lrange* range;
+	const comedi_lrange *range;
 };
-static struct boarddef_struct boards[]={
+static const struct boarddef_struct boards[]={
 	{
 	name:		"dac02",
 	iosize:		8,
@@ -95,7 +95,7 @@ static struct boarddef_struct boards[]={
 	},
 };
 #define n_boards (sizeof(boards)/sizeof(boards[0]))
-#define this_board ((struct boarddef_struct *)dev->board_ptr)
+#define this_board ((const struct boarddef_struct *)dev->board_ptr)
 
 static comedi_driver driver_poc=
 {
@@ -103,7 +103,7 @@ static comedi_driver driver_poc=
 	module:		THIS_MODULE,
 	attach:		poc_attach,
 	detach:		poc_detach,
-	board_name:	boards,
+	board_name:	&boards[0].name,
 	num_names:	n_boards,
 	offset:		sizeof(boards[0]),
 };
@@ -208,7 +208,7 @@ static int pcl733_insn_bits(comedi_device *dev,comedi_subdevice *s,
 	comedi_insn *insn,lsampl_t *data)
 {
 	if(insn->n!=2)return -EINVAL;
-	
+
 	data[1] = inb(dev->iobase + 0);
 	data[1] |= (inb(dev->iobase + 1) << 8);
 	data[1] |= (inb(dev->iobase + 2) << 16);
