@@ -1,7 +1,7 @@
 /*
     comedi/drivers/rtd520.c
     Comedi driver for Real Time Devices (RTD) PCI4520/DM7520
-    
+
     COMEDI - Linux Control and Measurement Device Interface
     Copyright (C) 2001 David A. Schleef <ds@schleef.org>
 
@@ -20,7 +20,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /*
-Driver: rtd520.o
+Driver: rtd520
 Description: Real Time Devices PCI4520/DM7520
 Author: Dan Christian
 Devices: [Real Time Devices] DM7520HR-1 (DM7520), DM7520HR-8 (DM7520-8),
@@ -86,7 +86,7 @@ Configuration options:
 /*
   driver status:
 
-  Analog-In supports instruction and command mode.  
+  Analog-In supports instruction and command mode.
 
   With DMA, you can sample at 1.15Mhz with 70% idle on a 400Mhz K6-2
   (single channel, 64K read buffer).  I get random system lockups when
@@ -96,7 +96,7 @@ Configuration options:
 
   Without DMA, you can do 620Khz sampling with 20% idle on a 400Mhz K6-2
   (with a 256K read buffer).
-  
+
   Digital-IO and Analog-Out only support instruction mode.
 
 */
@@ -188,7 +188,7 @@ Configuration options:
 #define DMA_TRANSFER_BITS (\
 /* descriptors in PCI memory*/ 	PLX_DESC_IN_PCI_BIT \
 /* interrupt at end of block */ | PLX_INTR_TERM_COUNT \
-/* from board to PCI */		| PLX_XFER_LOCAL_TO_PCI) 
+/* from board to PCI */		| PLX_XFER_LOCAL_TO_PCI)
 
 /*======================================================================
   Comedi specific stuff
@@ -197,7 +197,7 @@ Configuration options:
 /*
   The board has 3 input modes and the gains of 1,2,4,...32 (, 64, 128)
 */
-static comedi_lrange rtd_ai_7520_range = { 18, {
+static const comedi_lrange rtd_ai_7520_range = { 18, {
 					/* +-5V input range gain steps */
     BIP_RANGE(5.0),
     BIP_RANGE(5.0/2),
@@ -223,7 +223,7 @@ static comedi_lrange rtd_ai_7520_range = { 18, {
 }};
 
 /* PCI4520 has two more gains (6 more entries) */
-static comedi_lrange rtd_ai_4520_range = { 24, {
+static const comedi_lrange rtd_ai_4520_range = { 24, {
 					/* +-5V input range gain steps */
     BIP_RANGE(5.0),
     BIP_RANGE(5.0/2),
@@ -254,7 +254,7 @@ static comedi_lrange rtd_ai_4520_range = { 24, {
 }};
 
 /* Table order matches range values */
-static comedi_lrange rtd_ao_range = { 4, {
+static const comedi_lrange rtd_ao_range = { 4, {
     RANGE(0, 5),
     RANGE(0, 10),
     RANGE(-5, 5),
@@ -265,7 +265,7 @@ static comedi_lrange rtd_ao_range = { 4, {
   Board descriptions
  */
 typedef struct rtdBoard_struct{
-    char	*name;			/* must be first */
+    const char	*name;			/* must be first */
     int		device_id;
     int		aiChans;
     int		aiBits;
@@ -275,7 +275,7 @@ typedef struct rtdBoard_struct{
     int		rangeUniStart;		/* start of +10V range */
 } rtdBoard;
 
-static rtdBoard rtd520Boards[] = {
+static const rtdBoard rtd520Boards[] = {
     {
 	name:		"DM7520",
 	device_id:	0x7520,
@@ -328,7 +328,7 @@ MODULE_DEVICE_TABLE(pci, rtd520_pci_table);
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((rtdBoard *)dev->board_ptr)
+#define thisboard ((const rtdBoard *)dev->board_ptr)
 
 /*
    This structure is for data unique to this hardware driver.
@@ -713,7 +713,7 @@ static comedi_driver rtd520Driver={
     module:		THIS_MODULE,
     attach:		rtd_attach,
     detach:		rtd_detach,
-    board_name:		rtd520Boards,
+    board_name:		&rtd520Boards[0].name,
     offset:		sizeof(rtdBoard),
     num_names:		sizeof(rtd520Boards) / sizeof(rtdBoard),
 };
@@ -734,7 +734,7 @@ static int rtd_ai_cmd ( comedi_device *dev, comedi_subdevice *s);
 static int rtd_ai_cancel ( comedi_device *dev, comedi_subdevice *s);
 //static int rtd_ai_poll (comedi_device *dev,comedi_subdevice *s);
 static int rtd_ns_to_timer (unsigned int *ns, int roundMode);
-static irqreturn_t rtd_interrupt ( int irq, void *d, struct pt_regs *regs);
+static irqreturn_t rtd_interrupt ( int irq, void *d PT_REGS_ARG);
 
 
 /*
@@ -816,9 +816,9 @@ static int rtd_attach (
      * Initialize base addresses
      */
     /* Get the physical address from PCI config */
-	physLas0 = pci_resource_start(devpriv->pci_dev, LAS0_PCIINDEX); 
-	physLas1 = pci_resource_start(devpriv->pci_dev, LAS1_PCIINDEX); 
-	physLcfg = pci_resource_start(devpriv->pci_dev, LCFG_PCIINDEX); 
+	physLas0 = pci_resource_start(devpriv->pci_dev, LAS0_PCIINDEX);
+	physLas1 = pci_resource_start(devpriv->pci_dev, LAS1_PCIINDEX);
+	physLcfg = pci_resource_start(devpriv->pci_dev, LCFG_PCIINDEX);
     /* Now have the kernel map this into memory */
 					/* ASSUME page aligned */
     devpriv->las0 = ioremap_nocache(physLas0, LAS0_PCISIZE);
@@ -879,7 +879,7 @@ static int rtd_attach (
     dev->read_subdev=s;
     /* analog input subdevice */
     s->type=COMEDI_SUBD_AI;
-    s->subdev_flags=SDF_READABLE|SDF_GROUND|SDF_COMMON|SDF_DIFF;
+    s->subdev_flags = SDF_READABLE | SDF_GROUND | SDF_COMMON | SDF_DIFF | SDF_CMD_READ;
     s->n_chan=thisboard->aiChans;
     s->maxdata=(1<<thisboard->aiBits)-1;
     if (thisboard->aiMaxGain <= 32) {
@@ -922,9 +922,30 @@ static int rtd_attach (
     s->n_chan=3;
     s->maxdata=0xffff;
 
+					/* initialize board, per RTD spec */
+					/* also, initialize shadow registers */
+    RtdResetBoard (dev);
+    comedi_udelay (100);			/* needed? */
+    RtdPlxInterruptWrite (dev, 0);
+    RtdInterruptMask (dev,0);		/* and sets shadow */
+    RtdInterruptClearMask (dev,~0);	/* and sets shadow */
+    RtdInterruptClear(dev);		/* clears bits set by mask */
+    RtdInterruptOverrunClear(dev);
+    RtdClearCGT (dev);
+    RtdAdcClearFifo (dev);
+    RtdDacClearFifo (dev,0);
+    RtdDacClearFifo (dev,1);
+					/* clear digital IO fifo*/
+    RtdDioStatusWrite (dev, 0);		/* safe state, set shadow */
+    RtdUtcCtrlPut (dev, 0, 0x30);	/* safe state, set shadow */
+    RtdUtcCtrlPut (dev, 1, 0x30);	/* safe state, set shadow */
+    RtdUtcCtrlPut (dev, 2, 0x30);	/* safe state, set shadow */
+    RtdUtcCtrlPut (dev, 3, 0);		/* safe state, set shadow */
+    /* TODO: set user out source ??? */
+
 	/* check if our interrupt is available and get it */
 	if((ret=comedi_request_irq (devpriv->pci_dev->irq, rtd_interrupt,
-		SA_SHIRQ, "rtd520", dev)) < 0) {
+		IRQF_SHARED, "rtd520", dev)) < 0) {
 		printk("Could not get interrupt! (%u)\n", devpriv->pci_dev->irq);
 		return ret;
 	}
@@ -993,26 +1014,6 @@ static int rtd_attach (
 #else /* USE_DMA */
     printk("\n");			/* end configuration line */
 #endif /* USE_DMA */
-	
-					/* initialize board, per RTD spec */
-					/* also, initialize shadow registers */
-    RtdResetBoard (dev);
-    comedi_udelay (100);			/* needed? */
-    RtdInterruptMask (dev,0);		/* and sets shadow */
-    RtdInterruptClearMask (dev,~0);	/* and sets shadow */
-    RtdInterruptClear(dev);		/* clears bits set by mask */
-    RtdInterruptOverrunClear(dev);
-    RtdClearCGT (dev);
-    RtdAdcClearFifo (dev);
-    RtdDacClearFifo (dev,0);
-    RtdDacClearFifo (dev,1);
-					/* clear digital IO fifo*/
-    RtdDioStatusWrite (dev, 0);		/* safe state, set shadow */
-    RtdUtcCtrlPut (dev, 0, 0x30);	/* safe state, set shadow */
-    RtdUtcCtrlPut (dev, 1, 0x30);	/* safe state, set shadow */
-    RtdUtcCtrlPut (dev, 2, 0x30);	/* safe state, set shadow */
-    RtdUtcCtrlPut (dev, 3, 0);		/* safe state, set shadow */
-    /* TODO: set user out source ??? */
 
     if (dev->irq) {			/* enable plx9080 interrupts */
 	RtdPlxInterruptWrite (dev, ICS_PIE | ICS_PLIE);
@@ -1074,7 +1075,7 @@ static int rtd_attach (
 
 /*
  * _detach is called to deconfigure a device.  It should deallocate
- * resources.  
+ * resources.
  * This function is also called when _attach() fails, so it should be
  * careful not to release resources that were not necessarily
  * allocated by _attach().  dev->private and dev->subdevices are
@@ -1157,13 +1158,13 @@ static int rtd_detach (
 		{
 			pci_release_regions(devpriv->pci_dev);
 			pci_disable_device(devpriv->pci_dev);
-		}    
+		}
 		pci_dev_put(devpriv->pci_dev);
 	}
     }
 
     printk("comedi%d: rtd520: removed.\n",dev->minor);
-	
+
     return 0;
 }
 
@@ -1260,9 +1261,9 @@ static int rtd_ai_rinsn (
 					/* clear any old fifo data */
     RtdAdcClearFifo (dev);
 
-    /* write channel to multiplexer and clear channel gain table */ 
+    /* write channel to multiplexer and clear channel gain table */
     rtd_load_channelgain_list (dev, 1, &insn->chanspec);
-	
+
 					/* set conversion source */
     RtdAdcConversionSource (dev, 0);	/* software */
 
@@ -1501,14 +1502,18 @@ static int ai_process_dma (
 */
 static irqreturn_t rtd_interrupt (
     int irq,				/* interrupt number (ignored) */
-    void *d,				/* our data */
-    struct pt_regs *regs)		/* cpu context (ignored) */
+    void *d				/* our data */
+    PT_REGS_ARG)		/* cpu context (ignored) */
 {
 	comedi_device *dev = d;		/* must be called "dev" for devpriv */
 	u16 status;
 	u16 fifoStatus;
 	comedi_subdevice *s = dev->subdevices + 0; /* analog in subdevice */
 
+	if (!dev->attached) {
+		return IRQ_NONE;
+	}
+
 	devpriv->intCount++;		/* DEBUG statistics */
 
 	fifoStatus = RtdFifoStatus (dev);
@@ -1541,7 +1546,7 @@ static irqreturn_t rtd_interrupt (
 		DPRINTK("rtd520: Samples Done (DMA).\n");
 		goto transferDone;
 	    }
-	    comedi_event (dev, s, s->async->events);
+	    comedi_event (dev, s);
 	} else {
 	    /*DPRINTK ("rtd520: No DMA ready: istatus %x\n", istatus);*/
 	}
@@ -1572,7 +1577,7 @@ static irqreturn_t rtd_interrupt (
 			(fifoStatus ^ 0x6666) & 0x7777); /* should be all 0s */
 		goto transferDone;
 	    }
-	    comedi_event (dev, s, s->async->events);
+	    comedi_event (dev, s);
 	} else if (devpriv->transCount > 0) {	/* read often */
 	    /*DPRINTK("rtd520: Sample int, reading %d  fifo_status 0x%x\n",
 	      devpriv->transCount, (fifoStatus ^ 0x6666) & 0x7777);*/
@@ -1587,7 +1592,7 @@ static irqreturn_t rtd_interrupt (
 			    (fifoStatus ^ 0x6666) & 0x7777);
 		    goto transferDone;
 		}
-		comedi_event (dev, s, s->async->events);
+		comedi_event (dev, s);
 	    }
 	} else {			/* wait for 1/2 FIFO (old)*/
 	    DPRINTK("rtd520: Sample int.  Wait for 1/2. fifo_status 0x%x\n",
@@ -1644,7 +1649,7 @@ transferDone:
     }
 
     s->async->events |= COMEDI_CB_EOA;/* signal end to comedi */
-    comedi_event (dev, s, s->async->events);
+    comedi_event (dev, s);
 
 					/* clear the interrupt */
     status = RtdInterruptStatus (dev);
@@ -1676,7 +1681,7 @@ static int rtd_ai_poll (comedi_device *dev,comedi_subdevice *s)
   cmdtest tests a particular command to see if it is valid.
   Using the cmdtest ioctl, a user can create a valid cmd
   and then have it executed by the cmd ioctl (asyncronously).
- 
+
   cmdtest returns 1,2,3,4 or 0, depending on which tests
   the command passes.
 */
@@ -2050,7 +2055,7 @@ static int rtd_ai_cmd (
     else {			       /* 1/2 FIFO transfers */
 #ifdef USE_DMA
 	devpriv->flags |= DMA0_ACTIVE;
-	
+
 	/* point to first transfer in ring */
 	devpriv->dma0Offset = 0;
 	RtdDma0Mode (dev, DMA_MODE_BITS);
@@ -2064,7 +2069,7 @@ static int rtd_ai_cmd (
 	/* Must be 2 steps.  See PLX app note about "Starting a DMA transfer"*/
 	RtdDma0Control (dev, PLX_DMA_EN_BIT); /* enable DMA (clear INTR?) */
 	RtdDma0Control (dev, PLX_DMA_EN_BIT | PLX_DMA_START_BIT); /*start DMA*/
-	DPRINTK ("rtd520: Using DMA0 transfers. plxInt %x RtdInt %x\n", 
+	DPRINTK ("rtd520: Using DMA0 transfers. plxInt %x RtdInt %x\n",
 		 RtdPlxInterruptRead (dev), devpriv->intMask);
 #else /* USE_DMA */
 	RtdInterruptMask (dev, IRQM_ADC_ABOUT_CNT );
@@ -2232,10 +2237,10 @@ static int rtd_ao_rinsn (
     return i;
 }
 
-/* 
+/*
    Write a masked set of bits and the read back the port.
    We track what the bits should be (i.e. we don't read the port first).
-   
+
    DIO devices are slightly special.  Although it is possible to
  * implement the insn_read/insn_write interface, it is much more
  * useful to applications if you implement the insn_bits interface.
@@ -2255,7 +2260,7 @@ static int rtd_dio_insn_bits (
     if (data[0]) {
 	s->state &= ~data[0];
 	s->state |= data[0]&data[1];
-	
+
 	/* Write out the new digital output lines */
 	RtdDio0Write (dev, s->state);
     }
@@ -2281,7 +2286,7 @@ static int rtd_dio_insn_config (
 
     /* The input or output configuration of each digital line is
      * configured by a special insn_config instruction.  chanspec
-     * contains the channel to be changed, and data[0] contains the 
+     * contains the channel to be changed, and data[0] contains the
      * value COMEDI_INPUT or COMEDI_OUTPUT. */
 	switch(data[0])
 	{
