@@ -21,7 +21,7 @@
 
 */
 /*
-Driver: skel.o
+Driver: skel
 Description: Skeleton driver, an example for driver writers
 Devices:
 Author: ds
@@ -90,12 +90,12 @@ Configuration Options:
  * Some drivers use arrays such as this, other do not.
  */
 typedef struct skel_board_struct{
-	char *name;
+	const char *name;
 	int ai_chans;
 	int ai_bits;
 	int have_dio;
 }skel_board;
-static skel_board skel_boards[] = {
+static const skel_board skel_boards[] = {
 	{
 	name:		"skel-100",
 	ai_chans:	16,
@@ -125,7 +125,7 @@ MODULE_DEVICE_TABLE(pci, skel_pci_table);
 /*
  * Useful for shorthand access to the particular board structure
  */
-#define thisboard ((skel_board *)dev->board_ptr)
+#define thisboard ((const skel_board *)dev->board_ptr)
 
 /* this structure is for data unique to this hardware driver.  If
    several hardware drivers keep similar information in this structure,
@@ -176,7 +176,7 @@ static comedi_driver driver_skel={
 	 * the type of board in software.  ISA PnP, PCI, and PCMCIA
 	 * devices are such boards.
 	 */
-	board_name:	skel_boards,
+	board_name:	&skel_boards[0].name,
 	offset:		sizeof(skel_board),
 	num_names:	sizeof(skel_boards) / sizeof(skel_board),
 };
@@ -203,7 +203,7 @@ static int skel_attach(comedi_device *dev,comedi_devconfig *it)
 	comedi_subdevice *s;
 
 	printk("comedi%d: skel: ",dev->minor);
-	
+
 /*
  * If you can probe the device to determine what device in a series
  * it is, this is the place to do it.  Otherwise, dev->board_ptr
@@ -243,7 +243,8 @@ static int skel_attach(comedi_device *dev,comedi_devconfig *it)
 	s->len_chanlist=16;  /* This is the maximum chanlist length that
 				the board can handle */
 	s->insn_read = skel_ai_rinsn;
-	//s->do_cmd = skel_ai_cmd;
+// 	s->subdev_flags |= SDF_CMD_READ;
+// 	s->do_cmd = skel_ai_cmd;
 	s->do_cmdtest = skel_ai_cmdtest;
 
 	s=dev->subdevices+1;
@@ -269,16 +270,16 @@ static int skel_attach(comedi_device *dev,comedi_devconfig *it)
 	}else{
 		s->type = COMEDI_SUBD_UNUSED;
 	}
-	
+
 	printk("attached\n");
 
-	return 1;
+	return 0;
 }
 
 
 /*
  * _detach is called to deconfigure a device.  It should deallocate
- * resources.  
+ * resources.
  * This function is also called when _attach() fails, so it should be
  * careful not to release resources that were not necessarily
  * allocated by _attach().  dev->private and dev->subdevices are
@@ -287,7 +288,7 @@ static int skel_attach(comedi_device *dev,comedi_devconfig *it)
 static int skel_detach(comedi_device *dev)
 {
 	printk("comedi%d: skel: remove\n",dev->minor);
-	
+
 	return 0;
 }
 
