@@ -30,7 +30,7 @@
 int comedi_get_n_subdevices(comedi_t *d)
 {
 	comedi_device *dev = (comedi_device *)d;
-	
+
 	return dev->n_subdevices;
 }
 
@@ -39,14 +39,14 @@ int comedi_get_version_code(comedi_t *d)
 	return COMEDI_VERSION_CODE;
 }
 
-char *comedi_get_driver_name(comedi_t *d)
+const char *comedi_get_driver_name(comedi_t *d)
 {
 	comedi_device *dev = (comedi_device *)d;
 
 	return dev->driver->driver_name;
 }
 
-char *comedi_get_board_name(comedi_t *d)
+const char *comedi_get_board_name(comedi_t *d)
 {
 	comedi_device *dev = (comedi_device *)d;
 
@@ -151,7 +151,7 @@ int comedi_get_krange(comedi_t *d,unsigned int subdevice,unsigned int chan,unsig
 {
 	comedi_device *dev = (comedi_device *)d;
 	comedi_subdevice *s = dev->subdevices + subdevice;
-	comedi_lrange *lr;
+	const comedi_lrange *lr;
 
 	if (s->range_table_list) {
 		lr=s->range_table_list[chan];
@@ -191,8 +191,7 @@ int comedi_get_buffer_contents( comedi_t *d, unsigned int subdevice )
 	if( subdevice >= dev->n_subdevices ) return -1;
 	async = s->async;
 	if(async == NULL) return 0;
-	num_bytes = comedi_buf_read_n_available(s);
-	comedi_buf_munge( dev, s, async->buf_write_count - async->munge_count );
+	num_bytes = comedi_buf_read_n_available(s->async);
 	return num_bytes;
 }
 
@@ -211,7 +210,8 @@ int comedi_set_user_int_count(comedi_t *d,unsigned int subdevice,unsigned int bu
 
 	num_bytes =  buf_user_count - async->buf_read_count;
 	if( num_bytes < 0 ) return -1;
-	comedi_buf_read_free( async, num_bytes );
+	comedi_buf_read_alloc(async, num_bytes);
+	comedi_buf_read_free(async, num_bytes);
 
 	return 0;
 }
@@ -227,7 +227,8 @@ int comedi_mark_buffer_read( comedi_t *d, unsigned int subdevice,
 	async = s->async;
 	if( async == NULL ) return -1;
 
-	comedi_buf_read_free( async, num_bytes );
+	comedi_buf_read_alloc(async, num_bytes);
+	comedi_buf_read_free(async, num_bytes);
 
 	return 0;
 }
@@ -239,12 +240,11 @@ int comedi_mark_buffer_written( comedi_t *d, unsigned int subdevice,
 	comedi_subdevice *s = dev->subdevices + subdevice;
 	comedi_async *async;
 	int bytes_written;
-	
+
 	if( subdevice >= dev->n_subdevices ) return -1;
 	async = s->async;
 	if( async == NULL ) return -1;
 	bytes_written = comedi_buf_write_alloc(async, num_bytes);
-	comedi_buf_munge(dev, s, async->buf_write_alloc_count - async->munge_count);
 	comedi_buf_write_free(async, bytes_written);
 	if(bytes_written != num_bytes) return -1;
 	return 0;
