@@ -54,7 +54,7 @@
 struct comedi_irq_struct {
 	int rt;
 	int irq;
-	irqreturn_t (*handler)(int irq,void *dev_id,struct pt_regs *regs);
+	irqreturn_t (*handler)(int irq,void *dev_id PT_REGS_ARG);
 	unsigned long flags;
 	const char *device;
 	comedi_device *dev_id;
@@ -65,7 +65,8 @@ static int comedi_rt_release_irq(struct comedi_irq_struct *it);
 
 static struct comedi_irq_struct *comedi_irqs[NR_IRQS];
 
-int comedi_request_irq(unsigned irq, irqreturn_t (*handler)(int, void *,struct pt_regs *),
+int comedi_request_irq(unsigned irq, irqreturn_t (*handler)(int, void *
+			PT_REGS_ARG),
 		unsigned long flags,const char *device,comedi_device *dev_id)
 {
 	struct comedi_irq_struct *it;
@@ -73,12 +74,12 @@ int comedi_request_irq(unsigned irq, irqreturn_t (*handler)(int, void *,struct p
 	/* null shared interrupt flag, since rt interrupt handlers do not
 	* support it, and this version of comedi_request_irq() is only
 	* called for kernels with rt support */
-	unsigned long unshared_flags = flags & ~SA_SHIRQ;
+	unsigned long unshared_flags = flags & ~IRQF_SHARED;
 	
 	ret = request_irq(irq, handler, unshared_flags, device, dev_id);
 	if(ret<0){
 		// we failed, so fall back on allowing shared interrupt (which we won't ever make RT)
-		if(flags & SA_SHIRQ)
+		if(flags & IRQF_SHARED)
 		{
 			rt_printk("comedi: cannot get unshared interrupt, will not use RT interrupts.\n");
 			ret=request_irq(irq, handler, flags, device, dev_id);
@@ -88,10 +89,9 @@ int comedi_request_irq(unsigned irq, irqreturn_t (*handler)(int, void *,struct p
 		}
 	}else
 	{
-		it = kmalloc(sizeof(struct comedi_irq_struct), GFP_KERNEL);
+		it = kzalloc(sizeof(struct comedi_irq_struct), GFP_KERNEL);
 		if(!it)
 			return -ENOMEM;
-		memset(it, 0, sizeof(struct comedi_irq_struct));
 	
 		it->handler=handler;
 		it->irq=irq;
@@ -194,7 +194,7 @@ static void handle_void_irq(int irq)
 		rt_printk("comedi: null irq struct?\n");
 		return;
 	}
-	it->handler(irq, it->dev_id, NULL);
+	it->handler(irq, it->dev_id PT_REGS_NULL);
 	rt_enable_irq(irq);	//needed by rtai-adeos, seems like it shouldn't hurt earlier versions
 }
 
@@ -312,7 +312,7 @@ static void fusion_handle_irq(unsigned int irq, void *cookie)
 {
 	struct comedi_irq_struct *it = cookie;
 
-	it->handler(irq, it->dev_id, NULL);
+	it->handler(irq, it->dev_id PT_REGS_NULL);
 	rthal_irq_enable(irq);
 }
 
@@ -345,13 +345,13 @@ void comedi_rt_cleanup(void)
 /* RTLinux section */
 #ifdef CONFIG_COMEDI_RTL
 
-static unsigned int handle_rtl_irq(unsigned int irq,struct pt_regs *regs)
+static unsigned int handle_rtl_irq(unsigned int irq PT_REGS_ARG)
 {
 	struct comedi_irq_struct *it;
 
 	it = comedi_irqs[irq];
 	if( it == NULL ) return 0;
-	it->handler( irq, it->dev_id, regs );
+	it->handler( irq, it->dev_id PT_REGS_NULL );
 	rtl_hard_enable_irq(irq);
 	return 0;
 }
