@@ -38,7 +38,7 @@
 #include <linux/vmalloc.h>
 #include <linux/mm.h>
 #include <linux/interrupt.h>
-
+#include <linux/dma-mapping.h>
 #include <asm/uaccess.h>
 #include <asm/io.h>
 
@@ -96,6 +96,7 @@ typedef struct comedi_lrange_struct comedi_lrange;
 
 
 struct comedi_subdevice_struct{
+	comedi_device *device;
 	int type;
 	int n_chan;
 	volatile int subdev_flags;
@@ -107,20 +108,21 @@ struct comedi_subdevice_struct{
 
 	void *lock;
 	void *busy;
-	volatile unsigned int runflags;
+	unsigned runflags;
+	spinlock_t runflags_lock;
 
 	int io_bits;
 
 	lsampl_t maxdata;		/* if maxdata==0, use list */
-	lsampl_t *maxdata_list;		/* list is channel specific */
+	const lsampl_t *maxdata_list;	/* list is channel specific */
 
 	unsigned int flags;
-	unsigned int *flaglist;
+	const unsigned int *flaglist;
 
 	unsigned int settling_time_0;
 
-	comedi_lrange *range_table;
-	comedi_lrange **range_table_list;
+	const comedi_lrange *range_table;
+	const comedi_lrange * const *range_table_list;
 
 	unsigned int *chanlist;		/* driver-owned chanlist (not used) */
 
@@ -141,20 +143,34 @@ struct comedi_subdevice_struct{
 
 	void (*munge)( comedi_device *dev, comedi_subdevice *s, void *data,
 		unsigned int num_bytes, unsigned int start_chan_index );
+	enum dma_data_direction async_dma_dir;
 
 	unsigned int state;
+
+	struct class_device *class_dev;
+};
+
+struct comedi_buf_page
+{
+	void *virt_addr;
+	dma_addr_t dma_addr;
 };
 
 struct comedi_async_struct{
+	comedi_subdevice *subdevice;
+
 	void		*prealloc_buf;		/* pre-allocated buffer */
 	unsigned int	prealloc_bufsz;		/* buffer size, in bytes */
-	unsigned long	*buf_page_list;		/* physical address of each page */
+	struct comedi_buf_page *buf_page_list;		/* virtual and dma address of each page */
+	unsigned n_buf_pages;	/* num elements in buf_page_list */
+
 	unsigned int	max_bufsize;		/* maximum buffer size, bytes */
 	unsigned int	mmap_count;	/* current number of mmaps of prealloc_buf */
 
-	volatile unsigned int buf_write_count;	/* byte count for writer (write completed) */
-	volatile unsigned int buf_write_alloc_count;	/* byte count for writer (allocated for writing) */
-	volatile unsigned int buf_read_count;	/* byte count for reader (read completed)*/
+	unsigned int buf_write_count;	/* byte count for writer (write completed) */
+	unsigned int buf_write_alloc_count;	/* byte count for writer (allocated for writing) */
+	unsigned int buf_read_count;	/* byte count for reader (read completed)*/
+	unsigned int buf_read_alloc_count;	/* byte count for reader (allocated for reading)*/
 
 	unsigned int buf_write_ptr;	/* buffer marker for writer */
 	unsigned int buf_read_ptr;	/* buffer marker for reader */
@@ -173,6 +189,8 @@ struct comedi_async_struct{
 
 	comedi_cmd cmd;
 
+	wait_queue_head_t wait_head;
+
 	// callback stuff
 	unsigned int cb_mask;
 	int (*cb_func)(unsigned int flags,void *);
@@ -184,14 +202,14 @@ struct comedi_async_struct{
 struct comedi_driver_struct{
 	struct comedi_driver_struct *next;
 
-	char *driver_name;
+	const char *driver_name;
 	struct module *module;
 	int (*attach)(comedi_device *,comedi_devconfig *);
 	int (*detach)(comedi_device *);
 
 	/* number of elements in board_name and board_id arrays */
 	unsigned int num_names;
-	void *board_name;
+	const char * const *board_name;
 	/* offset in bytes from one board name pointer to the next */
 	int offset;
 };
@@ -200,8 +218,14 @@ struct comedi_device_struct{
 	int use_count;
 	comedi_driver *driver;
 	void *private;
-	unsigned int minor;
-	char *board_name;
+
+	struct class_device *class_dev;
+	unsigned minor;
+	/* hw_dev is passed to dma_alloc_coherent when allocating async buffers for subdevices
+	that have async_dma_dir set to something other than DMA_NONE */
+	struct device *hw_dev;
+
+	const char *board_name;
 	const void * board_ptr;
 	int attached;
 	int rt;
@@ -210,17 +234,13 @@ struct comedi_device_struct{
 
 	int n_subdevices;
 	comedi_subdevice *subdevices;
-	int options[COMEDI_NDEVCONFOPTS];
 
 	/* dumb */
 	unsigned long iobase;
 	unsigned int irq;
 
 	comedi_subdevice *read_subdev;
-	wait_queue_head_t read_wait;
-
 	comedi_subdevice *write_subdev;
-	wait_queue_head_t write_wait;
 
 	struct fasync_struct *async_queue;
 
@@ -228,7 +248,11 @@ struct comedi_device_struct{
 	void (*close)(comedi_device *dev);
 };
 
-
+struct comedi_inode_private
+{
+	comedi_device *device;
+	comedi_subdevice *subdevice;
+};
 
 extern comedi_device *comedi_devices;
 extern spinlock_t big_comedi_lock;
@@ -243,12 +267,74 @@ static const int comedi_debug = 0;
  * function prototypes
  */
 
-void comedi_event(comedi_device *dev,comedi_subdevice *s,unsigned int mask);
+void comedi_event(comedi_device *dev, comedi_subdevice *s);
 void comedi_error(const comedi_device *dev,const char *s);
 
-static inline comedi_device * comedi_get_device_by_minor( unsigned int minor )
+/* we can expand the number of bits used to encode devices/subdevices into
+ the minor number soon, after more distros support > 8 bit minor numbers
+ (like after Debian Etch gets released) */
+enum comedi_minor_bits
 {
-	return comedi_devices + minor;
+	COMEDI_DEVICE_MINOR_MASK = 0xf,
+	COMEDI_SUBDEVICE_MINOR_MASK = 0xf0
+};
+static const unsigned COMEDI_SUBDEVICE_MINOR_SHIFT = 4;
+static const unsigned COMEDI_SUBDEVICE_MINOR_OFFSET = 1;
+static const unsigned COMEDI_NUM_MINORS = 0x100;
+
+static inline comedi_device * comedi_get_device_by_minor(unsigned minor)
+{
+	unsigned device_index;
+	if(minor >= COMEDI_NUM_MINORS) return NULL;
+	device_index = minor & COMEDI_DEVICE_MINOR_MASK;
+	if(device_index >= COMEDI_NDEVICES) return NULL;
+	return comedi_devices + device_index;
+}
+
+static inline comedi_subdevice * comedi_get_subdevice_by_minor(unsigned minor)
+{
+	unsigned subdevice_index;
+	comedi_device * dev;
+
+	if((minor & COMEDI_SUBDEVICE_MINOR_MASK) == 0) return NULL;
+	dev = comedi_get_device_by_minor(minor);
+	if(dev == NULL) return NULL;
+	subdevice_index = ((minor & COMEDI_SUBDEVICE_MINOR_MASK) >> COMEDI_SUBDEVICE_MINOR_SHIFT) - COMEDI_SUBDEVICE_MINOR_OFFSET;
+	if(subdevice_index >= dev->n_subdevices) return NULL;
+	return dev->subdevices + subdevice_index;
+}
+
+static inline comedi_subdevice* comedi_get_read_subdevice(comedi_device *dev, unsigned minor)
+{
+	comedi_subdevice *read_subdev = comedi_get_subdevice_by_minor(minor);
+	if(read_subdev == NULL)
+	{
+		read_subdev = dev->read_subdev;
+	}
+	if(read_subdev == NULL || read_subdev->async == NULL || (read_subdev->subdev_flags & SDF_CMD_READ) == 0)
+		return NULL;
+	return read_subdev;
+}
+
+static inline comedi_subdevice* comedi_get_write_subdevice(comedi_device *dev, unsigned minor)
+{
+	comedi_subdevice *write_subdev = comedi_get_subdevice_by_minor(minor);
+	if(write_subdev == NULL)
+	{
+		write_subdev = dev->write_subdev;
+	}
+	if(write_subdev == NULL || write_subdev->async == NULL || (write_subdev->subdev_flags & SDF_CMD_WRITE) == 0)
+		return NULL;
+	return write_subdev;
+}
+
+static inline unsigned comedi_construct_minor_for_subdevice(comedi_device *dev, unsigned subdevice_index)
+{
+	unsigned minor = 0;
+	minor |= dev->minor & COMEDI_DEVICE_MINOR_MASK;
+	minor |= ((subdevice_index + COMEDI_SUBDEVICE_MINOR_OFFSET) << COMEDI_SUBDEVICE_MINOR_SHIFT) & COMEDI_SUBDEVICE_MINOR_MASK;
+	BUG_ON(minor >= COMEDI_NUM_MINORS);
+	return minor;
 }
 
 int comedi_device_detach(comedi_device *dev);
@@ -276,10 +362,14 @@ static inline void comedi_proc_cleanup(void) {}
 #endif
 
 /* subdevice runflags */
-#define SRF_USER		0x00000001
-#define SRF_RT			0x00000002
-/* indicates an COMEDI_CB_ERROR event has occurred since the last command was started */
-#define SRF_ERROR		0x00000004
+enum subdevice_runflags
+{
+	SRF_USER = 0x00000001,
+	SRF_RT = 0x00000002,
+	/* indicates an COMEDI_CB_ERROR event has occurred since the last command was started */
+	SRF_ERROR = 0x00000004,
+	SRF_RUNNING = 0x08000000
+};
 
 /*
    various internal comedi functions
@@ -287,6 +377,8 @@ static inline void comedi_proc_cleanup(void) {}
 
 int do_rangeinfo_ioctl(comedi_device *dev,comedi_rangeinfo *arg);
 int check_chanlist(comedi_subdevice *s,int n,unsigned int *chanlist);
+void comedi_set_subdevice_runflags(comedi_subdevice *s, unsigned mask, unsigned bits);
+unsigned comedi_get_subdevice_runflags(comedi_subdevice *s);
 
 /* range stuff */
 
@@ -298,12 +390,12 @@ int check_chanlist(comedi_subdevice *s,int n,unsigned int *chanlist);
 #define BIP_RANGE(a)		{-(a)*1e6,(a)*1e6,0}
 #define UNI_RANGE(a)		{0,(a)*1e6,0}
 
-extern comedi_lrange range_bipolar10;
-extern comedi_lrange range_bipolar5;
-extern comedi_lrange range_bipolar2_5;
-extern comedi_lrange range_unipolar10;
-extern comedi_lrange range_unipolar5;
-extern comedi_lrange range_unknown;
+extern const comedi_lrange range_bipolar10;
+extern const comedi_lrange range_bipolar5;
+extern const comedi_lrange range_bipolar2_5;
+extern const comedi_lrange range_unipolar10;
+extern const comedi_lrange range_unipolar5;
+extern const comedi_lrange range_unknown;
 
 #define range_digital		range_unipolar5
 
@@ -324,22 +416,26 @@ struct comedi_lrange_struct{
 
 static inline int alloc_subdevices(comedi_device *dev, unsigned int num_subdevices)
 {
-	int size=sizeof(comedi_subdevice)*num_subdevices;
+	unsigned i;
 
 	dev->n_subdevices = num_subdevices;
-	dev->subdevices=kmalloc(size,GFP_KERNEL);
+	dev->subdevices = kcalloc(num_subdevices,sizeof(comedi_subdevice),GFP_KERNEL);
 	if(!dev->subdevices)
 		return -ENOMEM;
-	memset(dev->subdevices,0,size);
+	for(i = 0; i < num_subdevices; ++i)
+	{
+		dev->subdevices[i].device = dev;
+		dev->subdevices[i].async_dma_dir = DMA_NONE;
+		spin_lock_init(&dev->subdevices[i].runflags_lock);
+	}
 	return 0;
 }
 
 static inline int alloc_private(comedi_device *dev,int size)
 {
-	dev->private=kmalloc(size,GFP_KERNEL);
+	dev->private=kzalloc(size,GFP_KERNEL);
 	if(!dev->private)
 		return -ENOMEM;
-	memset(dev->private,0,size);
 	return 0;
 }
 
@@ -351,21 +447,46 @@ static inline unsigned int bytes_per_sample( const comedi_subdevice *subd )
 		return sizeof( sampl_t );
 }
 
+/* must be used in attach to set dev->hw_dev if you wish to dma directly
+into comedi's buffer */
+static inline void comedi_set_hw_dev(comedi_device *dev, struct device *hw_dev)
+{
+	if(dev->hw_dev)
+	{
+		put_device(dev->hw_dev);
+	}
+	dev->hw_dev = hw_dev;
+	if(dev->hw_dev)
+	{
+		dev->hw_dev = get_device(dev->hw_dev);
+		BUG_ON(dev->hw_dev == NULL);
+	}
+}
+
 int comedi_buf_put(comedi_async *async, sampl_t x);
 int comedi_buf_get(comedi_async *async, sampl_t *x);
 
-unsigned int comedi_buf_write_n_available(comedi_subdevice *s);
+unsigned int comedi_buf_write_n_available(comedi_async *async);
 unsigned int comedi_buf_write_alloc(comedi_async *async, unsigned int nbytes);
 unsigned int comedi_buf_write_alloc_strict(comedi_async *async, unsigned int nbytes);
-void comedi_buf_write_free(comedi_async *async, unsigned int nbytes);
-void comedi_buf_read_free(comedi_async *async, unsigned int nbytes);
-unsigned int comedi_buf_read_n_available(comedi_subdevice *s);
+unsigned comedi_buf_write_free(comedi_async *async, unsigned int nbytes);
+unsigned comedi_buf_read_alloc(comedi_async *async, unsigned nbytes);
+unsigned comedi_buf_read_free(comedi_async *async, unsigned int nbytes);
+unsigned int comedi_buf_read_n_available(comedi_async *async);
 void comedi_buf_memcpy_to( comedi_async *async, unsigned int offset, const void *source,
 	unsigned int num_bytes );
 void comedi_buf_memcpy_from( comedi_async *async, unsigned int offset, void *destination,
 	unsigned int num_bytes );
-unsigned int comedi_buf_munge( comedi_device *dev, comedi_subdevice *s,
-	unsigned int num_bytes );
+static inline unsigned comedi_buf_write_n_allocated(comedi_async *async)
+{
+	return async->buf_write_alloc_count - async->buf_write_count;
+}
+static inline unsigned comedi_buf_read_n_allocated(comedi_async *async)
+{
+	return async->buf_read_alloc_count - async->buf_read_count;
+}
+
+void comedi_reset_async_buf(comedi_async *async);
 
 static inline void* comedi_aux_data(int options[], int n)
 {
@@ -389,6 +510,7 @@ static inline void* comedi_aux_data(int options[], int n)
 		address += options[COMEDI_DEVCONF_AUX_DATA1_LENGTH];
 	if(n >= 3)
 		address += options[COMEDI_DEVCONF_AUX_DATA2_LENGTH];
+	BUG_ON(n > 3);
 	return (void*) address;
 }
 //#ifdef CONFIG_COMEDI_RT
